package com.amx.duet.tools.comm;

import java.io.File;
import java.util.Properties;

import com.amx.duet.core.master.SystemConstants;
import com.amx.duet.core.master.netlinx.DPS;
import com.amx.duet.core.master.netlinx.Event;
import com.amx.duet.core.master.netlinx.Level;
import com.amx.duet.core.util.FormatUtil;
import com.amx.duet.core.util.NetLinxSystemUtil;
import com.amx.duet.da.NetLinxDevice;
import com.amx.duet.da.SerialDevice;
import com.amx.duet.devicesdk.base.AdvancedEvent;
import com.amx.duet.devicesdk.base.Module;
import com.amx.duet.devicesdk.base.ModuleComponentEvent;
import com.amx.duet.devicesdk.base.TypeSafeEnumBase;
import com.amx.duet.devicesdk.component.IPreAmpComponent;
import com.amx.duet.devicesdk.type.SurroundMode;
import com.amx.duet.tools.lang.Logger;
import com.amx.duet.tools.lang.ValidateUtil;
import com.amx.duet.tools.net.tcp.SocketConnection;
import com.amx.duet.tools.net.tcp.SocketConnectionListener;
import com.amx.duet.tools.text.NumericUtil;
import com.amx.duet.util.Timeline;
import com.amx.duet.util.TimelineListener;
import com.amx.duet.util.Timer;
import com.amx.duet.util.TimerListener;

/**
 * The DeviceUtil class wraps up the standard behavior and common functions of 
 * a device module. It works in conjuction with the IDeviceListener interface, 
 * which is an optional interface but improves the flexibility of this class.
 *
 */
public class DeviceUtil implements TimelineListener, TimerListener, SocketConnectionListener
{
//	public static final int QUEUE_TIME_MIN = 100;		// 100th sec.
//	public static final int QUEUE_TIME_MAX = 10000;		// 10 sec.
	
//	public static final int HEART_TIME_MIN = 10000;		// 10 sec.
//	public static final int HEART_TIME_MAX = 60000;		// 1 min.

//	public static final int TIMEOUT_MIN = 1000;			// 1 sec.
//	public static final int TIMEOUT_MAX = 10000;		// 10 sec.
	
	// Common constants
	/**
	 * The minimum polling time in milliseconds.
	 */
	public static final int POLL_TIME_MIN = 1000; 		// 1 sec.
	/**
	 * The maximum polling time in milliseconds.
	 */
	public static final int POLL_TIME_MAX = 300000;		// 5 min.

	/**
	 * The minimum IP reconnect time in milliseconds.
	 */
	public static final int RECONNECT_TIME_MIN = 1000;	// 1 sec.
	/**
	 * The maximum IP reconnect time in milliseconds.
	 */
	public static final int RECONNECT_TIME_MAX = 60000;	// 1 min.

	/**
	 * The default polling time in milliseconds.
	 */
	public static final int DEFAULT_POLL_TIME = 10000;

	/**
	 * The default IP reconnect time in milliseconds.
	 */
	public static final int DEFAULT_CONNECT_TIME = 15000;
	
	/**
	 * The default queing time in milliseconds.
	 */
	public static final int DEFAULT_QUEUE_TIME = 200;

	/**
	 * The default heartbeat time in milliseconds.
	 */
	public static final int DEFAULT_HEART_TIME = 30000;

	/**
	 * The default timeout count.
	 */
	public static final int DEFAULT_TIMEOUT_COUNT = 3;
	
	/**
	 * 5 second HeartBeat Response timer interval
	 */
	public static final long TIME_OUT = 5000;
	
	/**
	 * String constant used for getting or setting the Baud Rate property of a 
	 * SerialDevice in conjunction with a NetLinx PROPERTY call.
	 */
	public final static String BAUD_RATE  = "Baud_Rate";
	
	/**
	 * String constant used for getting or setting the Password property for a 
	 * Device in conjunction with a NetLinx PROPERTY call.
	 */
	public final static String PASSWORD = "Password";
	
	/**
	 * String constant used for getting or setting the User Name property for a 
	 * Device in conjunction with a NetLinx PROPERTY call.
	 */
	public final static String USERNAME = "User_Name";
	
	/**
	 * String constant used for getting or setting the Poll Time property for a 
	 * Device in conjunction with a NetLinx PROPERTY call.
	 */
	public final static String POLL_TIME = "Poll_Time";
	
	/**
	 * String constant used for getting or setting the Device ID property of a 
	 * Device in conjunction with a NetLinx PROPERTY call.
	 */
	public final static String DEVICE_ID  = "Device_ID";
	
	/**
	 * String constant used for getting or setting the Port property of an IP
	 * Device in conjunction with a NetLinx PROPERTY call.
	 */
	public final static String IP_PORT  = "Port";
	
	/**
	 * String constant used for getting or setting the IP Address property of an 
	 * IP Device in conjunction with a NetLinx PROPERTY call.
	 */
	public final static String IP_ADDRESS = "IP_Address";

	/**
	 * String constant used for getting or setting the Timeout count property 
	 * that allows the device port to go offline, used in conjunction with a 
	 * NetLinx PROPERTY call.
	 */
	public final static String TIMEOUT_COUNT = "Timeout_Count";
	
	/**
	 * String constant used for getting or setting the IP Reconnect Time 
	 * property of an IP Device in conjunction with a NetLinx PROPERTY call.
	 */
	public final static String RECONNECT_TIME = "Reconnect_Time";

	/**
	 * String constant used for getting or setting the System Diagnostic 
	 * property of a Module for debugging a module with no device, used in 
	 * conjunction with a NetLinx PROPERTY call.
	 */
	public final static String SYSTEM_DIAGNOSTIC = "System_Diagnostic";

	/**
	 * int constant used to define an IP only device.
	 */
	public final static int IP_ONLY = 1;
	
	/**
	 * int constant used to define an IP/Serial combonation device.
	 */
	public final static int IP_SERIAL = 2;
	
	/**
	 * int constant used to define an IP/Serial combonation device.
	 */
	public final static int SERIAL_ONLY = 3;
	
	/**
	 * minimum level range value for modules.
	 */
	public static final int MIN_LVL = 0;
	
	/**
	 * maximum level range value for modules.
	 */
	public static final int MAX_LVL = 255;
	
	/**
	 * maximum decremental adjust value for modules.
	 */
	public static final int MIN_ADJUST = -255;
	
	/**
	 * maximum incremental adjust value for modules.
	 */
	public static final int MAX_ADJUST = 255;
	
	/**
	 * Similar to the Service.DEBUG value.
	 */
	public static final int DEBUG = 4;

	/**
	 * Similar to the Service.INFO value.
	 */
	public static final int INFO = 4;

	/**
	 * Similar to the Service.WARNING value.
	 */
	public static final int WARNING = 4;

	/**
	 * Similar to the Service.ERROR value.
	 */
	public static final int ERROR = 4;
	
	private Module m_mod = null;
	
	private boolean m_bOnline 			= false;					// indicates that communication with the device has been established
	private boolean m_bInitialized		= false;					// indicates global variables have been initialized
	
	private NetLinxDevice m_dvDevice 	= null;
	private SocketConnection ipConnection = null;

	private boolean m_bIPControlled   	= false;
	private boolean m_bConnecting     	= false;

	private long[] lReConnect       	= { DEFAULT_CONNECT_TIME }; // interval between connection retries
	private Timeline tReConnectTimeline = null;						// IP connection timeline
	
	private long[] lDeQueTime 			= { DEFAULT_QUEUE_TIME };	// 1/5 second dequeue timeline interval
	private Timeline tDeQueTimeline 	= null;						// dequeue outgoing messages timeline

	private long[] lPollTime 			= { DEFAULT_POLL_TIME };	// 10 sec.				
	private Timeline tPollTimeline 		= null;	

	private long[] lHeartBeatTime 		= { DEFAULT_HEART_TIME };	// 30 seconds HeartBeat timeline interval
	private Timeline tHeartBeatTimeline = null;		

	private Timer tResponseTimer 		= null;

	private int nMAX_TIMEOUT			= DEFAULT_TIMEOUT_COUNT;	// 3 HeartBeat Responses missed
	private int m_nTimeoutCount			= 0;
	
	private boolean m_bSystemDiag		= false;					// indicates that the module is in diagnostic mode.
	private boolean bQueueTimePaused	= false;
	private boolean bQueueLock 			= false;
	private Object sync = new Object();
	
	private String[] m_rates			= null;						// A list of all the baud rates available for this device.
	private int m_baud					= 9600;						// Default Baud rate
	private String m_ipAddress			= "255.255.255.255";		// Default IP Address

	Properties m_props					= null;
	
	private boolean bDisposing			= false;
	private boolean bPolling			= false;
	private boolean bHeartbeating		= false;
	private boolean bQueueing			= false;
	
	/**
	 * The Property class was created to allow a method to return a property
	 * key/value pair.
	 */
	public class Property
	{
		public String key = "";
		public String value = "";
	}
	
	/**
	 * Default constructor. Do not use.
	 */
	public DeviceUtil() {}
	
	/**
	 * @deprecated
	 * Constructs the DeviceUtil class.
	 * 
	 * @param mod The current module (device class) we are working with. Pass in
	 * <strong>this</strong> as the parameter.
	 */
	public DeviceUtil(Module mod)
	{
		setCommonDevice(mod);
	}
	
	/**
	 * Constructs and initializes the DeviceUtil class. This will determine 
	 * whether a device is IP or serial, create the necessary timelines, and
	 * set the required default properties.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 * 	DeviceUtil utilities = null;
	 * 	...
	 *	public MakeModel(BundleContext bctxt, NetLinxDevice nd, Properties props)
	 *	{
	 *		super(bctxt, nd, props);
	 *		try
	 *		{
	 *			setDebugState(ERROR);
	 *			utilities = new DeviceUtil(this, nd, props);
	 *			
	 *			// Required code for serial port devices
	 *			utilities.setValidBaudRates(VALID_RATES);
	 *			utilities.setDefaultBaudRate(NumericUtil.atoi(DEFAULT_RATE));
	 *			
	 *			// Required code for IP connection devices
	 *			utilities.setDefaultIPAddress(DEFAULT_IP);
	 *			
	 *			// TODO: User defined code goes here...
	 *		}
	 *		catch (Exception e)
	 *		{
	 *			System.out.println("Exception thrown in constructor: " + e.toString());
	 *			e.printStackTrace();
	 *		}
	 *	}
	 * </pre>
	 * 
	 * @param mod The current module (device class) we are working with. Pass in
	 * <strong>this</strong> as the parameter.
	 * @param nd The NetLinxDevice object passed in from the module constructor.
	 * @param props The Properties object passed in from the module constructor.
	 */
	public DeviceUtil(Module mod, NetLinxDevice nd, Properties props)
	{
		setCommonDevice(mod);
		m_props = props;
		
		initialize(nd);
	}
	
	/*
	 * This method will determine whether as device is IP or serial, create the 
	 * necessary timelines, and set the required default properties.
	 * 
	 * @param nd The NetLinxDevice object passed in from the module constructor.
	 */
	protected void initialize(NetLinxDevice nd)
	{
		if (m_mod == null)
		{
			error("Common device not found");
			return;
		}
		
		log(DEBUG, "DeviceUtil.initialize(nd) called");
		
		try
		{
			int nDeviceType = isIPControlled();
			if (nDeviceType == IP_ONLY)
			{
				enableIPControl(true);
			}
			else if (nDeviceType == IP_SERIAL)
			{
				if (nd != null && ValidateUtil.isValidDPS(nd.getDPS()))
					setNetLinxDevice(nd);
				else
					enableIPControl(true);
			}
			else if (nDeviceType == SERIAL_ONLY)
			{
				setNetLinxDevice(nd);
			}

			// Creating all timelines in the constructor is the best way to go....
			tHeartBeatTimeline = new Timeline(this, lHeartBeatTime, Timeline.RELATIVE, Timeline.INFINITE);
			tDeQueTimeline = new Timeline(this, lDeQueTime, Timeline.RELATIVE, Timeline.INFINITE);
			tPollTimeline = new Timeline(this, lPollTime, Timeline.RELATIVE, Timeline.INFINITE);
			
			if (isIPControlEnabled())
			{
				this.setProperty(RECONNECT_TIME, NumericUtil.ltoa(DEFAULT_CONNECT_TIME));
				tReConnectTimeline = new Timeline(this, lReConnect, Timeline.RELATIVE, Timeline.INFINITE);
				tReConnectTimeline.start();
			}
			
			this.setProperty(POLL_TIME, NumericUtil.ltoa(DEFAULT_POLL_TIME));
			this.setProperty(TIMEOUT_COUNT, NumericUtil.itoa(DEFAULT_TIMEOUT_COUNT));
			
		}
		catch (Exception e)
		{
			error("Exception in DeviceUtil initialize() - " + e.toString());
			if (m_mod.getDebugState() == DEBUG)
				e.printStackTrace();
		}
	}
	
	/**
	 * Sets the device with which functions contained in this package will 
	 * interact with.
	 * 
	 * @param mod object of type Module 
	 */
	public void setCommonDevice(Module mod) { m_mod = mod; }
	
	/**
	 * Returns the currently set device in package common.
	 * 
	 * @return device object of type Module
	 */
	public Module getCommonDevice() { return m_mod; }
	
	/**
	 * Used to determine if physical device is online,
	 * not the NetLinx port, such as a serial port.
	 * <p>
	 * This is not the same thing as the ONLINE/OFFLINE
	 * NetLinx data events.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	public boolean isDeviceOnLine()
	 *	{
	 *		return utilities.isDeviceOnLine();
	 *	}
	 * </pre>
	 * 
	 * @return true if the device is online; false otherwise.
	 */
	public boolean isDeviceOnLine() { return m_bOnline; }
	
	/**
	 * Used to determine if the cached module data has 
	 * been synchronized with the physical device.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	public boolean isDataInitialized()
	 *	{
	 *		return utilities.isDataInitialized();
	 *	}
	 * </pre>
	 * 
	 * @return true if the data is initialized; false otherwise.
	 */
	public boolean isDataInitialized() { return m_bInitialized; }
	
	/**
	 * Allows the user to add their own NetLinxDevice to this class.
	 * 
	 * @param nd NetLinxDevice
	 */
	public void setNetLinxDevice(NetLinxDevice nd) { m_dvDevice = nd; }

	/**
	 * Retrieve the current NetLinxDevice object stored in this class.
	 * 
	 * @return NetLinxDevice
	 */
	public NetLinxDevice getNetLinxDevice() { return m_dvDevice; }
	
	/**
	 * Sets the maximum number of heartbeat retries before the module 
	 * disconnects from the physical device.
	 * 
	 * @param count Maximum number of retries
	 */
	public void setMaxTimeout(int count) { nMAX_TIMEOUT = count; }
	
	/**
	 * Returns the maximum number of heartbeat retries before the module 
	 * disconnects from the physical device.
	 * 
	 * @return Maximum number of retries
	 */
	public int getMaxTimeout() { return nMAX_TIMEOUT; }
	
	/**
	 * Sets the current timeout count value. This value is used to determine 
	 * when the number of heartbeat retries has reached their maximum. This is 
	 * mainly used to reset the value back to zero when necessary.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 * 	// Handle a single response here after parsing it from your buffered 
	 *	// responses. In other words, don't parse the entire buffer in this
	 *	// method.
	 *	private void parseResponse(ByteBuffer data)
	 *	{
	 *		if (!this.isDeviceOnLine())
	 *			utilities.setOnlineState(true);
	 *		try
	 *		{
	 *			// TODO: User defined code goes here...
	 *		}
	 *		catch (Exception e)
	 *		{
	 *			if (this.getDebugState() == DEBUG)
	 *				e.printStackTrace();
	 *			log(ERROR, "Exception thrown in parseResponse: " + e.toString());
	 *		}
	 *		finally
	 *		{
	 *			// Reset outgoing message states here...
	 *			utilities.killResponseTimer();
	 *			utilities.setTimeoutCount(0);
	 *			utilities.lockQueue(false);
	 *		}
	 *	}
	 * </pre>
	 * 
	 * @param count Timeout count value.
	 */
	public void setTimeoutCount(int count)
	{
		m_nTimeoutCount = count;
	}
	
	/**
	 * Increments the current timeout count value. This value is used to 
	 * determine when the number of heartbeat retries has reached their maximum.
	 */
	public void incrementTimeout()
	{
		m_nTimeoutCount++;
	}
	
	/**
	 * Returns the current timeout count value. This value is used to determine 
	 * when the number of heartbeat retries has reached their maximum.
	 * 
	 * @return Timeout count value.
	 */
	public int getTimeoutCount()
	{
		return m_nTimeoutCount;
	}
	
	/**
	 * Set all the valid baud rates that the device can accept.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	public MakeModel(BundleContext bctxt, NetLinxDevice nd, Properties props)
	 *	{
	 *		super(bctxt, nd, props);
	 *		try
	 *		{
	 *			setDebugState(ERROR);
	 *			utilities = new DeviceUtil(this, nd, props);
	 *			
	 *			// Required code for serial port devices
	 *			utilities.setValidBaudRates(VALID_RATES);
	 *			utilities.setDefaultBaudRate(NumericUtil.atoi(DEFAULT_RATE));
	 *		}
	 *		catch (Exception e)
	 *		{
	 *			System.out.println("Exception thrown in constructor: " + e.toString());
	 *			e.printStackTrace();
	 *		}
	 *	}
	 * </pre>
	 * 
	 * @param rates String[] of baud rates.
	 */
	public void setValidBaudRates(String rates[]) { m_rates = rates; }
	
	/**
	 * Returns all the valid baud rates that the device can accept.
	 * 
	 * @return String[] of baud rates.
	 */
	public String[] getValidBaudRates() { return m_rates; }
	
	/**
	 * Sets the default communications baud rate for the device.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	public MakeModel(BundleContext bctxt, NetLinxDevice nd, Properties props)
	 *	{
	 *		super(bctxt, nd, props);
	 *		try
	 *		{
	 *			setDebugState(ERROR);
	 *			utilities = new DeviceUtil(this, nd, props);
	 *			
	 *			// Required code for serial port devices
	 *			utilities.setValidBaudRates(VALID_RATES);
	 *			utilities.setDefaultBaudRate(NumericUtil.atoi(DEFAULT_RATE));
	 *		}
	 *		catch (Exception e)
	 *		{
	 *			System.out.println("Exception thrown in constructor: " + e.toString());
	 *			e.printStackTrace();
	 *		}
	 *	}
	 * </pre>
	 * 
	 * @param rate int containing the baud rate.
	 */
	public void setDefaultBaudRate(int rate)
	{
		m_baud = rate;
		this.setProperty(BAUD_RATE, NumericUtil.itoa(m_baud));
	}

	/**
	 * Sets the default communications IP address for the device.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	public MakeModel(BundleContext bctxt, NetLinxDevice nd, Properties props)
	 *	{
	 *		super(bctxt, nd, props);
	 *		try
	 *		{
	 *			setDebugState(ERROR);
	 *			utilities = new DeviceUtil(this, nd, props);
	 *			
	 *			// Required code for IP connection devices
	 *			utilities.setDefaultIPAddress(DEFAULT_IP);
	 *		}
	 *		catch (Exception e)
	 *		{
	 *			System.out.println("Exception thrown in constructor: " + e.toString());
	 *			e.printStackTrace();
	 *		}
	 *	}
	 * </pre>
	 * 
	 * @param ipAddress String containing the IP address.
	 */
	public void setDefaultIPAddress(String ipAddress)
	{
		m_ipAddress = ipAddress;
		this.setProperty(IP_ADDRESS, ipAddress);
	}
	
	/**
	 * Sets the default communications IP address and port for the device.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	public MakeModel(BundleContext bctxt, NetLinxDevice nd, Properties props)
	 *	{
	 *		super(bctxt, nd, props);
	 *		try
	 *		{
	 *			setDebugState(ERROR);
	 *			utilities = new DeviceUtil(this, nd, props);
	 *			
	 *			// Required code for IP connection devices
	 *			utilities.setDefaultIPAddress(DEFAULT_IP, DEFAULT_PORT);
	 *		}
	 *		catch (Exception e)
	 *		{
	 *			System.out.println("Exception thrown in constructor: " + e.toString());
	 *			e.printStackTrace();
	 *		}
	 *	}
	 * </pre>
	 * 
	 * @param ipAddress String containing the IP address.
	 * @param port int containing the IP port.
	 */
	public void setDefaultIPAddress(String ipAddress, int port)
	{
		this.setDefaultIPAddress(ipAddress);
		this.setProperty(IP_PORT, NumericUtil.itoa(port));
	}
	
	/**
	 * Used to determine if the DeviceUtil class (Module) is trying to connect 
	 * to the device via IP sockets. The difference between this method and the 
	 * isIPControlled() method is that this value is determined run time based 
	 * on the isIPControlled() method and the the other is based on static 
	 * properties, that don't necessarily tell the module what action to take.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	protected boolean doNetLinxDeviceInitialization()
	 *	{
	 *		return (!utilities.isIPControlEnabled());
	 *	}
	 * </pre>
	 * <P>
	 * See {@link #reinitialize()} for usage as well.
	 * 
	 * @return true if the connection type is IP; false otherwise.
	 */
	public boolean isIPControlEnabled() { return m_bIPControlled; }

	/**
	 * Changes the IP control enabled flag.
	 * 
	 * @param enable boolean
	 */
	protected void enableIPControl(boolean enable) { m_bIPControlled = enable; }
	
	/**
	 * Used to determine if this class is currently disposing itself.
	 * @return true if disposing; false otherwise.
	 */
	public boolean isDisposing()
	{
		return bDisposing;
	}
	
	/**
	 * Used to determine if the poll timeline has been started.
	 * @return true if polling; false otherwise.
	 */
	public boolean isPolling()
	{
		return bPolling;
	}
	
	/**
	 * Used to determine if the heartbeat timeline has been started.
	 * @return true if heartbeating; false otherwise.
	 */
	public boolean isHeartbeating()
	{
		return bHeartbeating;
	}
	
	/**
	 * Used to determine if the queue timeline has been started.
	 * @return true if queing; false otherwise.
	 */
	public boolean isQueing()
	{
		return bQueueing;
	}
	
	/*
	 * Gets the property from either the module or the passed in properties 
	 * object. This method was created because the properties in the module are 
	 * not set until after the module's construction is complete.
	 * 
	 * @param key
	 * @return
	 */
	private String getProperty(String key) 
	{
		try
		{
			if (m_mod != null)
			{
				String value = m_mod.getProperty(key);
				log(DEBUG, "getProperty(" + key + ") = " + value);
				return value; 
			}
		}
		catch (Exception e)
		{
			if (m_props != null)
			{
				String value = m_props.getProperty(key);
				log(DEBUG, "getProperty(" + key + ") = " + value);
				return value; 
			}
		}
		
		return null;
	}
	
	/*
	 * Sets the property for either the module or the passed in properties 
	 * object. This method was created because the properties in the module are 
	 * not set until after the module's construction is complete.
	 * 
	 * @param key
	 * @param value
	 */
	private void setProperty(String key, String value)
	{
		try
		{
			if (m_mod != null)
			{
				m_mod.setProperty(key, value);
				log(DEBUG, "setProperty(" + key + "," + value + ")");
			}
		}
		catch (Exception e)
		{
			if (m_props != null)
			{
				m_props.setProperty(key, value);
				log(DEBUG, "setProperty(" + key + "," + value + ")");
			}
		}
	}
	
	/**
	 * Gets the correctly formatted property key.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 * 	public String getProperty(String key) 
	 * 	{
	 *		key = utilities.getPropertyKey(key);
	 *		return super.getProperty(key);
	 *	}
	 * </pre>
	 * 
	 * @param key String used for matching.
	 * @return The String formatted property key.
	 */
	public String getPropertyKey(String key)
	{
		// Check for mixed case issues
		if (key.compareToIgnoreCase(DeviceUtil.BAUD_RATE) == 0)
			key = DeviceUtil.BAUD_RATE;
		else if (key.compareToIgnoreCase(DeviceUtil.IP_ADDRESS) == 0)
			key = DeviceUtil.IP_ADDRESS;
		else if (key.compareToIgnoreCase(SystemConstants.IP_ADDRESS) == 0)
			key = DeviceUtil.IP_ADDRESS;
		else if (key.compareToIgnoreCase(DeviceUtil.IP_PORT) == 0)
			key = DeviceUtil.IP_PORT;
		else if (key.compareToIgnoreCase(DeviceUtil.POLL_TIME) == 0)
			key = DeviceUtil.POLL_TIME;
		else if (key.compareToIgnoreCase(DeviceUtil.DEVICE_ID) == 0)
			key = DeviceUtil.DEVICE_ID;
		else if (key.compareToIgnoreCase(DeviceUtil.TIMEOUT_COUNT) == 0)
			key = DeviceUtil.TIMEOUT_COUNT;
		else if (key.compareToIgnoreCase(DeviceUtil.RECONNECT_TIME) == 0)
			key = DeviceUtil.RECONNECT_TIME;
		else if (key.compareToIgnoreCase(DeviceUtil.SYSTEM_DIAGNOSTIC) == 0)
			key = DeviceUtil.SYSTEM_DIAGNOSTIC;
		
		return key;
	}
	
	/**
	 * Validate a property of the Module as a name/value pair.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	public void setProperty(String key, String value)
	 *	{
	 *		DeviceUtil.Property prop = utilities.validateProperty(key, value);
	 *		super.setProperty(prop.key, prop.value);
	 *	}
	 * </pre>
	 * 
	 * @param key String to lookup.
	 * @param value String to validate and set.
	 * @return Property object containing the key/value pair.
	 */
	public Property validateProperty(String key, String value)
	{
		Property prop = new Property();
		
		// Check for mixed case issues
		if (key.compareToIgnoreCase(DeviceUtil.BAUD_RATE) == 0)
		{
			prop.value = ValidateUtil.validateBaudRate(value, getValidBaudRates(), NumericUtil.itoa(m_baud));
			prop.key = DeviceUtil.BAUD_RATE;
		}
		else if (key.compareToIgnoreCase(DeviceUtil.IP_ADDRESS) == 0)
		{
			prop.value = value;
			if (!ValidateUtil.isValidIP(value))
				prop.value = m_ipAddress;
			prop.key = DeviceUtil.IP_ADDRESS;
		}
		else if (key.compareToIgnoreCase(SystemConstants.IP_ADDRESS) == 0)
		{
			prop.value = value;
			if (!ValidateUtil.isValidIP(value))
				prop.value = m_ipAddress;
			prop.key = DeviceUtil.IP_ADDRESS;
		}
		else if (key.compareToIgnoreCase(DeviceUtil.IP_PORT) == 0)
		{
			prop.value = value;
			prop.key = DeviceUtil.IP_PORT;
		}
		if (key.compareToIgnoreCase(DeviceUtil.POLL_TIME) == 0)
		{
			prop.value = ValidateUtil.validatePollTime(value, POLL_TIME_MIN, POLL_TIME_MAX, DEFAULT_POLL_TIME);
			prop.key = DeviceUtil.POLL_TIME;
		}
		else if (key.compareToIgnoreCase(DeviceUtil.DEVICE_ID) == 0)
		{
			prop.value = ValidateUtil.validateDeviceID(value, 0, 255, "00");
			prop.key = DeviceUtil.DEVICE_ID;
		}
		else if (key.compareToIgnoreCase(DeviceUtil.TIMEOUT_COUNT) == 0)
		{
			prop.value = ValidateUtil.validateTimeoutCount(value, getMaxTimeout(), DEFAULT_TIMEOUT_COUNT);
			prop.key = DeviceUtil.TIMEOUT_COUNT;
		}
		else if (key.compareToIgnoreCase(DeviceUtil.RECONNECT_TIME) == 0)
		{
			prop.value = ValidateUtil.validateReconnnectTime(value, RECONNECT_TIME_MIN, RECONNECT_TIME_MAX, DEFAULT_CONNECT_TIME);
			prop.key = DeviceUtil.RECONNECT_TIME;
		}
		else if (key.compareToIgnoreCase(DeviceUtil.SYSTEM_DIAGNOSTIC) == 0)
		{
			// Module must be restarted for these values to go away.
			if (NumericUtil.atoi(value) == 1)
			{
				m_bSystemDiag = true;
				m_bOnline = setOnlineState(true);
				m_bInitialized = setDataInitializedState(true);
			}
			else
			{
				m_bSystemDiag = false;
				m_bInitialized = setDataInitializedState(false);
			}
			prop.value = value;
			prop.key = DeviceUtil.SYSTEM_DIAGNOSTIC;
		}
		else
		{
			prop.value = value;
			prop.key = key;
		}
		
		return prop;
	}
	/**
	 * Returns the module diagnostic state. If the module is currently in diagnostics mode this
	 * method will return true. To place the module in diagnostics mode you must set the 
	 * <code>SYSTEM_DIAGNOSTIC</code> module property to 1 (0 will turn it off).
	 * @return
	 */
	public boolean isInDiagnosticMode() { return m_bSystemDiag; }
	/**
	 * <code>refresh</code> the client listeners with the module's cached state 
	 * of the physical device by callback notifications.  This method will not 
	 * query the physical device for its current state, but should update the 
	 * client with the module's cached device state via callback notifications. 
	 * In this case, it will only update the device online and data initialized
	 * states.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	public void refresh() 
	 *	{
	 *		utilities.refresh();
	 *		
	 *		// TODO: User defined code goes here...
	 *		super.refresh();
	 *	}
	 * </pre>
	 */
	public void refresh()
	{
		if (m_mod != null)
		{
			log(DEBUG, "DeviceUtil.refresh() called");
			m_mod.processDeviceOnLineEvent(new ModuleComponentEvent(m_mod, isDeviceOnLine(), 1));
			m_mod.processDataInitializedEvent(new ModuleComponentEvent(m_mod, isDataInitialized(), 1));
		}
	}
	
	/**
	 * During reinitialization volatile member variables are reset to their 
	 * default values, timers and timelines are stopped, sockets are closed, 
	 * values are updated from their set properties, and the data initialized
	 * state is set to false. The data initialized flag is reset so that the 
	 * module can have to time reastablish communication with the device and 
	 * resynchronize states. The device does not go offline at this time, so 
	 * that module is given enough chances to reastablish communications with 
	 * the device.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	public void reinitialize() 
	 *	{
	 *		utilities.reinitialize();
	 *		
	 *		// TODO: User defined code goes here...
	 *		queue.clearQue();
	 *		sbRxMsg.clear();		// incoming message buffer for physical device
	 *		sbLastMsg.clear();
	 *
	 *		// Call the base class method
	 *		super.reinitialize();
	 *		
	 *		// If this is not an IP connection treat it as if the serial device
	 *		// was trying to come back online. Otherwise, attempt to reconnect 
	 *		// the disconnected socket.
	 *		if (!utilities.isIPControlEnabled())
	 *			handleOnlineEvent();
	 *		else
	 *			utilities.reconnect();
	 *	}
	 * </pre>
	 */
	public void reinitialize()
	{
		log(DEBUG, "DeviceUtil.reinitialize() called");
		
		killResponseTimer();
		killHeartbeatTimeline();
		killQueueTimeline();
		stopPolling();
		
		if (isIPControlEnabled())
		{
			if (tReConnectTimeline != null)
				tReConnectTimeline.kill();
		}

		// Set the maximum timeout count
		setMaxTimeout(NumericUtil.atoi(this.getProperty(TIMEOUT_COUNT)));

		// Close the socket
		if (isIPControlEnabled() && isConnected())
			ipClose();

		if (isDataInitialized())
			m_bInitialized = setDataInitializedState(false);

		// Unlock the queue and reset the pause flag
		lockQueue(false);
		bQueueTimePaused = false;
	}

	/**
	 * This operation is used as a destructor to release resources. It will 
	 * close and dispose of any sockets and stop and destroy and timers and 
	 * timelines that may be running. 
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	public void dispose() 
	 *	{
	 *		utilities.dispose();
	 *		
	 *		// TODO: User defined code goes here...
	 *		
	 *		// Be sure to dispose all of the components you created that may 
	 *		// have their own threads and listeners going on.
	 *		
	 *		super.dispose();
	 *	}
	 * </pre>
	 */
	public void dispose()
	{
		log(DEBUG, "DeviceUtil.dispose() called");
		
		bDisposing = true;
		
		try
		{
			// for garbage collection the ipConnection must be disconnected, then disposed of, and the set to null.
			if (isIPControlEnabled() && ipConnection != null)
			{
				ipClose();
				ipConnection.dispose();
				ipConnection = null;
			}
		}
		catch(Exception e)
		{
			System.out.println("Exception in dispose()" + e.toString());
			e.printStackTrace();
		}
		finally
		{
			killResponseTimer();
			if (tHeartBeatTimeline != null)
			{
				tHeartBeatTimeline.kill();
				tHeartBeatTimeline = null;
			}
			if (tDeQueTimeline != null)
			{
				tDeQueTimeline.kill();
				tDeQueTimeline = null;
			}
			if (tPollTimeline != null)
			{
				tPollTimeline.kill();
				tPollTimeline = null;
			}
			if (isIPControlEnabled())
			{
				if (tReConnectTimeline != null)
				{
					tReConnectTimeline.kill();
					tReConnectTimeline = null;
				}
			}
		}
		
		bPolling = false;
		bHeartbeating = false;
		bQueueing = false;
	}
	
	/**
	 * Block/unblock the queue from sending any more message to the device.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	public void deQueue()
	 *	{
	 *		try
	 *		{
	 *			if (utilities.isQueueLocked() || queue.getQueSize() == 0)
	 *				return;
	 *	
	 *			// stop sending messages and queue them up until we're done
	 *			if (isDeviceOnLine())
	 *				utilities.lockQueue(true);
	 *
	 *			// Remove the command from the queue
	 *			sbLastMsg.append((byte[])queue.doDeq());
	 *			if (sbLastMsg != null && sbLastMsg.length() > 0)
	 *			{
	 *				// TODO: User defined code goes here...
	 *				
	 *				// Send the command to the device
	 *				log(DEBUG, "Comm ----------> device: BUFFER '" + sbLastMsg.toHex() + "'");
	 *				utilities.sendString(sbLastMsg.toByteArray());
	 *
	 *				// times out the heart beat message if a response is not received
	 *				utilities.startResponseTimer();
	 *			}
	 *
	 *			// Pause the dequeue timeline if there are no more items
	 *			if (queue.getQueSize() == 0)
	 *				utilities.pauseQueueTimeline();
	 *		}
	 *		catch (Exception e)
	 *		{
	 *			if (this.getDebugState() == DEBUG)
	 *				e.printStackTrace();
	 *			log(ERROR, "Exception in deQueue: " + e.toString());
	 *		}
	 *	}
	 * </pre>
	 * <P>
	 * See {@link #killResponseTimer()} for usage as well.
	 * 
	 * @param bLock
	 */
	public void lockQueue(boolean bLock)
	{
		log(DEBUG, "lockQueue(" + bLock + ")");
		
		synchronized(sync)
		{
			bQueueLock = bLock;
		}
	}
	
	/**
	 * Is the queue flag set to blocked.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	public void deQueue()
	 *	{
	 *		try
	 *		{
	 *			if (utilities.isQueueLocked() || queue.getQueSize() == 0)
	 *				return;
	 *	
	 *			// stop sending messages and queue them up until we're done
	 *			if (isDeviceOnLine())
	 *				utilities.lockQueue(true);
	 *
	 *			// Remove the command from the queue
	 *			sbLastMsg.append((byte[])queue.doDeq());
	 *			if (sbLastMsg != null && sbLastMsg.length() > 0)
	 *			{
	 *				// TODO: User defined code goes here...
	 *				
	 *				// Send the command to the device
	 *				log(DEBUG, "Comm ----------> device: BUFFER '" + sbLastMsg.toHex() + "'");
	 *				utilities.sendString(sbLastMsg.toByteArray());
	 *
	 *				// times out the heart beat message if a response is not received
	 *				utilities.startResponseTimer();
	 *			}
	 *
	 *			// Pause the dequeue timeline if there are no more items
	 *			if (queue.getQueSize() == 0)
	 *				utilities.pauseQueueTimeline();
	 *		}
	 *		catch (Exception e)
	 *		{
	 *			if (this.getDebugState() == DEBUG)
	 *				e.printStackTrace();
	 *			log(ERROR, "Exception in deQueue: " + e.toString());
	 *		}
	 *	}
	 * </pre>
	 * 
	 * @return true if the queue is to be locked; false otherwise.
	 */
	public boolean isQueueLocked()
	{
		return bQueueLock;
	}

	/**
	 * Make an IP socket connection.
	 * 
	 * @return true if the socket connection was successful; false otherwise.
	 */
	public boolean ipConnect()
	{
		// Do not allow timers to restart if disposing
		if (bDisposing)
			return false;
		
		if (m_mod == null)
		{
			error("Common device not found");
			return false;
		}
		
		// We don't want to connect to while we are already connected or
		// while we are trying to make a connection
		if (isConnected() || m_bConnecting)
			return false;
		
		log(DEBUG, "connecting socket...");
		
		m_bConnecting = true;
		
		String ipAddress = this.getProperty(IP_ADDRESS);
		int nPort = NumericUtil.atoi(this.getProperty(IP_PORT));
		
		if (ipAddress != null && ipAddress.length() > 0 && nPort > 0)
		{
			log(DEBUG, "connecting to " + ipAddress + ":" + nPort);
			if (ipConnection == null)
				ipConnection = new SocketConnection(ipAddress, nPort, 1024, this);
			else
				ipConnection.setAddressAndPort(ipAddress, nPort);
			
			return ipConnection.connect();
		}
		
		m_bConnecting = false;
		
		if (m_mod instanceof IDeviceListener)
			((IDeviceListener)m_mod).handleErrorEvent(SocketConnection.INVALID_HOST);
		return false;
	}
	
	/**
	 * Close an IP socket connection.
	 */
	public void ipClose()
	{
		log(DEBUG, "closing socket...");
		
		// Make sure the connection is not already closed
		if (isConnected())
			ipConnection.disconnect();
	}
	
	/**
	 * Determine if an IP connection is closed.
	 * 
	 * @return true if the socket is connected; false otherwise.
	 */
	public boolean isConnected()
	{
		boolean connected = false;
			
		if (ipConnection != null)
			connected = ipConnection.isConnected();
		
		return connected;
	}

	/**
	 * Restart the IP connection timeline.
	 */
	public void reconnect()
	{
		// Do not allow timers to restart if disposing
		if (bDisposing)
			return;
		
		log(DEBUG, "DeviceUtil.reconnect() called");
		
		if (isIPControlEnabled())
		{
			lReConnect[0] = NumericUtil.atol(this.getProperty(DeviceUtil.RECONNECT_TIME));
			tReConnectTimeline.reload(lReConnect);
			tReConnectTimeline.start();
		}
	}

	/**
	 * If this is not an IP connection treat it as if the serial device was 
	 * trying to come back online. Otherwise, attempt to reconnect the 
	 * disconnected socket.
	 */
	public void intitializeConnection()
	{
		if (!this.isIPControlEnabled())
		{
			if (m_mod != null && m_mod instanceof IDeviceListener)
				((IDeviceListener)m_mod).handleOnlineEvent();
		}
		else
		{
			this.reconnect();
		}
	}
	
	

	/**
	 * Method for handling incoming advanced events. This method is called when the
	 * underlying device receives an advanced command.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	public void handleAdvancedEvent(Event evt)
	 *	{
	 *		utilities.handleAdvancedEvent(evt);
	 *	}
	 * </pre>
	 * @param advancedEvent an Event object containing all necessary event information.
	 */
	public void handleAdvancedEvent(Event advancedEvent) {
		log(ERROR, "handleAdvancedEvent(" + advancedEvent +")" );

		if (m_mod == null)
		{
			error("Common device not found");
			return;
		}
		
		try
		{
			if (m_mod instanceof IDeviceListener)
			{
				switch(advancedEvent.type)
				{
					case Event.E_COMMAND: {
						if (advancedEvent.dataType == Event.D_STRING8)
						{
							DPS dps = advancedEvent.dps;
							int zone = dps.getPort();

							// parse string from virtual device and call appropriate function
							String commandMessage = new String( (byte[]) advancedEvent.dataValue );
							log( DEBUG, "Comm received from UI: " + commandMessage );							
//							Please note the following line of code is used to fix a problem with SNAPI
							handleSurroundCommand( commandMessage, zone );						
							handleSurroundModesCommand( commandMessage, zone );						
							break;
						}
					}
					default: {
						log( WARNING, "UNHANDLED ADVANCED EVENT: event.type: " + advancedEvent.type );
						break;
					}
				}
			} // if (m_mod instanceof IDeviceListener)
		}
		catch (Exception e)
		{
			log(ERROR, 
				"Exception in handleAdvancedEvent: " + 
				e.getLocalizedMessage());
			if (m_mod.getDebugState() == DEBUG)
				e.printStackTrace();
		}
		
	}


	
	/**
	 * 
	 * This command was implemented to handle the command that SNAPI should have called.
	 * Once SNAPI is corrected, this command should be removed.
	 * 
	 * @param commandString
	 * @param zone
	 */
	private void handleSurroundCommand(String commandString, int zone)
	{
		String command = "";
		int nIndex = commandString.indexOf("-");	// find command
		
		// if the "-" is not there, then assume that part is not in string and just find str length
		if (nIndex == -1) { nIndex = commandString.length()-1; }
		
		command = commandString.substring(0, nIndex+1).trim().toUpperCase();	// get the command (ex. "SURROUND_TEXT-" )
		commandString = commandString.substring(nIndex+1).trim();	    // remove command from the beginning of the message IF it's there

		if( command.startsWith("SURROUND-") ) {
			if( m_mod instanceof IPreAmpComponent ) {
				IPreAmpComponent preAmp = (IPreAmpComponent)m_mod;
				preAmp=preAmp.getPreAmpComponent(zone);
				log(ERROR, "Handling: " + command );
				log(ERROR, "commandString: " + commandString );
				if( commandString.length() > 0 ) {
					TypeSafeEnumBase surroundModeResult = TypeSafeEnumBase.fromString(commandString,preAmp.getSurroundModeValues());
					if( surroundModeResult instanceof SurroundMode ){
						preAmp.setSurroundMode( (SurroundMode)surroundModeResult );							
					} else {
						log(WARNING, "Warning: "+commandString+" is not a valid SurroundMode." );													
					}
				} else {
					log(WARNING, "Warning: SURROUND- command missing parameter" );						
				}
			}
		}
	}

	/**
	 * @param commandMessage
	 * @param zone
	 */
	private void handleSurroundModesCommand(String commandString, int zone)
	{
		log(ERROR, "handleSurroundModesCommand(" + commandString + ", " + zone+")" );

		String command = "";
		int nIndex = commandString.indexOf("-");	// find command
		
		// if the "-" is not there, then assume that part is not in string and just find str length
		if (nIndex == -1) { nIndex = commandString.length()-1; }
		
		command = commandString.substring(0, nIndex+1).trim().toUpperCase();	// get the command (ex. "SURROUND_TEXT-" )
		commandString = commandString.substring(nIndex+1).trim();	    // remove command from the beginning of the message IF it's there

		TypeSafeEnumBase surroundModeCommand = SurroundModesAdvancedEvent.fromString(command);
		log(ERROR, "sCmd: " + command + ", mode: " + surroundModeCommand );
		
		if( surroundModeCommand instanceof SurroundModesAdvancedEvent ) {
			if( m_mod instanceof IPreAmpComponent ) {
				Object preAmpObject = ((IPreAmpComponent)m_mod).getPreAmpComponent( zone );
				if( preAmpObject instanceof ISurroundModes ) {
					ISurroundModes surroundModeSupport = (ISurroundModes)preAmpObject;
					log(ERROR, "mode: " + surroundModeCommand );
					if( surroundModeCommand.equals(SurroundModesAdvancedEvent.SURROUND_MODE_COUNT_INQUIRY) ) {
						int surroundModeIndex = surroundModeSupport.getSurroundModeCount();;							
						surroundModeSupport.processSurroundModeCountEvent( surroundModeIndex );
					}
					if( surroundModeCommand.equals(SurroundModesAdvancedEvent.SURROUND_MODE_INQUIRY) ) {
						int surroundModeIndex = NumericUtil.ATOI( commandString );							
						String propertyValue = surroundModeSupport.getSurroundModeProperty( surroundModeIndex );
						surroundModeSupport.processSurroundModePropertyEvent( propertyValue );
					}
					if( surroundModeCommand.equals(SurroundModesAdvancedEvent.SURROUND_MODES_INQUIRY) ) {
						String[] props = surroundModeSupport.getSurroundModeProperties();
						surroundModeSupport.processSurroundModeProperties(props);
					}
					if( surroundModeCommand.equals(SurroundModesAdvancedEvent.SURROUND_MODE_SELECT_INQUIRY) ) {
						int value = surroundModeSupport.getSurroundModeSelect();
						surroundModeSupport.updateSurroundModeSelect(value);
					}
					if( surroundModeCommand.equals(SurroundModesAdvancedEvent.SURROUND_MODE_SELECT) ) {
						int surroundModeIndex = NumericUtil.ATOI( commandString );							
						surroundModeSupport.setSurroundModeSelect( surroundModeIndex );
					}
				}
			}
		}
	}

	/**
	 * Method for handling incoming data events. This method is called when the
	 * underlying device receives a data event.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	public void handleDataEvent(Event evt)
	 *	{
	 *		utilities.handleDataEvent(evt);
	 *	}
	 * </pre>
	 * @param evt an Event object containing all necessary event information.
	 */
	public void handleDataEvent(Event evt)
	{
		if (m_mod == null)
		{
			error("Common device not found");
			return;
		}
		
		try
		{
			NetLinxDevice dvDevice = this.getNetLinxDevice();

			// this handles incoming data from the physical device  
			// online, offline, string etc events
			if (!evt.getDPS().equals(dvDevice.getDPS()))
			{
				log(ERROR, "Invalid event - device = " + evt.dps);
				return;
			}
			
			if (m_mod instanceof IDeviceListener)
			{
				switch(evt.type)
				{
					case Event.E_ONLINE:
					{
						((IDeviceListener)m_mod).handleOnlineEvent();
						break; 
					}
					case Event.E_OFFLINE:
					{
						((IDeviceListener)m_mod).handleOfflineEvent();
						break;
					}
					case Event.E_ERROR: 
					{
						((IDeviceListener)m_mod).handleErrorEvent(0);
						break; 
					}
					case Event.E_STRING:
					{
						byte[] data = (byte[])evt.dataValue;
						this.handleReceive(data.length, data);
					}
				}
				
			} // if (m_mod instanceof IDeviceListener)
			
		}
		catch (Exception e)
		{
			log(ERROR, 
				"Exception in handleDataEvent: " + 
				e.getLocalizedMessage());
			if (m_mod.getDebugState() == DEBUG)
				e.printStackTrace();
		}
		
	}
	
	/**
	 * Handle the device online event.
	 * <P> 
	 * If the module is using a serial port to communicate with a device then 
	 * this method will call back to the module so that the module can decide 
	 * it's own port settings. If the module is connected via an IP socket then 
	 * the timeline to reconnect the socket will be stopped here. The last thing 
	 * this method does is start both the queue and heartbeat timelines.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	public void handleOnlineEvent()
	 *	{
	 *		utilities.handleOnlineEvent();
	 *	}
	 * </pre>
	 */
	public void handleOnlineEvent()
	{
		// Do not allow timers to restart if disposing
		if (bDisposing)
			return;
		
		log(DEBUG, "DeviceUtil.handleOnlineEvent() called");
		
		if (!isIPControlEnabled())
		{
			if (m_dvDevice == null)
			{
				error("NetLinxDevice not found");
				return;
			}
			
			// Call the module to setup the serial port...
			if (m_mod instanceof IDeviceListener)
			{
				((IDeviceListener)m_mod).setCommPort();
			}
		}
		
		if (isIPControlEnabled())
		{
			if (tReConnectTimeline != null)
				tReConnectTimeline.kill();
			
			m_bConnecting = false;
		}
		
		// Startup all of your timers
		startQueueTimeline();
		startHeartbeatTimeline();
		
		((IDeviceListener)m_mod).heartbeatEvent();
	}
	
	/**
	 * Handle the device offline event.
	 * <P>
	 * If the device online state is set to true, then the timeout count will be 
	 * incremented until it reaches the max count. When it reaches its max it 
	 * set the device online state to false. This gives the module enough time 
	 * to try to reastablish communications with the device before (in the case 
	 * of Device Discovery) completely shutting down. The data initialized state 
	 * will be set to false as well as any connecting attempts. The last thing 
	 * this methods does is to call reinitialize in order to clear out any 
	 * erraneous data and try to reconnect to the device.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	public void handleOfflineEvent()
	 *	{
	 *		utilities.handleOfflineEvent();
	 *	}
	 * </pre>
	 */
	public void handleOfflineEvent()
	{
		log(DEBUG, "DeviceUtil.handleOfflineEvent() called");
		
		if (isDeviceOnLine())
		{
			incrementTimeout();
			if (getTimeoutCount() >= getMaxTimeout())
			{
				log(INFO, "Setting device offline.");

				setOnlineState(false);
				setTimeoutCount(0);
			}
		}
		
		if (isDataInitialized())
			setDataInitializedState(false);
		
		if (isIPControlEnabled())
		{
			log(INFO, "Socket closed by peer");
			
			m_bConnecting = false;
		}
		
		if (m_mod != null)
			m_mod.reinitialize();
	}
	
	/**
	 * Handle the device error event.
	 * <P>
	 * This method will log error passed in to the telnet session. It will also 
	 * close an IP socket if one is available or call the module's 
	 * handleOfflineEvent() if available.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	public void handleErrorEvent(int error)
	 *	{
	 *		utilities.handleErrorEvent(error);
	 *	}
	 * </pre>
	 */
	public void handleErrorEvent(int error)
	{
		log(DEBUG, "DeviceUtil.handleErrorEvent(" + error + ")");
		
		switch (error)
		{
			case SocketConnection.TIMEOUT:
			{
				log(ERROR,"handleErrorEvent: socket timed out");
				break;
			}
			case SocketConnection.FAILED_CONNECT:
			{
				log(ERROR,"handleErrorEvent: socket failed to connect");
				break;
			}
			default:
			{
				log(ERROR,"handleErrorEvent: unknown error - status [" + error + "]");
				break;
			}
		}
		
		// If the socket is not closed or was never open close it
		if (isIPControlEnabled() && isConnected())
		{
			ipClose();
		}
		// Otherwise, handle the error like a closed socket
		else if (m_mod instanceof IDeviceListener)
		{
			((IDeviceListener)m_mod).handleOfflineEvent();
		}
	}

	/* (non-Javadoc)
	 * @see com.amx.duet.util.TimelineListener#handleTimelineEvent(com.amx.duet.util.Timeline)
	 */
	public void handleTimelineEvent(Timeline timeline)
	{
		if (m_mod instanceof IDeviceListener)
		{
			if (timeline == tDeQueTimeline)
			{
				((IDeviceListener)m_mod).deQueue();
			}
			else if(timeline == tPollTimeline) //query all
			{
				((IDeviceListener)m_mod).pollEvent();
			}
			else if(timeline == tHeartBeatTimeline)
			{
				// put heartbeat message on the queue here...
				((IDeviceListener)m_mod).heartbeatEvent();
			}
		}
		
		if (timeline == tReConnectTimeline)
		{
			if (isConnected() || m_bConnecting) // redundant check
				return;
			
			ipConnect();
		}
		else if (m_mod instanceof TimelineListener)
		{
			((TimelineListener)m_mod).handleTimelineEvent(timeline);
		}
		
	}

	/* (non-Javadoc)
	 * @see com.amx.duet.util.TimerListener#handleTimerEvent(com.amx.duet.util.Timer)
	 */
	public void handleTimerEvent(Timer timer)
	{
		// The response timer timed out, which means communications were probably 
		// severed, so try to reestablish communications.
		if (timer == tResponseTimer)
		{
			log(INFO, "Response timer timed out.");
			
			tResponseTimer = null; // Make sure you do this here...
			lockQueue(false);
			
			if (isDeviceOnLine())
			{
				incrementTimeout();
				if (getTimeoutCount() >= getMaxTimeout())
				{
					setTimeoutCount(0);
					if (m_mod != null)
						m_mod.reinitialize(); // since build 2.0.36
				}
			}
			
		}
		
	}

	/* (non-Javadoc)
	 * @see com.amx.duet.tools.jupiter.fusion.net.tcp.dr1_0_0.SocketConnectionListener#handleReceive(int, byte[])
	 */
	public void handleReceive(int bytesReceived, byte[] data)
	{
		if (m_mod != null)
			m_mod.processPassBackEvent(new ModuleComponentEvent(m_mod, new String(data, 0, bytesReceived), 1));
		
		if (m_mod instanceof IDeviceListener)
		{
			((IDeviceListener)m_mod).handleIncomingData(bytesReceived, data);
		}
		else if (m_mod instanceof SocketConnectionListener)
		{
			((SocketConnectionListener)m_mod).handleReceive(bytesReceived, data);
		}
		
	}

	/* (non-Javadoc)
	 * @see com.amx.duet.tools.jupiter.fusion.net.tcp.dr1_0_0.SocketConnectionListener#handleSocketStatus(int)
	 */
	public void handleSocketStatus(int status)
	{
		log(DEBUG, "DeviceUtil.handleSocketStatus(" + status + ") called");
		
		if (m_mod instanceof IDeviceListener)
		{
			switch (status)
			{
				case SocketConnection.CLOSED:
				{
					((IDeviceListener)m_mod).handleOfflineEvent();
					break;
				}	
				case SocketConnection.CONNECTED:
				{
					((IDeviceListener)m_mod).handleOnlineEvent();
					break;
				}	
				default:
				{
					((IDeviceListener)m_mod).handleErrorEvent(status);
					break;
				}
			}
		}
		else if (m_mod instanceof SocketConnectionListener)
		{
			((SocketConnectionListener)m_mod).handleSocketStatus(status);
		}
	}

	/**
	 * Kills the running resonse timeout timer.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 * 	// Handle a single response here after parsing it from your buffered 
	 *	// responses. In other words, don't parse the entire buffer in this
	 *	// method.
	 *	private void parseResponse(ByteBuffer data)
	 *	{
	 *		if (!this.isDeviceOnLine())
	 *			utilities.setOnlineState(true);
	 *		try
	 *		{
	 *			// TODO: User defined code goes here...
	 *		}
	 *		catch (Exception e)
	 *		{
	 *			if (this.getDebugState() == DEBUG)
	 *				e.printStackTrace();
	 *			log(ERROR, "Exception thrown in parseResponse: " + e.toString());
	 *		}
	 *		finally
	 *		{
	 *			// Reset outgoing message states here...
	 *			utilities.killResponseTimer();
	 *			utilities.setTimeoutCount(0);
	 *			utilities.lockQueue(false);
	 *		}
	 *	}
	 * </pre>
	 */
	public void killResponseTimer()
	{
		if (tResponseTimer != null)
			tResponseTimer = stopTimer(tResponseTimer, "tResponseTimer");
	}

	/**
	 * Starts the response timeout timer.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	public void deQueue()
	 *	{
	 *		try
	 *		{
	 *			if (utilities.isQueueLocked() || queue.getQueSize() == 0)
	 *				return;
	 *	
	 *			// stop sending messages and queue them up until we're done
	 *			if (isDeviceOnLine())
	 *				utilities.lockQueue(true);
	 *
	 *			// Remove the command from the queue
	 *			sbLastMsg.append((byte[])queue.doDeq());
	 *			if (sbLastMsg != null && sbLastMsg.length() > 0)
	 *			{
	 *				// TODO: User defined code goes here...
	 *				
	 *				// Send the command to the device
	 *				log(DEBUG, "Comm ----------> device: BUFFER '" + sbLastMsg.toHex() + "'");
	 *				utilities.sendString(sbLastMsg.toByteArray());
	 *
	 *				// times out the heart beat message if a response is not received
	 *				utilities.startResponseTimer();
	 *			}
	 *
	 *			// Pause the dequeue timeline if there are no more items
	 *			if (queue.getQueSize() == 0)
	 *				utilities.pauseQueueTimeline();
	 *		}
	 *		catch (Exception e)
	 *		{
	 *			if (this.getDebugState() == DEBUG)
	 *				e.printStackTrace();
	 *			log(ERROR, "Exception in deQueue: " + e.toString());
	 *		}
	 *	}
	 * </pre>
	 */
	public void startResponseTimer()
	{
		if (tResponseTimer == null)
			tResponseTimer = startTimer("tResponseTimer", TIME_OUT);
	}
	
	/**
	 * Starts the response timeout timer with the specified amount of time.
	 * 
	 * @param lTimeout Amount of time before timer times out.
	 */
	public void startResponseTimer(long lTimeout)
	{
		if (tResponseTimer == null)
			tResponseTimer = startTimer("tResponseTimer", lTimeout);
	}

	/**
	 * Allows you to change the amount of time between each queue interval.
	 * 
	 * @param lTime long
	 */
	public void setDeQueueTime(long lTime)
	{
		if (lTime != lDeQueTime[0])
		{
			lDeQueTime[0] = lTime;
			restartTimeline(tDeQueTimeline, lDeQueTime);
			bQueueing = true;
			bQueueTimePaused = false;
		}
	}
	
	/**
	 * Returns current amount of time between each queue interval.
	 * 
	 * @return long
	 */
	public long getDeQueueTime()
	{
		return lDeQueTime[0];
	}

	/**
	 * Pauses the current queue timeline making sure the timeline is not already
	 * paused.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	public void deQueue()
	 *	{
	 *		try
	 *		{
	 *			if (utilities.isQueueLocked() || queue.getQueSize() == 0)
	 *				return;
	 *	
	 *			// stop sending messages and queue them up until we're done
	 *			if (isDeviceOnLine())
	 *				utilities.lockQueue(true);
	 *
	 *			// Remove the command from the queue
	 *			sbLastMsg.append((byte[])queue.doDeq());
	 *			if (sbLastMsg != null && sbLastMsg.length() > 0)
	 *			{
	 *				// TODO: User defined code goes here...
	 *				
	 *				// Send the command to the device
	 *				log(DEBUG, "Comm ----------> device: BUFFER '" + sbLastMsg.toHex() + "'");
	 *				utilities.sendString(sbLastMsg.toByteArray());
	 *
	 *				// times out the heart beat message if a response is not received
	 *				utilities.startResponseTimer();
	 *			}
	 *
	 *			// Pause the dequeue timeline if there are no more items
	 *			if (queue.getQueSize() == 0)
	 *				utilities.pauseQueueTimeline();
	 *		}
	 *		catch (Exception e)
	 *		{
	 *			if (this.getDebugState() == DEBUG)
	 *				e.printStackTrace();
	 *			log(ERROR, "Exception in deQueue: " + e.toString());
	 *		}
	 *	}
	 * </pre>
	 */
	public void pauseQueueTimeline()
	{
		log(DEBUG, "DeviceUtil.pauseQueueTimeline() called");
		
		if (tDeQueTimeline != null && bQueueing && !bQueueTimePaused)
		{
			tDeQueTimeline.pause();
			bQueueTimePaused = true;
		}
	}
	
	/**
	 * Restarts the currently paused queue timeline making sure the timeline was
	 * paused.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	public void enQueue(byte msg[])
	 *	{
	 *		synchronized (queue)
	 *		{
	 *			// Restart the dequeue timeline if it was paused
	 *			utilities.restartQueueTimeline();
	 *			queue.doEnq(msg);
	 *		}
	 *	}
	 * </pre>
	 */
	public void restartQueueTimeline()
	{
		log(DEBUG, "DeviceUtil.restartQueueTimeline() called");
		
		if (tDeQueTimeline != null && bQueueing && bQueueTimePaused)
		{
			tDeQueTimeline.restart();
			bQueueTimePaused = false;
		}
	}

	/**
	 * Stops the running queue timeline.<br>
	 * <strong>NOTE:</strong> This method should only be called if absolutely 
	 * necessary, because this is handled internally within this class.
	 */
	public void killQueueTimeline()
	{
		if (tDeQueTimeline != null)
		{
			tDeQueTimeline.kill();
			bQueueing = false;
		}
	}
	
	/**
	 * Start the queue timeline if it is not already running.<br>
	 * <strong>NOTE:</strong> This method should only be called if absolutely 
	 * necessary, because this is handled internally within this class.
	 */
	public void startQueueTimeline()
	{
		if (tDeQueTimeline != null && !bQueueing)
		{
			tDeQueTimeline.start();
			
			log(DEBUG, "Queue started...");
			bQueueTimePaused = false;
			bQueueing = true;
		}
	}

	/**
	 * Stops the running heartbeat timeline.<br>
	 * <strong>NOTE:</strong> This method should only be called if absolutely 
	 * necessary, because this is handled internally within this class.
	 */
	public void killHeartbeatTimeline()
	{
		if (tHeartBeatTimeline != null)
		{
			tHeartBeatTimeline.kill();
			bHeartbeating = false;
		}
	}
	
	/**
	 * Start the heartbeat timeline if it is not already running.<br>
	 * <strong>NOTE:</strong> This method should only be called if absolutely 
	 * necessary, because this is handled internally within this class.
	 */
	public void startHeartbeatTimeline()
	{
		if (tHeartBeatTimeline != null && !bHeartbeating)
		{
			tHeartBeatTimeline.start();
			
			log(DEBUG, "Heartbeat started...");
			bHeartbeating = true;
		}
	}

	/**
	 * Starts the polling timeline for devices that require polling. This should
	 * be done after the physical device has been initialized and synched with 
	 * the module.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	private void parseResponse(ByteBuffer data)
	 *	...
	 * 	if (!this.isDataInitialized())
	 * 	{
	 * 		utilities.setDataInitializedState(true);
	 * 		refresh();
	 * 		utilities.startPolling();
	 * 	}
	 * 	...
	 * </pre>
	 */
	public void startPolling()
	{
		// Do not allow timers to restart if disposing
		if (bDisposing)
			return;
		
		if (tPollTimeline != null && !bPolling)
		{
			lPollTime[0] = NumericUtil.atol(this.getProperty(DeviceUtil.POLL_TIME));
			tPollTimeline.reload(lPollTime);
			tPollTimeline.start();
			
			log(DEBUG, "Polling started...");
			bPolling = true;
		}
	}
	
	/**
	 * Stops the polling timeline for devices that require polling.
	 */
	public void stopPolling()
	{
		if (tPollTimeline != null)
		{
			tPollTimeline.kill();
			bPolling = false;
		}
	}
	
	/**
	 * @deprecated
	 * @see {@link sendCommandFeedback(String, AdvancedEvent, int, NetLinxDevice)}
	 * @see {@link sendCommandFeedback(String, AdvancedEvent, int, DPS)}
	 * Send string feedback back thru the virtual device to the UI.
	 * 
	 * @param msg is the response to return to the UI
	 * @param zone is the instantiation to which the response should be sent
	 * @param advEv contains information for Java users
	 * @param device is the virtual device to which the response should be sent
	 * Note: device MUST BE A 'NETLINXDEVICE' TYPE OBJECT! 'THIS' WILL NOT WORK! 
	 */
	public void fnSendStringFeedback(String msg, AdvancedEvent advEv, int zone, Object device)
	{
		sendCommandFeedback(msg, advEv, zone, (NetLinxDevice) device);
	}
	
	/**
	 * Send string feedback back thru the virtual device to the UI.
	 * 
	 * @param msg is the response to return to the UI
	 * @param zone is the instantiation to which the response should be sent
	 * @param advEv contains information for Java users
	 * @param device is the virtual device to which the response should be sent
	 */
	public void sendCommandFeedback(String msg, AdvancedEvent advEv, int zone, NetLinxDevice device)
	{
		DPS dps = null;
		try
		{
			if (device != null)
				dps = device.getDPS();
		}
		catch (Exception e) {}
		sendCommandFeedback(msg, advEv, zone, dps);
	}
	
	/**
	 * Send string feedback back thru the virtual device to the UI.
	 * 
	 * @param msg is the response to return to the UI
	 * @param zone is the instantiation to which the response should be sent
	 * @param advEv contains information for Java users
	 * @param dps is the virtual device to which the response should be sent
	 */
	public void sendCommandFeedback(String msg, AdvancedEvent advEv, int zone, DPS dps)
	{
		if (m_mod == null)
		{
			error("Common device not found");
			return;
		}
		
		Event e = new Event(dps);
		e.dps = dps;
		e.type = Event.E_COMMAND;
		e.dataType = Event.D_STRING8;
		e.dataValue = msg.getBytes();
		m_mod.processAdvancedEvent(new ModuleComponentEvent(m_mod, advEv, e, zone));
	}

	/**
	 * @deprecated
	 * @see {@link sendLevelFeedback(int, int, AdvancedEvent, int, NetLinxDevice)}
	 * @see {@link sendLevelFeedback(int, int, AdvancedEvent, int, DPS)}
	 * Send level feedback back thru the virtual device to the UI.
	 * 
	 * @param levelID - level number
	 * @param levelValue - level value
	 * @param advEv - contains information for Java users
	 * @param zone - the instantiation to which the response should be sent
	 * @param device - the virtual device to which the response should be sent
	 * Note: device MUST BE A 'NETLINXDEVICE' TYPE OBJECT! 'THIS' WILL NOT WORK! 
	 */
	public void fnSendLevelFeedback(int levelID, int levelValue, AdvancedEvent advEv, int zone, Object device)
	{
		sendLevelFeedback(levelID, levelValue, advEv, zone, (NetLinxDevice)device);
	}
	
	/**
	 * Send level feedback back thru the virtual device to the UI.
	 * 
	 * @param levelID level number
	 * @param levelValue level value
	 * @param advEv contains information for Java users
	 * @param zone the instantiation to which the response should be sent
	 * @param device is the virtual device to which the response should be sent
	 */
	public void sendLevelFeedback(int levelID, int levelValue, AdvancedEvent advEv, int zone, NetLinxDevice device)
	{
		DPS dps = null;
		try
		{
			if (device != null)
				dps = device.getDPS();
		}
		catch (Exception e) {}
		sendLevelFeedback(levelID, levelValue, advEv, zone, dps);
	}
	
	/**
	 * Send level feedback back thru the virtual device to the UI.
	 * 
	 * @param levelID level number
	 * @param levelValue level value
	 * @param advEv contains information for Java users
	 * @param zone the instantiation to which the response should be sent
	 * @param dps is the virtual device to which the response should be sent
	 */
	public void sendLevelFeedback(int levelID, int levelValue, AdvancedEvent advEv, int zone, DPS dps)
	{
		if (m_mod == null)
		{
			error("Common device not found");
			return;
		}
		
		Event e = new Event(dps);
		e.dps = dps;
		e.type = Event.E_LEVEL;
		e.idx = levelID;
		e.dataType = Event.D_LEVEL;
		e.dataValue = new Level(levelValue);
		m_mod.processAdvancedEvent(new ModuleComponentEvent(m_mod, advEv, e, zone));
	}

	/**
	 * @deprecated
	 * @see {@link sendChannelFeedback(int, boolean, AdvancedEvent, int, NetLinxDevice)}
	 * @see {@link sendChannelFeedback(int, boolean, AdvancedEvent, int, DPS)}
	 * Sends feedback for the specified channel.
	 * 
	 * @param chanID - integer, channel number
	 * @param chanValue - boolean, channel state
	 * @param advEv - AdvancedEvent Object
	 * @param zone - integer, port to return the data on
	 * @param device - the virtual device to which the response should be sent
	 * Note: device MUST BE A 'NETLINXDEVICE' TYPE OBJECT! 'THIS' WILL NOT WORK! 
	 */
	public void fnSendChannelFeedback(int chanID, boolean chanValue, AdvancedEvent advEv, int zone, Object device)
	{
		sendChannelFeedback(chanID, chanValue, advEv, zone, (NetLinxDevice)device);
	}
		
	/**
	 * Sends feedback for the specified channel.
	 * 
	 * @param chanID integer, channel number
	 * @param state boolean, channel state
	 * @param advEv AdvancedEvent Object
	 * @param zone integer, port to return the data on
	 * @param device the virtual device to which the response should be sent
	 */
	public void sendChannelFeedback(int chanID, boolean state, AdvancedEvent advEv, int zone, NetLinxDevice device)
	{
		DPS dps = null;
		try
		{
			if (device != null)
				dps = device.getDPS();
		}
		catch (Exception e) {}
		sendChannelFeedback(chanID, state, advEv, zone, dps);
	}
	
	/**
	 * Sends feedback for the specified channel.
	 * 
	 * @param chanID integer, channel number
	 * @param state boolean, channel state
	 * @param advEv AdvancedEvent Object
	 * @param zone integer, port to return the data on
	 * @param dps the virtual device to which the response should be sent
	 */
	public void sendChannelFeedback(int chanID, boolean state, AdvancedEvent advEv, int zone, DPS dps)
	{
		if (m_mod == null)
		{
			error("Common device not found");
			return;
		}
		
		Event e = new Event(dps);
		e.dps = dps;
		e.type = (state)?Event.E_ON:Event.E_OFF;
		e.idx = chanID;
		m_mod.processAdvancedEvent(new ModuleComponentEvent(m_mod, advEv, e, zone));
	}
	
	/**
	 * @deprecated make sure {@link #getPropertyKey(String)} and 
	 * {@link #validateProperty(String, String)} are in your module's overriden 
	 * getProperty and setProperty methods respectively.
	 * 
	 * Retrieve the baud rate from the properties bundle that is part of the
	 * device that is set in setCommonDevice.
	 * 
	 * @param sDefaultRate - String containing the device's default baud rate
	 * @param sValidRates - an array of Strings containing all valid baud rates for the device
	 * @return String containing the baud rate retrieved from the bundle, or the default value
	 */
	public String fnGetBaudRate(String sDefaultRate, String[] sValidRates)
	{
		if (m_mod == null)
		{
			error("Common device not found");
			return null;
		}
		
		String rate = this.getProperty(BAUD_RATE);
		String newrate = ValidateUtil.validateBaudRate(rate, sValidRates, sDefaultRate);
		if (!rate.equals(newrate))
		{
			log(DEBUG, "Changing Baud rate to " + newrate);
			this.setProperty(BAUD_RATE, newrate);
		}
		return newrate;
	}
	
	/**
	 * @deprecated make sure {@link #getPropertyKey(String)} and 
	 * {@link #validateProperty(String, String)} are in your module's overriden 
	 * getProperty and setProperty methods respectively.
	 * 
	 * Retrieve the default device ID from the properties bundle that is part of the
	 * device that is set in setCommonDevice.
	 * Note: This device ID is ONLY the device ID indicated in the physical device's protocol,
	 * which may be necessary for correct command formatting.  This is NOT the same as the device beacon ID.
	 * 
	 * @param sDefaultID - String containing the device's default ID
	 * @param nMinID - integer, represents the smallest valid ID
	 * @param nMaxID - integer, represents the largest valid ID
	 * @return byte containing the device ID retrieved from the bundle, or the default value
	 */
	public byte fnGetDeviceID(String sDefaultID, int nMinID, int nMaxID)
	{
		if (m_mod == null)
		{
			error("Common device not found");
			return 0;
		}
		
		String id = this.getProperty(DEVICE_ID);
		String newID = ValidateUtil.validateDeviceID(id, nMinID, nMaxID, sDefaultID);
		if (!id.equals(newID))
		{
			log(DEBUG, "Invalid value for the Device ID Property. Setting default.");
			this.setProperty(DEVICE_ID, newID);
		}
		return (byte)NumericUtil.ATOI(newID);
	}

	/**
	 * Changes the state of the bOnline variable and triggers the event.
	 * 
	 * @param state is the boolean value to be set
	 * 
	 * @return boolean value sent to the processDeviceOnlineEvent
	 */
	public boolean setOnlineState(boolean state)
	{
		if (m_mod == null)
		{
			error("Common device not found");
			return false;
		}
		
		if (m_mod.isDeviceOnLine() != state)
		{
			// Don't let the state reset...
			if (m_bSystemDiag && !state)
				return true;
				
			m_bOnline = state;
			m_mod.processDeviceOnLineEvent(new ModuleComponentEvent(m_mod, state, 1));
		}
		return state;
	}
	
	/**
	 * Changes the state of bInitialized and triggers the event.
	 * 
	 * @param state is the boolean value to be set
	 * 
	 * @return boolean value sent to the processDataInitializedEvent
	 */
	public boolean setDataInitializedState(boolean state)
	{
		if (m_mod == null)
		{
			error("Common device not found");
			return false;
		}
		
		if (m_mod.isDataInitialized() != state)
		{
//			 Don't let the state reset...
			if (m_bSystemDiag && !state)
				return true;
				
			m_bInitialized = state;
			m_mod.processDataInitializedEvent(new ModuleComponentEvent(m_mod, state, 1));
		}
		return state;
	}

	/**
	 * @deprecated make sure {@link #getPropertyKey(String)} and 
	 * {@link #validateProperty(String, String)} are in your module's overriden 
	 * getProperty and setProperty methods respectively.
	 * 
	 * Get the currently stored IP address in properties. If one has not been set yet, store the 
	 * default IP address in properties and return it.
	 * 
	 * @param defaultIP String containing the default IP address
	 * @return String
	 */
	public String getIPAddress(String defaultIP)
	{
		if (m_mod == null)
		{
			error("Common device not found");
			return null;
		}
		
		String addr = this.getProperty(IP_ADDRESS);
		if (addr != null && addr.length() > 0)
			return addr;
			
		addr = this.getProperty(SystemConstants.IP_ADDRESS);
		if (addr != null && addr.length() > 0)
			return addr;

		log(DEBUG, "Invalid value for the IP Address Property. Setting default.");
		this.setProperty(IP_ADDRESS, defaultIP);
		return defaultIP;
	}
	
	/**
	 * Checks the module properties to determine if this is an IP controlled device.
	 * 
	 * @param props
	 * @return 
	 * <UL>
	 * <LI>0 = not IP controlled
	 * <LI>1 = definately IP controlled
	 * <LI>2 = could be IP controlled or serial controlled
	 * </UL>
	 * @throws IllegalArgumentException Occurs when the DEVICE_CATEGORY field is empty
	 * @throws NullPointerException Occurs when the DEVICE_CATEGORY field returns null
	 */
	public int isIPControlled() throws IllegalArgumentException, NullPointerException
	{
		int nIPType = 0;
		
		if (m_mod != null)
		{
			String deviceType = this.getProperty(SystemConstants.DEVICE_CATEGORY);
			if (deviceType != null)
			{
				if (deviceType.length() > 0)
				{
					deviceType = deviceType.toUpperCase();
					if (deviceType.equals("IP"))
					{
						nIPType = IP_ONLY;
					}
					else if (deviceType.indexOf("IP") > -1 && (deviceType.indexOf("SERIAL") > -1 || deviceType.indexOf("RS-") > -1))
					{
						String ipAddress = this.getProperty(IP_ADDRESS);
						if (ipAddress == null || ipAddress.length() == 0)
							ipAddress = this.getProperty(SystemConstants.IP_ADDRESS);
						
						// If an ip address exists then assume ip control
						if (ValidateUtil.isValidIP(ipAddress))
							nIPType = IP_ONLY;
						else
							nIPType = IP_SERIAL;
					}
					else if (deviceType.indexOf("SERIAL") > -1 || deviceType.indexOf("RS-") > -1)
					{
						nIPType = SERIAL_ONLY;
					}
					// Just in case the category changes
					else if (deviceType.indexOf("IP") >= 0)
					{
						String ipAddress = this.getProperty(IP_ADDRESS);
						if (ipAddress == null || ipAddress.length() == 0)
							ipAddress = this.getProperty(SystemConstants.IP_ADDRESS);
						
						// category is IP, therefore the ID must match an IP address
						if (ValidateUtil.isValidIP(ipAddress))
							nIPType = IP_ONLY;
					}
				}
				else
				{
					throw new IllegalArgumentException(SystemConstants.DEVICE_CATEGORY + " not defined!");
				}
			}
			else
			{
				throw new NullPointerException(SystemConstants.DEVICE_CATEGORY + " not defined!");
			}
		}
		else
		{
			throw new NullPointerException("Module properties unavailable!");
		}
		
		return nIPType;
	}
	
	/**
	 * Checks the NetLinxDevice passed in to determine if this is a SerialDevice class.
	 * 
	 * @param nd
	 * @return true if this is a SerialDevice; false otherwise
	 * @throws NullPointerException Occurs when the NetLinxDevice parameter is null
	 */
	public boolean isSerialControlled(NetLinxDevice nd) throws NullPointerException
	{
		boolean bSerial = false;
		if (nd != null)
		{
			if (nd instanceof SerialDevice)
				bSerial = true;
			
			if (!bSerial)
				bSerial = ValidateUtil.isValidDPS(nd.getDPS());
		}
		else
		{
			throw new NullPointerException("Invalid NetLinx Device passed in!");
		}
		
		return bSerial;
	}

	/**
	 * Check to see if the Dynamic Device Property is set.
	 * 
	 * @return true if this was loaded dynamically; false otherwise.
	 */
	public boolean isDynamicDevice()
	{
		boolean bDynamic = false;
		
		if (m_mod != null)
		{
			String dynamicType = this.getProperty(SystemConstants.DYNAMIC_TYPE);
			if (dynamicType != null && dynamicType.length() > 0)
				bDynamic = true;
		}
		else
		{
			throw new NullPointerException("Module properties unavailable!");
		}
		
		return bDynamic;
	}

	/**
	 * Retrieve the main virtual device DPS value for this module.
	 * 
	 * @return DPS for the virtual device
	 */
	public DPS getVirtualDPS()
	{
		if (m_mod == null)
		{
			error("Common device not found");
			return null;
		}
		
		String vdStr = this.getProperty(SystemConstants.DUET_DEVICE);
		if (vdStr == null || vdStr.length() == 0)
		{
			error("Virtual device not found");
			return null;
		}
		
		DPS virtualDPS = new DPS(vdStr);
		return virtualDPS;
	}
	
	/**
	 * Validate and retrieve the port value from an advance Event object. The 
	 * port value can be used as the component or zone index on the device.
	 * 
	 * @param eventDPS Advance Event object's DPS
	 * @return port number from the advance event DPS
	 */
	public int getAdvancedIndex(DPS eventDPS)
	{
		int index = 0;
		
		DPS virtualDPS = getVirtualDPS();
		if (eventDPS.getDevice() == virtualDPS.getDevice() ||
			eventDPS.getSystem() == virtualDPS.getSystem())
			index = eventDPS.getPort();
			
		return index;
	}

	/**
	 * Checks a fully qualified path name for validity.
	 * 
	 * @param strFileName
	 * @return true if the name is valid; false otherwise.
	 */
	public boolean IsFileNameValid(String strFileName)
	{
		String strReservedName = "", strmsg = "";

		// check for valid characters within the file name
		// all of these characters are invalid "?\"><|/*"
		if ((strFileName.indexOf("?")  != -1) || 
			(strFileName.indexOf("\"") != -1) || 
			(strFileName.indexOf(">")  != -1) ||
			(strFileName.indexOf("<")  != -1) || 
			(strFileName.indexOf("|")  != -1) || 
			(strFileName.indexOf("/")  != -1) ||
			(strFileName.indexOf("*")  != -1))
		{
			strmsg = "The " + strFileName + " has invalid characters.";
			System.out.println(strmsg);
			return false;
		}

		//--------------------------------------------------------------------------
		// Check for reserved device names as part of the file name  
		// (i.e. LPTx, COMx, PRN, NUL). Thanks to Mike Morris for this sneaky trick!
		//--------------------------------------------------------------------------
		File splitFile = new File(strFileName);
		String strName = splitFile.getName();

		String strMsgFormat = "%s is a reserved device name and cannot be used for a files.";

		// test for COM1-COM10 and LPT1-10
		for (int i = 0; i < 10; i++)
		{
			// Construct COM name
			strReservedName = "COM" + (i+1);
			if (strName.compareToIgnoreCase(strReservedName) == 0)
			{
				Object [] msgList = new Object[3];
				msgList[0] = strReservedName;
				strmsg = FormatUtil.sprintf(strMsgFormat, msgList);
				System.out.println(strmsg);
				return false;
			}

			// Construct LPT name
			strReservedName = "LPT" + (i+1);
			if (strName.compareToIgnoreCase(strReservedName) == 0)
			{
				Object [] msgList = new Object[3];
				msgList[0] = strReservedName;
				strmsg = FormatUtil.sprintf(strMsgFormat, msgList);
				System.out.println(strmsg);
				return false;
			}
		}
		
		// test for NUL
		if (strName.compareToIgnoreCase("NUL") == 0)
		{
			Object [] msgList = new Object[3];
			msgList[0] = new String("NUL");
			strmsg = FormatUtil.sprintf(strMsgFormat, msgList);
			System.out.println(strmsg);
			return false;
		}
		
		// test for PRN
		if (strName.compareToIgnoreCase("PRN") == 0)
		{
			Object [] msgList = new Object[3];
			msgList[0] = new String("PRN");
			strmsg = FormatUtil.sprintf(strMsgFormat, msgList);
			System.out.println(strmsg);
			return false;
		}
		
		// test for AUX
		if (strName.compareToIgnoreCase("AUX") == 0)
		{
			Object [] msgList = new Object[3];
			msgList[0] = new String("AUX");
			strmsg = FormatUtil.sprintf(strMsgFormat, msgList);
			System.out.println(strmsg);
			return false;
		}

		// test for CON
		if (strName.compareToIgnoreCase("CON") == 0)
		{
			Object [] msgList = new Object[3];
			msgList[0] = new String("CON");
			strmsg = FormatUtil.sprintf(strMsgFormat, msgList);
			System.out.println(strmsg);
			return false;
		}

		return true;
	}
	
	/**
	 * Start a timer with debug messages naming the timer.
	 * 
	 * @param name String name of the timer to start.
	 * @param lTime Timeout value.
	 * @return The newly created timer.
	 */
	public Timer startTimer(String name, long lTime)
	{
		// Do not allow timers to restart if disposing
		if (bDisposing)
			return null;
		
		log(DEBUG, "startTimer(" + name + ", " + lTime + ")");
		Timer timer = new Timer(this, lTime);
		timer.start();
		return timer;
	}
	
	/**
	 * Reload and restart an existing timeline with a new set of times.
	 * 
	 * @param timeline Timeline to restart.
	 * @param alTimes Times to reload.
	 */
	public void restartTimeline(Timeline timeline, long alTimes[])
	{
		if (timeline != null)
		{
			timeline.reload(alTimes);
			timeline.start();
		}
	}
	
	/**
	 * Stop a timer with debug message naming the timer.
	 * 
	 * @param timer The Timer object to stop and destroy.
	 * @param name String name of the timer to destroy.
	 * @return The (hopefully) destroyed timer object.
	 */
	public Timer stopTimer(Timer timer, String name)
	{
		if (timer != null)
		{
			log(DEBUG, "stopTimer(" + name + ")");
			timer.cancel();
			timer = null;
		}
		
		return timer;
	}


	/**
	 * Log a message through the device.  If the device is null, then no 
	 * action is performed.
	 * 
	 * @param logLevel [1..4] the debug level of the device;
	 * @param message the string to send to the log.
	 */
	protected void log(int logLevel, String message )
	{
		if (m_mod != null)
			m_mod.log(logLevel, message);
	}

	
	/**
	 * Enable or disable CTS.
	 * 
	 * @param on
	 */
	public void setCTSEnable(boolean on)
	{
		if (on)
		{
			m_dvDevice.sendCommand("CTSPSH");
		}
		// no way to turn it off
	}
	
	/**
	 * Set a static baud rate for the device port.
	 * 
	 * @param baud
	 */
	public void setBaud(String baud)
	{
		m_dvDevice.sendCommand("SET BAUD " + baud);
	}
	
	/**
	 * Set a baud rate, which can be reset, for the device port.
	 * 
	 * @param baud
	 * @param temporary
	 */
	public void setBaud(String baud, boolean temporary)
	{
		if ((NetLinxSystemUtil.getDeviceID(m_dvDevice.getDPS()) > 255) && temporary)
			m_dvDevice.sendCommand("TSET BAUD " + baud);
		else
			m_dvDevice.sendCommand("SET BAUD " + baud);
	}
	
	/**
	 * 
	 * @param time
	 */
	public void setCharDelay(int time)
	{
		// NetLinx Serial ports have a 100 usec resolution
		// (we always round up)
		time = (time + 99) / 100;
		if (time < 256)
		{
			m_dvDevice.sendCommand("CHARD-"+time);
		}
		else
		{
			time = time / 10;
			if (time > 255)
			{
				// time was too large
				time = 255;
			}
			m_dvDevice.sendCommand("CHARDM-"+time);
		}
	}
	
	/**
	 * 
	 * @param on
	 */
	public void set9BitMode(boolean on)
	{
		if (on)
		{
			m_dvDevice.sendCommand("B9MON");
		}
		else
		{
			m_dvDevice.sendCommand("B9MOFF");
		}
	}
	
	/**
	 * 
	 * @param on
	 */
	public void setReceiveEnable(boolean on)
	{
		if (on)
		{
			m_dvDevice.sendCommand("RXON");
		}
		else
		{
			m_dvDevice.sendCommand("RXOFF");
		}
	}
	
	/**
	 * Enable or disable hardware handshaking.
	 *  
	 * @param on
	 */
	public void setHardwareHandshake(boolean on)
	{
		if (on)
		{
			m_dvDevice.sendCommand("HSON");
		}
		else
		{
			m_dvDevice.sendCommand("HSOFF");
		}
	}
	
	/**
	 * Enable or disable software handshaking.
	 * 
	 * @param on
	 */
	public void setSoftwareHandshake(boolean on)
	{
		if (on)
		{
			m_dvDevice.sendCommand("XON");
		}
		else
		{
			m_dvDevice.sendCommand("XOFF");
		}
	}
	
	/**
	 * Send a message to the device. If the device is connected via an IP socket
	 * then it will write the message out the socket, otherwise it will send the
	 * message on the device port.
	 * 
	 * @param msg Message to be sent to the device.
	 */
	public void sendString(String msg)
	{
		sendString(msg.getBytes());
	}
	
	/**
	 * Send a message to the device. If the device is connected via an IP socket
	 * then it will write the message out the socket, otherwise it will send the
	 * message on the device port.
	 * 
	 * @param msg Message to be sent to the device.
	 */
	public void sendString(byte[] msg)
	{
		if (isIPControlEnabled())			
		{
			try
			{
				ipConnection.write(msg);
			}
			catch(Exception e)
			{
				log(ERROR, "sendString: write failed " + new String(msg));
			}
		}
		else
		{	
			m_dvDevice.sendString(msg);
		}
		
	}
	
	/*
	 * 
	 * @param str
	 */
	private void error(String str)
	{
		Logger.error("DeviceUtil", str);
	}

}
