/*
 * DeviceListener.java
 * 
 * Created On Jun 13, 2007
 * @author vcharles
 * 
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.comm;

import com.amx.duet.tools.lang.ByteBuffer;



/**
 * 
 */
public abstract interface IDeviceListener
{
	/**
	 * Sets up the Serial port communications.<br>
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	public void setCommPort()
	 *	{
	 *		String sBaud = getProperty(DeviceUtil.BAUD_RATE);
	 *		utilities.setBaud(sBaud + COMM_PORT_STRING);
	 *		utilities.setHardwareHandshake(false);
	 *		utilities.setSoftwareHandshake(false);
	 *	}
	 * </pre>
	 */
	public abstract void setCommPort();
	
	/**
	 * Queue outgoing commands to the device and manage timeline.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	public void enQueue(byte msg[])
	 *	{
	 *		synchronized (queue)
	 *		{
	 *			// Restart the dequeue timeline if it was paused
	 *			utilities.restartQueueTimeline();
	 *			queue.doEnq(msg);
	 *		}
	 *	}
	 * </pre>
	 * 
	 * @param msg message to be sent to the device.
	 * @param priority the priority of the message being sent.
	 */
	public abstract void enQueue(byte[] msg, int priority);
	
	/**
	 * Remove a message from the queue and send it to the physical device.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	public void deQueue()
	 *	{
	 *		try
	 *		{
	 *			if (utilities.isQueueLocked() || queue.getQueSize() == 0)
	 *				return;
	 *	
	 *			// stop sending messages and queue them up until we're done
	 *			if (isDeviceOnLine())
	 *				utilities.lockQueue(true);
	 *
	 *			// Remove the command from the queue
	 *			sbLastMsg.append((byte[])queue.doDeq());
	 *			if (sbLastMsg != null && sbLastMsg.length() > 0)
	 *			{
	 *				// TODO: User defined code goes here...
	 *				
	 *				// Send the command to the device
	 *				log(DEBUG, "Comm ----------> device: BUFFER '" + sbLastMsg.toHex() + "'");
	 *				utilities.sendString(sbLastMsg.toByteArray());
	 *
	 *				// times out the heart beat message if a response is not received
	 *				utilities.startResponseTimer();
	 *			}
	 *
	 *			// Pause the dequeue timeline if there are no more items
	 *			if (queue.getQueSize() == 0)
	 *				utilities.pauseQueueTimeline();
	 *		}
	 *		catch (Exception e)
	 *		{
	 *			if (this.getDebugState() == DEBUG)
	 *				e.printStackTrace();
	 *			log(ERROR, "Exception in deQueue: " + e.toString());
	 *		}
	 *	}
	 * </pre>
	 */
	public abstract void deQueue();
	
	/**
	 * Query device for status here. This is a reocurring event at the specified 
	 * poll time interval.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	public void pollEvent()
	 *	{
	 *		// TODO: User defined code goes here...
	 *		enQueue("query_vol");
	 *	}
	 * </pre>
	 */
	public abstract void pollEvent();
	
	/**
	 * Put heartbeat messages on the queue here.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	public void heartbeatEvent()
	 *	{
	 *		byte cmd[] = {0x03,0x41};
	 *		sendCommand(cmd);
	 *	}
	 * </pre>
	 */
	public abstract void heartbeatEvent();

	/**
	 * Handle all incoming data in a single place.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	public void handleIncomingData(int bytesReceived, byte[] data)
	 *	{
	 *		// this would not be good...
	 *		if (data == null || data.length <= 0 || bytesReceived <= 0)
	 *			return;
	 *		
	 *		sbRxMsg.append(data);
	 *		log(DEBUG, "Comm <---------- device: INCOMING DATA '" + FormatUtil.toAsciiHexString(data) + "'");
	 *		log(DEBUG, "Comm <---------- device: BUFFER '" + sbRxMsg.toHex() + "'");
	 *		
	 *		// check for start and end bytes
	 *		int start = sbRxMsg.indexof(STX);
	 *		int end = sbRxMsg.indexof(ETX);
	 *		while (start > -1 && end > -1)
	 *		{
	 *			// TODO: User defined code goes here...
	 *			
	 *			// Parse a single response
	 *			parseResponse(sbRxMsg.substring(start, end));
	 *
	 *			// Delete processed response
	 *			sbRxMsg.delete(start, end+1);
	 *					
	 *			// check for another msg in the buffer
	 *			start = sbRxMsg.indexof(STX);
	 *			end = sbRxMsg.indexof(ETX);
	 *		}
	 *	}
	 * </pre>
	 *  
	 * @param data response received from the device.
	 */
	public abstract void handleIncomingData(int bytesReceived, byte[] data);

	/**
	 * Handle the device online event.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	public void handleOnlineEvent()
	 *	{
	 *		utilities.handleOnlineEvent();
	 *	}
	 * </pre>
	 */
	public abstract void handleOnlineEvent();
	
	/**
	 * Handle the device offline event.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	public void handleOfflineEvent()
	 *	{
	 *		utilities.handleOfflineEvent();
	 *	}
	 * </pre>
	 */
	public abstract void handleOfflineEvent();
	
	/**
	 * Handle the device error event.
	 * <P>
	 * <strong>Example usage:</strong>
	 * <pre>
	 *	public void handleErrorEvent(int error)
	 *	{
	 *		utilities.handleErrorEvent(error);
	 *	}
	 * </pre>
	 * 
	 * @param error int representing the status value.
	 */
	public void handleErrorEvent(int error);
	
	/**
	 * Parse a single response passed in from handleIncomingData().
	 * 
	 * @param response ByteBuffer object, which can be used for both byte 
	 *        arrays and Strings.
	 */
	public void parseResponse(ByteBuffer response);
}
