/*
 * InputSource.java
 * 
 * Created On May 24, 2007
 * @author vcharles
 * 
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.comm;

import com.amx.duet.devicesdk.type.AVType;
import com.amx.duet.devicesdk.type.InputSourceSelect;


/**
 * The InputProperty class is a container for the properties that define a
 * source input on a device. Input select group number has a 1:1 correlation 
 * with input source inputNumber. The input source should only be set to active 
 * if it has been configured within the group to be the only selectable source. 
 * Thus, in a device where all sources are selectable (mutually exclusive) each 
 * source must be set as an active source (see setActiveSource(boolean) for more 
 * detail).
 * 
 */
public class InputProperty
{
	private int m_nGroupNumber = Integer.MIN_VALUE;
	private String m_sLabel = "";
	private AVType m_avType = AVType.INVALID;
	private InputSourceSelect m_InputType = InputSourceSelect.INVALID;
	private boolean m_bActiveSource = false;

	/**
	 * Creates a new InputProperty object.
	 * 
	 * @param number integer value representing the selectable source (group number)
	 * @param sourceType type of input source (NamedInput or SignalType)
	 */
	public InputProperty(int number, InputSourceSelect sourceType)
	{
		m_nGroupNumber = number;
		m_InputType = sourceType;
	}
	
	/**
	 * Creates a new InputProperty object.
	 * 
	 * @param number integer value representing the selectable source (group number)
	 * @param label description provided by device to represent source
	 * @param avtype Audio/Video type
	 * @param sourceType type of input source (NamedInput or SignalType)
	 */
	public InputProperty(int number, String label, AVType avtype, InputSourceSelect sourceType)
	{
		m_nGroupNumber = number;
		m_sLabel = label;
		m_avType = avtype;
		m_InputType = sourceType;
	}
	
	/**
	 * Sets the device label property to the description provided by the device.
	 * 
	 * @param label description provided by device to represent source
	 */
	public void setDeviceLabel(String label)
	{
		m_sLabel = label;
	}
	
	/**
	 * Sets the Audio/Video property stating that this source is either an 
	 * AUDIO, VIDEO, or both source type.
	 * 
	 * @param type Audio/Video type
	 */
	public void setAVType(AVType type)
	{
		m_avType = type;
	}
	
	/**
	 * Sets the active selectable source property. In a mutually exclusive 
	 * source selection device all sources will be active by setting the active 
	 * state to true. In a device where input signal types are grouped together 
	 * and only one is configured to be selectable the active state must be set 
	 * to true for the selectable source and false for all of the other signal 
	 * types within the group.
	 * 
	 * @param bActive specifies that this is a selectable source (within a group)
	 */
	public void setActiveSource(boolean bActive)
	{
		m_bActiveSource = bActive;
	}
	
	/**
	 * Is the current InputProperty an active selectable source.
	 * 
	 * @return specifies that this is a selectable source (within a group)
	 */
	public boolean isActiveSource()
	{
		return m_bActiveSource;
	}

	/**
	 * Get the current input number or group number, where input select 
	 * groupNumber is a direct 1:1 correlation with input source inputNumber.
	 * 
	 * @return integer value representing the selectable source (group number)
	 */
	public int getInputNumber()
	{
		return m_nGroupNumber;
	}
	
	/**
	 * Get the current type of input (NamedInput, SignalType, etc.).
	 * 
	 * @return type of input source (NamedInput or SignalType)
	 */
	public InputSourceSelect getInputType()
	{
		return m_InputType;
	}
	
	/**
	 * Gets the device label property, which should be the description provided 
	 * by the device.
	 * 
	 * @return description provided by device to represent source
	 */
	public String getDeviceLabel()
	{
		return m_sLabel;
	}
	
	/**
	 * Gets the Audio/Video property, which states that this source is either an 
	 * AUDIO, VIDEO, or both source type.
	 * 
	 * @return Audio/Video type
	 */
	public AVType getAVType()
	{
		return m_avType;
	}

	/**
	 * Get a String representation of the properties that define an input 
	 * contained within this class.
	 * 
	 * @return Formatted property string
	 */
	public String getInputProperty()
	{
		return (m_nGroupNumber + "," + m_sLabel + "," + m_InputType.getName() + "," + m_avType.getName());
	}

	/*
	 *  (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
		return getInputProperty();
	}
}
