/**
 * 
 */
package com.amx.duet.tools.comm.component;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import com.amx.duet.core.master.DiagnosticString;
import com.amx.duet.core.osgi.Service;
import com.amx.duet.devicesdk.base.AdvancedEvent;
import com.amx.duet.devicesdk.component.IPreAmpComponent;
import com.amx.duet.devicesdk.component.PreAmpComponentImpl;
import com.amx.duet.devicesdk.type.SurroundMode;
import com.amx.duet.tools.comm.ICommonComponent;
import com.amx.duet.tools.comm.ISurroundModes;
import com.amx.duet.tools.comm.SurroundModesAdvancedEvent;

/**
 * The VirtualPreAmpComponent class stores and maintains a list of 
 * selectable surround modes for the device. Each source is added to both an 
 * internal list within this class and a list stored in the parent class. 
 * The active surroundMode can be changed with a call to the updateSurroundMode 
 * method.
 * 
 * In the constructor of the implemented class (i.e. MakeModelPreAmpComponent )
 * the surround modes for the device should be added.  One way to 
 * accomplish this with TSE's as described in the duet module programming guide 
 * is <code>setSurroundModeList( MakeModeSurroundModes.VALUES );</code>.
 * 
 */
public abstract class VirtualPreAmpComponent extends PreAmpComponentImpl implements ICommonComponent, ISurroundModes
{
	
	/**
	 * The InternalSurroundMode class provides the VirtualPreAmpComponent class 
	 * the mechanism to create and utilize surround modes automatically.  This 
	 * class will only be used for
	 */
	private class InternalSurroundMode extends SurroundMode {
		
		/**
		 * @param surroundModeLabel the name of the surround mode.
		 */
		protected InternalSurroundMode(String surroundModeLabel)
		{
			super(surroundModeLabel);
		}
		
	}
	
	// Because the class attributes may be used by the parent's reinitialize method they must be created before construction.
	private HashMap _surroundModes = new HashMap();
	private int _surroundModeSelect = Integer.MIN_VALUE;
	private IPreAmpComponent m_caller;
	
	/**
	 * @param caller
	 * @param index
	 */
	public VirtualPreAmpComponent(IPreAmpComponent caller, int index)
	{
		super(caller, index);
		m_caller = caller;
	}
	
	
	/**
	 * Creates an InternalSurroundMode for the surroundModeLabel speceified, then
	 * adds that SurroundMode object to the source select list.
	 * 
	 * @param surroundModeNumber integer value representing the selectable surround mode
	 * @param surroundModeLabel the name of the surround mode.
 	 * @return The SurroundMode added.
	 * 
	 * @see addSurroundMode( int, SurroundMode );
	 */
	public SurroundMode addSurroundMode(int surroundModeNumber, String surroundModeLabel)
	{
		logEntry("addSurroundMode(int, String)");
		InternalSurroundMode surroundMode;
		surroundMode = new InternalSurroundMode(surroundModeLabel);
		return addSurroundMode( surroundModeNumber, surroundMode);
	}
	
	/**
	 * Adds a SurroundMode object to the source select list. Each object is 
	 * added to both an internal list within this class and the list stored in
	 * the parent class. Input select group number has a 1:1 correlation with 
	 * input source inputNumber. The input source should only be set to active 
	 * if it has been configured within the group to be the only selectable 
	 * source. Thus, in a device where all sources are selectable (mutually 
	 * exclusive) each source must be set as an active source.
	 * 
	 * @param surroundModeNumber integer value representing the selectable surround mode
	 * @param surroundMode  a predefined SurroundMode used by the device
	 * @return The SurroundMode added.
	 */
	public SurroundMode addSurroundMode(int surroundModeNumber, SurroundMode surroundMode )
	{
		logEntry("addSurroundMode("+surroundModeNumber+", "+surroundMode+")");
		if( surroundModeNumber <= 0 ) {
			error("ERROR: Invalid surroundModeNumber sent to addSurroundMode");
		} else {
			Integer surroundSelection = new Integer(surroundModeNumber);
			_surroundModes.put(surroundSelection, surroundMode);
			Map sortedMap = new TreeMap(_surroundModes);
			
			List listValues = null;
			listValues = Collections.unmodifiableList(Arrays.asList( sortedMap.values().toArray() ) );
			super.setSurroundModeList( listValues );
			processAddSurroundModeEvent( surroundSelection.intValue(), surroundMode.getName() );
		}
		return surroundMode;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see com.amx.duet.tools.comm.ICommonComponent#dispose()
	 */
	public void dispose()
	{
	}
	
	
	private void error(String sMsg)
	{	
		DiagnosticString.send(0,0,this.getClass().getClass() +":"+sMsg);
	}
	
	/**
	 * Formats the source select property for SNAPI by prepending the offsetted index
	 * 
	 * @param surroundMode SurroundMode object which stores the current source attributes
	 * @param surroundModeNumber integer value representing the selectable surround mode
	 * @return Formatted property string
	 */
	private String formatSurroundSoundProperty(SurroundMode surroundMode, int surroundModeNumber)
	{
		logEntry("formatSurroundModeProperty(SurroundMode, int)");
		String fmt = "";
		if (surroundMode != null)
			fmt = (surroundModeNumber) + "," + surroundMode.getName();
		return fmt;
	}
	
	/**
	 * Returns the SurroundMode for the specified surroundModeNumber.  
	 * SurroundMode.INVALID is returned if no matching surroundModeNumber is found.
	 * @param surroundModeNumber integer value representing the selectable surround mode
	 * @return The matching SurroundMode or SurroundMode.INVALID if unable to match.
	 */
	public SurroundMode getSurroundMode(int surroundModeNumber)
	{
		logEntry("getSurroundMode("+surroundModeNumber+")");
		SurroundMode returnValue = SurroundMode.INVALID;
		Integer keyValue = new Integer(surroundModeNumber);
		if  ( _surroundModes.containsKey( keyValue ) ) {
			returnValue = (SurroundMode) _surroundModes.get( keyValue );
		}
		logEntry("returnValue: "+returnValue);
		return returnValue;
	}
	
	/**
	 * Returns the number of valid surround mode settings.
	 * 
	 * @return The number of surround mode settings.
	 */
	public int getSurroundModeCount() {
		logEntry("getSurroundModeCount()");
		return _surroundModes.size();
	}
	
	/**
	 * Find the number of the surround mode.  The SurroundMode is matched based
	 * on the name of surround modes set within this VirtualPreAmpComponent.  If 
	 * no matching SurroundMode is found, Integer.MIN_VALUE is returned.
	 * @param surroundMode the SurroundMode to match
	 * @return The number that specifies the SurroundMode, or Integer.MIN_VALUE if unable to match.
	 */
	public int getSurroundModeNumber( SurroundMode surroundMode )
	{
		int returnValue = Integer.MIN_VALUE;
		for (Iterator it = _surroundModes.entrySet().iterator(); it.hasNext();)
		{
			Map.Entry entry = (Map.Entry) it.next();
			SurroundMode listedSurroundMode = (SurroundMode) entry.getValue();
			if( listedSurroundMode.getName().equals(surroundMode.getName()) ) {
				Integer keyValue = (Integer) entry.getKey();
				returnValue = keyValue.intValue();
			}
		}
		return returnValue;
	}
	
	/**
	 * Returns the properties for all surround mode settings.
	 * 
	 * @return All the properties for the surround modes.
	 */
	public String[] getSurroundModeProperties()
	{
		logEntry("getSurroundModeProperties()");
		String props[] = new String[_surroundModes.size()];
		if (_surroundModes.size() > 0)
		{
			int i = 0;
			for (Iterator it = _surroundModes.entrySet().iterator(); it.hasNext(); )
			{
				Map.Entry entry = (Map.Entry) it.next();
				SurroundMode surroundMode = (SurroundMode) entry.getValue();
				Integer keyValue = (Integer) entry.getKey();
				props[i] = formatSurroundSoundProperty(surroundMode, keyValue.intValue());
				i++;
			}
		}
		return props;
	}
	
	
	/**
	 * Returns the properties for a single surround mode setting.
	 * 
	 * @param surroundModeNumber integer value representing the selectable surround mode
	 * @return String
 	 * @return Formatted property string.
	 */
	public String getSurroundModeProperty(int surroundModeNumber) {
		logEntry("getSurroundModeProperty(int)");
		SurroundMode surroundMode = getSurroundMode( surroundModeNumber );
		String prop = formatSurroundSoundProperty(surroundMode, surroundModeNumber);
		return prop;
		
	}
	

	
	/**
	 * @return The internal list of SurroundModes.
	 */
	protected HashMap getSurroundModes()
	{
		logEntry("getSurroundModes()");
		return _surroundModes;
	}
	
	
	/**
	 * Returns the currently selected surround mode.
	 * 
	 * @return int
	 */
	public int getSurroundModeSelect() {
		logEntry("getSurroundModeSelect()");
		return _surroundModeSelect;
	}
	
	private void logEntry(String sMethod)
	{	
		try		
		{
			if (m_caller instanceof Service)
			{
				Service m_service = (Service) m_caller;			
				m_service.log(DiagnosticString.SEVERITY_INFO,this.getClass().getClass()+"."+sMethod);
			}
			else
			{
				error("Error m_caller not instance of Service");
			}
		}
		catch (RuntimeException e)
		{
			error("Error in logEntry: " + e.getMessage());
		}
	}
	
	/**
	 * Processes the event when a Surround Mode is added.
	 * @param surroundModeNumber integer value representing the selectable surround mode
	 * @param surroundModeLabel the name of the surround mode.
	 */
	public void processAddSurroundModeEvent(int surroundModeNumber, String surroundModeLabel ) 
	{
		String surroundModeText = SurroundModesAdvancedEvent.SURROUND_MODE.getName()+surroundModeNumber+","+surroundModeLabel;
		AdvancedEvent advEv = new AdvancedEvent( surroundModeText, SurroundModesAdvancedEvent.SURROUND_MODE );
		sendCommandFeedback( advEv );
	}
	
	/**
	 * Processes the event when the PreAmp Surround Mode Count is requested.<br>
	 * @param surroundModeNumber integer value representing the selectable surround mode
	 */
	public void processSurroundModeCountEvent(int surroundModeNumber)
	{
		String surroundModeText = SurroundModesAdvancedEvent.SURROUND_MODE_COUNT.getName()+surroundModeNumber;
		AdvancedEvent advEv = new AdvancedEvent( surroundModeText, SurroundModesAdvancedEvent.SURROUND_MODE_COUNT );
		sendCommandFeedback( advEv );
	}
	
	/**
	 * Processes the event when the PreAmp Surround Mode properties are requested.<br>
	 * @param propertyArray
	 */
	public void processSurroundModeProperties(String[] propertyArray)
	{
		for( int i = 0; i < propertyArray.length; i++ ) {
			String surroundModeText = SurroundModesAdvancedEvent.SURROUND_MODE.getName()+propertyArray[i];
			AdvancedEvent advEv = new AdvancedEvent( surroundModeText, SurroundModesAdvancedEvent.SURROUND_MODE );
			sendCommandFeedback( advEv );
		}
	}
	
	/**
	 * Processes the event when the PreAmp Surround Mode Property is changed.<br>
	 * @param propertyValue 
	 */
	public void processSurroundModePropertyEvent(String propertyValue)
	{
		String surroundModeText = SurroundModesAdvancedEvent.SURROUND_MODE.getName()+propertyValue;
		AdvancedEvent advEv = new AdvancedEvent( surroundModeText, SurroundModesAdvancedEvent.SURROUND_MODE );
		sendCommandFeedback( advEv );
	}
	
	/**
	 * Processes the event when the PreAmp Surround Mode selection is made.<br>
	 * @param surroundModeNumber integer value representing the selectable surround mode
	 */
	public void processSurroundModeSelectEvent(int surroundModeNumber)
	{
		String surroundModeText = SurroundModesAdvancedEvent.SURROUND_MODE_SELECT.getName()+surroundModeNumber;
		AdvancedEvent advEv = new AdvancedEvent( surroundModeText, SurroundModesAdvancedEvent.SURROUND_MODE_SELECT );
		sendCommandFeedback( advEv );
	}
	
	/**
	 * Sends feedback to the device.
	 * @param advEv AdvancedEvent
	 */
	protected void sendCommandFeedback( AdvancedEvent advEv)
	{
		sendCommandFeedback( advEv.getValueObject().toString(), advEv, getPreAmpComponentIndex());
	}
	
	/**
	 * Sends feedback to the device.
	 * A sample implementation is shown below:<br>
	 * <code>utilities.sendCommandFeedback( message, advEv, zone, utilities.getNetLinxDevice()  );</code>
	 * @param message The feedback message to send back to the deive
	 * @param advEv The AdvancedEvent to process
	 * @param zone The zone of the virtual device the feedback is sent to.
	 */
	public abstract void sendCommandFeedback( String message, AdvancedEvent advEv, int zone);
	
	/**
	 * Send the command to the device to set the specified surround mode.
	 * 	 * // TODO: Send your command to the device here...
	 * enQueue( surroundModeCommand( surroundMode ) );
	 * where surroundCommand provides the correct command to send to device.
	 * 
	 * @param surroundMode  a predefined SurroundMode used by the device
	 */
	public abstract void setSurroundMode(SurroundMode surroundMode);
	
	/**
	 * Sets list of surround modes for the device.<br>
	 * <b>This method should not be overridden by the programmer unless 
	 *    non-standard behavior is desired.</b>
	 * @param surroundModes List
	 */
	public void setSurroundModeList(List surroundModes)
	{
		_surroundModes.clear();
		for( int i =1; i <= surroundModes.size(); i++ ) {
			Object listItem = surroundModes.get(i-1);
			if( listItem instanceof SurroundMode ) {
				addSurroundMode( i, (SurroundMode)listItem );
			} else if( listItem instanceof String ) {
				addSurroundMode( i, (String)listItem );				
			} else {
				logEntry("setSurroundModeList encountered invalid item: "+listItem);
			}
		}
	}
	
	/**
	 * A convenience method to set the list of surround modes for the device.  
	 * @param surroundModes
	 * 
	 * @see setSurroundModeList( List );
	 */
	public void setSurroundModeList( SurroundMode[] surroundModes ){
		setSurroundModeList(  Collections.unmodifiableList( Arrays.asList( surroundModes ) ));
	}
	
	/**
	 * Sets the current surround mode setting, where index is a virtual index number between 1 and
	 * the value returned by getSurroundModeCount().
	 * 
	 * @param surroundModeNumber integer value representing the selectable surround mode
	 */
	public void setSurroundModeSelect(int surroundModeNumber) {
		logEntry("setSurroundModeSelect("+surroundModeNumber+")");
		setSurroundMode( getSurroundMode(surroundModeNumber ) );
	}

	/**
	 * A convenience method to update the Surround Mode 
	 * @param surroundMode the desired surround mode
	 * 
	 * @see updateSurroundModeSelect( int );
	 */
	public void updateSurroundMode(SurroundMode surroundMode)
	{
		logEntry("updateSurroundMode("+surroundMode+")");
		updateSurroundModeSelect( getSurroundModeNumber(surroundMode) );
	}
	
	/**
	 * Set the SurroundMode for the device.<br>
	 * The new value is compared against the current value.  If the value has changed, the new
	 * value is saved and a process event for the SurroundMode is called.<br>
	 * @param surroundModeNumber the number of the surround mode selected 
	 */
	public void updateSurroundModeSelect( int surroundModeNumber )
	{
		logEntry("updateSurroundModeSelect("+surroundModeNumber+")");
		if( surroundModeNumber != _surroundModeSelect ) {
			_surroundModeSelect = surroundModeNumber;
			processSurroundModeSelectEvent( _surroundModeSelect );
			SurroundMode mode = getSurroundMode( surroundModeNumber );
			super.updateSurroundMode( mode );
		}
	}
	
}
