/*
 * VirtualSourceSelectComponent.java
 * 
 * Created On May 22, 2007
 * @author VCharles
 * 
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.comm.component;

import java.util.Vector;

import com.amx.duet.devicesdk.component.ISourceSelectComponent;
import com.amx.duet.devicesdk.component.ISourceSelectComponent2;
import com.amx.duet.devicesdk.component.SourceSelectComponentEvent;
import com.amx.duet.devicesdk.component.SourceSelectComponentImpl;
import com.amx.duet.devicesdk.type.AVType;
import com.amx.duet.devicesdk.type.InputSourceSelect;
import com.amx.duet.devicesdk.type.InputSourceSelectInfo;
import com.amx.duet.tools.comm.ICommonComponent;
import com.amx.duet.tools.comm.InputProperty;

/**
 * The VirtualSourceSelectComponent class store and maintains a list of 
 * selectable sources for the device. Each source is added to both an internal 
 * list within this class and a list stored in the parent class. Input select 
 * group number has a 1:1 correlation with input source inputNumber. The input 
 * source should only be set to active if it has been configured within the 
 * group to be the only selectable source. Thus, in a device where all sources 
 * are selectable (mutually exclusive) each source must be set as an active 
 * source. The active source can be changed with a call to the 
 * updateInputSource() method.
 */
public abstract class VirtualSourceSelectComponent extends SourceSelectComponentImpl implements ICommonComponent
{
	protected ISourceSelectComponent2 m_caller = null;
	
	private Vector inputs = new Vector();
	private int m_inputSelected = Integer.MIN_VALUE;
	
	/**
	 * @param caller
	 * @param index
	 */
	public VirtualSourceSelectComponent(ISourceSelectComponent caller, int index)
	{
		super(caller, index);
		
		m_caller = (ISourceSelectComponent2)caller;
		
		// TODO: Example of how to implement constructor...

		// Since each source for this device is mutually exclusive the group 
		// number is basically the same value as the list index. Thus, each 
		// source becomes the active source since there are no other sources in 
		// the group.
		
		//for (int i = 0; i < MAX_INPUTS; i++)
		//	addInputProperty(i+1, "Source " + (i+1), AVType.AUDIO_VIDEO, NamedInput.VIDEO, true);
	}

	/*
	 *  (non-Javadoc)
	 * @see com.amx.duet.tools.comm.ICommonComponent#dispose()
	 */
	public void dispose()
	{
	}

	/* (non-Javadoc)
	 * @see com.amx.duet.devicesdk.component.SourceSelectComponentImpl#getInputCount()
	 */
	public int getInputCount()
	{
		m_caller.processInputCountEvent(new SourceSelectComponentEvent(this, inputs.size(), this.getSourceSelectComponentIndex()));
		return inputs.size();
	}

	/**
	 * Adds an InputProperty object to the source select list. Each object is 
	 * added to both an internal list within this class and the list stored in
	 * the parent class. Input select group number has a 1:1 correlation with 
	 * input source inputNumber. The input source should only be set to active 
	 * if it has been configured within the group to be the only selectable 
	 * source. Thus, in a device where all sources are selectable (mutually 
	 * exclusive) each source must be set as an active source.
	 * 
	 * @param groupNumber integer value representing the selectable source
	 * @param deviceLabel description provided by device to represent source
	 * @param avType Audio/Video type
	 * @param sourceType type of input source (NamedInput or SignalType)
	 * @param bActive specifies that this is a selectable source
	 */
	public void addInputProperty(int groupNumber, String deviceLabel, AVType avType, InputSourceSelect sourceType, boolean bActive)
	{
		InputProperty input = new InputProperty(groupNumber, deviceLabel, avType, sourceType);
		input.setActiveSource(bActive);
		this.addInputSourceSelectInfo(new InputSourceSelectInfo(input.getInputType(), input.getInputNumber()), input.getDeviceLabel());
		inputs.add(input);
	}

	/*
	 * Formats the input property for SNAPI by prepending the offsetted index
	 * 
	 * @param input InputProperty object which stores the current source attributes
	 * @param index list position of the current source
	 * @return Formatted property string
	 */
	private String formatInputProperty(InputProperty input, int index)
	{
		String fmt = "";
		if (input != null)
			fmt = (index + 1) + "," + input.getInputProperty();
		return fmt;
	}

	/*
	 * Set the active source within a group. For instance, if a group has 
	 * multiple signal types within it and only one can be selected from the 
	 * group based on configuration. Then the configured signal type must be the
	 * active source. Input select group number has a 1:1 correlation with input 
	 * source inputNumber
	 * 
	 * @param sourceSelect type of input source (NamedInput or SignalType)
	 * @param inputNumber input source select group number
	 */
	private int setActiveSource(InputSourceSelect sourceSelect, int inputNumber)
	{
		int index = Integer.MIN_VALUE;
		for (int i = 0; i < inputs.size(); i++)
		{
			InputProperty input = (InputProperty)inputs.get(i);
			if (input != null)
			{
				// Find all those within the group
				if (input.getInputNumber() == inputNumber)
				{
					// Find the matching signal type
					if (sourceSelect.equals(input.getInputType()))
					{
						input.setActiveSource(true);
						index = i+1;
					}
					else
					{
						// Set all others to inactive
						input.setActiveSource(false);
					}
				}
			}
		}
		
		return index;
	}

	/* (non-Javadoc)
	 * @see com.amx.duet.devicesdk.component.SourceSelectComponentImpl#getInputProperties()
	 */
	public String[] getInputProperties()
	{
		if (inputs.size() > 0)
		{
			String props[] = new String[inputs.size()];
			for (int i = 0; i < inputs.size(); i++)
			{
				InputProperty input = (InputProperty)inputs.get(i);
				if (input != null)
					props[i] = formatInputProperty(input, i);
			}
			m_caller.processInputPropertiesEvent(new SourceSelectComponentEvent(this, props, this.getSourceSelectComponentIndex()));
			return props;
		}
		
		return super.getInputProperties();
	}

	/* (non-Javadoc)
	 * @see com.amx.duet.devicesdk.component.SourceSelectComponentImpl#getInputProperty(int)
	 */
	public String getInputProperty(int index)
	{
		if (inputs.size() > 0 && index > 0 && index <= inputs.size())
		{
			InputProperty input = (InputProperty)inputs.get(index-1);
			if (input != null)
			{
				String prop = formatInputProperty(input, index-1);
				m_caller.processInputPropertyEvent(new SourceSelectComponentEvent(this, prop, this.getSourceSelectComponentIndex()));
				return prop;
			}
		}
		
		return super.getInputProperty(index);
	}

	/* (non-Javadoc)
	 * @see com.amx.duet.devicesdk.component.SourceSelectComponentImpl#getInputSelect()
	 */
	public int getInputSelect()
	{
		m_caller.processInputSelectEvent(new SourceSelectComponentEvent(this, m_inputSelected, this.getSourceSelectComponentIndex()));
		return m_inputSelected;
	}
	
	/**
	 * Returns the currently selected input.
	 * 
	 * @return int selected input value.
	 */
	public int getSelectedInput()
	{
		return m_inputSelected;
	}
	
	/**
	 * Returns the entire list of inputs.
	 * 
	 * @return Vector
	 */
	public Vector getInputs()
	{
		return inputs;
	}
	
	/**
	 * Check to see if the InputSourceSelect matches up with the input number.
	 * 
	 * @param sourceSelect type of input source (NamedInput or SignalType)
	 * @param inputNumber input source select group number
	 * @return true if the two values go together; false otherwise.
	 */
	public boolean isValidInput(InputSourceSelect sourceSelect, int inputNumber)
	{
		for (int i = 0; i < inputs.size(); i++)
		{
			InputProperty input = (InputProperty)inputs.get(i);
			if (input != null)
			{
				// Find all those within the group
				if (input.getInputNumber() == inputNumber)
				{
					// Find the matching signal type
					if (sourceSelect.equals(input.getInputType()))
						return true;
				}
			}
		}
		
		return false;
	}

	/* (non-Javadoc)
	 * @see com.amx.duet.devicesdk.component.SourceSelectComponentImpl#setInputSelect(int)
	 */
	public void setInputSelect(int index)
	{
		if (inputs.size() > 0 && index > 0 && index <= inputs.size())
		{
			InputProperty input = (InputProperty)inputs.get(index-1);
			if (input != null)
				this.setInputSource(input.getInputType(), input.getInputNumber());
		}
	}

	/* (non-Javadoc)
	 * @see com.amx.duet.devicesdk.component.SourceSelectComponentImpl#setInputSource(com.amx.duet.devicesdk.type.InputSourceSelect, int)
	 * 
	 * // TODO: Send your command to the device here...
	 * 
	 * if (sourceSelectInfo != null && sourceSelectInfo.getInputSourceSelect().equals(NamedInput.VIDEO))
	 * {
	 * 	byte cmd[] = {0x05,(byte)0xA3,(byte)(getSourceSelectComponentIndex()-1),(byte)(sourceSelectInfo.getInputNumber()-1)};
	 * 	((MakeModel)m_caller).sendCommand(cmd);
	 * }
	 */
	public void setInputSource(InputSourceSelect sourceSelect, int inputNumber)
	{
		// TODO: Send your command to the device here...
	}

	/* (non-Javadoc)
	 * @see com.amx.duet.devicesdk.component.SourceSelectComponentImpl#setInputSource(com.amx.duet.devicesdk.type.InputSourceSelectInfo)
	 */
	public void setInputSource(InputSourceSelectInfo sourceSelectInfo)
	{
		this.setInputSource(sourceSelectInfo.getInputSourceSelect(), sourceSelectInfo.getInputNumber());
	}

	/**
	 * Sets the current Source Select Input Source.<br>
	 * The new value is compared against the current value.  If the value has changed, the new
	 * value is saved and a process event for the Source Select Input Source is called.<br>
	 * <b>This method should not be overridden by the programmer unless 
	 *    non-standard behavior is desired.</b>
	 * @param index int the selected index within the source list
	 */
	public void updateInputSelect(int index)
	{
		if (m_inputSelected != index)
		{
			if (inputs.size() > 0 && index > 0 && index <= inputs.size())
			{
				InputProperty input = (InputProperty)inputs.get(index-1);
				if (input != null)
				{
					m_inputSelected = index;
					
					// Make sure this is the active source now
					if (!input.isActiveSource())
						setActiveSource(input.getInputType(), input.getInputNumber());
					
					// Call both process events to keep the input selection in synch between to the two different APIs
					m_caller.processInputSelectEvent(new SourceSelectComponentEvent(m_caller, index, this.getSourceSelectComponentIndex()));
					super.updateInputSource(input.getInputType(), input.getInputNumber());
				}
			}
		}
	}

	/*
	 *  (non-Javadoc)
	 * @see com.amx.duet.devicesdk.component.SourceSelectComponentImpl#updateInputSource(com.amx.duet.devicesdk.type.InputSourceSelect, int)
	 */
	public void updateInputSource(InputSourceSelect sourceSelect, int inputNumber)
	{
		// Make this the active source
		int index = setActiveSource(sourceSelect, inputNumber);
		if (index >= 0)
			this.updateInputSelect(index);
	}

	/* (non-Javadoc)
	 * @see com.amx.duet.devicesdk.component.SourceSelectComponentImpl#refresh()
	 */
	public void refresh()
	{
		// This calls the process event anyway.
		this.getInputSelect();
		
		super.refresh();
	}

	/* (non-Javadoc)
	 * @see com.amx.duet.devicesdk.component.SourceSelectComponentImpl#reinitialize()
	 */
	public void reinitialize()
	{
		m_inputSelected = Integer.MIN_VALUE;
		
		super.reinitialize();
	}
	
	/**
	 * Returns the currently stored SourceSelectComponent.<br>
	 * @return ISourceSelectComponent
	 */
	public ISourceSelectComponent getSourceSelectComponent()
	{
		return m_caller;
	}
}
