/*
 * @author VGonzales
 * Created on May 21, 2005
 * 
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.io;

import java.io.EOFException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import com.amx.duet.tools.lang.ObjectRef;
import com.amx.duet.tools.lang.ObjectType;

/**
 * The FileManager class was created in order to simulate NetLinx file processing. It allows you to 
 * open multiple files and manipulate each one individualy until the user chooses to finish 
 * processing data by closing each file separately.<P>
 * 
 * Files are processed using the file handle retrieved during a call to FileOpen(). Once the file 
 * handle is obtained it can be used in subsequent read, write and close operations.<P>
 * 
 * How to use the FileManager:<br>
 * <PRE>
 * String str = "Hello World!";
 * long lRet = FileManager.FILE_OK;
 * 
 * FileManager mngr = FileManager.getInstance();
 * long handle = mngr.FileOpen("/user/mydir/myfile.txt", FileManager.IOFLAG_NEW);
 * lRet = mngr.FileWrite(handle, str, str.length());
 * if (lRet != FileManager.FILE_OK)
 * 	error("Error [" + lRet + "] occurred while writing to file"); 
 * 
 * ObjectRef rBuffer = new ObjectRef("");
 * long len = str.length();
 * lRet = mngr.FileRead(handle, rBuffer, len);
 * if (lRet == FileManager.FILE_OK)
 * 	diag("rBuffer = " + rBuffer.toString());
 * lRet = mngr.FileClose(handle); 
 * </PRE>
 * 
 * @version 1.0.0
 * @since AMXTools 1.0.0
 */
public class FileManager
{
	public final static int FILE_OK                   = 0;
	public final static int FILE_INVALID_HANDLE       = -1;
	public final static int FILE_INVALID_FILENAME     = -2;
	public final static int FILE_INVALID_IOFLAG       = -3;
	public final static int FILE_INVALID_DIRECTORY    = -4;
	public final static int FILE_DISK_ERROR           = -5;
	public final static int FILE_INVALID_PARAMETER    = -6;
	public final static int FILE_ALREADY_CLOSE        = -7;
	public final static int FILE_NAME_EXISTS          = -8;
	public final static int FILE_END_OF_FILE          = -9;
	public final static int FILE_BUFFER_TOO_SMALL     = -10;
	public final static int FILE_DISK_FULL            = -11;
	public final static int FILE_DIRECTORY_NOT_LOADED = -12;
	public final static int FILE_DIRECTORY_EXISTS     = -13;
	
	public final static int IOFLAG_READ               = 1;
	public final static int IOFLAG_NEW                = 2;
	public final static int IOFLAG_APPEND             = 3;

	private static FileManager m_instance = null;
	private Map fileMap = Collections.synchronizedMap(new HashMap());
	private Object sync = new Object();

	/*
	 * Do not allow this class to be instantiated by anyone else
	 */
	private FileManager() {}
	
	/**
	 * Get a single instance of this class
	 * @return a static instance of the FileManager class
	 */
	public static FileManager getInstance()
	{
		if (m_instance == null)
			m_instance = new FileManager();
		
		return m_instance;
	}

	/**
	 * Opens a file for reading or writing.<P>
	 *
	 * If the file is opened successfully, it must be closed (after all reading or writing is 
	 * completed) by calling FileClose(). If files are not closed, subsequent file open operations 
	 * may fail due to the limited number of file handles available.
	 *  
	 * @param sFilePath String containing the path to the file to be opened.
	 * @param lIOFlag
	 * <ul>
	 * <li>1 = Read: The file is opened with READONLY status.
	 * <li>2 = R/W New: The file is opened with READWRITE status. If the file currently exists, its 
	 * contents are erased.
	 * <li>3 = R/W Append: The file is opened with READWRITE status. The current contents of the file 
	 * are preserved and the file pointer is set to point to the end of the file.
	 * </ul>
	 * @return If the open operation is successful, this function returns a non-zero integer value 
	 * representing the handle to the file. This handle must be used in subsequent read, write and 
	 * close operations.
	 * <ul>
	 * <li>0 = handle to file (open was successful)
	 * <li>-2 = invalid file path or name
	 * <li>-5 = disk I/O error
	 * <li>-3 = invalid value supplied for IOFlag
	 * </ul>
	 */
	public long FileOpen(String sFilePath, long lIOFlag)
	{
		long lRet = FILE_OK;
	    try 
	    {
			if (lIOFlag < IOFLAG_READ && lIOFlag > IOFLAG_APPEND)
				return FILE_INVALID_IOFLAG;
			
	        ManagedFile file = new ManagedFile(sFilePath, lIOFlag);
	        
	        if (lIOFlag == IOFLAG_READ)
	        {
		        if (!file.exists())
		        {
		        	lRet = FILE_INVALID_FILENAME;
		        }
		        else
		        {
		        	file.setAccessor("r");
		        }
	        }
	        else if (lIOFlag == IOFLAG_APPEND)
	        {
		        if (!file.exists())
		        {
			        // Create file if it does not exist
			        if (!file.createNewFile())
			        {
			        	// Something bad must have happend
			        	lRet = FILE_DISK_ERROR;
			        }
		        }

				if (lRet == FILE_OK)
		        	file.setAccessor("rw");
	        }
	        else
	        {
		        if (file.exists())
		        	file.delete();
		        
		        // Create file if it does not exist
		        if (file.createNewFile())
	        		file.setAccessor("rw");
	        	else
		        	lRet = FILE_DISK_ERROR;
	        }

			
			if (file != null && lRet == FILE_OK)
			{
				fileMap.put(new Long(file.getHandle()), file);
		        lRet = file.getHandle();
			}
	    }
	    catch (IOException e)
	    {
        	lRet = FILE_DISK_ERROR;
	    }
	    catch (Exception e)
	    {
        	lRet = FILE_DISK_ERROR;
	    }
	    
	    // Send an error because we didn't get a file handle back
	    if (lRet == FILE_OK)
	    	lRet = FILE_INVALID_HANDLE;

		return lRet;
	}

	/**
	 * Closes a file opened with FileOpen(). This function should be called when all reading or 
	 * writing to the file is completed.
	 *  
	 * @param lFileHandle handle to the file returned by FileOpen().
	 * @return
	 * <ul>
	 * <li>0 = operation was successful
	 * <li>-1 = invalid file handle
	 * <li>-7 = file already closed
	 * <li>-5 = disk I/O error
	 * </ul>
	 */
	public long FileClose(long lFileHandle)
	{
		long lRet = FILE_OK;
		
		ManagedFile file = getFile(lFileHandle);
		if (file != null)
		{
			if (file.getAccessor() != null)
				lRet = file.setAccessor("");
			else
				lRet = FILE_ALREADY_CLOSE;
		}
		else
		{
			lRet = FILE_INVALID_HANDLE;
		}
		
		return lRet;
	}

	/**
	 * Reads a block of data from the specified file.
	 *
	 * This function reads (from the current location of the file pointer) the number of bytes 
	 * specified by lBufLen or fewer bytes if the end of file is reached. The bytes are read from 
	 * the file identified by lFileHandle and are stored in rStrBuffer. The file pointer will 
	 * automatically be advanced the correct number of bytes so the next read operation continues 
	 * where the last operation left off.
	 * 
	 * @param lFileHandle handle to the file returned by FileOpen().
     * @param rStrBuffer Must be of type <STRONG>String</STRONG>. Buffer to hold the data to be read.<P>
     *  
	 * Create rStrBuffer with a <STRONG>valueOf(String)</STRONG> Example: <br>
	 * ObjectRef rStrBuffer = new ObjectRef("") or rStrBuffer.valueOf("")
	 * @param lBufLen maximum number of bytes to read.
	 * @return
	 * <ul>
	 * <li>0 = the number of bytes actually read
	 * <li>-1 = invalid file handle
	 * <li>-9 = end-of-file reached
	 * <li>-5 = disk I/O error
	 * <li>-6 = invalid parameter
	 * </ul>
	 */
	public long FileRead(long lFileHandle, ObjectRef rStrBuffer, long lBufLen)
	{
		long lRet = FILE_OK;

		if (!ObjectType.STRING.equals(rStrBuffer.getType()))
			return FILE_INVALID_PARAMETER;
		
		RandomAccessFile access = getFileAccess(lFileHandle);
		if (access != null)
		{
			StringBuffer buffer = new StringBuffer();
			int count = 0;
			try
			{
				long len = access.length();
				if (len > 0)
				{
					if (len > lBufLen)
						len = lBufLen;
					
					while(buffer.length() < len)
					{
						// Read to a temp variable so that we can check for -1
						// values before appending data
						int temp = access.read();
						if (temp >= 0)
						{
							buffer.append((char)temp);
							count++;
						}
						else
						{
							lRet = FILE_END_OF_FILE;
							break;
						}
					}
					
					if (buffer.length() > 0)
					{
						rStrBuffer.valueOf(buffer.toString());
						lRet = buffer.length();
					}
				}
				else
					lRet = FILE_END_OF_FILE;
			}
			catch (EOFException e)
			{
				if (buffer.length() > 0)
				{
					rStrBuffer.valueOf(buffer.toString());
					lRet = buffer.length();
				}
				else
					lRet = FILE_END_OF_FILE;
			}
			catch (IOException e)
			{
				lRet = FILE_DISK_ERROR;
			}
		    catch (Exception e)
		    {
	        	lRet = FILE_DISK_ERROR;
		    }
		}
		else
		{
			lRet = FILE_INVALID_HANDLE;
		}
		
		return lRet;
	}
	
	/**
	 * Reads a line of data from the specified file.<P>
	 * 
	 * This function reads from the current location of the file pointer up to the next carriage 
	 * return or to the end-of-file (EOF), whichever comes first. A complete line will not be read 
	 * if the buffer length is exceeded before a carriage return (or EOF) is encountered. The bytes 
	 * are read from the file identified by lFileHandle and are stored in rStrBuffer. The <CR> or 
	 * <CR><LF> pair will not be stored in rStrBuffer. If a complete line is read, the file pointer 
	 * is advanced to the next character in the file after the <CR> or <CR><LF> pair or to the EOF 
	 * if the last line was read.
	 * 
	 * @param lFileHandle handle to the file returned by FileOpen().
     * @param rStrBuffer Must be of type <STRONG>String</STRONG>. Buffer to hold the data to be read.<P>
     *  
	 * Create rStrBuffer with a <STRONG>valueOf(String)</STRONG> Example: <br>
	 * ObjectRef rStrBuffer = new ObjectRef("") or rStrBuffer.valueOf("")
	 * @param lBufLen maximum number of bytes to read.
	 * @return
	 * <ul>
	 * <li>0 = the number of bytes actually read
	 * <li>-1 = invalid file handle
	 * <li>-9 = EOF (end-of-file) reached
	 * <li>-5 = disk I/O error
	 * <li>-6 = invalid parameter (buffer length must be greater than zero)
	 * </ul>
	 */
	public long FileReadLine(long lFileHandle, ObjectRef rStrBuffer, long lBufLen)
	{
		long lRet = FILE_OK;
		
		if (!ObjectType.STRING.equals(rStrBuffer.getType()))
			return FILE_INVALID_PARAMETER;

		RandomAccessFile access = getFileAccess(lFileHandle);
		if (access != null)
		{
			StringBuffer buffer = new StringBuffer();
			try
			{
				if (access.length() > 0)
				{
					// Read to a temp variable so that we can check for "null"
					// values before appending data
					Object temp = access.readLine();
					if (temp != null)
					{
						buffer.append((String)temp);
						if (lBufLen > 0 && lBufLen < buffer.length())
							rStrBuffer.valueOf(buffer.substring((int)lBufLen));
						else
							rStrBuffer.valueOf(buffer.toString());
							
						lRet = buffer.length();
					}
					else
					{
						lRet = FILE_END_OF_FILE;
					}
				}
				else
				{
					lRet = FILE_END_OF_FILE;
				}
			}
			catch (EOFException e)
			{
				if (buffer.length() > 0)
				{
					rStrBuffer.valueOf(buffer.toString());
					lRet = buffer.length();
				}
				else
					lRet = FILE_END_OF_FILE;
			}
			catch (IOException e)
			{
				lRet = FILE_DISK_ERROR;
			}
		    catch (Exception e)
		    {
	        	lRet = FILE_DISK_ERROR;
		    }
		}
		else
		{
			lRet = FILE_INVALID_HANDLE;
		}
		
		return lRet;
	}

	/**
	 * Sets the file pointer to the specified position.<P>
	 * 
	 * After FileSeek() is successfully called, subsequent read or write operations begin at the 
	 * byte number specified by lPos.
	 * 
	 * @param lFileHandle handle to the file returned by FileOpen().
	 * @param lPos The byte position to set the file pointer (0 = beginning of file, -1 = end of file)
	 * @return
	 * <ul>
	 * <li>0 = Operation was successful and the result is the current file pointer value
	 * <li>-1 = invalid file handle
	 * <li>-6 = Invalid parameter (pos points beyond the end-of-file (position is set to the end-of-file))
	 * <li>-5 = disk I/O error
	 * </ul>
	 */
	public long FileSeek(long lFileHandle, long lPos)
	{
		long lRet = FILE_OK;
		
		RandomAccessFile access = getFileAccess(lFileHandle);
		if (access != null)
		{
			try
			{
				if (access.length() >= lPos)
				{
					synchronized (sync)
					{
						access.seek(lPos);
					}
					lRet = access.getFilePointer();
				}
				else
				{
					lRet = FILE_INVALID_PARAMETER;
				}
			}
			catch (IOException e)
			{
				lRet = FILE_DISK_ERROR;
			}
		    catch (Exception e)
		    {
	        	lRet = FILE_DISK_ERROR;
		    }
		}
		else
		{
			lRet = FILE_INVALID_HANDLE;
		}
		
		return lRet;
	}
	
	/**
	 * Writes a block of data to the specified file.<P>
	 *
	 * The data will overwrite or append to the current contents of the file depending on the current 
	 * position of the file pointer.
	 * 
	 * @param lFileHandle handle to the file returned by FileOpen().
	 * @param sBuffer buffer containing the data to write.
	 * @param lBufLen number of bytes to write.
	 * @return
	 * <ul>
	 * <li>0 = the number of bytes actually written
	 * <li>-1 = invalid file handle
	 * <li>-11 = disk full
	 * <li>-5 = disk I/O error
	 * <li>-6 = invalid parameter (buffer length must be greater than zero)
	 * </ul>
	 */
	public long FileWrite(long lFileHandle, String sBuffer, long lBufLen)
	{
		long lRet = FILE_OK;
		
		if (sBuffer.length() == 0)
			return FILE_INVALID_PARAMETER;
		
		RandomAccessFile access = getFileAccess(lFileHandle);
		if (access != null)
		{
			try
			{
				String sub = sBuffer;
				if (sBuffer.length() > lBufLen)
					sub = sBuffer.substring((int)lBufLen);
				synchronized (sync)
				{
					access.writeBytes(sub);
				}
				lRet = sub.getBytes().length;
			}
			catch (IOException e)
			{
				lRet = FILE_DISK_ERROR;
			}
		    catch (Exception e)
		    {
	        	lRet = FILE_DISK_ERROR;
		    }
		}
		else
		{
			lRet = FILE_INVALID_HANDLE;
		}
		
		return lRet;
	}
	
	/**
	 * Writes a line of data to the specified file.<P>
	 *
	 * A <CR> <CR> <LF> character string is automatically appended to the end of the line.
	 * 
	 * @param lFileHandle handle to the file returned by FileOpen().
	 * @param sBuffer buffer containing the data to write.
	 * @param lLineLen number of bytes to write.
	 * @return
	 * <ul>
	 * <li>0 = the number of bytes actually written
	 * <li>-1 = invalid file handle
	 * <li>-11 = disk full
	 * <li>-5 = disk I/O error
	 * <li>-6 = invalid parameter (LineLen must be greater than zero)
	 * </ul>
	 */
	public long FileWriteLine(long lFileHandle, String sBuffer, long lLineLen)
	{
		long lRet = FILE_OK;
		
		if (sBuffer.length() == 0)
			return FILE_INVALID_PARAMETER;

		RandomAccessFile access = getFileAccess(lFileHandle);
		if (access != null)
		{
			try
			{
				String sub = sBuffer;
				if (sBuffer.length() > lLineLen)
					sub = sBuffer.substring((int)lLineLen);
				sub += System.getProperty("line.separator");
				synchronized (sync)
				{
					access.writeBytes(sub);
				}
				lRet = sub.getBytes().length;
			}
			catch (IOException e)
			{
				lRet = FILE_DISK_ERROR;
			}
		    catch (Exception e)
		    {
	        	lRet = FILE_DISK_ERROR;
		    }
		}
		else
		{
			lRet = FILE_INVALID_HANDLE;
		}
		
		return lRet;
	}
	
	/**
	 * Retrieves the size of a file on disk.
	 * 
	 * @param lFileHandle handle to the file returned by FileOpen().
	 * @return the length of data contained in the file or:
	 * <ul>
	 * <li>0 = Operation was successful and the file contains no data.
	 * <li>-1 = invalid file handle
	 * </ul>
	 */
	public long FileSize(long lFileHandle)
	{
		long lRet = FILE_OK;
		ManagedFile file = getFile(lFileHandle);
		if (file != null)
			lRet = file.length();
		else
			lRet = FILE_INVALID_HANDLE;
		return lRet;
	}
	
	/*
	 * Use the file handle passed in to get the file from the file map
	 * if the file exists.
	 * 
	 * @param lFileHandle handle to the file returned by FileOpen
	 * @return ManagedFile the file object stored
	 */
	private ManagedFile getFile(long lFileHandle)
	{
		ManagedFile file = null;
		Long lKey = new Long(lFileHandle);
		if (fileMap.containsKey(lKey))
			file = (ManagedFile)fileMap.get(lKey);
		return file;
	}
	
	/*
	 * Use the file handle passed in to get the accessor of the file if
	 * the file exists.
	 * 
	 * @param lFileHandle handle to the file returned by FileOpen
	 * @return RandomAccessFile the accessor that belongs to the object 
	 * stored
	 */
	private RandomAccessFile getFileAccess(long lFileHandle)
	{
		RandomAccessFile access = null;
		ManagedFile file = getFile(lFileHandle);
		if (file != null)
			access = file.getAccessor();
		return access;
	}

}
