/*
 * @author vgonzales
 * 
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.io;

import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import com.amx.duet.tools.lang.ObjectRef;
import com.amx.duet.tools.lang.ObjectType;

/**
 * A utility class that performs file operations similar to NetLinx file 
 * operations.
 */
public class FileUtil
{
	public final static int FILE_OK                   = 0;
	public final static int FILE_INVALID_HANDLE       = -1;
	public final static int FILE_INVALID_FILENAME     = -2;
	public final static int FILE_INVALID_IOFLAG       = -3;
	public final static int FILE_INVALID_DIRECTORY    = -4;
	public final static int FILE_DISK_ERROR           = -5;
	public final static int FILE_INVALID_PARAMETER    = -6;
	public final static int FILE_ALREADY_CLOSE        = -7;
	public final static int FILE_NAME_EXISTS          = -8;
	public final static int FILE_END_OF_FILE          = -9;
	public final static int FILE_BUFFER_TOO_SMALL     = -10;
	public final static int FILE_DISK_FULL            = -11;
	public final static int FILE_DIRECTORY_NOT_LOADED = -12;
	public final static int FILE_DIRECTORY_EXISTS     = -13;
	
	public final static int IOFLAG_READ               = 1;
	public final static int IOFLAG_NEW                = 2;
	public final static int IOFLAG_APPEND             = 3;
	
	/**
	 * Closes a file opened with FileOpen(). This function should be called when all reading or 
	 * writing to the file is completed.
	 *  
	 * @param lFileHandle handle to the file returned by FileOpen().
	 * @return
	 * <ul>
	 * <li>0 = operation was successful
	 * <li>-1 = invalid file handle
	 * <li>-7 = file already closed
	 * <li>-5 = disk I/O error
	 * </ul>
	 */
	public final static long FileClose(long lFileHandle)
	{
		return FileManager.getInstance().FileClose(lFileHandle);
	}
	
	/**
	 * Copies the specified file from the source location to the specified destination or name. 
	 * If either path name fails to specify a directory, the current working directory is assumed.
	 * 
	 * @param sSrcFilePath path name of the file to copy (source).
	 * @param sDstFilePath path name of the copied file (destination).
	 * @return
	 * <ul>
	 * <li>0 = operation was successful
	 * <li>-2 = invalid file name
	 * <li>-11 = disk full
	 * <li>-5 = disk I/O error
	 * </ul>
	 */
	public final static long FileCopy(String sSrcFilePath, String sDstFilePath)
	{
		long lRet = FILE_OK;
		
		// Copies src file to dst file.
		// If the dst file does not exist, it is created
		try
		{
			if (!(new File(sSrcFilePath)).exists())
				return FILE_INVALID_FILENAME;
				
			InputStream in = new FileInputStream(sSrcFilePath);
			OutputStream out = new FileOutputStream(sDstFilePath);
    
			// Transfer bytes from in to out
			byte[] buf = new byte[1024];
			int len = 0;
			while ((len = in.read(buf)) > 0)
			{
				out.write(buf, 0, len);
			}
			
			in.close();
			out.close();
		}
		catch (IOException e)
		{
			lRet = FILE_DISK_ERROR;
		}
		return lRet;
	}
	
	/**
	 * Create the specified directory path. This method will also create the number of 
	 * subdirectories needed to complete the directory path.
	 * 
	 * @param sDirPath string containing the directory path to create.
	 * @return
	 * <ul>
	 * <li>0 = operation was successful
	 * <li>-4 = invalid directory path
	 * <li>-5 = disk I/O error
	 * <li>-13 = directory name exists
	 * </ul>
	 */
	public final static long FileCreateDir(String sDirPath)
	{
		long lRet = FILE_OK;

		try
		{
			sDirPath = getUserDirectory(sDirPath);
			
			if ((new File(sDirPath)).exists())
				return FILE_DIRECTORY_EXISTS;

			// Create a directory; all non-existent ancestor directories are
			// automatically created
			if (!(new File(sDirPath)).mkdirs())
			{
				// Directory creation failed
				lRet = FILE_INVALID_DIRECTORY;
			}
		}
		catch (Exception e)
		{
			lRet = FILE_DISK_ERROR;
		}
		
		return lRet;
	}
	
	/**
	 * Deletes the specified file(s).
	 * 
	 * @param sFilePath path name of the file to delete. Wildcard characters (* and ?) are 
	 * permitted in the path name.
	 * @return
	 * <ul>
	 * <li>0 = operation was successful
	 * <li>-2 = invalid file path or name
	 * <li>-5 = disk I/O error
	 * </ul>
	 */
	public final static long FileDelete(String sFilePath)
	{
		long lRet = FILE_OK;
		try
		{
			if (!(new File(sFilePath)).delete()) 
			{
				// Deletion failed
				lRet = FILE_INVALID_FILENAME;
			}
		}
		catch (Exception e)
		{
			lRet = FILE_DISK_ERROR;
		}
		return lRet;
	}
	
	/**
	 * Returns a list of files located at the specified path.<P>
	 * 
	 * Each directory entry will have a <CR><LF> character pair appended to the end.
	 * 
	 * @param sDirPath String containing the path to the requested directory.
     * @param rStrBuffer Must be of type <STRONG>String</STRONG>. buffer to hold the directory list.<P> 
     *  
	 * Create rStrBuffer with a <STRONG>valueOf(String)</STRONG> Example: <br>
	 * <code>ObjectRef rStrBuffer = new ObjectRef("") or rStrBuffer.valueOf("")</code>
     * @param rEntry Must be of type <STRONG>String</STRONG>. requested directory entry..<P> 
     *  
	 * Create rEntry with a <STRONG>valueOf(String)</STRONG> Example: <br>
	 * <code>ObjectRef rEntry = new ObjectRef("") or rEntry.valueOf("")</code>
	 * @return This method returns the number of remaining files in the directory, or:
	 * <ul>
	 * <li>-4 = invalid directory path
	 * <li>-10 = buffer too small
	 * <li>-6 = invalid parameter (i.e. Entry points beyond the end of the directory)
	 * <li>-12 = directory not loaded
	 * <li>-5 = Disk I/O error
	 * </ul>
	 */
	public final static long FileDir(String sDirPath, ObjectRef rStrBuffer, ObjectRef rEntry)
	{
		long lRet = FILE_OK;
		try
		{
			if ((!ObjectType.STRING.equals(rStrBuffer.getType())) ||
				(!ObjectType.LONG.equals(rEntry.getType())))
				return FILE_INVALID_PARAMETER;
				
			sDirPath = getUserDirectory(sDirPath);
				
			if (!(new File(sDirPath)).exists())
				return FILE_INVALID_DIRECTORY;
				
			File dir = new File(sDirPath);
    
			// The list of files can also be retrieved as File objects
			File[] files = dir.listFiles();
    
			// This filter only returns directories
			FileFilter fileFilter = new FileFilter()
			{
				public boolean accept(File file)
				{
					return file.isFile();
				}
			};
			
			files = dir.listFiles(fileFilter);
			if (files.length == 0)
				return FILE_DIRECTORY_NOT_LOADED;
				
			if (rEntry.toLong() > files.length)
				return FILE_INVALID_PARAMETER;
				
			rStrBuffer.valueOf(files[rEntry.toInt()].getName() + 
				System.getProperty("line.separator"));
				
			lRet = files.length;
		}
		catch (Exception e)
		{
			lRet = FILE_DISK_ERROR;
		}
		return lRet;
	}
	
	/**
	 * Returns the current working directory.
	 * 
     * @param rStrDirPath Must be of type <STRONG>String</STRONG>. Buffer to receive the current 
     * working directory.<P> 
     *  
	 * Create rStrDirPath with a <STRONG>valueOf(String)</STRONG> Example: <br>
	 * <code>ObjectRef rStrDirPath = new ObjectRef("") or rStrDirPath.valueOf("")</code>
	 * @return
	 * <ul>
	 * <li>0 = operation was successful
	 * <li>-10 = size of DirPath buffer insufficient to hold directory path name
	 * </ul>
	 */
	public final static long FileGetDir(ObjectRef rStrDirPath)
	{
		long lRet = FILE_OK;
		
		rStrDirPath.valueOf(System.getProperty("user.dir"));

		return lRet;
	}
	
	/**
	 * Opens a file for reading or writing.<P>
	 *
	 * If the file is opened successfully, it must be closed (after all reading or writing is 
	 * completed) by calling FileClose(). If files are not closed, subsequent file open operations 
	 * may fail due to the limited number of file handles available.
	 *  
	 * @param sFilePath String containing the path to the file to be opened.
	 * @param lIOFlag<br>
	 * 1 = Read: The file is opened with READONLY status.<br>
	 * <br>
	 * 2 = R/W New: The file is opened with READWRITE status. If the file currently exists, its 
	 * contents are erased.<br>
	 * <br>
	 * 3 = R/W Append: The file is opened with READWRITE status. The current contents of the file 
	 * are preserved and the file pointer is set to point to the end of the file.
	 * @return If the open operation is successful, this function returns a non-zero integer value 
	 * representing the handle to the file. This handle must be used in subsequent read, write and 
	 * close operations.
	 * <ul>
	 * <li>0 = handle to file (open was successful)
	 * <li>-2 = invalid file path or name
	 * <li>-5 = disk I/O error
	 * <li>-3 = invalid value supplied for IOFlag
	 * </ul>
	 */
	public final static long FileOpen(String sFilePath, long lIOFlag)
	{
		sFilePath = getUserDirectory(sFilePath);
		return FileManager.getInstance().FileOpen(sFilePath, lIOFlag);
	}
	
	/**
	 * Reads a block of data from the specified file.
	 *
	 * This function reads (from the current location of the file pointer) the number of bytes 
	 * specified by lBufLen or fewer bytes if the end of file is reached. The bytes are read from 
	 * the file identified by lFileHandle and are stored in rStrBuffer. The file pointer will 
	 * automatically be advanced the correct number of bytes so the next read operation continues 
	 * where the last operation left off.
	 * 
	 * @param lFileHandle handle to the file returned by FileOpen().
     * @param rStrBuffer Must be of type <STRONG>String</STRONG>. Buffer to hold the data to be read.<P>
     *  
	 * Create rStrBuffer with a <STRONG>valueOf(String)</STRONG> Example: <br>
	 * <code>ObjectRef rStrBuffer = new ObjectRef("") or rStrBuffer.valueOf("")</code>
	 * @param lBufLen maximum number of bytes to read.
	 * @return
	 * <ul>
	 * <li>0 = the number of bytes actually read
	 * <li>-1 = invalid file handle
	 * <li>-9 = end-of-file reached
	 * <li>-5 = disk I/O error
	 * <li>-6 = invalid parameter
	 * </ul>
	 */
	public final static long FileRead(long lFileHandle, ObjectRef rStrBuffer, long lBufLen)
	{
		return FileManager.getInstance().FileRead(lFileHandle, rStrBuffer, lBufLen);
	}
	
	/**
	 * Reads a line of data from the specified file.<P>
	 * 
	 * This function reads from the current location of the file pointer up to the next carriage 
	 * return or to the end-of-file (EOF), whichever comes first. A complete line will not be read 
	 * if the buffer length is exceeded before a carriage return (or EOF) is encountered. The bytes 
	 * are read from the file identified by lFileHandle and are stored in rStrBuffer. The <CR> or 
	 * <CR><LF> pair will not be stored in rStrBuffer. If a complete line is read, the file pointer 
	 * is advanced to the next character in the file after the <CR> or <CR><LF> pair or to the EOF 
	 * if the last line was read.
	 * 
	 * @param lFileHandle handle to the file returned by FileOpen().
     * @param rStrBuffer Must be of type <STRONG>String</STRONG>. Buffer to hold the data to be read.<P>
     *  
	 * Create rStrBuffer with a <STRONG>valueOf(String)</STRONG> Example: <br>
	 * <code>ObjectRef rStrBuffer = new ObjectRef("") or rStrBuffer.valueOf("")</code>
	 * @param lBufLen maximum number of bytes to read.
	 * @return
	 * <ul>
	 * <li>0 = the number of bytes actually read
	 * <li>-1 = invalid file handle
	 * <li>-9 = EOF (end-of-file) reached
	 * <li>-5 = disk I/O error
	 * <li>-6 = invalid parameter (buffer length must be greater than zero)
	 * </ul>
	 */
	public final static long FileReadLine(long lFileHandle, ObjectRef rStrBuffer, long lBufLen)
	{
		return FileManager.getInstance().FileReadLine(lFileHandle, rStrBuffer, lBufLen);
	}
	
	/**
	 * Removes the specified directory path and all subdirectories below it. This function should be 
	 * used with extreme caution.<P>
	 * 
	 * This function will remove the specified directory and all subdirectories below that directory. 
	 * It will also delete all files contained in any of these directories.
	 * 
	 * @param sDirPath string containing the directory path to remove.
	 * @return
	 * <ul>
	 * <li>0 = operation was successful
	 * <li>-4 = invalid directory path
	 * <li>-5 = disk I/O error
	 * </ul>
	 */
	public final static long FileRemoveDir(String sDirPath)
	{
		long lRet = FILE_OK;
		try
		{
			sDirPath = getUserDirectory(sDirPath);
			File dir = new File(sDirPath);
			if (dir.exists())
			{
				if (!deleteDir(dir))
					lRet = FILE_DISK_ERROR;
			}
			else
			{
				lRet = FILE_INVALID_DIRECTORY;
			}
		}
		catch (Exception e)
		{
			lRet = FILE_DISK_ERROR;
		}
		return lRet;
	}
	
	/**
	 * Renames the specified file.
	 * 
	 * @param sFilePath path name of the file to rename.
	 * @param sNewFileName new file name. This name must not contain a directory path.
	 * @return
	 * <ul>
	 * <li>0 = operation was successful
	 * <li>-2 = invalid file name
	 * <li>-8 = file name exists
	 * <li>-5 = disk I/O error
	 * </ul>
	 */
	public final static long FileRename(String sFilePath, String sNewFileName)
	{
		long lRet = FILE_OK;
		try
		{
			sFilePath = getUserDirectory(sFilePath);
			// File (or directory) with old name
			File file = new File(sFilePath);
			if (!file.exists())
				return FILE_INVALID_FILENAME;
    
			sNewFileName = getUserDirectory(sNewFileName);
			// File (or directory) with new name
			File file2 = new File(sNewFileName);
			if (file2.exists())
				return FILE_NAME_EXISTS;
    
			// Rename file (or directory)
			if (!file.renameTo(file2))
			{
				// File was not successfully renamed
				lRet = FILE_DISK_ERROR;
			}
		}
		catch (Exception e)
		{
			lRet = FILE_DISK_ERROR;
		}
		return lRet;
	}
	
	/**
	 * Retrieves the size of a file on disk.
	 *  
	 * @param sFilePath path name of the file to get the size of.
	 * @return the length of data contained in the file or:
	 * <ul>
	 * <li>0 = Operation was successful and the file contains no data.
	 * <li>-2 = invalid file name
	 * <li>-5 = disk I/O error
	 * </ul>
	 */
	public final static long FileSize(String sFilePath)
	{
		long lRet = FILE_OK;
		try
		{
			sFilePath = getUserDirectory(sFilePath);
			// File (or directory) with old name
			File file = new File(sFilePath);
			if (!file.exists())
				return FILE_INVALID_FILENAME;
				
			lRet = file.length();
		}
		catch (Exception e)
		{
			lRet = FILE_DISK_ERROR;
		}
		return lRet;
	}

	/**
	 * Retrieves the size of a file on disk.
	 * 
	 * @param lFileHandle handle to the file returned by FileOpen().
	 * @return the length of data contained in the file or:
	 * <ul>
	 * <li>0 = Operation was successful and the file contains no data.
	 * <li>-1 = invalid file handle
	 * </ul>
	 */
	public final static long FileSize(long lFileHandle)
	{
		return FileManager.getInstance().FileSize(lFileHandle);
	}
	
	/**
	 * Sets the file pointer to the specified position.<P>
	 * 
	 * After FileSeek() is successfully called, subsequent read or write operations begin at the 
	 * byte number specified by lPos.
	 * 
	 * @param lFileHandle handle to the file returned by FileOpen().
	 * @param lPos The byte position to set the file pointer (0 = beginning of file, -1 = end of file)
	 * @return
	 * <ul>
	 * <li>0 = Operation was successful and the result is the current file pointer value
	 * <li>-1 = invalid file handle
	 * <li>-6 = Invalid parameter (pos points beyond the end-of-file (position is set to the end-of-file))
	 * <li>-5 = disk I/O error
	 * </ul>
	 */
	public final static long FileSeek(long lFileHandle, long lPos)
	{
		return FileManager.getInstance().FileSeek(lFileHandle, lPos);
	}
	
	/**
	 * Writes a block of data to the specified file.<P>
	 *
	 * The data will overwrite or append to the current contents of the file depending on the current 
	 * position of the file pointer.
	 * 
	 * @param lFileHandle handle to the file returned by FileOpen().
	 * @param sBuffer buffer containing the data to write.
	 * @param lBufLen number of bytes to write.
	 * @return
	 * <ul>
	 * <li>0 = the number of bytes actually written
	 * <li>-1 = invalid file handle
	 * <li>-11 = disk full
	 * <li>-5 = disk I/O error
	 * <li>-6 = invalid parameter (buffer length must be greater than zero)
	 * </ul>
	 */
	public final static long FileWrite(long lFileHandle, String sBuffer, long lBufLen)
	{
		return FileManager.getInstance().FileWrite(lFileHandle, sBuffer, lBufLen);
	}
	
	/**
	 * Writes a line of data to the specified file.<P>
	 *
	 * A <CR> <CR> <LF> character string is automatically appended to the end of the line.
	 * 
	 * @param lFileHandle handle to the file returned by FileOpen().
	 * @param sBuffer buffer containing the data to write.
	 * @param lLineLen number of bytes to write.
	 * @return
	 * <ul>
	 * <li>0 = the number of bytes actually written
	 * <li>-1 = invalid file handle
	 * <li>-11 = disk full
	 * <li>-5 = disk I/O error
	 * <li>-6 = invalid parameter (LineLen must be greater than zero)
	 * </ul>
	 */
	public final static long FileWriteLine(long lFileHandle, String sBuffer, long lLineLen)
	{
		return FileManager.getInstance().FileWriteLine(lFileHandle, sBuffer, lLineLen);
	}

	/**
	 * Deletes all files and subdirectories under the specified directory. Returns true if 
	 * all deletions were successful. If a deletion fails, the method stops attempting 
	 * to delete and returns false.
	 * 
	 * @param dir
	 * @return
	 */
	public final static boolean deleteDir(File dir)
	{
		if (dir.isDirectory())
		{
			String[] children = dir.list();
			for (int i=0; i<children.length; i++)
			{
				boolean success = deleteDir(new File(dir, children[i]));
				if (!success)
				{
					return false;
				}
			}
		}
    
		// The directory is now empty so delete it
		return dir.delete();
	}
	
	private final static String USER_DIRECTORY = "/user"; 
	
	/**
	 * Determines if the path passed in has the user directory prepended to it. If it does
	 * not, then this method will automatically prepend the user directory to the selected 
	 * path in order to avoid security exceptions. If the path passed in does not contain any
	 * data, then the default user directory will be returned.
	 * 
	 * @param path String to check for user directory (can be null or empty)
	 * @return Default user directory prepended String
	 */
	public final static String getUserDirectory(String path)
	{
		if (path == null || path.length() == 0)
			return USER_DIRECTORY;
		
		String dir = "";	
		if (path.indexOf(USER_DIRECTORY) != 0)
		{
			if (path.indexOf("/") != 0)
				dir = USER_DIRECTORY + "/" + path;
			else
				dir = USER_DIRECTORY + path;
		}
		else
		{
			dir = path;
		}
		
		return dir;
	}
	

}
