/*
 * Created on May 18, 2005
 * @author VGonzales
 * 
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.io;

import com.amx.duet.tools.lang.ObjectRef;


/**
 * The IMarshall interface provides a structure for creating other marshalled objects.  It 
 * provides standard error values, standard marshalled byte types, and standard data sizes.  
 * It also provides a <code>binaryEncode()</code> and a <code>binaryDecode()</code> method, which 
 * give users access to marshalled and unmarshalled data implemented by instances of this interface.
 * <p>
 * The mashalled and unmarshalled data should contained within the implementing object.  Accessors to
 * this data are left to user discretion.
 * <p>
 * <pre>
 * public class <strong>MarshalledObject</strong> <i>implements</i> <strong>IMarshall</strong>
 * {
 * 	private Object <strong>data</strong> = null;
 * 	
 * 	public int <strong>binaryEncode</strong>(ObjectRef buffer, ObjectRef rPos)
 * 	{
 * 		// Take the values stored in <strong>data</strong>
 * 		// Encode them
 * 		// Pass the values back to <strong>buffer</strong>
 * 	}
 * 	public int <strong>binaryDecode</strong>(ObjectRef buffer, ObjectRef rPos)
 * 	{
 * 		// Take the value from <strong>buffer</strong>
 * 		// Decode it
 * 		// Store the values in <strong>data</strong>
 * 	}
 * 	public Object getData()
 * 	{
 * 		return <strong>data</strong>;
 * 	}
 * }
 * </pre>
 * 
 * @version 1.0.0
 * @since AMXTools 1.0.0
 *
 */
public interface IBinaryMarshall
{
	// Extended Errors
	public static final int STV_DECODE_EOS = 100;
	
	// Errors
	public static final int STV_DECODE_OK = 0;
	public static final int STV_DECODE_VARIABLE_MISMATCH = -1;
	public static final int STV_DECODE_DATA_INCOMPLETE = -2;
	public static final int STV_DECODE_DATA_TOO_SMALL = -3;
	public static final int STV_DECODE_OBJECT_TOO_SMALL = -4;
	
	public static final int STV_ENCODE_OK = 0;
	public static final int STV_ENCODE_VARIABLE_UNKNOWN = -1;
	public static final int STV_ENCODE_BUFFER_TOO_SMALL = -2;
	
	// Encoding/Decoding
	public static final byte TYPE_CHAR = (byte)0xe1;
	public static final byte TYPE_WORD = (byte)0xe2;
	public static final byte TYPE_DWORD = (byte)0xe3;
	public static final byte TYPE_QWORD = (byte)0xe4;
	public static final byte TYPE_CHAR_ARRAY = (byte)0xe5;
	public static final byte TYPE_WORD_ARRAY = (byte)0xe6;
	public static final byte TYPE_DWORD_ARRAY = (byte)0xe7;
	public static final byte TYPE_QWORD_ARRAY = (byte)0xe8;
	public static final byte TYPE_LONG_CHAR_ARRAY = (byte)0xe9;
	public static final byte TYPE_STRUCT_START = (byte)0xea;
	public static final byte TYPE_STRUCT_END = (byte)0xeb;
	public static final byte TYPE_ARRAY = (byte)0xec;
	public static final byte TYPE_SKIP = (byte)0xed;

	public static final byte SIZE_CHAR = 1;
	public static final byte SIZE_WORD = 2;
	public static final byte SIZE_DWORD = 4;
	public static final byte SIZE_QWORD = 8;
	
	/**
	 * The binaryEncode function encodes the internal set of values and places them in a single binary value.
	 * 
	 * @param rBuffer Must be of type <STRONG>String</STRONG>. This is where the encode data is placed.<P>
	 *  
	 * Create rbuffer with a <STRONG>valueOf(String)</STRONG> Example: <br>
	 * <code>ObjectRef rBuffer = new ObjectRef("") or rBuffer.valueOf("")</code>
	 * 
	 * @param rPos Must be of type <STRONG>long</STRONG>. This is where the first byte of the encoding 
	 * is placed. It is also modified to point to the next location after the last encoded byte. 
	 * That means that successive calls to this function can be made without modifying position. 
	 * Position should be set to 0 on the first call.<P>
	 * 
	 * Create rPos with a <STRONG>valueOf(long)</STRONG> Example: <br>
	 * <code>ObjectRef rPos = new ObjectRef((long)0) or rPos.valueOf((long)0)</code>
	 * 
	 * @return
	 * <ul>
	 * <li>0 Encoded OK
	 * <li>-1 Encoded variable unrecognized type
	 * <li>-2 Encoded data would not fit into buffer, buffer too small
	 * </ul>
	 */
	public abstract int binaryEncode(ObjectRef buffer, ObjectRef rPos);
	
	/**
	 * The binaryDecode function decodes binary data passed in and places it into an internal set of values.
	 * 
	 * @param rBuffer Must be of type <STRONG>String</STRONG>. This is where the data to be decode is found.<P>
	 * 
	 * Create rbuffer with a <STRONG>valueOf(String)</STRONG> Example: <br>
	 * <code>ObjectRef rBuffer = new ObjectRef("") or rBuffer.valueOf("")</code>
	 * 
	 * @param rPos Must be of type <STRONG>long</STRONG>. This is where the first byte of the decode 
	 * data. Is it also modified to point to the next location after the last decoded byte. That means 
	 * that successive calls to this function can be made without modifying position. Position should 
	 * be set to 0 on the first call.<P>
	 * 
	 * Create rPos with a <STRONG>valueOf(long)</STRONG> Example: <br>
	 * <code>ObjectRef rPos = new ObjectRef((long)0) or rPos.valueOf((long)0)</code>
	 * 
	 * @return
	 * <ul>
	 * <li>2 - Decode data to small, more members in structure<br>
	 * <li>1 - Structure to small, more members in decode string<br>
	 * <li>0 - Decoded OK<br>
	 * <li>-1 - Decode variable type mismatch<br>
	 * <li>-2 - Decode data too small, decoder ran out of data
	 * </ul>
	 */
	public abstract int binaryDecode(ObjectRef buffer, ObjectRef rPos);
}
