/*
 * @author vgonzales
 * 
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.io;

import java.io.ByteArrayOutputStream;

import com.amx.duet.tools.lang.ObjectRef;
import com.amx.duet.tools.lang.ObjectType;

/**
 * Utility class used to marshall data similar to NetLinx marshalling.
 */
public class MarshallUtil
{
	/*
	 * Convert an object to a byte array
	 * (skips the header bit if data is in an array)
	 * 
	 * @param encode
	 * @param rBuffer
	 * @param rPos
	 * @param bSkipHeader
	 * @return
	 */
	private final static int VariableToString(Marshall encode, ObjectRef rBuffer, ObjectRef rPos, boolean bSkipHeader)
	{
		int ret = IBinaryMarshall.STV_ENCODE_OK;
		int pos = 0;
		
		ByteArrayOutputStream stream = new ByteArrayOutputStream();
		
		pos = rPos.toInt();
		if (pos > 0)
		{
			String str = rBuffer.toString();
			if (str.length() != pos)
			{
				byte[] ba = new byte[pos];
				stream.write(ba, 0, pos);
				rBuffer.valueOf(str + stream.toString());
			}
			
			Object obj = encode.getValue();
			if (obj instanceof IStruct)
				ret = ((IStruct)obj).binaryEncode(rBuffer, rPos);
			else
			{
				ret = encode.binaryEncode(rBuffer, bSkipHeader);
				if (ret > 0)
				{
					rPos.valueOf(pos + ret);
					ret = IBinaryMarshall.STV_ENCODE_OK;
				}
			}
		}
		else
		{
			Object obj = encode.getValue();
			if (obj instanceof IStruct)
				ret = ((IStruct)obj).binaryEncode(rBuffer, rPos);
			else
				ret = encode.binaryEncode(rBuffer, rPos);
		}

		return ret;
	}

	/**
	 * This routine takes the variable encode and creates entries in the buffer to represent that 
	 * variable. The variable passed in can be of any intrinsic type or a class that implements the
	 * IStruct interface.
	 * <P><STRONG>Note: </STRONG>NetLinx intrinsic types are smaller than java types<br>
	 * Example: INTEGER = short and LONG = int
	 * 
	 * @param encode This is the variable to be encoded.<P>
	 * 
	 * Example: <br>
	 * <pre>
	 * ObjectRef encoder = new ObjectRef(0);
	 * ObjectRef buffer = new ObjectRef("");
	 * ObjectRef rPos = new rPos((long)0);
	 * String sDate = "";
	 * int ret = 0;
	 * 
	 * // do some date processing...
	 * 
	 * encoder.valueOf(sDate);
	 * if ((ret = NetLinx.VariableToString(encoder, buffer, rPos)) &lt 0) return ret;
	 * </pre>
	 * @param rBuffer Must be of type <STRONG>String</STRONG>. This is where the encode data is placed.<P>
	 *  
	 * Create rbuffer with a <STRONG>valueOf(String)</STRONG> Example: <br>
	 * <code>ObjectRef rBuffer = new ObjectRef("") or rBuffer.valueOf("")</code>
	 * @param rPos Must be of type <STRONG>long</STRONG>. This is where the first byte of the encoding 
	 * is placed. It is also modified to point to the next location after the last encoded byte. 
	 * That means that successive calls to this function can be made without modifying position. 
	 * Position should be set to 0 on the first call.<P>
	 * 
	 * Create rPos with a <STRONG>valueOf(long)</STRONG> Example: <br>
	 * <code>ObjectRef rPos = new ObjectRef((long)0) or rPos.valueOf((long)0)</code>
	 * @return
	 * <ul>
	 * <li>0 Encoded OK
	 * <li>-1 Encoded variable unrecognized type
	 * <li>-2 Encoded data would not fit into buffer, buffer too small
	 * </ul>
	 */
	public final static int VariableToString(Marshall encode, ObjectRef rBuffer, ObjectRef rPos)
	{
		return VariableToString(encode, rBuffer, rPos, false);
	}
	
	/**
	 * This routine takes the variable encodeArray and creates entries in the buffer to represent  
	 * that array. The array passed in can be of any intrinsic type or an array of classes that 
	 * implement the IStruct interface.
	 * <P><STRONG>Note: </STRONG>NetLinx intrinsic types are smaller than java types<br>
	 * Example: INTEGER = short and LONG = int
	 * 
	 * @param encodeArray This is the variable to be encoded.<P>
	 * Example (class Appointment implements IStruct): <br>
	 * <pre>
	 * ObjectRef buffer = new ObjectRef("");
	 * ObjectRef rPos = new rPos((long)0);
	 * Appointment sAppts[] = new Appointment[10];
	 * int ret = 0;
	 * 
	 * // do some appointment processing...
	 * 
	 * ObjectRef[] encodeArray = new ObjectRef[sAppts.length];
	 * for (int i = 0; i &lt sAppts.length; i++)
	 * 	encodeArray[i] = new ObjectRef(sAppts[i]);
	 * if ((ret = NetLinx.VariableToString(encodeArray, buffer, rPos)) &lt 0) return ret;
	 * </pre>
	 * @param rBuffer Must be of type <STRONG>String</STRONG>. This is where the encode data is placed.<P>
	 *  
	 * Create rbuffer with a <STRONG>valueOf(String)</STRONG> Example: <br>
	 * <code>ObjectRef rBuffer = new ObjectRef("") or rBuffer.valueOf("")</code>
	 * @param rPos Must be of type <STRONG>long</STRONG>. This is where the first byte of the encoding 
	 * is placed. It is also modified to point to the next location after the last encoded byte. 
	 * That means that successive calls to this function can be made without modifying position. 
	 * Position should be set to 0 on the first call.<P>
	 * 
	 * Create rPos with a <STRONG>valueOf(long)</STRONG> Example: <br>
	 * <code>ObjectRef rPos = new ObjectRef((long)0) or rPos.valueOf((long)0)</code>
	 * @return
	 * <ul>
	 * <li>0 Encoded OK
	 * <li>-1 Encoded variable unrecognized type
	 * <li>-2 Encoded data would not fit into buffer, buffer too small
	 * </ul>
	 */
	public final static int VariableToString(Marshall encodeArray[], ObjectRef rBuffer, ObjectRef rPos)
	{
		int ret = IBinaryMarshall.STV_ENCODE_OK;
		ByteArrayOutputStream stream = new ByteArrayOutputStream();
		for (int i = 0; i < encodeArray.length; i++)
		{
			if (i == 0)
			{
				int len = encodeArray.length;
				if ((ObjectType.STRING.equals(encodeArray[i].getType())) ||
					(ObjectType.OBJECT.equals(encodeArray[i].getType())))
				{
					stream.write(IBinaryMarshall.TYPE_ARRAY);
					stream.write((byte)(len >> 24));
					stream.write((byte)(len >> 16));
					stream.write((byte)(len >>  8));
					stream.write((byte)(len >>  0));
					rPos.valueOf(rPos.toInt() + 5);
				}
				else
				{
					if (ObjectType.BOOL.equals(encodeArray[i].getType()))
						stream.write(IBinaryMarshall.TYPE_CHAR_ARRAY);
					else if (ObjectType.BYTE.equals(encodeArray[i].getType()))
						stream.write(IBinaryMarshall.TYPE_CHAR_ARRAY);
					else if (ObjectType.CHAR.equals(encodeArray[i].getType()))
						stream.write(IBinaryMarshall.TYPE_WORD_ARRAY);
					else if (ObjectType.SHORT.equals(encodeArray[i].getType()))
						stream.write(IBinaryMarshall.TYPE_WORD_ARRAY);
					else if (ObjectType.INTEGER.equals(encodeArray[i].getType()))
						stream.write(IBinaryMarshall.TYPE_DWORD_ARRAY);
					else if (ObjectType.LONG.equals(encodeArray[i].getType()))
						stream.write(IBinaryMarshall.TYPE_QWORD_ARRAY);
					else if (ObjectType.FLOAT.equals(encodeArray[i].getType()))
						stream.write(IBinaryMarshall.TYPE_DWORD_ARRAY);
					else if (ObjectType.DOUBLE.equals(encodeArray[i].getType()))
						stream.write(IBinaryMarshall.TYPE_QWORD_ARRAY);
					stream.write((byte)(len >> 8));
					stream.write((byte)(len >> 0));
					rPos.valueOf(rPos.toInt() + 3);
				}
				
				rBuffer.valueOf(rBuffer.toString() + stream.toString());
			}
			
			ret = VariableToString(encodeArray[i], rBuffer, rPos, true);
			if (ret < 0)
				break;
			
			Thread.yield();
		}
		
		return ret;
	}

	/*
	 * Convert a byte array to an object
	 * (skips the header bit if data is in an array)
	 * 
	 * @param decode
	 * @param rBuffer
	 * @param rPos
	 * @param bSkipHeader
	 * @return
	 */
	private final static int StringToVariable(Marshall decode, ObjectRef rBuffer, ObjectRef rPos, boolean bSkipHeader)
	{
		int ret = IBinaryMarshall.STV_DECODE_OK;
		
		int pos = rPos.toInt();
		if (pos > 0)
		{
			String str = rBuffer.toString();
			ObjectRef rTempBuffer = new ObjectRef(str.substring(pos, str.length()));
			Object obj = decode.getValue();
			if (obj instanceof IStruct)
			{
				ret = ((IStruct)obj).binaryDecode(rTempBuffer, rPos);
			}
			else
			{
				ret = decode.binaryDecode(rTempBuffer, bSkipHeader);
				if (ret > 0)
				{
					rPos.valueOf(pos + ret);
					ret = IBinaryMarshall.STV_DECODE_OK;
				}
			}
		}
		else
		{
			Object obj = decode.getValue();
			if (obj instanceof IStruct)
				ret = ((IStruct)obj).binaryDecode(rBuffer, rPos);
			else
				ret = decode.binaryDecode(rBuffer, rPos);
		}
		
		return ret;
	}
	
	/**
	 * This routine takes the Encode data from rBuffer and loads the values into the decode variable. 
	 * The decode variable must match the type of the encoded variable. In the case where the Encode 
	 * variable was a structure then the decode variable members must match in type and order. 
	 * However if the number of members of the structures doesnt match then the routine will fill 
	 * all it can or skip any unused data members.
	 * <P><STRONG>Note: </STRONG>NetLinx intrinsic types are smaller than java types<br>
	 * Example: INTEGER = short and LONG = int
	 * 
	 * @param decode This is the variable to be decoded into.<P>
	 * Example: <br>
	 * <pre>
	 * ObjectRef decoder = new ObjectRef(0);
	 * ObjectRef buffer = new ObjectRef("");
	 * ObjectRef rPos = new rPos((long)0);
	 * String sDate = "";
	 * int ret = 0;
	 * 
	 * // do some date processing...
	 * 
	 * decoder.valueOf(sDate);
	 * if ((ret = NetLinx.StringToVariable(decoder, buffer, rPos)) &lt 0) return ret;
	 * sDate = decoder.toString();
	 * </pre>
	 * @param rBuffer Must be of type <STRONG>String</STRONG>. This is where the decode data is found.<P>
	 * 
	 * Create rbuffer with a <STRONG>valueOf(String)</STRONG> Example: <br>
	 * <code>ObjectRef rBuffer = new ObjectRef("") or rBuffer.valueOf("")</code>
	 * @param rPos Must be of type <STRONG>long</STRONG>. This is where the first byte of the decode 
	 * data. Is it also modified to point to  the next location after the last decoded byte. That means 
	 * that successive calls to this function can be made without modifying position. Position should 
	 * be set to 0 on the first call.<P>
	 * 
	 * Create rPos with a <STRONG>valueOf(long)</STRONG> Example: <br>
	 * <code>ObjectRef rPos = new ObjectRef((long)0) or rPos.valueOf((long)0)</code>
	 * @return
	 * <ul>
	 * <li>2 - Decode data to small, more members in structure
	 * <li>1 - Structure to small, more members in decode string
	 * <li>0 - Decoded OK
	 * <li>-1 - Decode variable type mismatch
	 * <li>-2 - Decode data too small, decoder ran out of data
	 * </ul>
	 */
	public final static int StringToVariable(Marshall decode, ObjectRef rBuffer, ObjectRef rPos)
	{
		return StringToVariable(decode, rBuffer, rPos, false);
	}

	/**
	 * This routine takes the Encode array data from rBuffer and loads the values into the decode array 
	 * variable. The decode array variable must match the type of the encoded array variable. In the 
	 * case where the Encode variable was a structure array then the decode variable members must match 
	 * in type and order. However if the number of members of the structures doesnt match then the 
	 * routine will fill all it can or skip any unused data members.
	 * <P><STRONG>Note: </STRONG>NetLinx intrinsic types are smaller than java types<br>
	 * Example: INTEGER = short and LONG = int
	 * 
	 * @param decodeArray This is the variable to be decoded into.<P>
	 * 
	 * Example (class Appointment implements IStruct): <br>
	 * <pre>
	 * ObjectRef buffer = new ObjectRef("");
	 * ObjectRef rPos = new rPos((long)0);
	 * Appointment sAppts[] = new Appointment[10];
	 * int ret = 0;
	 * 
	 * // do some appointment processing...
	 * 
	 * ObjectRef[] decodeArray = new ObjectRef[sAppts.length];
	 * for (int i = 0; i &lt sAppts.length; i++)
	 * 	decodeArray[i] = new ObjectRef(sAppts[i]);
	 * if ((ret = NetLinx.StringToVariable(decodeArray, buffer, rTempPos)) &lt 0) return ret;
	 * </pre>
	 * <P><STRONG>Note: </STRONG>Class objects such as the Appointment class are passed by reference to 
	 * the ObjectRef class. Therefore, the values will change along with the decodeArray value and  
	 * don't need to be passed back to the sAppts array, but intrinsic type arrays do need their values 
	 * to be passed back. So you will need to something like this:<P>
	 * 
	 * <pre>
	 * public int [] nApptList = new int[10];
	 * decodeArray = new ObjectRef[nApptList.length];
	 * for (int i = 0; i &lt nApptList.length; i++)
	 * 	decodeArray[i] = new ObjectRef((short)nApptList[i]);
	 * if ((ret = NetLinx.StringToVariable(decodeArray, buffer, rTempPos)) &lt 0) return ret;
	 * for (int i = 0; i &lt nApptList.length; i++)
	 * 	nApptList[i] = decodeArray[i].toInt();
	 * </pre>
	 * @param rBuffer Must be of type <STRONG>String</STRONG>. This is where the decode data is found.<P>
	 * 
	 * Create rbuffer with a <STRONG>valueOf(String)</STRONG> Example: <br>
	 * <code>ObjectRef rBuffer = new ObjectRef("") or rBuffer.valueOf("")</code>
	 * @param rPos Must be of type <STRONG>long</STRONG>. This is where the first byte of the decode 
	 * data. Is it also modified to point to  the next location after the last decoded byte. That means 
	 * that successive calls to this function can be made without modifying position. Position should 
	 * be set to 0 on the first call.<P>
	 * 
	 * Create rPos with a <STRONG>valueOf(long)</STRONG> Example: <br>
	 * <code>ObjectRef rPos = new ObjectRef((long)0) or rPos.valueOf((long)0)</code>
	 * @return
	 * <ul>
	 * <li>2 - Decode data to small, more members in structure
	 * <li>1 - Structure to small, more members in decode string
	 * <li>0 - Decoded OK
	 * <li>-1 - Decode variable type mismatch
	 * <li>-2 - Decode data too small, decoder ran out of data
	 * </ul>
	 */
	public final static int StringToVariable(Marshall decodeArray[], ObjectRef rBuffer, ObjectRef rPos)
	{
		int ret = IBinaryMarshall.STV_DECODE_OK;
		int len = 0;
		String str = "";
		
		if (decodeArray.length > 0)
		{
			str = rBuffer.toString();
			if ((ObjectType.STRING.equals(decodeArray[0].getType())) ||
				(ObjectType.OBJECT.equals(decodeArray[0].getType())))
			{
				byte[] header = (str.substring(0, 5)).getBytes();
				len = (header[4] <<  0) + 
				      (header[3] <<  8) + 
				      (header[2] << 16) + 
				      (header[1] << 24); 
				rPos.valueOf(rPos.toInt() + 5);
			}
			else
			{
				byte[] header = (str.substring(0, 3)).getBytes();
				len = (header[2] << 0) + 
				      (header[1] << 8);
				rPos.valueOf(rPos.toInt() + 3);
			}
			
			for (int x = 0; x < len; x++)
			{
				ret = StringToVariable(decodeArray[x], rBuffer, rPos, true);
				if (ret < 0)
					break;
				
				Thread.yield();
			}
			
		}
		
		return ret;
	}
	
	/**
	 * This routine calculates how many bytes it takes to encode a variable.
	 * 
	 * @param encode any type of variable. This is the variable to be encoded. Example: <br>
	 * <code>ObjectRef encode = new ObjectRef(0) or encode.valueOf(0)</code>
	 * @return Number of bytes required to encode variable. 0 if there is an error.
	 */
	public final static int LengthVariableToString(Marshall encode)
	{
		// Init
		int length = 0;
		ObjectRef buffer = new ObjectRef("");
		ObjectRef rPos = new ObjectRef(0);
		
		if (IBinaryMarshall.STV_ENCODE_OK == VariableToString(encode, buffer, rPos))
		{
			length = rPos.toInt();
		}
		
		// Done
		return length;
	}
	
	/**
	 * This routine calculates how many bytes it takes to encode an array variable.
	 * 
	 * @param encodeArray any type of array variable. This is the array to be encoded. Example: <br>
	 * <pre>
	 * ObjectRef[] encodeArray = new ObjectRef[nValues.length];
	 * for (int i = 0; i &lt nValues.length; i++)
	 * 	encodeArray[i] = new ObjectRef(nValues[i]);
	 * </pre>
	 * @return Number of bytes required to encode variable. 0 if there is an error.
	 */
	public final static int LengthVariableToString(Marshall[] encodeArray)
	{
		// Init
		int length = 0;
		ObjectRef buffer = new ObjectRef("");
		ObjectRef rPos = new ObjectRef(0);
		
		if (IBinaryMarshall.STV_ENCODE_OK == VariableToString(encodeArray, buffer, rPos))
		{
			length = rPos.toInt();
		}
		
		// Done
		return length;
	}
	

}
