package com.amx.duet.tools.lang;


import java.io.ByteArrayOutputStream;
import java.io.IOException;

/**
 * This is similar to the Java StringBuffer class except that it deals with data 
 * at the byte level.
 * 
 * @version 1.1.0
 * @since AMXTools 1.0.0
 */
public class ByteBuffer
{
	private static String className = "ByteBuffer";
	
	private ByteArrayOutputStream stream = new ByteArrayOutputStream();
	private String digits = "0123456789ABCDEF";
	
	/**
     * Initializes a newly created <TT>byte[]</TT> object 
	 */
	public ByteBuffer()
	{
	}
	
	/**
     * Initializes a newly created <TT>byte[]</TT> object so that it
     * represents a single byte as the argument. 
     *
     * @param b the initial value of the byte array
	 */
	public ByteBuffer(int b)
	{
		append(b);
	}
	
    /**
     * Initializes a newly created <TT>byte[]</TT> object so that it
     * represents the same sequence of characters as the argument; in other
     * words, the newly created string is a copy of the argument string. Unless 
     * an explicit copy of <code>original</code> is needed, use of this 
     * constructor is unnecessary. 
     *
     * @param original a <TT>byte[]</TT>.
     */
	public ByteBuffer(byte[] original)
	{
		append(original);
	}
	
	/**
     * Allocates a new <TT>byte[]</TT> that contains characters from
     * a subarray of the byte array argument. The <code>offset</code>
     * argument is the index of the first character of the subarray and
     * the <code>size</code> argument specifies the length of the
     * subarray. The contents of the subarray are copied; subsequent
     * modification of the byte array does not affect the newly
     * created byte array.
	 * 
     * @param value array that is the source of bytes.
     * @param offset the initial offset.
     * @param size the length.
	 */
	public ByteBuffer(byte[] value, int offset, int size)
	{
		append(value, offset, size);
	}

	/**
     * Allocates a new <TT>byte[]</TT> so that it represents the
     * sequence of characters currently contained in the <code>String</code>
     * argument. The contents of the string are copied; subsequent
     * modification of the string does not affect the newly created
     * byte array.
	 * 
	 * @param value the initial value of the byte array
	 */
	public ByteBuffer(String value)
	{
		append(value);
	}

	/**
     * Allocates a new <TT>byte[]</TT> so that it represents the
     * sequence of characters currently contained in the character array
     * argument. The contents of the character array are copied; subsequent
     * modification of the character array does not affect the newly created
     * byte array.
	 * 
	 * @param value the initial value of the byte array
	 */
	public ByteBuffer(char[] value)
	{
		append(value);
	}

	/**
	 * 
	 * @param b
	 */
	public void append(int b)
	{
		stream.write(b);
	}

	/**
	 * 
	 * @param value
	 */
	public void append(byte[] value)
	{
		try
		{
			stream.write(value);
		}
		catch (IOException e)
		{
			error("IOException thrown: " + e.getMessage());
		}
	}
	
	/**
	 * 
	 * @param value
	 * @param offset
	 * @param size
	 */
	public void append(byte[] value, int offset, int size)
	{
		stream.write(value, offset, size);
	}
	
	/**
	 * 
	 * @param value
	 */
	public void append(String value)
	{
		append(value.getBytes());
	}

	/**
	 * 
	 * @param value
	 */
	public void append(char[] value)
	{
		String str = new String(value);
		append(str.getBytes());
	}
	
	/**
	 * Return a byte at the specified index
	 * 
	 * @param index
	 * @return
	 */
	public byte byteAt(int index)
	{
		byte[] bytes = toByteArray();
		if (index < bytes.length)
			return bytes[index];
		throw new IndexOutOfBoundsException();
	}

	/**
	 * Return an 8-bit unsigned byte at the specified index
	 *  
	 * @param index
	 * @return
	 */
	public int UByteAt(int index)
	{
		byte[] bytes = toByteArray();
		if (index < bytes.length)
			return (bytes[index] & 0xFF); // Covert to 8-bit unsigned byte
		throw new IndexOutOfBoundsException();
	}

	/**
	 * Return a character at the specified index
	 * 
	 * @param index
	 * @return
	 */
	public char charAt(int index)
	{
		char[] chars = toCharArray();
		if (index < chars.length)
			return chars[index];
		throw new IndexOutOfBoundsException();
	}

	/**
	 * Return the boolean value at the specified index
	 * 
	 * @param index
	 * @return
	 */
	public boolean getBoolean(int index)
	{
		byte[] bytes = toByteArray();
		if (index < bytes.length)
			return Bits.getBoolean(bytes, index);
		throw new IndexOutOfBoundsException();
	}

	/**
	 * Return the char value at the specified index
	 * 
	 * @param index
	 * @return
	 */
	public char getChar(int index)
	{
		byte[] bytes = toByteArray();
		if (index < bytes.length && (index + 1) < bytes.length)
		{
			return Bits.getChar(bytes, index);
		}
		throw new IndexOutOfBoundsException();
	}
	
	/**
	 * Return the short value at the specified index
	 * 
	 * @param index
	 * @return
	 */
	public short getShort(int index)
	{
		byte[] bytes = toByteArray();
		if (index < bytes.length && (index + 1) < bytes.length)
		{
			return Bits.getShort(bytes, index);
		}
		throw new IndexOutOfBoundsException();
	}
	
	/**
	 * Return the int value at the specified index
	 * 
	 * @param index
	 * @return
	 */
	public int getInt(int index)
	{
		byte[] bytes = toByteArray();
		if (index < bytes.length && (index + 3) < bytes.length)
		{
			return Bits.getInt(bytes, index);
		}
		throw new IndexOutOfBoundsException();
	}
	
	/**
	 * Return the long value at the specified index
	 * 
	 * @param index
	 * @return
	 */
	public long getLong(int index)
	{
		byte[] bytes = toByteArray();
		if (index < bytes.length && (index + 7) < bytes.length)
		{
			return Bits.getLong(bytes, index);
		}
		throw new IndexOutOfBoundsException();
	}
	
	/**
	 * Return the float value at the specified index
	 * 
	 * @param index
	 * @return
	 */
	public float getFloat(int index)
	{
		byte[] bytes = toByteArray();
		if (index < bytes.length && (index + 3) < bytes.length)
		{
			return Bits.getFloat(bytes, index);
		}
		throw new IndexOutOfBoundsException();
	}
	
	/**
	 * Return the double value at the specified index
	 * 
	 * @param index
	 * @return
	 */
	public double getDouble(int index)
	{
		byte[] bytes = toByteArray();
		if (index < bytes.length && (index + 7) < bytes.length)
		{
			return Bits.getDouble(bytes, index);
		}
		throw new IndexOutOfBoundsException();
	}
	
	/**
	 * Return the double value at the specified index
	 * 
	 * @param index
	 * @param length
	 * @return
	 */
	public String getString(int index, int length)
	{
		byte[] bytes = toByteArray();
		if (index < bytes.length && (index + length < bytes.length))
		{
			return new String(bytes);
		}
		throw new IndexOutOfBoundsException();
	}
	
	/**
	 * 
	 * @return
	 */
	public byte[] toByteArray()
	{
		return stream.toByteArray();
	}
	
	/**
	 * 
	 * @return
	 */
	public char[] toCharArray()
	{
		String str = new String(toByteArray());
		return str.toCharArray();
	}
	
	/**
     * Converts the buffer's contents into a string, translating bytes into
     * characters according to the platform's default character encoding.
	 * 
	 * @return String representation of a byte array
	 */
	public String toString()
	{
		return stream.toString();
	}

	/**
	 * Generates an ASCII hex string from the stored byte array.
	 * ex. {0x1a, 0x1b, 0x1c} returns "1A1B1C"
	 * 
	 * @return ASCII hex representation of a byte array
	 */
	public String toHex()
	{
		// This code is the same as formatUtil.toAsciiHexString(byte[]). It was rewritten so that
		// this class could be run in a stand alone application.
		
		byte[] bArr = toByteArray();
		int dig1;
		int dig2;
		
		byte[] tempArr = new byte[bArr.length*2];
		
		for (int i=0; i<bArr.length; i++)
		{
			dig1 = (bArr[i] >> 4) & 0x0F;
			dig2 = bArr[i] & 0x0F;
			
			tempArr[i*2] = (byte)digits.charAt(dig1);
			tempArr[(i*2)+1] = (byte)digits.charAt(dig2);
		}
		
		return new String(tempArr);
	}
	
	/**
     * Returns the current size of the buffer.
	 * 
     * @return the number of valid bytes in the byte array.
	 */
	public int length()
	{
		return stream.size();
	}
	
	/**
	 * Clear all of the contents of the ByteBuffer
	 */
	public void clear()
	{
		stream.reset();
	}
	
    /**
     * Returns the index within this string of the first occurrence of the
     * specified byte. If a byte with value <code>value</code> occurs 
     * in the byte sequence represented by this <code>ByteBuffer</code> 
     * object, then the index of the first such occurrence is returned -- 
     * that is, the smallest value <i>k</i> such that: 
     * <blockquote><pre>
     * this.byteAt(<i>k</i>) == value
     * </pre></blockquote>
     * is <code>true</code>. If no such byte occurs in this ByteBuffer, 
     * then <code>-1</code> is returned.
     *
     * @param   value   a byte.
     * @return  the index of the first occurrence of the byte in the
     *          byte sequence represented by this object, or
     *          <code>-1</code> if the byte does not occur.
     */
	public int indexof(int value)
	{
		return indexof(value, 0);
	}

	/**
	 * 
	 * @param value
	 * @param fromIndex
	 * @return
	 */
	public int indexof(int value, int fromIndex)
	{
		if (length() > 0)
			return toString().indexOf(value, fromIndex);
		return -1;
	}

	/**
	 * 
	 * @param value
	 * @return
	 */
	public int indexof(byte value[])
	{
		return indexof(value, 0);
	}

	/**
	 * 
	 * @param value
	 * @param fromIndex
	 * @return
	 */
	public int indexof(byte value[], int fromIndex)
	{
		if (length() > 0)
			return toString().indexOf(new String(value), fromIndex);
		return -1;
	}

	/**
	 * 
	 * @param value
	 * @return
	 */
	public int indexof(char value[])
	{
		return indexof(value, 0);
	}

	/**
	 * 
	 * @param value
	 * @param fromIndex
	 * @return
	 */
	public int indexof(char value[], int fromIndex)
	{
		if (length() > 0)
			return toString().indexOf(new String(value), fromIndex);
		return -1;
	}
	
    /**
     * Returns the index within this ByteBuffer of the first occurrence of the
     * specified substring. The integer returned is the smallest value 
     * <i>k</i> such that:
     * <blockquote><pre>
     * this.startsWith(str, <i>k</i>)
     * </pre></blockquote>
     * is <code>true</code>.
     *
     * @param   str   any string.
     * @return  if the string argument occurs as a substring within this
     *          object, then the index of the first character of the first
     *          such substring is returned; if it does not occur as a
     *          substring, <code>-1</code> is returned.
     * @exception java.lang.NullPointerException if <code>str</code> is 
     *          <code>null</code>.
     * @see        java.lang.String#indexOf(java.lang.String)
     */
	public int indexof(String value)
	{
		return indexof(value, 0);
	}

	/**
	 * 
	 * @param value
	 * @param fromIndex
	 * @return
	 */
	public int indexof(String value, int fromIndex)
	{
		if (length() > 0)
			return toString().indexOf(value, fromIndex);
		return -1;
	}

    /**
     * Tests if this byte buffer starts with the specified prefix.
     * 
     * @param str the prefix
     * @return true if the character sequence represented by the argument is a prefix of the character sequence 
     * represented by this byte buffer; false otherwise. Note also that true will be returned if the argument 
     * is an empty string or is equal to this ByteBuffer object as determined by the equals(Object) method.
     */
    public synchronized boolean startsWith(String str)
    {
    	return toString().startsWith(str);
    }

    /**
     * Tests if this byte buffer ends with the specified suffix.
     * 
     * @param str the suffix.
     * @return true if the character sequence represented by the argument is a suffix of the character sequence 
     * represented by this object; false otherwise. Note that the result will be true if the argument is the 
     * empty string or is equal to this ByteBuffer object as determined by the equals(Object) method.
     */
	public synchronized boolean endsWith(String str)
    {
    	return toString().endsWith(str);
    }
	
    /**
     * Removes the entire contents of this byte buffer from beginning to end.
     * 
     * @return     An empty byte buffer.
     */
    public synchronized ByteBuffer delete()
    {
    	return delete(0, length());
    }

    /**
     * Removes the characters in a substring of this <code>ByteBuffer</code>
     * up to the specified length.  The substring starts from the beginning and  
     * extends to the character at index <code>len - 1</code> or to the end of the
     * <code>ByteBuffer</code> if no such character exists.
     *
     * @param      len  The number of characters to remove from beginning to <code>len</code>.
     * @return     This byte buffer.
     */
    public synchronized ByteBuffer delete(int len)
    {
    	return delete(0, len);
    }

    /**
     * Removes the characters in a substring of this <code>ByteBuffer</code>.
     * The substring begins at the specified <code>start</code> and extends to
     * the character at index <code>end - 1</code> or to the end of the
     * <code>ByteBuffer</code> if no such character exists. If
     * <code>start</code> is equal to <code>end</code>, no changes are made.
     *
     * @param      start  The beginning index, inclusive.
     * @param      end    The ending index, exclusive.
     * @return     This byte buffer.
     * @exception  StringIndexOutOfBoundsException  if <code>start</code>
     *             is negative, greater than <code>length()</code>, or
     *		   greater than <code>end</code>.
     */
    public synchronized ByteBuffer delete(int start, int end)
    {
    	StringBuffer sb = new StringBuffer(toString());
    	sb.delete(start, end);
    	this.clear();
    	if (sb.length() > 0)
    		this.append(sb.toString());
    	
    	return this;
    }

    /**
     * Replaces the bytes in a substring of this <code>ByteBuffer</code>
     * with bytes in the specified <code>byte[]</code>. The substring
     * begins at the specified <code>start</code> and extends to the byte
     * at index <code>end - 1</code> or to the end of the
     * <code>ByteBuffer</code> if no such byte exists. First the
     * byte in the substring are removed and then the specified
     * <code>byte[]</code> is inserted at <code>start</code>. (The
     * <code>ByteBuffer</code> will be lengthened to accommodate the
     * specified byte[] if necessary.)
     * 
     * @param      start    The beginning index, inclusive.
     * @param      end      The ending index, exclusive.
     * @param      value    Byte[] that will replace previous contents.
     * @return     This byte buffer.
     * @exception  StringIndexOutOfBoundsException  if <code>start</code>
     *             is negative, greater than <code>length()</code>, or
     *		   greater than <code>end</code>.
     */ 
    public synchronized ByteBuffer replace(int start, int end, byte[] value)
    {
    	StringBuffer sb = new StringBuffer(toString());
    	sb.replace(start, end, new String(value));
    	this.clear();
    	if (sb.length() > 0)
    		this.append(sb.toString());
    	
    	return this;
    }

    /**
	 * Removes every instance of the passed in pattern from the ByteBuffer and replaces them with the 
	 * replace pattern.
     * 
	 * @param pattern Pattern to remove from source.
	 * @param replace Pattern to replace removed pattern with.
     * @return the newly replaced bytes
     */
    public synchronized ByteBuffer replace(byte[] pattern, byte[] replace)
	{
    	return this.replace(0, this.length(), pattern, replace);
	}

    /**
	 * Removes every instance of the passed in pattern from the ByteBuffer and replaces them with the 
	 * replace pattern.
     * 
     * @param start The beginning index, inclusive.
     * @param end The ending index, exclusive.
	 * @param pattern Pattern to remove from source.
	 * @param replace Pattern to replace removed pattern with.
     * @return the newly replaced bytes
     */
    public synchronized ByteBuffer replace(int start, int end, byte[] pattern, byte[] replace)
	{
    	StringBuffer sb = new StringBuffer(toString());
    	sb = new StringBuffer(Replace(sb.toString(), start, end, new String(pattern), new String(replace)));
    	this.clear();
    	if (sb.length() > 0)
    		this.append(sb.toString());
    	return this;
	}

    /**
     * Replaces a sequence of patterns with a sequence of replace values.
     * 
	 * @param pattern Patterns to remove from source.
	 * @param replace Patterns to replace removed patterns with.
     * @return the newly replaced bytes
     */
    public synchronized ByteBuffer replace(byte[][] pattern, byte[][] replace)
	{
    	return this.replace(0, this.length(), pattern, replace);
	}
    
    /**
     * Replaces a sequence of patterns with a sequence of replace values.
     * 
     * @param start The beginning index, inclusive.
     * @param end The ending index, exclusive.
	 * @param pattern Patterns to remove from source.
	 * @param replace Patterns to replace removed patterns with.
     * @return the newly replaced bytes
     */
    public synchronized ByteBuffer replace(int start, int end, byte[][] pattern, byte[][] replace)
	{
    	StringBuffer sb = new StringBuffer(toString());
    	
    	String[] patterns = new String[pattern.length];
    	for (int i = 0; i < pattern.length; i++)
    		patterns[i] = new String(pattern[i]);
    	
    	String[] replaces = new String[replace.length];
    	for (int i = 0; i < replace.length; i++)
    		replaces[i] = new String(replace[i]);
    	
    	sb = new StringBuffer(Replace(sb.toString(), start, end, patterns, replaces));
    	this.clear();
    	if (sb.length() > 0)
    		this.append(sb.toString());
    	return this;
	}
    
    /**
	 * Removes every instance of the passed in pattern from the string and replaces them with the 
	 * replace pattern.
	 * 
	 * @param str String to be manipulated
     * @param start The beginning index, inclusive.
     * @param end The ending index, exclusive.
	 * @param pattern Pattern to remove from source string
	 * @param replace Pattern to replace removed pattern with
	 * @return the newly replaced string
     */
    private String Replace(String str, int start, int end, String pattern, String replace) 
    {
        int s = start;
        int e = 0;
        StringBuffer result = new StringBuffer();
    
        while (e >= 0 && e <= end) 
        {
        	e = str.indexOf(pattern, s);
        	if (e >= 0 && e <= end)
        	{
                result.append(str.substring(s, e));
                result.append(replace);
                s = e+pattern.length();
        	}
        }
        
        result.append(str.substring(s));
        return result.toString();
    }
    
	/**
	 * Removes every instance of the passed in pattern from the string and replaces them with the 
	 * replace pattern.
	 * 
	 * @param str String to be manipulated
     * @param start The beginning index, inclusive.
     * @param end The ending index, exclusive.
	 * @param pattern Pattern to remove from source string
	 * @param replace Pattern to replace removed pattern with
	 * @return the newly replaced string
	 */
    private String Replace(String str, int start, int end, String pattern[], String replace[]) 
    {
        int s = start;
        int e = 0;
        int index = 0;
        int len = pattern.length;
        StringBuffer result = new StringBuffer();
        
        if (replace.length < len)
        	len = replace.length;

        while (e >= 0 && e <= end && index >= 0)
        {
            index = findFirst(str, s, pattern, len);
            if (index >= 0)
            {
            	e = str.indexOf(pattern[index], s);
                if (e >= 0 && e <= end)
                {
                    result.append(str.substring(s, e));
                    result.append(replace[index]);
                    s = e+pattern[index].length();
                }
            }
        }
        
        result.append(str.substring(s));
        return result.toString();
    }
    
    /**
     * 
     * @param str
     * @param start The beginning index, inclusive.
     * @param pattern
     * @param patternCount
     * @return
     */
    private int findFirst(String str, int start, String pattern[], int patternCount)
    {
        int e = -1;
        int temp = -1;
        int res = -1;
        for (int i = 0; i < patternCount; i++)
        {
        	temp = str.indexOf(pattern[i], start);
        	if (temp >= 0)
        	{
        		if (e < 0 || temp < e)
        		{
            		e = temp;
            		res = i;
        		}
        	}
        }
        return res;
    }
    
    /**
     * Returns a new <code>ByteBuffer</code> that contains a subsequence of
     * bytes currently contained in this <code>ByteBuffer</code>.The 
     * substring begins at the specified index and extends to the end of the
     * <code>ByteBuffer</code>.
     * 
     * @param      start    The beginning index, inclusive.
     * @return     The new ByteBuffer.
     * @exception  StringIndexOutOfBoundsException  if <code>start</code> is
     *             less than zero, or greater than the length of this
     *             <code>ByteBuffer</code>.
     */
    public synchronized ByteBuffer substring(int start)
    {
        return substring(start, length());
    }

    /**
     * Returns a new <code>ByteBuffer</code> that contains a subsequence of
     * characters currently contained in this <code>ByteBuffer</code>. The 
     * substring begins at the specified <code>start</code> and 
     * extends to the byte at index <code>end - 1</code>. An
     * exception is thrown if 
     *
     * @param      start    The beginning index, inclusive.
     * @param      end      The ending index, exclusive.
     * @return     The new ByteBuffer.
     * @exception  StringIndexOutOfBoundsException  if <code>start</code>
     *             or <code>end</code> are negative or greater than
     *		   <code>length()</code>, or <code>start</code> is
     *		   greater than <code>end</code>.
     */
    public synchronized ByteBuffer substring(int start, int end)
    {
    	StringBuffer sb = new StringBuffer(toString());
    	return new ByteBuffer(sb.substring(start, end));
    }

	/*
	 * 
	 * @param str
	 */
	private void diag(String str)
	{
		System.out.println(className + ": " + str);
	}
	
	/*
	 * 
	 * @param str
	 */
	private void error(String str)
	{
		diag("ERROR - " + str);
	}

	/**
	 * Converts a byte value to a byte array.
	 * 
	 * @param value
	 * @return 
	 */
	public static byte[] toBytes(boolean value)
	{
		byte[] bytes = new byte[1];
		Bits.putBoolean(bytes, 0, value);
		return bytes;
	}
	
	/**
	 * Converts a byte value to a byte array.
	 * 
	 * @param value
	 * @return 
	 */
	public static byte[] toBytes(byte value)
	{
		byte[] bytes = new byte[1];
		bytes[0] = value;
		return bytes;
	}
	
	/**
	 * Converts a char value to a byte array.
	 * 
	 * @param value
	 * @return 
	 */
	public static byte[] toBytes(char value)
	{
		byte[] bytes = new byte[2];
		Bits.putChar(bytes, 0, value);
		return bytes;
	}
	
	/**
	 * Converts a short value to a byte array.
	 * 
	 * @param value
	 * @return 
	 */
	public static byte[] toBytes(short value)
	{
		byte[] bytes = new byte[2];
		Bits.putShort(bytes, 0, value);
		return bytes;
	}
	
	/**
	 * Converts an int value to a byte array.
	 * 
	 * @param value
	 * @return 
	 */
	public static byte[] toBytes(int value)
	{
		byte[] bytes = new byte[4];
		Bits.putInt(bytes, 0, value);
		return bytes;
	}

	/**
	 * Converts a long value to a byte array.
	 * 
	 * @param value
	 * @return 
	 */
	public static byte[] toBytes(long value)
	{
		byte[] bytes = new byte[8];
		Bits.putLong(bytes, 0, value);
		return bytes;
	}
	
	/**
	 * Converts a float value to a byte array.
	 * 
	 * @param value
	 * @return 
	 */
	public static byte[] toBytes(float value)
	{
		byte[] bytes = new byte[4];
		Bits.putFloat(bytes, 0, value);
		return bytes;
	}
	
	/**
	 * Converts a double value to a byte array.
	 * 
	 * @param value
	 * @return 
	 */
	public static byte[] toBytes(double value)
	{
		byte[] bytes = new byte[8];
		Bits.putDouble(bytes, 0, value);
		return bytes;
	}
	
	/**
	 * Converts a String value to a byte array.
	 * 
	 * @param value
	 * @return 
	 */
	public static byte[] toBytes(String value)
	{
		return value.getBytes();
	}
}
