/*
 * Created on Nov 18, 2004
 * @author VGonzales & CAyers
 *
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.lang; 

import java.net.InetAddress;

import com.amx.duet.core.master.netlinx.DPS;


/**
 * A utility class used to perform common validation routines in Duet modules.
 * 
 * @version 1.0.0
 */
public class ValidateUtil
{
	private static String className = ClassName.getClassName(ValidateUtil.class.getName());
	
	/**
	 * Validates the current baud rate. If the current baud rate is invalid or does not match 
	 * one of the valid rates in the list, the passed in default baud rate is returned.
	 * 
	 * @param rate String containing the device's current baud rate setting
	 * @param validRates an array of Strings containing all valid baud rates for the device
	 * @param defaultRate String containing the device's default baud rate
	 * @return String containing the current baud rate setting, or the default value
	 */
	public static String validateBaudRate(String rate, String[] validRates, String defaultRate)
	{
		if (rate != null && rate.length() > 0)
		{
			for (int n = 0; n < validRates.length; n++)
			{
				if (rate.equals(validRates[n]))
					return rate;
			}
		}

		for (int n = 0; n < validRates.length; n++)
		{
			if (defaultRate.equals(validRates[n]))
			{
				error("Invalid value for the Baud Rate. Using default rate.");
				return defaultRate;
			}
		}

		error("Invalid value for the Baud Rate, and default rate. Using Null");
		return null;
	}
	
	/**
	 * Validates the current device ID. If the current ID is invalid or not with the minimum 
	 * and maximum ID range, the passed in default ID is returned.
	 * 
	 * @param deviceID String containing the device's current device ID setting
	 * @param nMinID integer, represents the smallest valid ID
	 * @param nMaxID integer, represents the largest valid ID
	 * @param defaultID String containing the device's default ID
	 * @return String containing the current device ID setting, or the default value
	 */
	public static String validateDeviceID(String deviceID, int nMinID, int nMaxID, String defaultID)
	{
		if (deviceID != null && deviceID.length() > 0)
		{
			int nID = ATOI(deviceID);
			if (nID >= nMinID && nID <= nMaxID)
				return deviceID;
		}
		int dID = ATOI(defaultID);
		if (dID >= nMinID && dID <= nMaxID)
		{
			error("Invalid value for the Device ID. Returning default ID");
			return defaultID;
		}
		
		error("Invalid value for the Device ID and default ID. Returning Null");
		return null;
	}

	/**
	 * @deprecated Use {@link #validatePollTime(String, long, long, long)} instead.
	 * 
	 * Validates the current poll time. If the current poll time is invalid, the passed in 
	 * time value (in seconds) is returned.
	 * 
	 * @param polltime String containing the device's current poll time setting
	 * @param nTime integer, represents the default poll time in milliseconds
	 * @return String containing the current poll time setting, or the default value in 
	 * milliseconds converted to seconds
	 */
	public static String validatePollTime(String polltime, long nTime)
	{
		if (polltime != null && polltime.length() > 0)
			return polltime;
			
		error("Invalid value for the Poll Time.");
		return Integer.toString((int)(nTime / 1000));
	}
	
	/**
	 * Validates the current poll time. If the current poll time is invalid, the passed in 
	 * time value (in seconds) is returned.
	 * 
	 * @param polltime String containing the device's current poll time setting
	 * @param lMin minimum poll time value.
	 * @param lMax maximum poll time value.
	 * @param lTime integer, represents the default poll time in milliseconds
	 * @return String containing the current poll time setting, or the default value in 
	 * milliseconds converted to seconds
	 */
	public static String validatePollTime(String polltime, long lMin, long lMax, long lTime)
	{
		if (polltime != null && polltime.length() > 0)
		{
			int nPoll = ATOI(polltime);
			if (nPoll >= lMin && nPoll <= lMax)
				return polltime;
			
			error("Poll Time out of range. Returning default Poll Time.");
		}
		else
		{
			error("Invalid value for the Poll Time. Returning default Poll Time.");
		}
		
		return Long.toString(lTime);
	}
	
	/**
	 * 
	 * @param timeout
	 * @param nMax
	 * @param nDefault
	 * @return
	 */
	public static String validateTimeoutCount(String timeout, int nMax, int nDefault)
	{
		if (timeout != null && timeout.length() > 0)
		{
			int nTime = ATOI(timeout);
			if (nTime >= 0 && nTime <= nMax)
				return timeout;

			error("Timeout Count out of range. Returning default Timeout Count.");
		}
		else
		{
			error("Invalid value for the Timeout Count. Returning default Timeout Count.");
		}
		
		return Integer.toString(nDefault);
	}

	/**
	 * 
	 * @param polltime
	 * @param lMin
	 * @param lMax
	 * @param lTime
	 * @return
	 */
	public static String validateReconnnectTime(String connectTime, long lMin, long lMax, long lTime)
	{
		if (connectTime != null && connectTime.length() > 0)
		{
			int nConnect = ATOI(connectTime);
			if (nConnect >= lMin && nConnect <= lMax)
				return connectTime;
			
			error("Reconnnect Time out of range. Returning default Reconnnect Time.");
		}
		else
		{
			error("Invalid value for the Reconnnect Time. Returning default Reconnnect Time.");
		}
		
		return Long.toString(lTime);
	}
	
	/**
	 * Checks to see if the string passed in is a NetLinx D:P:S or not
	 * 
	 * @param dps String containing the DPS to validate
	 * @return true if this is a valid DPS; false otherwise
	 */
	public final static boolean isValidDPS(String dps)
	{
		boolean bValid = false;
		
		if (dps != null && dps.length() > 0)
		{
			try
			{
				DPS dev = new DPS(dps);
				bValid = isValidDPS(dev);
				dev = null;
			}
			catch (Exception e) {}
		}
		
		return bValid;
	}

	/**
	 * Checks to see if the DPS passed in does not contain a 0 for the device or port
	 * numbers
	 * 
	 * @param dps DPS to validate
	 * @return true if this is a valid DPS; false otherwise
	 */
	public final static boolean isValidDPS(DPS dps)
	{
		boolean bValid = false;
		
		if (dps != null)
		{
			if (dps.getDevice() > 0 && dps.getPort() > 0)
				bValid = true;
		}

		return bValid;
	}
	
	/**
	 * Checks to see if the string passed in is a valid IP address or not
	 * 
	 * @param ip String containing the address to validate
	 * @return true if this is a valid IP address; false otherwise
	 */
	public final static boolean isValidIP(String ip)
	{
		boolean bValid = false;
		
		if (ip != null && ip.length() > 0)
		{
			try
			{
				InetAddress addr = InetAddress.getByName(ip);
				if (addr != null)
					bValid = true;
				addr = null;
			}
			catch (Exception e) {} // Don't worry about this
		}
		
		return bValid;
	}
	
	/*
	 * 
	 * @param str
	 */
	private static void error(String str)
	{
		Logger.error(className, str);
	}

	/**
	 * Simulates the functionality of the NetLinx ATOI function.
	 * This function returns Integer.MIN_VALUE if no numerical characters are found.
	 * Note: the numerical characters must be at the start of the string
	 * 
	 * @param str - String containing an int value
	 * 
	 * @return int value contained in the string
	 */
	public static int ATOI(String str)
	{
		StringBuffer sbString = new StringBuffer("");
		boolean minusFailsNow = false;
		
		// find first non-numerical character and filter out all other characters 
		for (int i=0;i<str.length(); i++)
		{
			char cChar = str.charAt(i);
			if ( (cChar == '-') || ( (cChar >= '0') && (cChar <= '9') ) )
			{
				if ((cChar == '-') && (minusFailsNow)) { break; }
				
				minusFailsNow = true;
				sbString.append(str.charAt(i));
			}
			else if(sbString.length() > 0) { break; }
		}
		if (sbString.length() == 0) 
		{ 
			error("'ATOI():Number formatException, String was of zero length"); 
			return Integer.MIN_VALUE; 
		}

		try { return Integer.parseInt(sbString.toString()); }
		catch(NumberFormatException e) 
		{
			error("'ATOI():Number formatException, String was "+sbString);
			return Integer.MIN_VALUE;
		}
	}
}