package com.amx.duet.tools.math;

import com.amx.duet.tools.lang.Logger;
/*
 * @author MManners
 * Created on July 11, 2007
 * 
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */

/**
 * A utility class to handle ranges used within Duet modules. These ranges can
 * be especially useful when adjusting values between the panel and the device.
 * 
 * @version 1.0.4
 * @since AMXTools 2.0.20
 * 
 */
public class Range
{
	private double _minValue = 0;

	private double _maxValue = 255;

	private double _increment = 1;

	private double currentValue = _minValue;

	/**
	 * Constructs and initializes the Range class. This will determine the
	 * minimum and maximum values of the range, as well as the increment used
	 * within the range. The current value of the range is also set. These
	 * values will be used when setting the value based on a level input, and
	 * when processing the value sent from the device to set the level feedback.
	 * 
	 * <P>
	 * <strong>Example usage:</strong>
	 * 
	 * <pre>
	 * Range panelRange  = new Range( 0, 255, 1);
	 * Range volumeRange = new Range( -95.5, 31.5, .5);
	 *  ...
	 *       
	 * public void setVolume(int level)
	 * {
	 *    panelRange.setCurrentValue( level );
	 *    double deviceVolume = volumeRange.getScaledValue(panelRange);
	 *    // If your device only needs an integer value, you can cast 
	 *    // the deviceVolume returned from getScaled value as shown below:
	 *    // int deviceVolume = (int) volumeRange.getScaledValue(panelRange);
	 *    enQueue( setVolumeCommand(deviceVolume) );
	 * }
	 *    
	 * private void parseResponse(ByteBuffer data)
	 * {
	 *    ...
	 *    // Handling Volume response
	 *    String volumeValue = data.toString().substring(volumeStart,volumeEnd);
	 *    double deviceVolume = Double.parseDouble(volumeValue);
	 *    // If your device only needs an integer value, you can cast 
	 *    // the deviceVolume returned from getScaled value as shown below:
	 *    // int deviceVolume = Integer.parseInt(volumeValue);
	 *
	 *    volumeRange.setCurrentValue( deviceVolume );
	 *    int panelVolume = panelRange.scale( volumeRange );
	 *    updateVolume( panelVolume );
	 * }
	 * </pre>
	 * 
	 * @param minValue
	 *            The minimum value of the range.
	 * @param maxValue
	 *            The maximum value of the range.
	 * @param increment
	 *            The increment between each discrete value of the range.
	 * @param currentValue
	 *            The current value of range.
	 */
	public Range(double minValue, double maxValue, double increment,
			double currentValue)
	{
		this._minValue = minValue;
		this._maxValue = maxValue;
		if (minValue >= maxValue)
		{
			Logger.diag(Range.class,
					"Warning: UtilityRange is encounterd a minValue >= maxValue in constructor.");
		}
		this._increment = increment;
		if (range() % increment != 0)
		{
			Logger.diag(Range.class,
					"Warning: UtilityRange is working with an increment of "
							+ increment + " for a range of " + range()
							+ ", which is not evenly divisible.");
		}
		setCurrentValue(currentValue);
	}

	/**
	 * Constructs and initializes the Range class. This will determine the
	 * minimum and maximum values of the range, as well as the increment used
	 * within the range. These values will be used when setting the value based
	 * on a level input, and when processing the value sent from the device to
	 * set the level feedback. The current Value is set to the minimum value of the range.
	 *
	 * 
	 * @param minValue
	 *            The minimum value of the range.
	 * @param maxValue
	 *            The maximum value of the range.
	 * @param increment
	 *            The increment between each discrete value of the range.
	 *            
	 * @see #Range(double,double,double,double)
	 */
	public Range(double minValue, double maxValue, double increment)
	{
		this(minValue, maxValue, increment, minValue);
	}

	/**
	 * Returns the current value of the rnage.
	 * 
	 * @return Returns the currentValue.
	 */
	public double getCurrentValue()
	{
		return currentValue;
	}

	/**
	 * Returns the value of the increment used to determin discrete values
	 * within the range.
	 * 
	 * @return Returns the _increment.
	 */
	public double getIncrement()
	{
		return _increment;
	}

	/**
	 * Returns the maximum value of the range.
	 * @return Returns the maxValue.
	 */
	public double getMaxValue()
	{
		return _maxValue;
	}

	/**
	 * Returns the minimum value of the Range.
	 * 
	 * @return Returns the minValue.
	 */
	public double getMinValue()
	{
		return _minValue;
	}

	/**
	 * Returns the discrete position of the currentValue of the range.
	 *  
	 * @return The discrete position that most closely matches the currentValue.
	 * @see #getPosition(double)
	 */
	public int getPosition()
	{
		return getPosition(currentValue);
	}

	/**
	 * Returns the discrete position of the range that most closely matches
	 * the value. If the value is outside the minimum or maximum values of the
	 * range, a warning message is displayed. Also, if the value is not a
	 * discrete value of the range, a warning message is displayed.
	 * 
	 * @param value The value you want to evaluate the position of.
	 * @return The discrete position that most closely matches the value.
	 */
	public int getPosition(double value)
	{
		int result = 0;

		if (value < _minValue)
		{
			Logger.diag(this, "Warning: incrementOfValue: value(" + value
					+ ") < minValue(" + _minValue + ").");
		}
		if (value > _maxValue)
		{
			Logger.diag(this, "Warning: incrementOfValue: value(" + value
					+ ") > maxValue(" + _maxValue + ").");
		}

		double workingValue = value;
		workingValue = workingValue - _minValue;
		if ((workingValue) % _increment != 0)
		{
			Logger.diag(this,
					"Warning: incrementOfValue is working with an increment of "
							+ _increment + " for a value of " + value + ".");
		}

		result = (int)Math.round(workingValue / _increment);
		return result;
	}

	/**
	 * Returns the value of this range based on the scaled position of the
	 * sourceRange.
	 * 
	 * 
	 * @param sourceRange
	 *            The range you want to evaluate the discrete position of the
	 *            value.
	 * @return The value within this range that most closely matches the
	 *         position of the value within sourceRange.
	 *         
	 * @see #getValue(int)
	 * @see #scale(Range)
	 */
	public double getScaledValue(Range sourceRange)
	{
		return getValue(scale(sourceRange));
	}

	/**
	 * Returns the value of the discrete position of this range. If the position
	 * requested is less than zero, then the minimum value is returned. If the
	 * position is greater than the number of steps, then the maximum value of
	 * the range is calculated.
	 * 
	 * @param step
	 *            The discrete position of this range.
	 * @return The value of the position within the range.
	 */
	public double getValue(int step)
	{
		double result = 0.0;
		if (step > numberOfSteps())
		{
			Logger.diag(this,
					"Warning: valueOfIncrement: incrementPosition out of range for incrementPosition of "
							+ step
							+ " with only "
							+ numberOfSteps()
							+ " increments.");
			step = numberOfSteps();
		}
		else if (step < 0)
		{
			Logger.diag(this,
					"Warning: valueOfIncrement: incrementPosition out of range for incrementPosition of "
							+ step
							+ " with only "
							+ numberOfSteps()
							+ " increments.");
			step = 0;
		}
		result = _minValue + (_increment * step);
		return result;
	}

	/**
	 * Returns the value of the discrete position of the current value plus the
	 * offset. The offset can be positive or negative.
	 * 
	 *  
	 * <P>
	 * <strong>Example usage:</strong>
	 * 
	 * <pre>
	 * public void adjustVolume(int offset)
	 * {
	 *    double deviceVolume = volumeRange.getValueForOffset( offset );
	 *    enQueue( setVolumeCommand(deviceVolume) );
	 * }
	 * </pre>
	 * 
	 * Review {@link #getValue(int)} for more information....
	 * 
	 * @param offset The number of discrete units to adjust.
	 * @return The value for this range adjusted by the offset.
	 */
	public double getValueForOffset(int offset)
	{
		return (getValue(getPosition() + offset));
	}

	/**
	 * Returns the number of discrete values contained within this range. If the
	 * range is not evenly divisible by the increment of the range, a warning
	 * message is displayed.
	 * 
	 * @return The number of discrete values within the range.
	 * 
	 */
	public int numberOfSteps()
	{
		int result = 1;
		if (range() % _increment != 0)
		{
			Logger.diag(this,
					"Warning: increments() is working with an increment of "
							+ _increment + " for a range of " + range() + ".");
		}
		result = (int)Math.round(range() / _increment);
		return result;
	}

	/**
	 * Returns the difference between the minimum and maximum values of this range.
	 * @return The range between the minimum and maximum values of the range.
	 */
	private double range()
	{
		double result = 0;

		if (_maxValue > _minValue)
		{
			result = _maxValue - _minValue;
		}
		else
		{
			result = 0;
		}
//		if ((_minValue == 0 || _maxValue == 0) && (_minValue != _maxValue))
//		{
//			result = result + 1;
//		}
		if (result == 0)
		{
			Logger.diag(this,
					"Warning: range() == 0");
		}

		return result;
	}

	/**
	 * Returns the position the value of the sourceRange would have within this
	 * range. If a loss of percision will occur, a warning message is displayed.
	 * 
	 * @param sourceRange
	 *            The range you want to evaluate the discrete position of the
	 *            value.
	 * @return The discrete position of this range that most closely matches the
	 *         value contained in the sourceRange.
	 */
	public int scale(Range sourceRange)
	{
		int returnValue = -1;
		if (sourceRange.numberOfSteps() > numberOfSteps())
		{
			Logger.diag(this,
					"Warning: scaledIncrement: sourceRange has more increments current range.  Scaling will loose percision.");
		}
		if (sourceRange.currentValue >= sourceRange._maxValue)
		{
			returnValue = numberOfSteps();
		}
		else if (sourceRange.currentValue <= sourceRange._minValue)
		{
			returnValue = 0;//(int)this._minValue;
		}
		else
		{
			int sourceIncrement = sourceRange.getPosition();
			double preciseValue = (((sourceIncrement * 1.0) / (sourceRange.numberOfSteps() * 1.0)) * numberOfSteps());
			returnValue = (int)Math.round(preciseValue);
		}
		return returnValue;
	}

	/**
	 * Sets the current value of the range. If the value is not one of the
	 * discrete values defined by the range, a warning messages is displayed.
	 * 
	 * @param value
	 *            The currentValue to set.
	 */
	public void setCurrentValue(double value)
	{
		if ((value - _minValue) % _increment != 0)
		{
			Logger.diag(this,
					"Warning: setCurrentValue is working with an increment of "
							+ _increment + " for a value of " + value + ".");
		}
		this.currentValue = value;
	}

}
