/*
 * Range.java
 * 
 * Created on Jul 7, 2006
 * Created by VGonzales
 *
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.math;

/**
 * A utility class used for range checking and scaling.
 * 
 * @version 1.0.0
 */
public class RangeUtil
{
	/**
	 * Used by adjust functions to set the scaled offset within the valid range.
	 * 
	 * @param value is the value to be adjusted
	 * @param increment is the currently evaluated step to be added to value
	 * @param min is the minimum number that value can be set to
	 * @param max is the maximum number that value can be set to
	 * 
	 * @return integer containing the newly calculated value which can be sent to a setter method directly
	 */
	public static int evaluateValue(int value, int increment, int min, int max)
	{
		if (value + increment > max) { return max; }
		else if (value + increment < min) { return min; }
		else { return value + increment; }
	}
	
	/**
	 * @deprecated
	 * @see {@link rangeCheck(int, int, int)}
	 * Determines if a specific value falls within the specified bounds.
	 * 
	 * @param value - int value in question
	 * @param lowerBound - int lower limit
	 * @param upperBound - int upper limit
	 * 
	 * @return boolean - result
	 */
	public static boolean fnRangeCheck(int value, int lowerBound, int upperBound)
	{
		return rangeCheck(value, lowerBound, upperBound);
	}

	/**
	 * Determines if a specific value falls within the specified bounds.
	 * 
	 * @param value - int value in question
	 * @param lowerBound - int lower limit
	 * @param upperBound - int upper limit
	 * 
	 * @return boolean - result
	 */
	public static boolean rangeCheck(int value, int lowerBound, int upperBound)
	{
		if ( (value >= lowerBound) && (value <=upperBound))
			return true;
		return false;      			
	}
	
	/**
	 * @deprecated
	 * @see {@link scaleRange(int, int, int, int, int)}
	 * Scale incoming UI ranges to device ranges and scale incoming device ranges to UI ranges.
	 * Note: this method assumes that range checking has already been performed on the inputs.
	 * 
	 * @param value is the value to be scaled
	 * @param sourceMin is the minimum value of the current scale that value is in
	 * @param sourceMax is the maximum value of the current scale that value is in
	 * @param targetMin is the minimum value of the scale to which value should be converted to
	 * @param targetMax is the maximum value of the scale to which value should be converted to
	 * 
	 * @return integer value of the scaled value 
	 */
	public static int fnScaleRange(int value, int sourceMin, int sourceMax, int targetMin, int targetMax)
	{
		return scaleRange(value, sourceMin, sourceMax, targetMin, targetMax);
	}
	
	/**
	 * @deprecated
	 * @see {@link scaleRange(float, float, float, float, float)}
	 * Scale incoming UI ranges to device ranges and scale incoming device ranges to UI ranges.
	 * Note: this method assumes that range checking has already been performed on the inputs.
	 * 
	 * @param value is the value to be scaled
	 * @param sourceMin is the minimum value of the current scale that value is in
	 * @param sourceMax is the maximum value of the current scale that value is in
	 * @param targetMin is the minimum value of the scale to which value should be converted to
	 * @param targetMax is the maximum value of the scale to which value should be converted to
	 * 
	 * @return float value of the scaled value 
	 */
	public static float fnScaleRange(float value, float sourceMin, float sourceMax, float targetMin, float targetMax)
	{
		return scaleRange(value, sourceMin, sourceMax, targetMin, targetMax);
	}
	
	/**
	 * Scale incoming UI ranges to device ranges and scale incoming device ranges to UI ranges.
	 * Note: this method assumes that range checking has already been performed on the inputs.
	 * 
	 * @param value is the value to be scaled
	 * @param sourceMin is the minimum value of the current scale that value is in
	 * @param sourceMax is the maximum value of the current scale that value is in
	 * @param targetMin is the minimum value of the scale to which value should be converted to
	 * @param targetMax is the maximum value of the scale to which value should be converted to
	 * 
	 * @return integer value of the scaled value 
	 */
	public static int scaleRange(int value, int sourceMin, int sourceMax, int targetMin, int targetMax)
	{
		int sourceScale;
		int targetScale;
		float resultValue;

		if (value <= sourceMin) { resultValue = targetMin; }		// handle end points
		else if (value >= sourceMax) { resultValue = targetMax; }	// handle end points
		else	// otherwise scale...
		{
			sourceScale = sourceMax - sourceMin;	// establish input range
			targetScale = targetMax - targetMin;	// establish output range
			value = value - sourceMin;				// remove input offset
			
			// same as resultValue = ((value * targetScale) / sourceScale) + targetMin;
			resultValue = value * targetScale;			// multiply by output range
			resultValue = resultValue / sourceScale;	// then divide by input range
			resultValue = resultValue + targetMin;		// add in minimum output value
			
			resultValue = Math.round(resultValue);
		}
		return (int) resultValue;
	}
	
	/**
	 * Scale incoming UI ranges to device ranges and scale incoming device ranges to UI ranges.
	 * Note: this method assumes that range checking has already been performed on the inputs.
	 * 
	 * @param value is the value to be scaled
	 * @param sourceMin is the minimum value of the current scale that value is in
	 * @param sourceMax is the maximum value of the current scale that value is in
	 * @param targetMin is the minimum value of the scale to which value should be converted to
	 * @param targetMax is the maximum value of the scale to which value should be converted to
	 * 
	 * @return float value of the scaled value 
	 */
	public static float scaleRange(float value, float sourceMin, float sourceMax, float targetMin, float targetMax)
	{
		float sourceScale;
		float targetScale;
		float resultValue;

		if (value <= sourceMin) { resultValue = targetMin; }		// handle end points
		else if (value >= sourceMax) { resultValue = targetMax; }	// handle end points
		else	// otherwise scale...
		{
			sourceScale = sourceMax - sourceMin;	// establish input range
			targetScale = targetMax - targetMin;	// establish output range
			value = value - sourceMin;				// remove input offset
			
			// same as resultValue = ((value * targetScale) / sourceScale) + targetMin;
			resultValue = value * targetScale;			// multiply by output range
			resultValue = resultValue / sourceScale;	// then divide by input range
			resultValue = resultValue + targetMin;		// add in minimum output value
		}
		return resultValue;
	}

}
