package com.amx.duet.tools.net.http;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Enumeration;
import java.util.Properties;
import java.util.Vector;

import com.amx.duet.util.Timer;
import com.amx.duet.util.TimerListener;

//#####################################################################//
//# Copyright Notice :                                                #//
//#    Copyright, AMX, Inc., 2008                                     #//
//#    Private, proprietary information, the sole property of AMX.    #//
//#    The contents, ideas, and concepts expressed herein are not to  #//
//#    be disclosed except within the confines of a confidential      #//
//#    relationship and only then on a need to know basis.            #//
//#####################################################################//
/**
 * This class will handle any and all HTTP type post requests. It works similar to how a web-browser issues an HTTP request to a Web server, by properly formatting your request as per HTTP v1.1 RFC 2616
 * <li>This <code>HttppPostWrapper</code> is to be instantiated by the module.</li>
 * <li>The <code>HttpPostWrapper</code> provides a reusable HTTP connection by keeping the connection alive. This is desirable for a number of reason from better resource use to more efficient communications.</li>
 * <li>The <code>HttpPostWrapper</code> establishes an HTTP connection on a separate thread, therefore it is non-blocking. Events are generated by invoking asynchronous callback methods as implemented in the <code>IHttpPostWrapper</code> interface.</li>
 * You must implement the <code>IHttpPostWrapper</code> interface if you want to use this class.
 * You should not create several instances of this class unless you are trying to post HTTP request to several different hosts. If you will always use the same host (i.e. http://192.168.103.201 for example) you should not create several instances of this class.<p>
 * <p>
 * By default, this Http wrapper will assume a Content-Type of text/html and a Keep-Alive connection.
 * </p>
 * <p>
 * <b><u>This wrapper can use the <code>POST</code> or <code>GET</code> methods to send information to a Web server. Default is <code>POST</code>.</u></b>
 * </p>
 * When the user enters information there are two ways the information can be sent from the browser to the server: in the URL, or within the body of the HTTP request.
 * The <code>GET</code> method, appends name/value pairs to the URL. Unfortunately, the length of a URL is limited, so this method only works if there are only a few parameters. The URL could be truncated if the form uses a large number of parameters, or if the parameters contain large amounts of data. Also, parameters passed on the URL are visible in the address field of the browser not the best place for a password to be displayed.
 * The alternative to the <code>GET</code> method is the <code>POST</code> method. This method packages the name/value pairs inside the body of the HTTP request, which makes for a cleaner URL and imposes no size limitations on the forms output. It is also more secure.
 * @author Cnina 
 * @version 1.0.3
 * <p>Updates from previous version</p>
 * <li>Added HttpRead Timeout feature in case the http server is not responding. Default is 5 seconds.</li>
 * <ii>Added HttpConnect Timeout feature. Default is 5 seconds.</li>
 */
public class HttpPostWrapper
{
	public static final int HTTP_POST_SUCCESS = 0;
	public static final int INVALID_URL = 1;
	public static final int HTTP_CONNECTION_FAILED = 2;
	public static final int HTTP_OUTPUT_STREAM_FAILED = 3;
	public static final int HTTP_INPUT_STREAM_FAILED = 4;
	public static final int HTTP_CONNECTION_ALREADY_CONNECTED = 5;
	public static final int HTTP_CONNECTION_GENERAL_EXCEPTION = 6;
	public static final int HTTP_CONNECTION_DISPOSED = 7;
	public static final int HTTP_READ_TIMEOUT = 8;
	public static final int HTTP_CONNECT_TIMEOUT = 9;
	
	protected static final String[] STATE = {"HTTP_POST_SUCCESS","INVALID_URL","HTTP_CONNECTION_FAILED",
		                               "HTTP_OUTPUT_STREAM_FAILED","HTTP_INPUT_STREAM_FAILED",
		                               "HTTP_CONNECTION_ALREADY_CONNECTED","HTTP_CONNECTION_GENERAL_EXCEPTION",
		                               "HTTP_CONNECTION_DISPOSED","HTTP_READ_TIMEOUT","HTTP_CONNECT_TIMEOUT"};
    
	protected HttpPostHandler thread = null;
	private Properties m_RequestProperties = new Properties();
	private static final String m_Empty = "";
	protected boolean usePost = true;
	protected int m_HttpReadTimeout = 5000;
	protected int m_HttpConnectTimeout = 5000;
    
	public HttpPostWrapper(IHttpPostWrapper caller, long handle)
	{
		m_RequestProperties.setProperty("Content-Type", "text/html");
		m_RequestProperties.setProperty("connection", "Keep-Alive"); // default for HTTP v1.1
		thread = new HttpPostHandler(caller,handle);
		thread.start();
	}
    /**
     * Sets the Request method.
     * @param state True if POST otherwise GET
     */
    public void UsePost(boolean state)
    {
    	usePost = state;
    }
    /**
     * Sets the http read timeout in milliseconds.
     * Use 0 for infinite or no timeout.
     * If a negative time is specified, infinite is assumed.
     * @param time
     */
    public void setReadTimeout(int time)
    {
    	m_HttpReadTimeout = Math.max(0,time);
    }
    /**
     * Returns the current http read timeout time being used in milliseconds.
     * @return
     */
    public int getReadTimeout(){return m_HttpReadTimeout;}
    /**
     * Sets the http connect timeout in milliseconds.
     * Use 0 for infinite or no timeout.
     * If a negative time is specified, infinite is assumed.
     * @param time
     */
    public void setConnectTimeout(int time)
    {
    	m_HttpConnectTimeout = Math.max(0,time);
    }
    /**
     * Returns the current http connect timeout time being used in milliseconds.
     * @return
     */
    public int getConnectTimeout(){return m_HttpConnectTimeout;}
    /**
     * Sets the HTTP request property (also known as HTTP connection headers).
     * <p>This property will persist, for this object instance, until changed. If the property already
     * exists, it will be overriden.
     * <p>If a null parameter is passed it is ignored.
     * This is an optional method. By default, this wrapper will assume a Content-Type of text/html and a Keep-Alive connection.
     * <pre>Example of use: name="Content-Type" and value="text/html" (w/o the double-quotes)</pre>
     * @param name
     * @param value
     */
    public void setRequestProperty(String name, String value)
    {
    	if(name!=null && value!=null)
    		m_RequestProperties.setProperty(name,value);
    }
    /**
     * Sets the HTTP request properties (also known as HTTP connection headers).
     * <p>These properties will persist, for this object instance, until changed. This method will
     * override all previously established properties. If you need to change a specific property w/o affecting
     * any of the others, use the <code>setRequestProperty()</code>method instead.
     * <p>If a null <code>props</code> parameter is passed, it is ignored.
     * This is an optional method. By default, this wrapper will assume a Content-Type of text/html and a Keep-Alive connection.
     * @param props
     */
    public void setRequestProperties(Properties props)
    {
    	if(props!=null)
    		m_RequestProperties = props;
    }
    /**
     * Retrieves the associated property value.
     * If the property name does not exist, an empty string is returned.
     * @param name
     * @return
     */
    public String getRequestProperty(String name)
    {
    	if(m_RequestProperties.containsKey(name))
    	{
    		return m_RequestProperties.getProperty(name);
    	}
    	return m_Empty;
    }
    /**
     * Retrieves the Http request properties.
     * <p>The returned <code>Properties</code> object is never null.
     * @param name
     * @return
     */
    public Properties getRequestProperties()
    {
    	return m_RequestProperties;
    }
    /**
     * Provide an URL to Post(or Get) an HTTP request to.
     * The method used (POST or GET) is determined by setting the <code>UsePost(boolean)</code> method. Default is POST.
     * The URL provided can include query parameters if needed.
     * For example: http://192.168.103.201/lib/ajaxresults.php?cmd=nowplaying&zoneid=1
     * @param url
     */
	public void doPost(String url)
	{
		synchronized(thread)
		{
			thread.pleaseWait = true;
		}
		
		thread.doPost(url,this.getRequestProperties());
		
		synchronized(thread)
		{
    		thread.pleaseWait = false;
			thread.notify();
		}
	}
	/**
	 * This method will dispose of this wrapper instance. You should call this method if this Http connection is no longer needed.
	 */
	public void dispose()
	{
		this.m_RequestProperties.clear();
		if(thread!=null)
		{
			thread.dispose();
			thread = null;
		}
	}
	private class HttpPostHandler extends Thread implements TimerListener
	{
		private long m_handle = Long.MIN_VALUE;
	    private boolean m_keepRunning = true;
	    protected boolean pleaseWait = true;
	    protected HttpURLConnection   urlConn;
	    private DataOutputStream  printout;
        protected BufferedReader  input;
    	private IHttpPostWrapper m_callback = null;
    	private Vector messages = new Vector(10);
    	protected Timer m_HttpReadTimer = null;
    	protected Timer m_HttpConnectTimer = null;
    	/*
	     * Constructor.
	     * @param caller
	     * @param handle
	     */
	    private HttpPostHandler(IHttpPostWrapper caller, long handle)
		{
	    	m_callback = caller;
			m_handle = handle; // provides a handle to the caller
		}
	    /*
	     * Provide an URL to Post an HTTP request to.
	     * The URL provided can include query parameters if needed.
	     * For example: http://192.168.103.201/lib/ajaxresults.php?cmd=nowplaying&zoneid=1
	     * @param url
	     */
	    protected void doPost(String url, Properties props)
		{
			try
			{
				messages.add(new HtmlObject(props,new URL(url)));
			}
			catch (MalformedURLException e)
			{
				handleHttpPostState(INVALID_URL);
			}
		}
		protected void handleHttpPostState(int state)
		{
			if(m_callback!=null) 
				m_callback.handleHttpPostState(m_handle,state);
		}
		private void handleHttpPostReply(long handle, String data, HttpURLConnection url)
		{
			if(m_callback!=null) 
				m_callback.handleHttpPostReply(handle,data,url);
		}
		class HtmlObject
		{
			public Properties Props = null;
			public URL Url = null;
			public HtmlObject(Properties props, URL url)
			{
				Props = props;
				Url = url;
			}
		}
		
		protected void disposeHttpConnectTimer()
		{
			if(m_HttpConnectTimer!=null)
			{
				m_HttpConnectTimer.cancel();
				m_HttpConnectTimer = null;
			}
		}
		protected void disposeHttpReadTimer()
		{
			if(m_HttpReadTimer!=null)
			{
				m_HttpReadTimer.cancel();
				m_HttpReadTimer = null;
			}
		}
		public void run()
		{
			while(m_keepRunning)
			{
				synchronized(thread)
				{
					while(pleaseWait)
					{
						try
						{
							thread.wait();
						}
						catch(Exception e){}
					}
					if(messages.size()>0)
					{
						this.post();
						messages.remove(0);
					}
					if(messages.size()==0)
					{
						pleaseWait = true;
					}
				}
			}
		}
		protected void dispose()
		{
			this.interrupt();
			this.disposeHttpReadTimer();
			this.disposeHttpConnectTimer();
			m_keepRunning = false;
			try
			{
				if(printout!=null)
					printout.close();
			}
			catch(Exception e){}
			printout = null;
			try
			{
				if(input!=null)
					input.close();
			}
			catch(Exception e){}
			input = null;
			try
			{
				if(urlConn!=null)
					urlConn.disconnect();
			}
			catch(Exception e){}
			urlConn = null;
			try
			{
				this.join(1000);
			}
			catch (InterruptedException e)
			{
				
			}
			handleHttpPostState(HTTP_CONNECTION_DISPOSED);
		}
		private void post()
	    {
			try
			{
	    		HtmlObject message = null; 
	    		if(messages.size()>0)
	    			message = (HtmlObject)messages.get(0);
	    		else return;
	    		if(m_HttpConnectTimeout>0)
	    		{
	    			m_HttpConnectTimer = new Timer(this,m_HttpConnectTimeout);
	    			m_HttpConnectTimer.start();
		        }
	        	urlConn = (HttpURLConnection)message.Url.openConnection();
				//	Let the run-time system (RTS) know that we want input.
		        urlConn.setDoInput (true);
		        // Let the RTS know that we want to do output.
		        urlConn.setDoOutput (true);
		        // No caching, we want the real thing.
		        urlConn.setUseCaches (false);
		        // I'm gonna do POST. Specifically call POST (the default seems to be POST anyway but just in case...)
		        urlConn.setRequestMethod((usePost==false)? "GET":"POST");
		        Enumeration enm = message.Props.keys();
		        while(enm.hasMoreElements())
		        {  // Set the HTTP connection headers....
		        	String name  = (String)enm.nextElement();
		        	String value = message.Props.getProperty(name);
		        	if(value!=null && !value.equalsIgnoreCase("")) 
		        	{
		        		urlConn.setRequestProperty(name,value);
		        	}
		        }
		        try
				{// Send POST output.
		            String query = message.Url.getQuery();
		            if(query!=null)
		            {
		            	printout = new DataOutputStream (urlConn.getOutputStream ());
		            	printout.writeBytes (query);
		            	printout.flush ();
				        printout.close ();
				        printout = null;
				    }
		            this.disposeHttpConnectTimer();
			        try
			        {
			        	if(m_HttpReadTimeout>0)
			        	{
			        		m_HttpReadTimer = new Timer(this,m_HttpReadTimeout);
			        		m_HttpReadTimer.start();
				        }
			        	input = new BufferedReader (new InputStreamReader(urlConn.getInputStream()));
		            	String str;
			        	while ((str = input.readLine())!= null)
				        {
				        	handleHttpPostReply(m_handle,str,urlConn);
				        }
			        	disposeHttpReadTimer();
				        input.close();
				        input = null;
				    }
			        catch (IOException e)
					{ // getInputStream() exception...Reading the response body, by calling getErrorStream(), cleans up the connection...
			        	try
						{
			        		InputStream es = urlConn.getErrorStream();
			        		byte buf[] = new byte[1024];
			        		// read the response body
							while (es.read(buf) > 0) 
							{
								/*do nothing.*/
							}
							// close the errorstream
							es.close();
						}
						catch(IOException ex){}
						disposeHttpReadTimer();
						if(input!=null)
						{
							input.close();
							input = null;
						}
						handleHttpPostState(HTTP_INPUT_STREAM_FAILED);
						return;
					}
			    }
				catch (IOException e)
				{
					if(printout!=null)
					{
					    printout.flush ();
				        printout.close ();
				        printout = null;
				    }
					this.disposeHttpConnectTimer();
			        handleHttpPostState(HTTP_OUTPUT_STREAM_FAILED);
					return;
				}
			}
			catch (IOException e)
			{
				this.disposeHttpConnectTimer();
				handleHttpPostState(HTTP_CONNECTION_FAILED);
				return;
			}
			catch (IllegalStateException e)
			{
				this.disposeHttpConnectTimer();
				handleHttpPostState(HTTP_CONNECTION_ALREADY_CONNECTED);
				return;
			}
			catch(Exception e)
			{
				this.disposeHttpConnectTimer();
				handleHttpPostState(HTTP_CONNECTION_GENERAL_EXCEPTION);
				return;
			}
			handleHttpPostState(HTTP_POST_SUCCESS);
	    }
		public void handleTimerEvent(Timer arg0)
		{
			if(arg0 == m_HttpConnectTimer)
			{
				if(urlConn!=null)
				{
					urlConn.disconnect();
					urlConn = null;
				}
				handleHttpPostState(HTTP_CONNECT_TIMEOUT);
				disposeHttpConnectTimer();
			}
			else if(arg0 == m_HttpReadTimer)
			{
				if(input!=null)
				{
					try
					{
						input.close();
					}
					catch (IOException e)
					{
					}
					input = null;
				}
				if(urlConn!=null)
				{
					urlConn.disconnect();
					urlConn = null;
				}
				handleHttpPostState(HTTP_READ_TIMEOUT);
				disposeHttpReadTimer();
			}
		}
	}
}
