/*
 * @author VGonzales
 * 
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.net.snmp;

import java.net.InetAddress;
import java.net.SocketException;
import java.net.UnknownHostException;

import com.amx.duet.tools.lang.Logger;
import com.amx.duet.tools.net.snmp.api.SNMPBadValueException;
import com.amx.duet.tools.net.snmp.api.SNMPCounter32;
import com.amx.duet.tools.net.snmp.api.SNMPCounter64;
import com.amx.duet.tools.net.snmp.api.SNMPGauge32;
import com.amx.duet.tools.net.snmp.api.SNMPGetException;
import com.amx.duet.tools.net.snmp.api.SNMPIPAddress;
import com.amx.duet.tools.net.snmp.api.SNMPInteger;
import com.amx.duet.tools.net.snmp.api.SNMPNSAPAddress;
import com.amx.duet.tools.net.snmp.api.SNMPObject;
import com.amx.duet.tools.net.snmp.api.SNMPObjectIdentifier;
import com.amx.duet.tools.net.snmp.api.SNMPOctetString;
import com.amx.duet.tools.net.snmp.api.SNMPSequence;
import com.amx.duet.tools.net.snmp.api.SNMPSetException;
import com.amx.duet.tools.net.snmp.api.SNMPTimeTicks;
import com.amx.duet.tools.net.snmp.api.SNMPUInteger32;
import com.amx.duet.tools.net.snmp.api.SNMPVarBindList;
import com.amx.duet.tools.net.snmp.api.SNMPv1CommunicationInterface;

/**
 * The SNMPConnection class is a wrapper around all of the basic SNMP operations.
 * It can be used to perform operations such as GET, GETNEXT, and SET.
 * 
 * Each operation runs in its own thread, but the class itself has been 
 * throttled back to work as if each thread were one. In otherwords, the class 
 * will not allow you to perform another operation until the previous operation 
 * is complete. The reason behind this is because each SNMP operation performs 
 * its request and response action in a single instance. Thus, the request is 
 * not complete until the agent has finished responding, which stalls all other 
 * processes on the master.
 * 
 * <pre>
 * // This is an example of how to create the connection object
 * connection = new SNMPConnection("10.0.0.10", "1.3.6.1.4", "public", this);
 * connection.setDebugState(DEBUG);
 * 
 * // Here is an operation example using the connection object
 * if (!connection.isBusy())
 * {
 * 	// Get object at "1.3.6.1.4.1.9.3.3.1"
 * 	connection.getMIBEntry("1.9.3.3.1");
 * }
 * 
 * // This is an example of how to handle the response in the implementing listener 
 * public void handleSNMPReceive(SNMPObjectIdentifier itemID, SNMPObject snmpValue)
 * {
 * 	String OID = "";
 * 	if (itemID != null)
 * 	{
 * 		OID = "." + itemID.toString();
 * 		if (OID.startsWith(connection.getEnterpriseID()))
 * 		{
 * 			// Remove "1.3.6.1.4" (Enterprise OID) from OID string
 * 		}
 * 	}
 * 
 * 	try
 * 	{
 * 		String data = snmpValue.toString();
 * 		this.processPassBackEvent(new ModuleComponentEvent(this, itemID + "," + data, 1));
 * 		if (OID.equals("1.9.3.3.1"))
 * 		{
 * 			// Do feedback processing...
 * 		}
 * 	}
 * }
 * </pre>
 */
public class SNMPConnection implements Runnable
{
	public static final int CONNECTED = 1;
	public static final int TIMEOUT = -1;
	public static final int CLOSED = -2;
	public static final int INVALID_HOST = -3;
	public static final int FAILED_CONNECT = -4;
	public static final int FAILED_GET = -5;
	public static final int FAILED_SET = -6;
	public static final int UNKNOWN_ERROR = -7;
	public static final int INVALID_OID = -8;
	
	public static final int INFINITE = 0;
	private static final int MAX_VERSION = 2;
	
	private SNMPConnectionListener m_listener = null;
	private InetAddress m_hostAddress = null;
	private String m_community = "";
	private int m_version = MAX_VERSION;
	
	private boolean m_bIsRunning = false;
	private int nDebug = Logger.LOG_ERROR;
	
	private Object connectLock = new Object();
	MIBMessage m_msg = null;
	
	private SNMPObjectIdentifier m_DeviceMakeID = null;
	
	public static final int NONE           = 0;
	
	// Operations
	public static final int OP_GET         = 1;
	public static final int OP_GETNEXT     = 2;
	public static final int OP_SET         = 3;
	
	// Expected Data Type
	public static final int DT_INTEGER     = 1;
	public static final int DT_UINTEGER32  = 2;
	public static final int DT_GAUGE       = 3;
	public static final int DT_COUNTER32   = 4;
	public static final int DT_COUNTER64   = 5;
	public static final int DT_TIMETICKS   = 6;
	public static final int DT_OCTETSTRING = 7;
	public static final int DT_IPADDRESS   = 8;
	public static final int DT_MACADDRESS  = 9;
	public static final int DT_OBJECTID    = 10;
	public static final int DT_MIBTABLE    = 14;
	
    private class RetrieveThread extends Thread
    {
        public RetrieveThread(Runnable r)
        {
            super(r);
        }
        
    }
    
    /**
     * Default constructor
     */
    public SNMPConnection()
    {
    	
    }
    
	/**
	 * Setup the SNMP connection with all initial values.
	 * 
	 * @param version [1..2] SNMP version
	 * @param hostAddress (InetAddress) Address to connect to.
	 * @param deviceMakeOID Identifies the type of managed object (Enterprise OID)
	 * @param community Defines the access environment for a group of NMSs
	 * @param listener Object which will receive SNMP events.
	 */
	public SNMPConnection(int version, InetAddress hostAddress, String deviceMakeOID, String community, SNMPConnectionListener listener)
	{
		initConnection(version, hostAddress, deviceMakeOID, community, listener);
	}

	/**
	 * Setup the SNMP connection with initial values using a String IP address.
	 * 
	 * @param version [1..2] SNMP version
	 * @param hostAddress (String) Address to connect to.
	 * @param deviceMakeOID Identifies the type of managed object (Enterprise OID)
	 * @param community Defines the access environment for a group of NMSs
	 * @param listener Object which will receive SNMP events.
	 */
	public SNMPConnection(int version, String hostAddress, String deviceMakeOID, String community, SNMPConnectionListener listener)
	{
		initConnection(version, hostAddress, deviceMakeOID, community, listener);
	}
	
	/**
	 * Setup the SNMP connection with initial values using the default SNMP version (latest supported).
	 * 
	 * @param hostAddress (InetAddress) Address to connect to.
	 * @param deviceMakeOID Identifies the type of managed object (Enterprise OID)
	 * @param community Defines the access environment for a group of NMSs
	 * @param listener Object which will receive SNMP events.
	 */
	public SNMPConnection(InetAddress hostAddress, String deviceMakeOID, String community, SNMPConnectionListener listener)
	{
		initConnection(m_version, hostAddress, deviceMakeOID, community, listener);
	}

	/**
	 * Setup the SNMP connection with initial values using the default SNMP version (latest supported) and a String IP address.
	 * 
	 * @param hostAddress (String) Address to connect to.
	 * @param deviceMakeOID Identifies the type of managed object (Enterprise OID)
	 * @param community Defines the access environment for a group of NMSs
	 * @param listener Object which will receive SNMP events.
	 */
	public SNMPConnection(String hostAddress, String deviceMakeOID, String community, SNMPConnectionListener listener)
	{
		initConnection(m_version, hostAddress, deviceMakeOID, community, listener);
	}
	
	/**
	 * Use this method to initialize the SNMP connection with initial values if created using the default constructor.
	 * 
	 * @param version [1..2] SNMP version
	 * @param hostAddress (String) Address to connect to.
	 * @param deviceMakeOID Identifies the type of managed object (Enterprise OID)
	 * @param community Defines the access environment for a group of NMSs
	 * @param listener Object which will receive SNMP events.
	 */
	public void initConnection(int version, String hostAddress, String deviceMakeOID, String community, SNMPConnectionListener listener)
	{
		try
		{
			initConnection(version, InetAddress.getByName(hostAddress), deviceMakeOID, community, listener);
		}
		catch (UnknownHostException e)
		{
			error("UnknownHostException - " + e.toString());
			if (m_listener != null)
				m_listener.handleSocketStatus(INVALID_HOST);
		}
	}
	
	/**
	 * Use this method to initialize the SNMP connection with initial values if created using the default constructor.
	 * 
	 * @param version [1..2] SNMP version
	 * @param hostAddress (InetAddress) Address to connect to.
	 * @param deviceMakeOID Identifies the type of managed object (Enterprise OID)
	 * @param community Defines the access environment for a group of NMSs
	 * @param listener Object which will receive SNMP events.
	 * @throws IllegalArgumentException
	 */
	public void initConnection(int version, InetAddress hostAddress, String deviceMakeOID, String community, SNMPConnectionListener listener) throws IllegalArgumentException
	{
		if (listener == null)
			throw new NullPointerException("Invalid listener");
		
		if (hostAddress == null)
			throw new NullPointerException("Invalid host address");
		
		if (community == null || community.length() == 0)
			throw new IllegalArgumentException("Invalid community");
		
		if (version <= 0 && version > MAX_VERSION)
			throw new IllegalArgumentException("Invalid version number; must be 1 or 2");
		
		if (!setEnterpriseID(deviceMakeOID))
			throw new IllegalArgumentException("Invalid device make OID");
		
		m_version = version - 1;
		m_hostAddress = hostAddress;
		m_community = community;
		m_listener = listener;
	}
	
	/**
	 * Set the current SNMP version to use for communications
	 * 
	 * @param version [1..2]
	 */
	public void setSNMPVersion(int version)
	{
		if (version > 0 && version <= MAX_VERSION)
			m_version = version;
		else
			error("Invalid version number; must be 1 or 2");
	}
	
	/**
	 * Sets the internet address for the host you are trying to connect to.
	 * 
	 * @param hostAddress String containing the host address.
	 */
	public void setHostAddress(String hostAddress)
	{
		try
		{
			setHostAddress(InetAddress.getByName(hostAddress));
		}
		catch (UnknownHostException e)
		{
			error("UnknownHostException - " + e.toString());
			if (m_listener != null)
				m_listener.handleSocketStatus(INVALID_HOST);
		}
	}

	/**
	 * Sets the internet address for the host you are trying to connect to.
	 * 
	 * @param hostAddress InetAddress containing the host address.
	 */
	public void setHostAddress(InetAddress hostAddress)
	{
		if (hostAddress == null)
			throw new NullPointerException("Invalid host address");
		
		m_hostAddress = hostAddress;
	}
	
	/**
	 * Defines the access environment for a group of Network Management Systems. 
	 * NMSs within the community are said to exist within the same 
	 * administrative domain. Community names serve as a weak form of 
	 * authentication because devices that do not know the proper community name 
	 * are precluded from SNMP operations.
	 *  
	 * @param community String containing message group access.
	 */
	public void setCommunityAccess(String community)
	{
		if (community == null || community.length() == 0)
			throw new IllegalArgumentException("Invalid community");
		
		m_community = community;
	}
	
	/**
	 * Defines the device's company Object Identifier. The company OID should be 
	 * the prefix used for every OID used in the device's list of managed 
	 * objects in its MIB hierarchy. 
	 * <p>
	 * The MIB hierarchy can be depicted as a tree with a nameless root, the 
	 * levels of which are assigned by different organizations. Vendors usually 
	 * define private branches that include managed objects for their products. 
	 * The level prior to the products branch is the company's OID.
	 * <p>
	 * Example:
	 * <pre>
	 * If the OID to get to a device variable looked like this:
	 * 
	 * 1.3.6.1.4.1.9.3.3.1
	 * 
	 * The product OID might be:
	 * 
	 * 1.3.6.1.4.1.9.3
	 * 
	 * And the company OID might be:
	 * 
	 * 1.3.6.1.4
	 * 
	 * The call should look like this:
	 * boolean success = setEnterpriseID("1.3.6.1.4");
	 * </pre>
	 * Basically, the company OID is the prefix ID for the product ID and the 
	 * product ID is the prefix for the variable OID.
	 * 
	 * @param makeOID String containing the company OID.
	 * @return <code>true</code> if the OID was valid; <code>false</code> 
	 * otherwise.
	 */
	public boolean setEnterpriseID(String makeOID)
	{
		boolean bSuccess = false;
		try
		{
			m_DeviceMakeID = new SNMPObjectIdentifier(makeOID);
			if (m_DeviceMakeID != null)
				bSuccess = true;
		}
		catch (SNMPBadValueException e)
		{
			error("Invalid OID for setEnterpriseID(): " + e.toString());
			if (m_listener != null)
				m_listener.handleSocketStatus(INVALID_OID);
		}
		return bSuccess;
	}

	/**
	 * Retrieves the device's company Object Identifier in a String object.
	 * 
	 * @return String containing the company OID.
	 */
	public String getEnterpriseID()
	{
		String oid = "";
		if (m_DeviceMakeID != null)
			oid = m_DeviceMakeID.toString();
		return oid;
	}
	
	/*
	 * This operation is used to retrieve the value of an object instance from 
	 * an SNMP agent (device).
	 * 
	 * @param itemID String containing the friendly named device variable key.
	 * @param dataType
	 * @param operation
	 */
	private void getMIBEntry(String itemID, int operation)
	{
		debug("getMIBEntry(" + itemID + ")");
		if (m_bIsRunning)
		{
			error("Connection busy");
			return;
		}
		
		SNMPObjectIdentifier oid = createOID(itemID);
		if (oid == null)
			return;
		
		MIBMessage msg = new MIBMessage();
		msg.oid = new SNMPObjectIdentifier[1];
		msg.sIdentifier = new String[1];
		
		msg.sIdentifier[0] = itemID;
		msg.oid[0] = oid;
		msg.nOperation = operation;
		msg.nOIDCount = 1;
		
		synchronized (connectLock) { m_msg = msg; }
		startThread();
	}
	
	/**
	 * This operation is used to retrieve the value of an object instance from 
	 * an SNMP agent (device).
	 * 
	 * @param itemID String containing the OID of the data to access
	 * (you do not need to include the Enterprise ID).
	 */
	public void getMIBEntry(String itemID)
	{
		getMIBEntry(itemID, OP_GET);
	}

	/**
	 * This operation is used to retrieve the NEXT value of an object instance, 
	 * after a GET was called, from an SNMP agent (device).
	 * 
	 * @param itemID String containing the OID of the data to access
	 * (you do not need to include the Enterprise ID).
	 */
	public void getNextMIBEntry(String itemID)
	{
		getMIBEntry(itemID, OP_GETNEXT);
	}
	
	/*
	 * 
	 * @param itemIDs
	 * @param dataType
	 * @param operation
	 */
	private void getMIBEntry(String[] itemIDs, int operation)
	{
		debug("getMIBEntry(" + itemIDs + ")");
		if (m_bIsRunning)
		{
			error("Connection busy");
			return;
		}
		
		SNMPObjectIdentifier[] oids = createOID(itemIDs);
		if (oids != null && itemIDs != null)
		{
			if (oids.length == itemIDs.length)
			{
				MIBMessage msg = new MIBMessage();
				
				msg.sIdentifier = itemIDs;
				msg.oid = oids;
				msg.nOperation = operation;
				msg.nOIDCount = 1;
				
				synchronized (connectLock) { m_msg = msg; }
				startThread();
			}
			else
			{
				error("One or more OIDs is not valid");
			}
		}
	}

	/**
	 * This operation is used to retrieve the value of an object instance from 
	 * an SNMP agent (device).
	 * 
	 * @param itemID Array of Strings containing the OID of the data to access
	 * (you do not need to include the Enterprise ID).
	 */
	public void getMIBEntry(String[] itemIDs)
	{
		getMIBEntry(itemIDs, OP_GET);
	}

	/**
	 * This operation is used to retrieve the NEXT value of an object instance, 
	 * after a GET was called, from an SNMP agent (device).
	 * 
	 * @param itemID Array of Strings containing the OID of the data to access
	 * (you do not need to include the Enterprise ID).
	 */
	public void getNextMIBEntry(String[] itemIDs)
	{
		getMIBEntry(itemIDs, OP_GETNEXT);
	}
	
	/**
	 * This operation is used to set the value of an object instance on 
	 * an SNMP agent (device).
	 * 
	 * @param itemID String containing the OID of the data to access
	 * (you do not need to include the Enterprise ID).
	 * @param newValue Value to be updated
	 * @param dataType Data type of the value to be updated
	 */
	public void setMIBEntry(String itemID, SNMPObject newValue, int dataType)
	{
        // the setMIBEntry method of the communications interface returns the SNMPVarBindList
        // corresponding to the supplied OID and value
		debug("setMIBEntry(" + itemID + "," + newValue.toString() + "," + dataTypeToString(dataType) + ")");
		if (m_bIsRunning)
		{
			error("Connection busy");
			return;
		}
		if (dataType <= NONE || dataType > DT_MIBTABLE)
		{
			error("Invalid return type");
			return;
		}
		
		SNMPObjectIdentifier oid = createOID(itemID);
		if (oid == null)
			return;
		
		MIBMessage msg = new MIBMessage();
		msg.oid = new SNMPObjectIdentifier[1];
		msg.sIdentifier = new String[1];
		msg.value = new SNMPObject[1];
		
		msg.sIdentifier[0] = itemID;
		msg.oid[0] = oid;
		msg.value[0] = newValue;
		msg.nOperation = OP_SET;
		msg.nDataType = dataType;
		msg.nOIDCount = 1;
		
		synchronized (connectLock) { m_msg = msg; }
		startThread();
	}
	
	/**
	 * This operation is used to set the value of an object instance on 
	 * an SNMP agent (device).
	 * 
	 * @param itemID String containing the OID of the data to access
	 * (you do not need to include the Enterprise ID).
	 * @param newValue String form of value to be updated
	 * @param dataType Data type of the value to be updated
	 */
	public void setMIBEntry(String itemID, String value, int dataType)
	{
		try
		{
			switch (dataType)
			{
				case DT_INTEGER:
					setIntValue(itemID, Long.parseLong(value));
					break;
				case DT_UINTEGER32:
					setUIntValue(itemID, Long.parseLong(value));
					break;
				case DT_GAUGE:
					setGuageValue(itemID, Long.parseLong(value));
					break;
				case DT_COUNTER32:
					setCounterValue(itemID, Long.parseLong(value));
					break;
				case DT_COUNTER64:
					setCounter64Value(itemID, Long.parseLong(value));
					break;
				case DT_TIMETICKS:
					setTimeTicksValue(itemID, Long.parseLong(value));
					break;
				case DT_OCTETSTRING:
					setStringValue(itemID, value);
					break;
				case DT_IPADDRESS:
					setIPAddressValue(itemID, value);
					break;
				case DT_MACADDRESS:
					setMACAddressValue(itemID, value);
					break;
				case DT_OBJECTID:
//					SNMPObject object = new SNMPObject();
//					setMIBEntry(itemID, (value), dataType);
					break;
				case DT_MIBTABLE:
					break;
			}
			
		}
		catch (Exception e)
		{
			error("Exception in setMIBEntry(String itemID, String value, int dataType): " + e.toString());
			if (getDebugState() == Logger.LOG_DEBUG)
				e.printStackTrace();
		}
		
	}
	
	/**
	 * This operation is used to set the INTEGER value of an object instance on 
	 * an SNMP agent (device).
	 * 
	 * @param itemID String containing the OID of the data to access
	 * (you do not need to include the Enterprise ID).
	 * @param value Value to be updated
	 */
	public void setIntValue(String itemID, int value)
	{
		setMIBEntry(itemID, new SNMPInteger(value), DT_INTEGER);
	}
	
	/**
	 * This operation is used to set the INTEGER value of an object instance on 
	 * an SNMP agent (device).
	 * 
	 * @param itemID String containing the OID of the data to access
	 * (you do not need to include the Enterprise ID).
	 * @param value Value to be updated
	 */
	public void setIntValue(String itemID, long value)
	{
		setMIBEntry(itemID, new SNMPInteger(value), DT_INTEGER);
	}
	
	/**
	 * This operation is used to set the UNSIGNED INTEGER value of an object 
	 * instance on an SNMP agent (device).
	 * 
	 * @param itemID String containing the OID of the data to access
	 * (you do not need to include the Enterprise ID).
	 * @param value Value to be updated
	 */
	public void setUIntValue(String itemID, long value)
	{
		setMIBEntry(itemID, new SNMPUInteger32(value), DT_UINTEGER32);
	}
	
	/**
	 * This operation is used to set the GUAGE value of an object instance on 
	 * an SNMP agent (device).
	 * 
	 * @param itemID String containing the OID of the data to access
	 * (you do not need to include the Enterprise ID).
	 * @param value Value to be updated
	 */
	public void setGuageValue(String itemID, long value)
	{
		setMIBEntry(itemID, new SNMPGauge32(value), DT_GAUGE);
	}
	
	/**
	 * This operation is used to set the 32 BIT COUNTER value of an object 
	 * instance on an SNMP agent (device).
	 * 
	 * @param itemID String containing the OID of the data to access
	 * (you do not need to include the Enterprise ID).
	 * @param value Value to be updated
	 */
	public void setCounterValue(String itemID, long value)
	{
		setMIBEntry(itemID, new SNMPCounter32(value), DT_COUNTER32);
	}
	
	/**
	 * This operation is used to set the 64 BIT COUNTER value of an object 
	 * instance on an SNMP agent (device).
	 * 
	 * @param itemID String containing the OID of the data to access
	 * (you do not need to include the Enterprise ID).
	 * @param value Value to be updated
	 */
	public void setCounter64Value(String itemID, long value)
	{
		setMIBEntry(itemID, new SNMPCounter64(value), DT_COUNTER64);
	}

	/**
	 * This operation is used to set the TIME TICK value of an object instance 
	 * on an SNMP agent (device).
	 * 
	 * @param itemID String containing the OID of the data to access
	 * (you do not need to include the Enterprise ID).
	 * @param value Value to be updated
	 */
	public void setTimeTicksValue(String itemID, long value)
	{
		setMIBEntry(itemID, new SNMPTimeTicks(value), DT_TIMETICKS);
	}
	
	/**
	 * This operation is used to set the OCTET STRING value of an object 
	 * instance on an SNMP agent (device).
	 * 
	 * @param itemID String containing the OID of the data to access
	 * (you do not need to include the Enterprise ID).
	 * @param value Value to be updated
	 */
	public void setStringValue(String itemID, String value)
	{
		setMIBEntry(itemID, new SNMPOctetString(value), DT_OCTETSTRING);
	}
	
	/**
	 * This operation is used to set the IP ADDRESS value of an object instance 
	 * on an SNMP agent (device).
	 * 
	 * @param itemID String containing the OID of the data to access
	 * (you do not need to include the Enterprise ID).
	 * @param value Value to be updated
	 */
	public void setIPAddressValue(String itemID, String value)
	{
		try
		{
			setMIBEntry(itemID, new SNMPIPAddress(value), DT_IPADDRESS);
		}
		catch (SNMPBadValueException e)
		{
			error("Invalid IP address");
		}
	}
	
	/**
	 * This operation is used to set the MAC ADDRESS value of an object instance 
	 * on an SNMP agent (device).
	 * 
	 * @param itemID String containing the OID of the data to access
	 * (you do not need to include the Enterprise ID).
	 * @param value Value to be updated
	 */
	public void setMACAddressValue(String itemID, String value)
	{
		try
		{
			setMIBEntry(itemID, new SNMPNSAPAddress(value), DT_MACADDRESS);
		}
		catch (SNMPBadValueException e)
		{
			error("Invalid MAC address");
		}
	}
	
	/**
	 * This operation is used to set the value of an object instance on 
	 * an SNMP agent (device).
	 * 
	 * @param itemID Array of Strings containing the OID of the data to access
	 * (you do not need to include the Enterprise ID).
	 * @param value Array of values to be updated
	 * @param dataType Data type of the values to be updated
	 */
	public void setMIBEntry(String[] itemIDs, SNMPObject[] newValues, int dataType)
	{
		debug("setMIBEntry(" + itemIDs + "," + newValues + "," + dataTypeToString(dataType) + ")");
		if (m_bIsRunning)
		{
			error("Connection busy");
			return;
		}
		if (dataType <= NONE || dataType > DT_MIBTABLE)
		{
			error("Invalid return type");
			return;
		}
		
		SNMPObjectIdentifier[] oids = createOID(itemIDs);
		if (oids != null && itemIDs != null && newValues != null)
		{
			if (oids.length == itemIDs.length && newValues.length == itemIDs.length)
			{
				MIBMessage msg = new MIBMessage();
				
				msg.sIdentifier = itemIDs;
				msg.oid = oids;
				msg.value = newValues;
				msg.nOperation = OP_SET;
				msg.nDataType = dataType;
				msg.nOIDCount = 1;
				
				synchronized (connectLock) { m_msg = msg; }
				startThread();
			}
			else
			{
				error("One or more OIDs is not valid");
			}
		}
	}
	
	/**
	 * Validates the OID string passed in.
	 * 
	 * @param oid String containing the OID to validate.
	 * @return String containing the valid OID; emtpy String otherwise.
	 */
	public String validateOID(String oid)
	{
		String newOID = "";
		try
		{
			SNMPObjectIdentifier temp = new SNMPObjectIdentifier(oid);
			if (temp != null)
				newOID = temp.toString();
		}
		catch (SNMPBadValueException e)
		{
			error("Invalid OID: " + e.toString());
		}
		return newOID;
	}

	/**
	 * Converts an SNMPObjectIdentifier to a String object.
	 * 
	 * @param oid SNMPObjectIdentifier to be converted
	 * @return String
	 */
	public static String oidToString(SNMPObjectIdentifier oid)
	{
		if (oid != null)
			return oid.toString();
		return "";
	}
	
	/**
	 * Converts an array SNMPObjectIdentifier to an array of String objects.
	 * 
	 * @param oids SNMPObjectIdentifiers to be converted
	 * @return String[]
	 */
	public static String[] oidToString(SNMPObjectIdentifier[] oids)
	{
		String [] items = null;
		if (oids.length > 0)
		{
			items = new String[oids.length];
			for(int i = 0; i < oids.length; i++)
				items[i] = oidToString(oids[i]);
			
		} // if (names.length > 0)
		return items;
	}

	/*
	 * Convert a string into an SNMPObjectIdentifier object.
	 * 
	 * @param itemID
	 * @return
	 */
	private SNMPObjectIdentifier createOID(String itemID)
	{
		SNMPObjectIdentifier oid = null;
		String make = getEnterpriseID();
		try
		{
			if (make.length() > 0)
				oid = new SNMPObjectIdentifier(make + "." + itemID);
			else
				oid = new SNMPObjectIdentifier(itemID);
		}
		catch (SNMPBadValueException e)
		{
			error("Invalid OID - " + e.toString());
		}
		return oid;
	}
	
	/*
	 * Convert an array of strings into an array of SNMPObjectIdentifier objects.
	 * 
	 * @param names
	 * @return
	 */
	private SNMPObjectIdentifier[] createOID(String[] itemIDs)
	{
		SNMPObjectIdentifier[] oids = null;
		if (itemIDs.length > 0)
		{
			oids = new SNMPObjectIdentifier[itemIDs.length];
			for(int i = 0; i < itemIDs.length; i++)
				oids[i] = createOID(itemIDs[i]);
			
		} // if (names.length > 0)
		
		return oids;
	}
	
	/**
	 * Cleans up the connection.
	 */
	public void dispose()
	{
		debug("dispose()");
//		stopThread();
	}
	
	/**
	 * Checks to see if an operation is currently in progress.
	 * 
	 * @return TRUE if the connection is busy; FALSE otherwise.
	 */
	public boolean isBusy()
	{
		return m_bIsRunning;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#finalize()
	 */
	protected void finalize() throws Throwable
	{
		dispose();
	}

	/*
	 * Starts the communication thread.
	 */
	private void startThread()
	{
		RetrieveThread retrievalThread = new RetrieveThread(this);
		retrievalThread.start();
	}

	/*
	 *  (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	public void run()
	{
		if (m_bIsRunning)
			return;

		// Use this flag to make sure another thread isn't started until this
		// one is finished.
		m_bIsRunning = true;
		
		try
		{
			SNMPv1CommunicationInterface connection = new SNMPv1CommunicationInterface(m_version, m_hostAddress, m_community);

			if (m_msg != null)
			{
				SNMPVarBindList newVars = null;
				
				if (m_msg.nOIDCount == 1)
				{
					if (m_msg.oid[0] != null)
					{
			            // the getMIBEntry method of the communications interface returns an SNMPVarBindList
			            // object; this is essentially a Vector of SNMP (OID,value) pairs. In this case, the
			            // returned Vector has just one pair inside it.
						if (m_msg.nOperation == OP_GET)
						{
							debug("get MIB for " + m_msg);
							newVars = connection.getMIBEntry(m_msg.oid[0].toString());
						}
						else if (m_msg.nOperation == OP_GETNEXT)
						{
							debug("get next MIB for " + m_msg);
							newVars = connection.getNextMIBEntry(m_msg.oid[0].toString());
						}
						else if (m_msg.nOperation == OP_SET)
						{
							debug("set MIB for " + m_msg);
							newVars = connection.setMIBEntry(m_msg.oid[0].toString(), m_msg.value[0]);
						}
					}
				}
				else if (m_msg.nOIDCount > 1)
				{
					if (m_msg.sIdentifier != null)
					{
						String[] oids = oidToString(m_msg.oid);
						if (oids.length == m_msg.nOIDCount)
						{
							if (m_msg.nOperation == OP_GET)
							{
								debug("get MIB for " + m_msg);
								newVars = connection.getMIBEntry(oids);
							}
							else if (m_msg.nOperation == OP_GETNEXT)
							{
								debug("get next MIB for " + m_msg);
								newVars = connection.getNextMIBEntry(oids);
							}
							if (m_msg.nOperation == OP_SET)
							{
								debug("set MIB for " + m_msg);
								newVars = connection.setMIBEntry(oids, m_msg.value);
							}
						}
					}
				}
				
				if (newVars != null)
				{
					// Check return types here...
					debug("response = " + newVars);
					handleResponse(m_msg, newVars);
				}
				
			} // if (m_msg != null)
			
		}
		
		// Handled Exception Types
		catch (SocketException e)
		{
			error("SNMP: SocketException - " + e.toString());
			debug(m_msg.toString());
			if (getDebugState() == Logger.LOG_DEBUG)
				e.printStackTrace();
			if (m_listener != null)
				m_listener.handleSocketStatus(FAILED_CONNECT);
		}
		catch (SNMPGetException e)
		{
			error("SNMP: failed to get data - " + e.toString());
			debug(m_msg.toString());
			if (getDebugState() == Logger.LOG_DEBUG)
				e.printStackTrace();
			if (m_listener != null)
				m_listener.handleSocketStatus(FAILED_GET);
		}
		catch (SNMPSetException e)
		{
			error("SNMP: failed to set data - " + e.toString());
			debug(m_msg.toString());
			if (getDebugState() == Logger.LOG_DEBUG)
				e.printStackTrace();
			if (m_listener != null)
				m_listener.handleSocketStatus(FAILED_SET);
		}
		catch (SNMPBadValueException e)
		{
			error("Invalid OID: " + e.toString());
			debug(m_msg.toString());
			if (getDebugState() == Logger.LOG_DEBUG)
				e.printStackTrace();
			if (m_listener != null)
				m_listener.handleSocketStatus(INVALID_OID);
		}
		catch (Exception e)
		{
			error("run: failed - " + e.toString());
			debug(m_msg.toString());
			if (getDebugState() == Logger.LOG_DEBUG)
				e.printStackTrace();
			if (m_listener != null)
				m_listener.handleSocketStatus(UNKNOWN_ERROR);
		}
		
		m_bIsRunning = false;
	}

	/*
	 * Handle all of the responses for a single message.
	 * 
	 * @param msg
	 * @param response
	 */
	private void handleResponse(MIBMessage msg, SNMPVarBindList response)
	{
		try
		{
	    	if (msg != null)
			{
	    		if (response != null && response.size() > 0)
	    		{
	    			for (int i = 0; i < response.size(); i++)
	    				handleResponse(msg, (SNMPSequence)response.getSNMPObjectAt(i));
	    		}
	    		else
	    		{
	    			error("Invalid response");
	    		}
		        
			} // if (msg != null)
	    	else
	    	{
	    		error("Invalid message handler");
	    	}
		}
		catch (Exception e)
		{
			error("Exception in handleResponse(MIBMessage, SNMPVarBindList): " + e.toString());
			debug(msg.toString());
			if (getDebugState() == Logger.LOG_DEBUG)
				e.printStackTrace();
		}
		
	}
	
	/*
	 * Handle an individual response.
	 * 
	 * @param msg
	 * @param pair
	 */
	private void handleResponse(MIBMessage msg, SNMPSequence pair)
	{
		try
		{
	        if (pair != null && pair.size() == 2)
	        {
	            // extract the object identifier from the pair; it's the first element in the sequence
	            SNMPObjectIdentifier snmpOID = (SNMPObjectIdentifier)pair.getSNMPObjectAt(0);
	            if (snmpOID != null)
	            {
	        		int index = msg.indexOf(snmpOID);
	        		if (index >= 0)
	            	{
	            		// extract the corresponding value from the pair; it's the second element in the sequence
	            		SNMPObject snmpValue = pair.getSNMPObjectAt(1);
		                if (snmpValue != null)
		                {
	                		if (msg.sIdentifier != null && msg.sIdentifier[index] != null)
	                		{
	                			// Sending msg.oid[index], because we want to send the short  
	                			// version of the oid to use for matching responses
	                			m_listener.handleSNMPReceive(msg.oid[index], snmpValue);
	                		}
	                		else
	                		{
	                			error("Invalid message identifier");
	                		}
		                	
		                } // if (snmpValue != null)
		                else
		                {
		                	error("Missing response value");
		                }
		                
	            	} // if (index >= 0)
	        		else
	        		{
	        			error("Response does not match message");
	        		}
	            	
	            } // if (snmpOID != null)
	            else
	            {
	            	error("Missing response identifier");
	            }
	        	
	        } // if (pair != null && pair.size() == 2)
	        else
	        {
	        	error("Invalid response sequence");
	        }

		}
		catch (Exception e)
		{
			error("Exception in handleResponse(MIBMessage, SNMPSequence): " + e.toString());
			debug(msg.toString());
			if (getDebugState() == Logger.LOG_DEBUG)
				e.printStackTrace();
		}
        
	}

//	/*
//	 * 
//	 * @param str
//	 */
//	private void diag(String str)
//	{
//		System.out.println("SNMPConnection: " + str);
//	}

	/*
	 * 
	 * @param str
	 */
	private void debug(String str)
	{
		if (nDebug > Logger.LOG_WARNING)
			System.out.println("SNMPConnection: " + str);
	}
	
	/*
	 * 
	 * @param str
	 */
	private void error(String str)
	{
		System.out.println("ERROR - SNMPConnection: " + str);
	}
	
	/**
	 * @deprecated Do not use this method; use setDebugState instead
	 * @param bState
	 */
	public void setDebug(boolean bState)
	{
		if (bState)
			nDebug = Logger.LOG_DEBUG;
		else
			nDebug = Logger.LOG_ERROR;
	}
	
	/**
	 * Puts the class into a debug diagnostic state.
	 * 
	 * @param nState
	 */
	public void setDebugState(int nState)
	{
		nDebug = nState;
	}
	
	/**
	 * Gets the current diagnostic state.
	 * 
	 * @return
	 */
	public int getDebugState()
	{
		return nDebug;
	}
	
	/*
	 * 
	 *
	 */
	private class MIBMessage
	{
		public String [] sIdentifier = null;
		public SNMPObjectIdentifier [] oid = null;
		public SNMPObject[] value = null;
		public int nOIDCount = 0;
		public int nOperation = NONE;
		public int nDataType = NONE;

		public int indexOf(SNMPObjectIdentifier OID)
		{
			int index = Integer.MIN_VALUE;
			if (oid != null)
			{
				for (int i = 0; i < oid.length; i++)
				{
					if (oid[i] != null && oid[i].equals(OID))
					{
						index = i;
						break;
					}
				}
			}
			return index;
		}
		
		public String toString()
		{
			StringBuffer msg = new StringBuffer();
			msg.append("OPERATION: [" + operationToString(nOperation) + "]; DATA TYPE: [" + dataTypeToString(nDataType) + "]; ITEM COUNT: [" + nOIDCount + "]\n");
			for (int i = 0; i < nOIDCount; i++)
			{
				msg.append("ITEM[" + i + "] NAME: [");
				if (sIdentifier != null && sIdentifier[i] != null)
					msg.append(sIdentifier[i]);
				else
					msg.append("null");
				msg.append("]; OID: [");
				if (oid != null && oid[i] != null)
					msg.append(oid[i]);
				else
					msg.append("null");
				msg.append("]; VALUE: [");
				if (value != null && value[i] != null)
					msg.append(value[i]);
				else
					msg.append("null");
				msg.append("]\n");
			}
			int index = msg.toString().lastIndexOf('\n');
			msg.deleteCharAt(index);
			
			return msg.toString();
		}

	}
	
	/**
	 * Converts an operation type constant to it's equivalent string representation.
	 * 
	 * @param oper Operation type to be converted.
	 * @return String
	 */
	public static String operationToString(int oper)
	{
		String str = "";
		switch (oper)
		{
			case OP_GET:
				str = "GET";
				break;
			case OP_GETNEXT:
				str = "GET NEXT";
				break;
			case OP_SET:
				str = "SET";
				break;
			default:
				str = "UNKNOWN";
				break;
		}
		return str;
	}
	
	/**
	 * Converts a data type constant to it's equivalent string representation.
	 * 
	 * @param dataType Data type to be converted.
	 * @return String
	 */
	public static String dataTypeToString(int dataType)
	{
		String str = "";
		switch (dataType)
		{
			case DT_INTEGER:
				str = "INTEGER";
				break;
			case DT_UINTEGER32:
				str = "UINTEGER32";
				break;
			case DT_GAUGE:
				str = "GAUGE";
				break;
			case DT_COUNTER32:
				str = "COUNTER32";
				break;
			case DT_COUNTER64:
				str = "COUNTER64";
				break;
			case DT_TIMETICKS:
				str = "TIMETICKS";
				break;
			case DT_OCTETSTRING:
				str = "OCTETSTRING";
				break;
			case DT_IPADDRESS:
				str = "IPADDRESS";
				break;
			case DT_MACADDRESS:
				str = "MACADDRESS";
				break;
			case DT_OBJECTID:
				str = "OBJECTID";
				break;
			case DT_MIBTABLE:
				str = "MIBTABLE";
				break;
			default:
				str = "UNKNOWN";
				break;
		}
		return str;
	}

	/**
	 * Converts a data type string representation to its equivalent contant value.
	 * 
	 * @param str String to be converted.
	 * @return int
	 */
	public static int stringToDataType(String str)
	{
		int dataType = Integer.MIN_VALUE;
		if (str.equalsIgnoreCase("INTEGER"))
			dataType = DT_INTEGER;
		else if (str.equalsIgnoreCase("UINTEGER32"))
			dataType = DT_UINTEGER32;
		else if (str.equalsIgnoreCase("GAUGE"))
			dataType = DT_GAUGE;
		else if (str.equalsIgnoreCase("COUNTER32"))
			dataType = DT_COUNTER32;
		else if (str.equalsIgnoreCase("COUNTER64"))
			dataType = DT_COUNTER64;
		else if (str.equalsIgnoreCase("TIMETICKS"))
			dataType = DT_TIMETICKS;
		else if (str.equalsIgnoreCase("OCTETSTRING"))
			dataType = DT_OCTETSTRING;
		else if (str.equalsIgnoreCase("IPADDRESS"))
			dataType = DT_IPADDRESS;
		else if (str.equalsIgnoreCase("MACADDRESS"))
			dataType = DT_MACADDRESS;
		else if (str.equalsIgnoreCase("OBJECTID"))
			dataType = DT_OBJECTID;
		else if (str.equalsIgnoreCase("MIBTABLE"))
			dataType = DT_MIBTABLE;
		return dataType;
	}
	
}
