/*
 * SocketConnection.java
 *
 * Copyright  2004 AMX Corporation.  All rights reserved.  
 *
 * PROPRIETARY
 *
 * Use is subject to license terms.
 *
 */
package com.amx.duet.tools.net.tcp;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.InterruptedIOException;
import java.net.InetAddress;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.LinkedList;

import com.amx.duet.tools.lang.ClassName;
import com.amx.duet.tools.lang.Logger;

/**
 * This class is used to manage TCP/IP socket connections on a single thread making it 
 * safe for AMX masters.
 *  
 * @version 2.0.6
 * @since AMXTools 1.0.1
 */
public class SocketConnection extends Thread
{
	public static final int CONNECTED = 1;
	public static final int TIMEOUT = -1;
	public static final int CLOSED = -2;
	public static final int INVALID_HOST = -3;
	public static final int FAILED_CONNECT = -4;
	
	public static final int INFINITE = 0;
	
	public static final String className = ClassName.getClassName(SocketConnection.class);
	
	private Socket m_socket = null;
	private boolean m_connected = false;
	private InetAddress m_addr = null;
	private String m_addrStr = null;
	private int m_port = 0;
	private SocketConnectionListener m_listener = null;
	private int m_timeout = INFINITE;
	private int m_bufferSize = 1024;
	private boolean m_TcpNoDelayOn = false;
	private byte[] m_readBuff;
	
	private BufferedInputStream m_inStream;
	private BufferedOutputStream m_outStream;
	
	private boolean m_isRunning = false;
	private boolean m_doConnect = false;
	private Object connectLock = new Object();
	
	private boolean m_continueRunning = true;
	
	private Object writeSync = new Object();
	
	private boolean m_okToSend = true;
	
	private LinkedList m_queue;

	private int nDebug = Logger.LOG_ERROR;
	
	public void initConnection(String addrStr, int port, int bufferSize, SocketConnectionListener listener, 
			  int timeout, boolean TcpNoDelayOn)
	{
		if(this.getDebugState()>=Logger.LOG_DEBUG)
			log(Logger.LOG_DEBUG, "initConnection(" + addrStr + ", " + port + ", " + timeout + ")");
		if (listener == null)
		{
			log(Logger.LOG_ERROR, "Invalid Listener");
			return;
		}
		
		if (addrStr == null)
		{
			log(Logger.LOG_ERROR, "Invalid Host address");
			return;
		}
		
		m_queue = new LinkedList();
		m_addr = null;
		m_addrStr = addrStr;
		m_port = port;
		m_listener = listener;
		m_timeout = Math.max(0,timeout);
		m_bufferSize = bufferSize;
		m_TcpNoDelayOn = TcpNoDelayOn;
		// go ahead and start the connect/receive thread
		this.start();
	}
	public void initConnection(String addrStr, int port, int bufferSize, SocketConnectionListener listener, 
										  int timeout)
	{
		this.initConnection(addrStr,port,bufferSize,listener,timeout,false);
	}
	
	private SocketConnection(){}
	
	/**
	 * Creates a simple socket connection along with a receive thread for reading the socket.
	 * When data is received on the socket, <i>listener</i> will be called.  The socket receive
	 * has no initial timeout value (INFINITE).
	 * 
	 * @param addr InetAddress object indicating the host to connect to
	 * @param port integer port number to connect to
	 * @param bufferSize integer size of maximum receive buffer
	 * @param listener SocketReceiveHandler object that will be called when data is received on a
	 * 		  sockect or when an error occurs.
	 */
	public SocketConnection(InetAddress addr, int port, int bufferSize, SocketConnectionListener listener)
	{
		if(this.getDebugState()>=Logger.LOG_DEBUG)
			log(Logger.LOG_DEBUG, "SocketConnection(" + addr + ", " + port + ")");
		initConnection(addr.getHostAddress(),port,bufferSize,listener,INFINITE);
	}
	
	/**
	 * Creates a simple socket along with a receive thread for reading the socket.
	 * When data is received on the socket, <i>listener</i> will be called.  The socket receive
	 * has no initial timeout value (INFINITE).
	 * 
	 * @param host String name of host to connect to
	 * @param port port number to connect to
	 * @param bufferSize integer size of maximum receive buffer
	 * @param listener SocketReceiveHandler object that will be called when data is received on a 
	 * 		  sockect or when an error occurs.
	 */
	public SocketConnection(String host, int port, int bufferSize, SocketConnectionListener listener)
	{
		if(this.getDebugState()>=Logger.LOG_DEBUG)
			log(Logger.LOG_DEBUG, "SocketConnection(" + host + ", " + port + ")");
		initConnection(host,port,bufferSize,listener,INFINITE);
	}
	
	/**
	 *  Creates a simple socket along with a receive thread for reading the socket.
	 * When data is received on the socket, <i>listener</i> will be called.  The socket recieve
	 * has the specified timeout.  If a timeout occurs, the ConnectionListener.handleSocketStatus
	 *  will be called with a status of TIMEOUT.
	 * 
	 * @param addr host String name of host to connect to
	 * @param port port number to connect to
	 * @param bufferSize integer size of maximum receive buffer
	 * @param listener SocketReceiveHandler object that will be called when data is received on a
	 * 		  sockect or when an error occurs.
	 * @param timeout number of milliseconds to wait for received data
	 */
	public SocketConnection(InetAddress addr, int port, int bufferSize, SocketConnectionListener listener, int timeout)
	{
		if(this.getDebugState()>=Logger.LOG_DEBUG)
			log(Logger.LOG_DEBUG, "SocketConnection(" + addr + ", " + port + ", " + timeout + ")");
		initConnection(addr.getHostAddress(),port,bufferSize,listener,timeout);
	}
		
	/**
	 * Creates a simple socket along with a receive thread for reading the socket.
	 * When data is received on the socket, <i>listener</i> will be called.  The socket recieve
	 * has the specified timeout.   If a timeout occurs, the ConnectionListener.handleSocketStatus
	 *  will be called with a status of TIMEOUT.
	 * 
	 * @param host String name of host to connect to
	 * @param port port number to connect to
	 * @param bufferSize integer size of maximum receive buffer
	 * @param listener SocketReceiveHandler object that will be called when data is received on a 
	 * 		  sockect or when an error occurs.
	 * @param timeout number of milliseconds to wait for received data
	 */
	public SocketConnection(String host, int port, int bufferSize, SocketConnectionListener listener, int timeout)
	{
		if(this.getDebugState()>=Logger.LOG_DEBUG)
			log(Logger.LOG_DEBUG, "SocketConnection(" + host + ", " + port + ", " + timeout + ")");
		initConnection(host,port,bufferSize,listener,timeout);
	}
	
	/**
	 *  Creates a simple socket along with a receive thread for reading the socket.
	 * When data is received on the socket, <i>listener</i> will be called.  The socket recieve
	 * has the specified timeout.  If a timeout occurs, the ConnectionListener.handleSocketStatus
	 *  will be called with a status of TIMEOUT.
	 * 
	 * @param addr host String name of host to connect to
	 * @param port port number to connect to
	 * @param bufferSize integer size of maximum receive buffer
	 * @param listener SocketReceiveHandler object that will be called when data is received on a
	 * 		  sockect or when an error occurs.
	 * @param timeout number of milliseconds to wait for received data
	 * @param TcpNoDelayOn Enable/disable TCP_NODELAY (disable/enable Nagle's algorithm)
	 */
	public SocketConnection(InetAddress addr, int port, int bufferSize, SocketConnectionListener listener, int timeout, boolean TcpNoDelayOn)
	{
		if(this.getDebugState()>=Logger.LOG_DEBUG)
			log(Logger.LOG_DEBUG, "SocketConnection(" + addr + ", " + port + ", " + timeout + ", "+TcpNoDelayOn + ")");
		initConnection(addr.getHostAddress(),port,bufferSize,listener,timeout,TcpNoDelayOn);
	}
		
	/**
	 * Creates a simple socket along with a receive thread for reading the socket.
	 * When data is received on the socket, <i>listener</i> will be called.  The socket recieve
	 * has the specified timeout.   If a timeout occurs, the ConnectionListener.handleSocketStatus
	 *  will be called with a status of TIMEOUT.
	 * 
	 * @param host String name of host to connect to
	 * @param port port number to connect to
	 * @param bufferSize integer size of maximum receive buffer
	 * @param listener SocketReceiveHandler object that will be called when data is received on a 
	 * 		  sockect or when an error occurs.
	 * @param timeout number of milliseconds to wait for received data
	 * @param TcpNoDelayOn Enable/disable TCP_NODELAY (disable/enable Nagle's algorithm)
	 */
	public SocketConnection(String host, int port, int bufferSize, SocketConnectionListener listener, int timeout, boolean TcpNoDelayOn)
	{
		if(this.getDebugState()>=Logger.LOG_DEBUG)
			log(Logger.LOG_DEBUG, "SocketConnection(" + host + ", " + port + ", " + timeout + ", "+TcpNoDelayOn + ")");
		initConnection(host,port,bufferSize,listener,timeout,TcpNoDelayOn);
	}
	
	/**
	 * Sets the socket connections IP address and port to the specified values.  The new values
	 * will not take affect until a connection is created.  If a connection already exists, it 
	 * must be disconnected and re-connected with the new values.
	 * @param host String containing the new host name to connect to.
	 * @param port integer value containing the new port number to connect to.
	 * @return <i>true</i> if successful set, <i>false</i> if set failed due to invalid host.
	 */
	public boolean setAddressAndPort(String host, int port)
	{
		if(this.getDebugState()>=Logger.LOG_DEBUG)
			log(Logger.LOG_DEBUG, "setAddressAndPort(" + host + ", " + port + ")");
		m_addr = null;
		m_addrStr = host;
		m_port = port;
		return true;
	}
	
	/**
	 * Sets the socket connections IP address and port to the specified values.  The new values
	 * will not take affect until a connection is created.  If a connection already exists, it 
	 * must be disconnected and re-connected with the new values.
	 * @param addr <b>InetAddress</b> object containing the new IP address
	 * @param port integer value containing the new port.
	 * @return ,i>true</i> if set was successful, else <i>false</i>
	 */
	public boolean setAddressAndPort(InetAddress addr, int port)
	{
		if(this.getDebugState()>=Logger.LOG_DEBUG)
			log(Logger.LOG_DEBUG, "setAddressAndPort(" + addr + ", " + port + ")");
		setAddressAndPort(addr.getHostAddress(),port);
		return true;
	}
	
	/**
	 * Sets the socket connection's receive timeout in milliseconds. The new timeout will take
	 * affect upon the next receive iteration.
	 * Use 0 for NO_TIMEOUT.
	 * If a negative timeout is specified, infinite is assumed.
	 * @param timeout [0..n]
	 */
	public void setTimeout(int timeout)
	{
		if(this.getDebugState()>=Logger.LOG_DEBUG)
			log(Logger.LOG_DEBUG, "setTimeout(" + timeout + ")");
		m_timeout = Math.max(0,timeout);
	}
	
	/**
	 * The first time <i>connect</i> is called for a <b>SocketConnection</b>, the receive
	 * thread is started and a <b>socket</b> object is created connected to the address and port.
	 * The connection is made asynchronously so the connect call will return immediately.  A
	 * call into the listeners <i>handleSocketStatus</i> routine with a status of <b>CONNECTED</b>
	 * will occur when the socket has been created.
	 * @return true if the connection was initiated, false if the address is invalid
	 */
	public boolean connect()
	{
		if(this.getDebugState()>=Logger.LOG_DEBUG)
			log(Logger.LOG_DEBUG, "connect()");
		if (m_addrStr == null) return false;
		
		synchronized (connectLock)
		{
			// check for existing connection or pending connection
			if (m_connected || m_doConnect)  return true;
		
			// trigger the flag so that the receive thread will execute the connection asynchronously
			m_doConnect = true;
			
			connectLock.notify();
		}
		
		return true;
	}
	
	/**
	 * Closes the socket.  The receive thread remains running in an idle state waiting
	 * for <i>connect</i> to be called.
	 *
	 */
	public void disconnect()
	{	
		if(this.getDebugState()>=Logger.LOG_DEBUG)
			log(Logger.LOG_DEBUG, "disconnect()");
		if (m_socket != null)
		{
			try
			{
				
				m_inStream = null;
				m_outStream = null;
				m_socket.close();
				m_socket = null;
				 
			} 
			catch (IOException e)
			{
				log(Logger.LOG_ERROR, "disconnect(): close failure");
				if (getDebugState() == Logger.LOG_DEBUG)
					e.printStackTrace();
			}
		}
	}
	
	public boolean isConnected()
	{
//		log(Logger.LOG_DEBUG, "isConnected(" + (m_socket != null && m_connected) + ")");
		return (m_socket != null && m_connected);
	}

	/**
	 * Destroys the socket connection and terminates the receive thread.
	 *
	 */
	public void dispose()
	{
		if(this.getDebugState()>=Logger.LOG_DEBUG)
			log(Logger.LOG_DEBUG, "dispose()");
		m_continueRunning = false;
		
		if (m_connected)
			disconnect();
			
		// interrupt the receive thread in case it is in the middle of a wait.
		if (m_isRunning) 
		{
			this.interrupt();
		
			try
			{
				this.join(1000);
			}
			catch (InterruptedException e)
			{
				log(Logger.LOG_ERROR, "dispose(): failed to terminate socket thread");
			}
		}
	}
	
	public void finalize()
	{
		if(this.getDebugState()>=Logger.LOG_DEBUG)
			log(Logger.LOG_DEBUG, "finalize()");
		dispose();
	}
	
	private void createSocket()
	{
		if(this.getDebugState()>=Logger.LOG_DEBUG)
			log(Logger.LOG_DEBUG, "createSocket: creating socket");
		resetDoConnect();
		try
		{
			if(this.getDebugState()>=Logger.LOG_DEBUG)
				log(Logger.LOG_DEBUG, "createSocket(): initializing...");
			m_socket = new Socket(m_addr, m_port);
			
			if (m_timeout != INFINITE)
				m_socket.setSoTimeout(m_timeout); 
			m_socket.setTcpNoDelay(m_TcpNoDelayOn);		
			m_inStream = new BufferedInputStream(m_socket.getInputStream());
			m_outStream = new BufferedOutputStream(m_socket.getOutputStream());
			
			m_connected = true;
			if(this.getDebugState()>=Logger.LOG_DEBUG)
				log(Logger.LOG_DEBUG, "createSocket: socket created");
			
			// notify the listener that the connection completed.
			if (m_listener != null) m_listener.handleSocketStatus(CONNECTED);
				
		}
		catch (Exception e)
		{
			// failed to create socket, call listener with failed status
			try
			{
				m_outStream = null;
				m_inStream = null;
				if (m_socket != null) m_socket.close();
				m_socket = null;
			}
			catch (IOException ex)
			{
				m_socket = null;
				if (getDebugState() == Logger.LOG_DEBUG)
					ex.printStackTrace();
			}
			
			log(Logger.LOG_ERROR, "createSocket: socket create failed");
			
			// notify the listener that the connection failed.
			if (m_listener != null) m_listener.handleSocketStatus(FAILED_CONNECT);
		}
	}
	
	/**
	 * Writes data to the output stream of the Socket connection.  
	 * @param data byte array containing data to write
	 * @return <b>true</b> if write was successful, else <b>false</b>
	 */
	public boolean write(byte [] data)
	{
//		log(Logger.LOG_DEBUG, "write()");
		try
		{
			if (m_outStream != null)
			{ 
				synchronized (writeSync)
				{
					m_outStream.write(data);
					m_outStream.flush(); 
					return true;
				}
			}
			
			log(Logger.LOG_ERROR, "write: Invalid output stream");
		}
		catch (IOException e)
		{
			log(Logger.LOG_ERROR, "write: IOException during socket write");
			if (getDebugState() == Logger.LOG_DEBUG)
				e.printStackTrace();
		}
		
		return false;
	}
	
	/**
	 * Write data to the <b>SocketConnection</b>'s output stream if output is OK, otherwise
	 * the output is queued awaiting send until OK-to-send is triggered via <i>setOKtoSendTrue()</i>
	 * @param data byte array containing data to send.
	 * @return <i>true</i> if write or queuing was successful, else <i>false</i>
	 */
	public boolean writeQueued(byte [] data)
	{
//		log(Logger.LOG_DEBUG, "writeQueued()");
		synchronized (m_queue)
		{
			if ((m_queue.size() == 0) && m_okToSend && (m_outStream != null))
			{
				if(this.getDebugState()>=Logger.LOG_DEBUG)
					log(Logger.LOG_DEBUG, "writeQueued(): no queue, sending msg");
				// nothing in the queue and we're OK to send
				m_okToSend = false;
				return write(data);
			}

			if(this.getDebugState()>=Logger.LOG_DEBUG)
				log(Logger.LOG_DEBUG, "writeQueued(): queueing message");
			// queue up the message
			byte[] copy = new byte[data.length];
			System.arraycopy(data,0,copy,0,data.length);
			m_queue.add(copy);
			return true;
		}
	}
	
	/**
	 * Flushes any message awaiting send by either sending them to the <b>SocketConnection</b>'s
	 * output stream or discarding the message based on the <i>send</i> argument.
	 * @param send boolean flag indicating if the queued messages should be send (<i>true</i>) or
	 * discarded (<i>false</i>)
	 */
	public void flushQueue(boolean send)
	{
//		log(Logger.LOG_DEBUG, "flushQueue()");
		synchronized (m_queue)
		{
			if (send)
			{
				byte[] data;
				while (m_queue.size() > 0)
				{
					data = (byte [])m_queue.removeFirst();
					write(data);
				}
			}
			else
				m_queue.clear();
		}
	}
	
	/**
	 * Triggers the <b>SocketConnection</b>'s thread that it is OK to send the next queued message.
	 *
	 */
	public void setOKtoSendTrue()
	{
//		log(Logger.LOG_DEBUG, "setOKtoSendTrue()");
		m_okToSend = true;
	}
	
	private void resetDoConnect()
	{
//		log(Logger.LOG_DEBUG, "resetDoConnect()");
		synchronized (connectLock)
		{
			m_doConnect = false;
		}
	}
	
	public void run()
	{
//		log(Logger.LOG_DEBUG, "run()");
		m_readBuff = new byte[m_bufferSize];
		m_isRunning = true;
		
		while (m_continueRunning)
		{
			if (m_doConnect)
			{
				if (m_addrStr == null)
				{
					log(Logger.LOG_ERROR, "no host for connection");
					m_addr = null;
					resetDoConnect();
				}
				
				if (m_addr == null)
				{
					try
					{
						if(this.getDebugState()>=Logger.LOG_DEBUG)
							log(Logger.LOG_DEBUG, "run(): Getting host address...");
						m_addr = InetAddress.getByName(m_addrStr);
					}
					catch (UnknownHostException e)
					{
						log(Logger.LOG_ERROR, "could not resolve host: "+m_addrStr);
						resetDoConnect();
					}
				}
				
				if (m_addr != null)
				{
					if(this.getDebugState()>=Logger.LOG_DEBUG)
						log(Logger.LOG_DEBUG, "run(): Creating socket...");
					createSocket();
					m_addr = null;
				}
			} 
			
			if (m_connected)
			{
				// receive data
				try
				{
					int bytesRead = 0;
					if (m_inStream != null)
					{
						if(m_socket.getSoTimeout()!=m_timeout)
							m_socket.setSoTimeout(m_timeout);
						bytesRead = m_inStream.read(m_readBuff);
					}
//					log(Logger.LOG_DEBUG, "Bytes received[" + bytesRead + "]: " + new String(m_readBuff, 0, bytesRead));
					if ((bytesRead > 0) && (m_listener != null))
					{
						m_listener.handleReceive(bytesRead,m_readBuff);
					} 
					else if(bytesRead <= 0)
					{
						if(this.getDebugState()>=Logger.LOG_DEBUG)
							log(Logger.LOG_DEBUG, "run(): Closing socket...");
						// server has closed the socket
						m_connected = false;
						disconnect(); // close out our socket  object
						flushQueue(false); // clear out any unsent messages
						if (m_listener != null) m_listener.handleSocketStatus(CLOSED);
					}
					
				}
				catch (InterruptedIOException e)
				{
					if(this.getDebugState()>=Logger.LOG_DEBUG)
						log(Logger.LOG_DEBUG, "run(): InterruptedIOException - " + e.toString());
					m_connected = false;
					disconnect(); // close out our socket  object
					// notify the listener that a timeout occurred on the receive
					if (m_listener != null) m_listener.handleSocketStatus(TIMEOUT);
				}
				catch (Exception e)
				{
					if(this.getDebugState()>=Logger.LOG_DEBUG)
						log(Logger.LOG_DEBUG, "run(): Exception - " + e.toString());
//					log(Logger.LOG_ERROR, "run: failed data read");
					if (getDebugState() == Logger.LOG_DEBUG)
						e.printStackTrace();
					if(this.getDebugState()>=Logger.LOG_DEBUG)
						log(Logger.LOG_DEBUG, "run(): Closing socket...");
					// Bad Exception, Close the socket!
					m_connected = false;
					disconnect(); // close out our socket  object
					flushQueue(false); // clear out any unsent messages
					if (m_listener != null) m_listener.handleSocketStatus(CLOSED);
				}
				
				// check for pending queued message send
				synchronized (m_queue)
				{
					if ((m_queue.size()>0) && m_okToSend)
					{						
						//log(Logger.LOG_DEBUG, "run() sending queued message");
						byte[] data;
						
						m_okToSend = false;
						data = (byte [])m_queue.removeFirst();
						write(data);
					}
				}
			}
			else
			{
				// wait to be notified to connect
				synchronized (connectLock)
				{
					try
					{
						connectLock.wait(5000);
					}
					catch (InterruptedException e)
					{
						if(this.getDebugState()>=Logger.LOG_DEBUG)
							log(Logger.LOG_DEBUG, "run(): InterruptedException - " + e.toString());
					}
				}
			}

		}
		m_isRunning = false;
	}

	/**
	 * @deprecated Do not use this method; use setDebugState instead
	 * @param bState
	 */
	public void setDebug(boolean bState)
	{
		if (bState)
			nDebug = Logger.LOG_DEBUG;
		else
			nDebug = Logger.LOG_ERROR;
	}
	
	/**
	 * Puts the class into a debug diagnostic state.
	 * 
	 * @param nState
	 */
	public void setDebugState(int nState)
	{
		nDebug = nState;
	}
	
	/**
	 * Gets the current diagnostic state.
	 * 
	 * @return
	 */
	public int getDebugState()
	{
		return nDebug;
	}

	/**
	 * 
	 * @param level
	 * @param msg
	 */
	public void log(int level, String msg)
	{
		if (getDebugState() >= level && level == Logger.LOG_ERROR)
			Logger.error(className, "[" + m_addr + "]" + msg); 
		else if (getDebugState() >= level) 
			Logger.diag(className, "[" + m_addr + "]" + msg);
	}
}
