/*
 * SocketConnection.java
 *
 * Copyright  2004 AMX Corporation.  All rights reserved.  
 *
 * PROPRIETARY
 *
 * Use is subject to license terms.
 *
 */
package com.amx.duet.tools.net.udp;
//#####################################################################//
//# Copyright Notice :                                                #//
//#    Copyright, AMX, Inc., 2008                                     #//
//#    Private, proprietary information, the sole property of AMX.    #//
//#    The contents, ideas, and concepts expressed herein are not to  #//
//#    be disclosed except within the confines of a confidential      #//
//#    relationship and only then on a need to know basis.            #//
//#####################################################################//
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketTimeoutException;
import java.net.UnknownHostException;

import com.amx.duet.tools.lang.ClassName;
import com.amx.duet.tools.lang.Logger;

/**
 * This class is used to manage datagram socket connections on a single thread making it 
 * safe for AMX masters.
 *  
 * @author CNina
 * @version 2.0.5
 */
public class DatagramSocketConnection extends Thread
{
	public static final int INVALID_HOST = -3;
	public static final int FAILED_CREATE_DATAGRAM_SOCKET = -4;
	public static final int FAILED_DATAGRAM_WRITE = -2;
	public static final int FAILED_DATAGRAM_READ= -1;
	public static final int TIMEOUT = -5;
	public static final int INFINITE = 0;
	
	private DatagramSocket m_udpsocket = null;
	private InetAddress m_addr = null;
	private int m_port = 0;
	private ISocketConnectionListener m_listener = null;
	private int m_timeout = INFINITE;
	private int m_bufferSize = 4096; // 4k buffer
	private int nDebug = 0;
	private final String className = ClassName.getClassName(DatagramSocketConnection.class);
	
	private byte[] m_readBuff;
	
	private boolean m_isRunning = false;
	private boolean m_continueRunning = true;
	
	private Object writeSync = new Object();
	
		
	protected void initConnection(InetAddress addr, int port, int bufferSize, ISocketConnectionListener listener, 
										  int timeout)
	{
		if (listener == null)
		{
			return;
		}
		
		m_addr = addr;
		m_port = port;
		m_listener = listener;
		m_timeout = Math.max(0,timeout);
		m_bufferSize = bufferSize;
		if(createSocket())
		{
			// go ahead and start the receive thread if not started already
			if(m_isRunning==false)
				this.start();
		}
	}
	
	/**
	 * Creates a simple socket connection along with a receive thread for reading the socket.
	 * When data is received on the socket, <i>listener</i> will be called.  The socket receive
	 * has no initial timeout value (INFINITE).
	 * 
	 * @param addr InetAddress object indicating the host to connect to
	 * @param port integer port number to connect to
	 * @param bufferSize integer size of maximum receive buffer
	 * @param listener SocketReceiveHandler object that will be called when data is received on a
	 * 		  sockect or when an error occurs.
	 */
	public DatagramSocketConnection(InetAddress addr, int port, int bufferSize, ISocketConnectionListener listener)
	{
		initConnection(addr,port,bufferSize,listener,INFINITE);
	}
	
	/**
	 * Creates a simple socket along with a receive thread for reading the socket.
	 * When data is received on the socket, <i>listener</i> will be called.  The socket receive
	 * has no initial timeout value (INFINITE).
	 * 
	 * @param host String name of host to connect to
	 * @param port port number to connect to
	 * @param bufferSize integer size of maximum receive buffer
	 * @param listener SocketReceiveHandler object that will be called when data is received on a 
	 * 		  sockect or when an error occurs.
	 */
	public DatagramSocketConnection(String host, int port, int bufferSize, ISocketConnectionListener listener)
	{
		try
		{
			initConnection(InetAddress.getByName(host),port,bufferSize,listener,INFINITE);
		}
		catch (UnknownHostException e)
		{
			log("ERROR: SocketConnection() UnknownHost "+e.getMessage());
		}
	}
	
	/**
	 *  Creates a simple socket along with a receive thread for reading the socket.
	 * When data is received on the socket, <i>listener</i> will be called.  The socket receive
	 * has the specified timeout. If a timeout occurs, the ISocketConnectionListener.handleSocketStatus
	 * will be called with a status of TIMEOUT. 
	 * 
	 * @param addr host String name of host to connect to
	 * @param port port number to connect to
	 * @param bufferSize integer size of maximum receive buffer
	 * @param listener SocketReceiveHandler object that will be called when data is received on a
	 * 		  sockect or when an error occurs.
	 * @param timeout number of milliseconds to wait for received data
	 */
	public DatagramSocketConnection(InetAddress addr, int port, int bufferSize, ISocketConnectionListener listener, int timeout)
	{
		initConnection(addr,port,bufferSize,listener,timeout);
	}
	
	/**
	 * Creates a simple socket along with a receive thread for reading the socket.
	 * When data is received on the socket, <i>listener</i> will be called.  The socket receive
	 * has no initial timeout value (INFINITE). If a timeout occurs, the ISocketConnectionListener.handleSocketStatus
	 * will be called with a status of TIMEOUT.
	 * 
	 * @param host String name of host to connect to
	 * @param port port number to connect to
	 * @param bufferSize integer size of maximum receive buffer
	 * @param listener SocketReceiveHandler object that will be called when data is received on a 
	 * 		  sockect or when an error occurs.
	 * @param timeout number of milliseconds to wait for received data
	 */
	public DatagramSocketConnection(String host, int port, int bufferSize, ISocketConnectionListener listener, int timeout)
	{
		try
		{
			initConnection(InetAddress.getByName(host),port,bufferSize,listener,timeout);
		}
		catch (UnknownHostException e)
		{
			log("ERROR: SocketConnection() UnknownHost "+e.getMessage());
		}
	}
	/**
	 * Sets the IP address and Port to be used with a datagram packet.
	 * @param host
	 * @param port
	 */
	public void setAddressAndPort(String host, int port)
	{
		try
		{
			m_addr = InetAddress.getByName(host);
			m_port = port;
		}
		catch (UnknownHostException e)
		{
			log("ERROR: setAddressAndPort() UnknownHost "+e.getMessage());
		}
	}
	/**
	 * Closes this socket connection and sets it to null.
	 * This method will also terminate the receive thread and release all resources.
	 */
	public void dispose()
	{
		m_continueRunning = false;
		
		this.close();
		
			
		// interrupt the receive thread in case it is in the middle of a wait.
		if (m_isRunning) 
		{
			this.interrupt();
		
			try
			{
				this.join(1000);
			}
			catch (InterruptedException e)
			{
			}
		}
	}
	/**
	 * Closes this socket and sets it to null.
	 *
	 */
	protected void close()
	{
		if (m_udpsocket != null)
		{
			m_udpsocket.close();
			m_udpsocket = null;
		}
	}
	
	private boolean createSocket()
	{
		try
		{
			if(m_udpsocket==null)
			{
				m_udpsocket = new DatagramSocket();
			}
			else 
			{
				m_udpsocket.disconnect();
			}
			m_udpsocket.setSoTimeout(m_timeout);
		}
		catch (Exception e)
		{
			// failed to create socket, call listener with failed status
			m_udpsocket = null;
			// notify the listener that the connection failed.
			if (m_listener != null) m_listener.handleSocketStatus(FAILED_CREATE_DATAGRAM_SOCKET);
			return false;
		}
		return true;
	}
	
	/**
	 * Writes data to the datagram Socket.  
	 * @param data byte array containing data to write
	 * @return <b>true</b> if write was successful, else <b>false</b>
	 */
	public boolean write(byte [] data)
	{
		try
		{
			if (m_udpsocket != null)
			{ 
				synchronized (writeSync)
				{
					DatagramPacket dp = new DatagramPacket(data,data.length,m_addr,m_port);
					m_udpsocket.send(dp);
					return true;
				}
			}
			
		}
		catch (Exception e)
		{
			if (m_listener != null) m_listener.handleSocketStatus(FAILED_DATAGRAM_WRITE);
		}
		
		return false;
	}
	
	public void run()
	{
		m_readBuff = new byte[m_bufferSize];
		m_isRunning = true;
		while (m_continueRunning)
		{
			// receive data
			try
			{
				DatagramPacket incoming = new DatagramPacket(m_readBuff, m_bufferSize);
				m_udpsocket.receive(incoming);
				if(incoming.getAddress().equals(m_addr) && incoming.getPort()==m_port)
				{ // make sure I don't get UDP packets from somewhere else...
					byte[] data = incoming.getData();
					int length = incoming.getLength();
					if ((length > 0) && (m_listener != null))
					{
						m_listener.handleReceive(length,data);
					} 
				}
			}
			catch(SocketTimeoutException ste)
			{
				if (m_listener != null && m_continueRunning==true) 
					m_listener.handleSocketStatus(TIMEOUT);
			}
			catch (Exception e)
			{
				if (m_listener != null && m_continueRunning==true) 
					m_listener.handleSocketStatus(FAILED_DATAGRAM_READ);
			}
				
		}
		m_isRunning = false;
	}
	
	/**
	 * Puts the class into a debug diagnostic state.
	 * 
	 * @param nState
	 */
	public void setDebugState(int nState)
	{
		nDebug = nState;
	}
	
	/**
	 * Gets the current diagnostic state.
	 * 
	 * @return
	 */
	public int getDebugState()
	{
		return nDebug;
	}

	/**
	 * @param msg
	 */
	private void log(String msg)
	{
		Logger.diag(className, "[" + m_addr + "]" + msg);
	}
}
