/*
 * Numeric.java
 * 
 * Created on Jul 7, 2006
 * Created by VGonzales
 *
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.text;

import com.amx.duet.tools.lang.ObjectRef;
import com.amx.duet.tools.lang.ValidateUtil;

/**
 * A utility class commonly used for safely converting String values to numeric 
 * values and numeric values to String values.
 * 
 * @version 1.0.0
 */
public class NumericUtil
{
	/**
	 * Check to see if the character passed in is a numeric value.
	 * 
	 * @param cValue character value to check.
	 * @return true if this a numeric value; false otherwise.
	 */
	public final static boolean isNumeric(char cValue)
	{
		if (cValue >= '0' && cValue <= '9')
			return true;
		return false;
	}

	/**
	 * Check to see if the string passed in contains all numeric values.
	 * 
	 * @param sValue String value to check.
	 * @return true if this is a numeric string; false otherwise.
	 */
	public final static boolean isNumeric(String sValue)
	{
		if (sValue != null && sValue.length() > 0)
		{
			for (int i = 0; i < sValue.length(); i++)
			{
				// The first non numeric exits the method
				if (!isNumeric(sValue.charAt(i)))
					return false;
			}
			
			// The entire string contains numeric values
			return true;
		}
		
		// Empty string
		return false;
	}

	/**
	 * Simulates the functionality of the NetLinx ATOI function.
	 * This function returns Integer.MIN_VALUE if no numerical characters are found.
	 * Note: the numerical characters must be at the start of the string
	 * 
	 * @param str - String containing an int value
	 * 
	 * @return int value contained in the string
	 */
	public static int ATOI(String str)
	{
		return ValidateUtil.ATOI(str);
	}

	/**
	 * Simulates the functionality of the NetLinx ATOF function.
	 * This function returns Float.MIN_VALUE if no numerical characters are found.
	 * Note: the numerical characters must be at the start of the string
	 * 
	 * @param str - String containing a float value
	 * 
	 * @return float value contained in the string
	 */
	public static float ATOF(String str)
	{
		StringBuffer sbString = new StringBuffer("");
		boolean minusFailsNow = false;
		boolean decimalFailsNow = false;
		// find first non-numerical character and filter out all other characters 
		for (int i=0;i<str.length(); i++)
		{
			char cChar = str.charAt(i);
			if ( (cChar == '-') || ( (cChar >= '0') && (cChar <= '9') ) || (cChar == '.'))
			{
				if ((cChar == '-') && (minusFailsNow)) { break; }

				minusFailsNow = true;
				
				if (cChar == '.')
				{
					if (!decimalFailsNow) { decimalFailsNow = true;}
					else { break; }
				}
				sbString.append(str.charAt(i));
			}
			else if(sbString.length() > 0){ break; }
		}
		if (sbString.length() == 0) 
		{ 
			error("'fnStrToFloat():Number formatException, String was of zero length.");
			return Float.MIN_VALUE; 
		}
		
		try { return Float.parseFloat(sbString.toString()); }
		catch(NumberFormatException e) 
		{
			error("'fnStrToFloat():Number formatException, String was "+sbString);
			return Float.MIN_VALUE;
		}
	}

	/**
	 * Converts a character representation of a number to a 32-bit integer value
	 * 
	 * @param value String containing the character representation of the integer
	 * @return A 32-bit integer representing the converted string. Any non-numeric 
	 * characters in the string are ignored. <code>atoi</code> returns the value representing 
	 * the first complete set of characters that represent an integer. Valid characters 
	 * are "0" through "9" as well as a prepended "-" representing a negative sign. If 
	 * invalid characters are found, zero is returned as a result.
	 */
	public final static int atoi(String value)
	{
		int i = 0;
		try
		{
			ObjectRef val = new ObjectRef(value);
			i = val.toInt();
		}
		catch (Exception e) {} // Ignore the exception
		
		return i;
	}

	/**
	 * Converts a character representation of a number to a 64-bit long value
	 * 
	 * @param value String containing the character representation of the long
	 * @return A 64-bit long representing the converted string. Any non-numeric 
	 * characters in the string are ignored. <code>atol</code> returns the value representing 
	 * the first complete set of characters that represent a long. Valid characters 
	 * are "0" through "9" as well as a prepended "-" representing a negative sign. 
	 * If invalid characters are found, zero is returned as a result.
	 */
	public final static long atol(String value)
	{
		long l = 0;
		try
		{
			ObjectRef val = new ObjectRef(value);
			l = val.toLong();
		}
		catch (Exception e) {} // Ignore the exception
		
		return l;
	}

	/**
	 * Converts a character representation (with or without exponent) of a number to 
	 * a 32-bit floating point value
	 * 
	 * @param value String containing the character representation of the floating point number.
	 * @return A 32-bit floating point number representing the converted string. Any non-numeric 
	 * characters in the string are ignored. <code>atof</code> returns the value representing the first 
	 * complete set of characters that represents a floating point value. Valid characters are 
	 * "0" through "9" and "." along with the sign designators ("+" and "-") and the exponent 
	 * ("e" or "E"). If invalid characters are found, zero is returned as a result.
	 */
	public final static float atof(String value)
	{
		float f = 0;
		try
		{
			ObjectRef val = new ObjectRef(value);
			f = val.toFloat();
		}
		catch (Exception e) {} // Ignore the exception
		
		return f;
	}

	/**
	 * Converts a character representation (with or without exponent) of a number to 
	 * a 64-bit floating point value
	 * 
	 * @param value String containing the character representation of the floating point number.
	 * @return A 64-bit floating point number representing the converted string. Any non-numeric 
	 * characters in the string are ignored. <code>atod</code> returns the value representing the first 
	 * complete set of characters that represents a floating point value. Valid characters are 
	 * "0" through "9" and "." along with the sign designators ("+" and "-") and the exponent 
	 * ("e" or "E"). If invalid characters are found, zero is returned as a result.
	 */
	public final static double atod(String value)
	{
		double d = 0;
		try
		{
			ObjectRef val = new ObjectRef(value);
			d = val.toDouble();
		}
		catch (Exception e) {} // Ignore the exception
		
		return d;
	}

	/**
	 * Converts a character representation to a boolean value
	 * 
	 * @param value String containing the character representation of the boolean value.
	 * @return A boolean value (either <code>true</code> or <code>false</code>) representing the 
	 * converted string. <code>atob</code> returns the value representing the first complete set
	 * of characters that represent a boolean like value. Valid <STRONG>true</STRONG> characters are
	 * "1" or "true" (case sensitive). Everything else is considered <STRONG>false</STRONG>.
	 */
	public final static boolean atob(String value)
	{
		boolean b = false;
		try
		{
			ObjectRef val = new ObjectRef(value);
			b = val.toBoolean();
		}
		catch (Exception e) {} // Ignore the exception
		
		return b;
	}
	
	/**
	 * Converts a 32-bit integer to an ASCII string.
	 * 
	 * @param value a 32-bit integer to convert to a string.
	 * @return A character string that contains the representation of the specified integer.
	 */
	public final static String itoa(int value)
	{
		String str = "";
		try
		{
			ObjectRef val = new ObjectRef(value);
			str = val.toString();
		}
		catch (Exception e) {} // Ignore the exception
		
		return str;
	}

	/**
	 * Converts a 64-bit long to an ASCII string.
	 * 
	 * @param value a 64-bit long to convert to a string.
	 * @return A character string that contains the representation of the specified long.
	 */
	public final static String ltoa(long value)
	{
		String str = "";
		try
		{
			ObjectRef val = new ObjectRef(value);
			str = val.toString();
		}
		catch (Exception e) {} // Ignore the exception
		
		return str;
	}

	/**
	 * Converts a 32-bit floating-point value to an ASCII string containing the representation  
	 * of the number
	 * 
	 * @param value a 32-bit floating-point number to convert to a string.
	 * @return A character string that contains the representation of the specified floating 
	 * point number. The character representation will use exponents if necessary.
	 */
	public final static String ftoa(float value)
	{
		String str = "";
		try
		{
			ObjectRef val = new ObjectRef(value);
			str = val.toString();
		}
		catch (Exception e) {} // Ignore the exception
		
		return str;
	}

	/**
	 * Converts a 64-bit floating-point value to an ASCII string containing the representation  
	 * of the number
	 * 
	 * @param value a 64-bit floating-point number to convert to a string.
	 * @return A character string that contains the representation of the specified double 
	 * value. The character representation will use exponents if necessary.
	 */
	public final static String dtoa(double value)
	{
		String str = "";
		try
		{
			ObjectRef val = new ObjectRef(value);
			str = val.toString();
		}
		catch (Exception e) {} // Ignore the exception
		
		return str;
	}

	/**
	 * Converts a boolean value to an ASCII string.
	 * 
	 * @param value a boolean to convert to a string.
	 * @return A character string that contains the representation of the specified boolean. If 
	 * value is <STRONG>true</STRONG>, a "1" will be returned. Otherwise, "0" will be returned.
	 */
	public final static String btoa(boolean value)
	{
		String str = "";
		try
		{
			ObjectRef val = new ObjectRef(value);
			str = val.toString();
		}
		catch (Exception e) {} // Ignore the exception
		
		return str;
	}
	
	/*
	 * 
	 * @param str
	 */
	private static void error(String str)
	{
		System.out.println("NumericUtil: ERROR - " + str);
	}
}
