/*
 * @author vgonzales
 * 
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.text;

/**
 * Extension to the J2ME StringBuffer class.
 *
 */
public class StringBufferExt
{
	/**
     * The buffer is used for character storage.
	 */
	StringBuffer buffer = null;
	
    /**
     * Constructs a string buffer with no characters in it and an 
     * initial capacity of 16 characters. 
     */
    public StringBufferExt()
    {
		this(16);
    }

    /**
     * Constructs a string buffer with no characters in it and 
     * the specified initial capacity. 
     *
     * @param      capacity  the initial capacity.
     * @exception  NegativeArraySizeException  if the <code>capacity</code>
     *               argument is less than <code>0</code>.
     */
    public StringBufferExt(int capacity)
    {
		buffer = new StringBuffer(capacity);
    }

    /**
     * Constructs a string buffer initialized to the contents of the 
     * specified string. The initial capacity of the string buffer is 
     * <code>16</code> plus the length of the string argument.
     *
     * @param   str   the initial contents of the buffer.
     * @exception NullPointerException if <code>str</code> is <code>null</code>
     */
    public StringBufferExt(String str)
    {
		this(str.length() + 16);
		append(str);
    }

    /**
     * Returns the length (character count) of this string buffer.
     *
     * @return  the length of the sequence of characters currently 
     *          represented by this string buffer.
     */
    public synchronized int length()
    {
    	return buffer.length();
    }

    /**
     * Returns the current capacity of the String buffer. The capacity
     * is the amount of storage available for newly inserted
     * characters; beyond which an allocation will occur.
     *
     * @return  the current capacity of this string buffer.
     */
    public synchronized int capacity()
    {
    	return buffer.capacity();
    }


    /**
     * Ensures that the capacity of the buffer is at least equal to the
     * specified minimum.
     * If the current capacity of this string buffer is less than the 
     * argument, then a new internal buffer is allocated with greater 
     * capacity. The new capacity is the larger of: 
     * <ul>
     * <li>The <code>minimumCapacity</code> argument. 
     * <li>Twice the old capacity, plus <code>2</code>. 
     * </ul>
     * If the <code>minimumCapacity</code> argument is nonpositive, this
     * method takes no action and simply returns.
     *
     * @param   minimumCapacity   the minimum desired capacity.
     */
    public synchronized void ensureCapacity(int minimumCapacity)
    {
    	buffer.ensureCapacity(minimumCapacity);
    }

    /**
     * Sets the length of this String buffer.
     * This string buffer is altered to represent a new character sequence 
     * whose length is specified by the argument. For every nonnegative 
     * index <i>k</i> less than <code>newLength</code>, the character at 
     * index <i>k</i> in the new character sequence is the same as the 
     * character at index <i>k</i> in the old sequence if <i>k</i> is less 
     * than the length of the old character sequence; otherwise, it is the 
     * null character <code>'\u0000'</code>. 
     *  
     * In other words, if the <code>newLength</code> argument is less than 
     * the current length of the string buffer, the string buffer is 
     * truncated to contain exactly the number of characters given by the 
     * <code>newLength</code> argument. 
     * <p>
     * If the <code>newLength</code> argument is greater than or equal 
     * to the current length, sufficient null characters 
     * (<code>'&#92;u0000'</code>) are appended to the string buffer so that 
     * length becomes the <code>newLength</code> argument. 
     * <p>
     * The <code>newLength</code> argument must be greater than or equal 
     * to <code>0</code>. 
     *
     * @param      newLength   the new length of the buffer.
     * @exception  IndexOutOfBoundsException  if the
     *               <code>newLength</code> argument is negative.
     * @see        com.amx.duet.tools.text.StringBufferExt#length()
     */
    public synchronized void setLength(int newLength)
    {
    	buffer.setLength(newLength);
    }

    /**
     * The specified character of the sequence currently represented by 
     * the string buffer, as indicated by the <code>index</code> argument, 
     * is returned. The first character of a string buffer is at index 
     * <code>0</code>, the next at index <code>1</code>, and so on, for 
     * array indexing. 
     * <p>
     * The index argument must be greater than or equal to 
     * <code>0</code>, and less than the length of this string buffer. 
     *
     * @param      index   the index of the desired character.
     * @return     the character at the specified index of this string buffer.
     * @exception  IndexOutOfBoundsException  if <code>index</code> is 
     *             negative or greater than or equal to <code>length()</code>.
     * @see        com.amx.duet.tools.text.StringBufferExt#length()
     */
    public synchronized char charAt(int index)
    {
    	return buffer.charAt(index);
    }

    /**
     * Characters are copied from this string buffer into the 
     * destination character array <code>dst</code>. The first character to 
     * be copied is at index <code>srcBegin</code>; the last character to 
     * be copied is at index <code>srcEnd-1</code>. The total number of 
     * characters to be copied is <code>srcEnd-srcBegin</code>. The 
     * characters are copied into the subarray of <code>dst</code> starting 
     * at index <code>dstBegin</code> and ending at index:
     * <p><blockquote><pre>
     * dstbegin + (srcEnd-srcBegin) - 1
     * </pre></blockquote>
     *
     * @param      srcBegin   start copying at this offset in the string buffer.
     * @param      srcEnd     stop copying at this offset in the string buffer.
     * @param      dst        the array to copy the data into.
     * @param      dstBegin   offset into <code>dst</code>.
     * @exception  NullPointerException if <code>dst</code> is 
     *             <code>null</code>.
     * @exception  IndexOutOfBoundsException  if any of the following is true:
     *             <ul>
     *             <li><code>srcBegin</code> is negative
     *             <li><code>dstBegin</code> is negative
     *             <li>the <code>srcBegin</code> argument is greater than 
     *             the <code>srcEnd</code> argument.
     *             <li><code>srcEnd</code> is greater than 
     *             <code>this.length()</code>, the current length of this 
     *             string buffer.
     *             <li><code>dstBegin+srcEnd-srcBegin</code> is greater than 
     *             <code>dst.length</code>
     *             </ul>
     */
    public synchronized void getChars(int srcBegin, int srcEnd, char dst[], int dstBegin)
    {
    	buffer.getChars(srcBegin, srcEnd, dst, dstBegin);
    }

    /**
     * The character at the specified index of this string buffer is set 
     * to <code>ch</code>. The string buffer is altered to represent a new 
     * character sequence that is identical to the old character sequence, 
     * except that it contains the character <code>ch</code> at position 
     * <code>index</code>. 
     * <p>
     * The offset argument must be greater than or equal to 
     * <code>0</code>, and less than the length of this string buffer. 
     *
     * @param      index   the index of the character to modify.
     * @param      ch      the new character.
     * @exception  IndexOutOfBoundsException  if <code>index</code> is 
     *             negative or greater than or equal to <code>length()</code>.
     * @see        com.amx.duet.tools.text.StringBufferExt#length()
     */
    public synchronized void setCharAt(int index, char ch)
    {
    	buffer.setCharAt(index, ch);
    }

    /**
     * Sets this string buffer to the string representation of the <code>Object</code> 
     * argument. 
     * <p>
     * The argument is converted to a string as if by the method 
     * <code>String.valueOf</code>, and the characters of that 
     * string replace the original contents of this string buffer. 
     *
     * @param   obj   an <code>Object</code>.
     * @return  a reference to this <code>StringBufferExt</code> object.
     * @see     java.lang.String#valueOf(java.lang.Object)
     * @see     com.amx.duet.tools.text.StringBufferExt#append(java.lang.Object)
     */
    public synchronized StringBufferExt set(Object obj)
    {
    	delete();
    	return append(obj);
    }

    /**
     * Sets this string buffer to the string argument. 
     * <p>
     * The characters of the <code>String</code> argument are added, in order,  
     * deleting the contents of this string buffer, and replacing the length of 
     * this string buffer with the length of the argument. If <code>str</code> is 
     * <code>null</code>, then the four characters <code>"null"</code> are 
     * appended to this string buffer.
     *
     * @param   str   a string.
     * @return  a reference to this <code>StringBufferExt</code>.
     * @see     com.amx.duet.tools.text.StringBufferExt#append(java.lang.String)
     */
    public synchronized StringBufferExt set(String str)
    {
    	delete();
    	return append(str);
    }

    /**
     * Sets this sequence to the specified <tt>StringBufferExt</tt>.
     * <p>
     * The characters of the <tt>StringBuffer</tt> argument are added, in order, 
     * deleting the old contents from this <tt>StringBufferExt</tt>, and replacing 
     * the length of this <tt>StringBufferExt</tt> with the length of the argument. 
     * If <tt>sb</tt> is <tt>null</tt>, then the four characters <tt>"null"</tt> 
     * are appended to this <tt>StringBufferExt</tt>.
     * <p>
     * This method synchronizes on <code>this</code> (the destination) 
     * object but does not synchronize on the source (<code>sb</code>).
     *
     * @param   sb   the <tt>StringBuffer</tt> to set.
     * @return  a reference to this object.
     * @see     com.amx.duet.tools.text.StringBufferExt#append(java.lang.StringBuffer)
     */
    public synchronized StringBufferExt set(StringBuffer sb)
    {
    	delete();
    	return append(sb);
    }


    /**
     * Sets this string buffer to the specified <tt>StringBufferExt</tt> sequence.
     * <p>
     * The characters of the <tt>StringBufferExt</tt> argument are added, in order, 
     * deleting the old contents from this <tt>StringBufferExt</tt>, and replacing 
     * the length of this <tt>StringBufferExt</tt> with the length of the argument. 
     * If <tt>sb</tt> is <tt>null</tt>, then the four characters <tt>"null"</tt> 
     * are appended to this <tt>StringBufferExt</tt>.
     * <p>
     * This method synchronizes on <code>this</code> (the destination) 
     * object but does not synchronize on the source (<code>sb</code>).
     *
     * @param   sb   the <tt>StringBufferExt</tt> to set.
     * @return  a reference to this object.
     * @see     com.amx.duet.tools.text.StringBufferExt#append(com.amx.duet.tools.text.StringBufferExt)
     */
    public synchronized StringBufferExt set(StringBufferExt sb)
    {
    	delete();
    	return append(sb);
    }

    /**
     * Sets this string buffer to the string representation of the <code>char</code> 
     * array argument. 
     * <p>
     * The characters of the array argument are added, in order, deleting the old
     * contents from this string buffer, and replacing the length of this string 
     * buffer with the length of the argument. 
     * <p>
     * The overall effect is exactly as if the argument were converted to 
     * a string by the method {@link String#valueOf(char[])} and the 
     * characters of that string were then {@link #set(String) set} 
     * to this <code>StringBufferExt</code> object.
     *
     * @param   str   the characters to set.
     * @return  a reference to this <code>StringBufferExt</code> object.
     * @see     com.amx.duet.tools.text.StringBufferExt#append(char[])
     */
    public synchronized StringBufferExt set(char str[])
    { 
    	delete();
    	return append(str);
    }

    /**
     * Sets this string buffer to the string representation of a subarray of the 
     * <code>char</code> array argument. 
     * <p>
     * Characters of the character array <code>str</code>, starting at 
     * index <code>offset</code>, are added, in order, deleting the previous contents 
     * from this string buffer, and replacing the length of this string buffer with 
     * the length of the argument <code>len</code>. 
     * <p>
     * The overall effect is exactly as if the arguments were converted to 
     * a string by the method {@link String#valueOf(char[],int,int)} and the
     * characters of that string were then {@link #append(String) appended} 
     * to this <code>StringBufferExt</code> object.
     *
     * @param   str      the characters to be added.
     * @param   offset   the index of the first character to set.
     * @param   len      the number of characters to set.
     * @return  a reference to this <code>StringBufferExt</code> object.
     */
    public synchronized StringBufferExt set(char str[], int offset, int len)
    {
    	delete();
    	return append(str, offset, len);
    }

    /**
     * Set this string buffer to the string representation of the <code>boolean</code> 
     * argument. 
     * <p>
     * The argument is converted to a string as if by the method 
     * <code>String.valueOf</code>, and then this string buffer is set to the characters 
     * of that string. 
     *
     * @param   b   a <code>boolean</code>.
     * @return  a reference to this <code>StringBufferExt</code>.
     * @see     java.lang.String#valueOf(boolean)
     * @see     com.amx.duet.tools.text.StringBufferExt#set(java.lang.String)
     */
    public synchronized StringBufferExt set(boolean b)
    {
    	delete();
    	return append(b);
    }

    /**
     * Sets this string buffer to the string representation of the <code>char</code> 
     * argument. 
     * <p>
     * The contents of this string buffer are set to the argument. 
     * The length of this string buffer is set to <code>1</code>. 
     * <p>
     * The overall effect is exactly as if the argument were converted to 
     * a string by the method {@link String#valueOf(char)} and the character 
     * in that string was then {@link #set(String) set} to this 
     * <code>StringBufferExt</code> object.
     *
     * @param   c   a <code>char</code>.
     * @return  a reference to this <code>StringBufferExt</code> object.
     */
    public synchronized StringBufferExt set(char c)
    {
    	delete();
    	return append(c);
    }

    /**
     * Set this string buffer to the string representation of the <code>int</code> 
     * argument. 
     * <p>
     * The argument is converted to a string as if by the method 
     * <code>String.valueOf</code>, and then this string buffer is set to the characters 
     * of that string. 
     *
     * @param   i   an <code>int</code>.
     * @return  a reference to this <code>StringBufferExt</code> object.
     * @see     java.lang.String#valueOf(int)
     * @see     com.amx.duet.tools.text.StringBufferExt#set(java.lang.String)
     */
    public synchronized StringBufferExt set(int i)
    {
    	delete();
    	return append(i);
    }

    /**
     * Sets this string buffer to the string representation of the <code>long</code> 
     * argument. 
     * <p>
     * The argument is converted to a string as if by the method 
     * <code>String.valueOf</code>, and then this string buffer is set to the characters 
     * of that string. 
     *
     * @param   l   a <code>long</code>.
     * @return  a referenct to this <code>StringBufferExt</code> object.
     * @see     java.lang.String#valueOf(long)
     * @see     com.amx.duet.tools.text.StringBufferExt#set(java.lang.String)
     */
    public synchronized StringBufferExt set(long l)
    {
    	delete();
    	return append(l);
    }

    /**
     * Sets this string buffer to the string representation of the <code>float</code> 
     * argument. 
     * <p>
     * The argument is converted to a string as if by the method 
     * <code>String.valueOf</code>, and then this string buffer is set to the characters 
     * of that string. 
     *
     * @param   f   a <code>float</code>.
     * @return  a reference to this <code>StringBufferExt</code> object.
     * @see     java.lang.String#valueOf(float)
     * @see     com.amx.duet.tools.text.StringBufferExt#set(java.lang.String)
     */
    public synchronized StringBufferExt set(float f)
    {
    	delete();
    	return append(f);
    }

    /**
     * Sets this string buffer to the string representation of the <code>double</code> 
     * argument. 
     * <p>
     * The argument is converted to a string as if by the method 
     * <code>String.valueOf</code>, and then this string buffer is set to the characters 
     * of that string. 
     *
     * @param   d   a <code>double</code>.
     * @return  a reference to this <code>StringBufferExt</code> object.
     * @see     java.lang.String#valueOf(double)
     * @see     com.amx.duet.tools.text.StringBufferExt#set(java.lang.String)
     */
    public synchronized StringBufferExt set(double d)
    {
    	delete();
    	return append(d);
    }

    /**
     * Appends the string representation of the <code>Object</code> 
     * argument to this string buffer. 
     * <p>
     * The argument is converted to a string as if by the method 
     * <code>String.valueOf</code>, and the characters of that 
     * string are then appended to this string buffer. 
     *
     * @param   obj   an <code>Object</code>.
     * @return  a reference to this <code>StringBufferExt</code> object.
     * @see     java.lang.String#valueOf(java.lang.Object)
     * @see     com.amx.duet.tools.text.StringBufferExt#append(java.lang.String)
     */
    public synchronized StringBufferExt append(Object obj)
    {
    	buffer.append(obj);
    	return this;
    }

    /**
     * Appends the string to this string buffer. 
     * <p>
     * The characters of the <code>String</code> argument are appended, in 
     * order, to the contents of this string buffer, increasing the 
     * length of this string buffer by the length of the argument. 
     * If <code>str</code> is <code>null</code>, then the four characters 
     * <code>"null"</code> are appended to this string buffer.
     * <p>
     * Let <i>n</i> be the length of the old character sequence, the one 
     * contained in the string buffer just prior to execution of the 
     * <code>append</code> method. Then the character at index <i>k</i> in 
     * the new character sequence is equal to the character at index <i>k</i> 
     * in the old character sequence, if <i>k</i> is less than <i>n</i>; 
     * otherwise, it is equal to the character at index <i>k-n</i> in the 
     * argument <code>str</code>.
     *
     * @param   str   a string.
     * @return  a reference to this <code>StringBufferExt</code>.
     */
    public synchronized StringBufferExt append(String str)
    {
    	buffer.append(str);
    	return this;
    }

    /**
     * Appends the specified <tt>StringBuffer</tt> to this sequence.
     * <p>
     * The characters of the <tt>StringBuffer</tt> argument are appended, 
     * in order, to the contents of this <tt>StringBuffer</tt>, increasing the 
     * length of this <tt>StringBuffer</tt> by the length of the argument. 
     * If <tt>sb</tt> is <tt>null</tt>, then the four characters 
     * <tt>"null"</tt> are appended to this <tt>StringBuffer</tt>.
     * <p>
     * Let <i>n</i> be the length of the old character sequence, the one 
     * contained in the <tt>StringBuffer</tt> just prior to execution of the 
     * <tt>append</tt> method. Then the character at index <i>k</i> in 
     * the new character sequence is equal to the character at index <i>k</i> 
     * in the old character sequence, if <i>k</i> is less than <i>n</i>; 
     * otherwise, it is equal to the character at index <i>k-n</i> in the 
     * argument <code>sb</code>.
     * <p>
     * This method synchronizes on <code>this</code> (the destination) 
     * object but does not synchronize on the source (<code>sb</code>).
     *
     * @param   sb   the <tt>StringBuffer</tt> to append.
     * @return  a reference to this object.
     * @since 1.4
     */
    public synchronized StringBufferExt append(StringBuffer sb)
    {
    	buffer.append(sb);
    	return this;
    }


    /**
     * Appends the specified <tt>StringBufferExt </tt> to this sequence.
     * <p>
     * The characters of the <tt>StringBufferExt </tt> argument are appended, 
     * in order, to the contents of this <tt>StringBufferExt </tt>, increasing the 
     * length of this <tt>StringBufferExt </tt> by the length of the argument. 
     * If <tt>sb</tt> is <tt>null</tt>, then the four characters 
     * <tt>"null"</tt> are appended to this <tt>StringBufferExt </tt>.
     * <p>
     * Let <i>n</i> be the length of the old character sequence, the one 
     * contained in the <tt>StringBufferExt </tt> just prior to execution of the 
     * <tt>append</tt> method. Then the character at index <i>k</i> in 
     * the new character sequence is equal to the character at index <i>k</i> 
     * in the old character sequence, if <i>k</i> is less than <i>n</i>; 
     * otherwise, it is equal to the character at index <i>k-n</i> in the 
     * argument <code>sb</code>.
     * <p>
     * This method synchronizes on <code>this</code> (the destination) 
     * object but does not synchronize on the source (<code>sb</code>).
     *
     * @param   sb   the <tt>StringBufferExt </tt> to append.
     * @return  a reference to this object.
     * @since 1.4
     */
    public synchronized StringBufferExt append(StringBufferExt  sb)
    {
    	buffer.append(sb);
    	return this;
    }


    /**
     * Appends the string representation of the <code>char</code> array 
     * argument to this string buffer. 
     * <p>
     * The characters of the array argument are appended, in order, to 
     * the contents of this string buffer. The length of this string 
     * buffer increases by the length of the argument. 
     * <p>
     * The overall effect is exactly as if the argument were converted to 
     * a string by the method {@link String#valueOf(char[])} and the 
     * characters of that string were then {@link #append(String) appended} 
     * to this <code>StringBufferExt</code> object.
     *
     * @param   str   the characters to be appended.
     * @return  a reference to this <code>StringBufferExt</code> object.
     */
    public synchronized StringBufferExt append(char str[])
    { 
    	buffer.append(str);
    	return this;
    }

    /**
     * Appends the string representation of a subarray of the 
     * <code>char</code> array argument to this string buffer. 
     * <p>
     * Characters of the character array <code>str</code>, starting at 
     * index <code>offset</code>, are appended, in order, to the contents 
     * of this string buffer. The length of this string buffer increases 
     * by the value of <code>len</code>. 
     * <p>
     * The overall effect is exactly as if the arguments were converted to 
     * a string by the method {@link String#valueOf(char[],int,int)} and the
     * characters of that string were then {@link #append(String) appended} 
     * to this <code>StringBufferExt</code> object.
     *
     * @param   str      the characters to be appended.
     * @param   offset   the index of the first character to append.
     * @param   len      the number of characters to append.
     * @return  a reference to this <code>StringBufferExt</code> object.
     */
    public synchronized StringBufferExt append(char str[], int offset, int len)
    {
    	buffer.append(str, offset, len);
    	return this;
    }

    /**
     * Appends the string representation of the <code>boolean</code> 
     * argument to the string buffer. 
     * <p>
     * The argument is converted to a string as if by the method 
     * <code>String.valueOf</code>, and the characters of that 
     * string are then appended to this string buffer. 
     *
     * @param   b   a <code>boolean</code>.
     * @return  a reference to this <code>StringBufferExt</code>.
     * @see     java.lang.String#valueOf(boolean)
     * @see     com.amx.duet.tools.text.StringBufferExt#append(java.lang.String)
     */
    public synchronized StringBufferExt append(boolean b)
    {
    	buffer.append(b);
    	return this;
    }

    /**
     * Appends the string representation of the <code>char</code> 
     * argument to this string buffer. 
     * <p>
     * The argument is appended to the contents of this string buffer. 
     * The length of this string buffer increases by <code>1</code>. 
     * <p>
     * The overall effect is exactly as if the argument were converted to 
     * a string by the method {@link String#valueOf(char)} and the character 
     * in that string were then {@link #append(String) appended} to this 
     * <code>StringBufferExt</code> object.
     *
     * @param   c   a <code>char</code>.
     * @return  a reference to this <code>StringBufferExt</code> object.
     */
    public synchronized StringBufferExt append(char c)
    {
    	buffer.append(c);
    	return this;
    }

    /**
     * Appends the string representation of the <code>int</code> 
     * argument to this string buffer. 
     * <p>
     * The argument is converted to a string as if by the method 
     * <code>String.valueOf</code>, and the characters of that 
     * string are then appended to this string buffer. 
     *
     * @param   i   an <code>int</code>.
     * @return  a reference to this <code>StringBufferExt</code> object.
     * @see     java.lang.String#valueOf(int)
     * @see     com.amx.duet.tools.text.StringBufferExt#append(java.lang.String)
     */
    public synchronized StringBufferExt append(int i)
    {
    	buffer.append(i);
    	return this;
    }

    /**
     * Appends the string representation of the <code>long</code> 
     * argument to this string buffer. 
     * <p>
     * The argument is converted to a string as if by the method 
     * <code>String.valueOf</code>, and the characters of that 
     * string are then appended to this string buffer. 
     *
     * @param   l   a <code>long</code>.
     * @return  a referenct to this <code>StringBufferExt</code> object.
     * @see     java.lang.String#valueOf(long)
     * @see     com.amx.duet.tools.text.StringBufferExt#append(java.lang.String)
     */
    public synchronized StringBufferExt append(long l)
    {
    	buffer.append(l);
    	return this;
    }

    /**
     * Appends the string representation of the <code>float</code> 
     * argument to this string buffer. 
     * <p>
     * The argument is converted to a string as if by the method 
     * <code>String.valueOf</code>, and the characters of that 
     * string are then appended to this string buffer. 
     *
     * @param   f   a <code>float</code>.
     * @return  a reference to this <code>StringBufferExt</code> object.
     * @see     java.lang.String#valueOf(float)
     * @see     com.amx.duet.tools.text.StringBufferExt#append(java.lang.String)
     */
    public synchronized StringBufferExt append(float f)
    {
    	buffer.append(f);
    	return this;
    }

    /**
     * Appends the string representation of the <code>double</code> 
     * argument to this string buffer. 
     * <p>
     * The argument is converted to a string as if by the method 
     * <code>String.valueOf</code>, and the characters of that 
     * string are then appended to this string buffer. 
     *
     * @param   d   a <code>double</code>.
     * @return  a reference to this <code>StringBufferExt</code> object.
     * @see     java.lang.String#valueOf(double)
     * @see     com.amx.duet.tools.text.StringBufferExt#append(java.lang.String)
     */
    public synchronized StringBufferExt append(double d)
    {
    	buffer.append(d);
    	return this;
    }
    
    /**
     * Removes the entire contents of this string buffer from beginning to end.
     * 
     * @return     An empty string buffer.
     */
    public synchronized StringBufferExt delete()
    {
    	return delete(0, buffer.length());
    }

    /**
     * Removes the characters in a substring of this <code>StringBufferExt</code>
     * up to the specified length.  The substring starts from the beginning and  
     * extends to the character at index <code>len - 1</code> or to the end of the
     * <code>StringBufferExt</code> if no such character exists.
     *
     * @param      len  The number of characters to remove from beginning to <code>len</code>.
     * @return     This string buffer.
     */
    public synchronized StringBufferExt delete(int len)
    {
    	return delete(0, len);
    }

    /**
     * Removes the characters in a substring of this <code>StringBufferExt</code>.
     * The substring begins at the specified <code>start</code> and extends to
     * the character at index <code>end - 1</code> or to the end of the
     * <code>StringBufferExt</code> if no such character exists. If
     * <code>start</code> is equal to <code>end</code>, no changes are made.
     *
     * @param      start  The beginning index, inclusive.
     * @param      end    The ending index, exclusive.
     * @return     This string buffer.
     * @exception  StringIndexOutOfBoundsException  if <code>start</code>
     *             is negative, greater than <code>length()</code>, or
     *		   greater than <code>end</code>.
     */
    public synchronized StringBufferExt delete(int start, int end)
    {
    	buffer.delete(start, end);
    	return this;
    }

    /**
     * Removes the character at the specified position in this
     * <code>StringBufferExt</code> (shortening the <code>StringBufferExt</code>
     * by one character).
     *
     * @param       index  Index of character to remove
     * @return      This string buffer.
     * @exception   StringIndexOutOfBoundsException  if the <code>index</code>
     *		    is negative or greater than or equal to
     *		    <code>length()</code>.
     */
    public synchronized StringBufferExt deleteCharAt(int index)
    {
    	buffer.deleteCharAt(index);
    	return this;
    }

    /**
     * Replaces the characters in a substring of this <code>StringBufferExt</code>
     * with characters in the specified <code>String</code>. The substring
     * begins at the specified <code>start</code> and extends to the character
     * at index <code>end - 1</code> or to the end of the
     * <code>StringBufferExt</code> if no such character exists. First the
     * characters in the substring are removed and then the specified
     * <code>String</code> is inserted at <code>start</code>. (The
     * <code>StringBufferExt</code> will be lengthened to accommodate the
     * specified String if necessary.)
     * 
     * @param      start    The beginning index, inclusive.
     * @param      end      The ending index, exclusive.
     * @param      str   String that will replace previous contents.
     * @return     This string buffer.
     * @exception  StringIndexOutOfBoundsException  if <code>start</code>
     *             is negative, greater than <code>length()</code>, or
     *		   greater than <code>end</code>.
     */ 
    public synchronized StringBufferExt replace(int start, int end, String str)
    {
    	buffer.replace(start, end, str);
    	return this;
    }

    /**
     * Returns a new <code>String</code> that contains a subsequence of
     * characters currently contained in this <code>StringBufferExt</code>.The 
     * substring begins at the specified index and extends to the end of the
     * <code>StringBufferExt</code>.
     * 
     * @param      start    The beginning index, inclusive.
     * @return     The new string.
     * @exception  StringIndexOutOfBoundsException  if <code>start</code> is
     *             less than zero, or greater than the length of this
     *             <code>StringBufferExt</code>.
     */
    public synchronized String substring(int start)
    {
        return substring(start, buffer.length());
    }

    /**
     * Returns a new <code>String</code> that contains a subsequence of
     * characters currently contained in this <code>StringBufferExt</code>. The 
     * substring begins at the specified <code>start</code> and 
     * extends to the character at index <code>end - 1</code>. An
     * exception is thrown if 
     *
     * @param      start    The beginning index, inclusive.
     * @param      end      The ending index, exclusive.
     * @return     The new string.
     * @exception  StringIndexOutOfBoundsException  if <code>start</code>
     *             or <code>end</code> are negative or greater than
     *		   <code>length()</code>, or <code>start</code> is
     *		   greater than <code>end</code>.
     */
    public synchronized String substring(int start, int end)
    {
    	if (start >= buffer.length() || end > buffer.length() || start >= end)
    		return "";
        return buffer.substring(start, end);
    }

    /**
     * Inserts the string representation of a subarray of the <code>str</code>
     * array argument into this string buffer. The subarray begins at the
     * specified <code>offset</code> and extends <code>len</code> characters.
     * The characters of the subarray are inserted into this string buffer at
     * the position indicated by <code>index</code>. The length of this
     * <code>StringBufferExt</code> increases by <code>len</code> characters.
     *
     * @param      index    position at which to insert subarray.
     * @param      str       A character array.
     * @param      offset   the index of the first character in subarray to
     *		   to be inserted.
     * @param      len      the number of characters in the subarray to
     *		   to be inserted.
     * @return     This string buffer.
     * @exception  StringIndexOutOfBoundsException  if <code>index</code>
     *             is negative or greater than <code>length()</code>, or
     *		   <code>offset</code> or <code>len</code> are negative, or
     *		   <code>(offset+len)</code> is greater than
     *		   <code>str.length</code>.
     */
    public synchronized StringBufferExt insert(int index, char str[], int offset, int len) 
    {
        buffer.insert(index, str, offset, len);
    	return this;
    }

    /**
     * Inserts the string representation of the <code>Object</code> 
     * argument into this string buffer. 
     * <p>
     * The second argument is converted to a string as if by the method 
     * <code>String.valueOf</code>, and the characters of that 
     * string are then inserted into this string buffer at the indicated 
     * offset. 
     * <p>
     * The offset argument must be greater than or equal to 
     * <code>0</code>, and less than or equal to the length of this 
     * string buffer. 
     *
     * @param      offset   the offset.
     * @param      obj      an <code>Object</code>.
     * @return     a reference to this <code>StringBufferExt</code> object.
     * @exception  StringIndexOutOfBoundsException  if the offset is invalid.
     * @see        java.lang.String#valueOf(java.lang.Object)
     * @see        com.amx.duet.tools.text.StringBufferExt#insert(int, java.lang.String)
     * @see        com.amx.duet.tools.text.StringBufferExt#length()
     */
    public synchronized StringBufferExt insert(int offset, Object obj)
    {
    	buffer.insert(offset, obj);
    	return this;
    }

    /**
     * Inserts the string into this string buffer. 
     * <p>
     * The characters of the <code>String</code> argument are inserted, in 
     * order, into this string buffer at the indicated offset, moving up any 
     * characters originally above that position and increasing the length 
     * of this string buffer by the length of the argument. If 
     * <code>str</code> is <code>null</code>, then the four characters 
     * <code>"null"</code> are inserted into this string buffer.
     * <p>
     * The character at index <i>k</i> in the new character sequence is 
     * equal to:
     * <ul>
     * <li>the character at index <i>k</i> in the old character sequence, if 
     * <i>k</i> is less than <code>offset</code> 
     * <li>the character at index <i>k</i><code>-offset</code> in the 
     * argument <code>str</code>, if <i>k</i> is not less than 
     * <code>offset</code> but is less than <code>offset+str.length()</code> 
     * <li>the character at index <i>k</i><code>-str.length()</code> in the 
     * old character sequence, if <i>k</i> is not less than 
     * <code>offset+str.length()</code>
     * </ul><p>
     * The offset argument must be greater than or equal to 
     * <code>0</code>, and less than or equal to the length of this 
     * string buffer. 
     *
     * @param      offset   the offset.
     * @param      str      a string.
     * @return     a reference to this <code>StringBufferExt</code> object.
     * @exception  StringIndexOutOfBoundsException  if the offset is invalid.
     * @see        com.amx.duet.tools.text.StringBufferExt#length()
     */
    public synchronized StringBufferExt insert(int offset, String str)
    {
        buffer.insert(offset, str);
    	return this;
    }

    /**
     * Inserts the string representation of the <code>char</code> array 
     * argument into this string buffer. 
     * <p>
     * The characters of the array argument are inserted into the 
     * contents of this string buffer at the position indicated by 
     * <code>offset</code>. The length of this string buffer increases by 
     * the length of the argument. 
     * <p>
     * The overall effect is exactly as if the argument were converted to 
     * a string by the method {@link String#valueOf(char[])} and the 
     * characters of that string were then 
     * {@link #insert(int,String) inserted} into this 
     * <code>StringBufferExt</code>  object at the position indicated by
     * <code>offset</code>.
     *
     * @param      offset   the offset.
     * @param      str      a character array.
     * @return     a reference to this <code>StringBufferExt</code> object.
     * @exception  StringIndexOutOfBoundsException  if the offset is invalid.
     */
    public synchronized StringBufferExt insert(int offset, char str[])
    {
        buffer.insert(offset, str);
    	return this;
    }

    /**
     * Inserts the string representation of the <code>boolean</code> 
     * argument into this string buffer. 
     * <p>
     * The second argument is converted to a string as if by the method 
     * <code>String.valueOf</code>, and the characters of that 
     * string are then inserted into this string buffer at the indicated 
     * offset. 
     * <p>
     * The offset argument must be greater than or equal to 
     * <code>0</code>, and less than or equal to the length of this 
     * string buffer. 
     *
     * @param      offset   the offset.
     * @param      b        a <code>boolean</code>.
     * @return     a reference to this <code>StringBufferExt</code> object.
     * @exception  StringIndexOutOfBoundsException  if the offset is invalid.
     * @see        java.lang.String#valueOf(boolean)
     * @see        com.amx.duet.tools.text.StringBufferExt#insert(int, java.lang.String)
     * @see        com.amx.duet.tools.text.StringBufferExt#length()
     */
    public StringBufferExt insert(int offset, boolean b)
    {
    	buffer.insert(offset, b);
    	return this;
    }

    /**
     * Inserts the string representation of the <code>char</code> 
     * argument into this string buffer. 
     * <p>
     * The second argument is inserted into the contents of this string 
     * buffer at the position indicated by <code>offset</code>. The length 
     * of this string buffer increases by one. 
     * <p>
     * The overall effect is exactly as if the argument were converted to 
     * a string by the method {@link String#valueOf(char)} and the character 
     * in that string were then {@link #insert(int, String) inserted} into 
     * this <code>StringBufferExt</code> object at the position indicated by
     * <code>offset</code>.
     * <p>
     * The offset argument must be greater than or equal to 
     * <code>0</code>, and less than or equal to the length of this 
     * string buffer. 
     *
     * @param      offset   the offset.
     * @param      c        a <code>char</code>.
     * @return     a reference to this <code>StringBufferExt</code> object.
     * @exception  IndexOutOfBoundsException  if the offset is invalid.
     * @see        com.amx.duet.tools.text.StringBufferExt#length()
     */
    public synchronized StringBufferExt insert(int offset, char c)
    {
    	buffer.insert(offset, c);
    	return this;
    }

    /**
     * Inserts the string representation of the second <code>int</code> 
     * argument into this string buffer. 
     * <p>
     * The second argument is converted to a string as if by the method 
     * <code>String.valueOf</code>, and the characters of that 
     * string are then inserted into this string buffer at the indicated 
     * offset. 
     * <p>
     * The offset argument must be greater than or equal to 
     * <code>0</code>, and less than or equal to the length of this 
     * string buffer. 
     *
     * @param      offset   the offset.
     * @param      i        an <code>int</code>.
     * @return     a reference to this <code>StringBufferExt</code> object.
     * @exception  StringIndexOutOfBoundsException  if the offset is invalid.
     * @see        java.lang.String#valueOf(int)
     * @see        com.amx.duet.tools.text.StringBufferExt#insert(int, java.lang.String)
     * @see        com.amx.duet.tools.text.StringBufferExt#length()
     */
    public StringBufferExt insert(int offset, int i)
    {
    	buffer.insert(offset, i);
    	return this;
    }

    /**
     * Inserts the string representation of the <code>long</code> 
     * argument into this string buffer. 
     * <p>
     * The second argument is converted to a string as if by the method 
     * <code>String.valueOf</code>, and the characters of that 
     * string are then inserted into this string buffer at the position 
     * indicated by <code>offset</code>. 
     * <p>
     * The offset argument must be greater than or equal to 
     * <code>0</code>, and less than or equal to the length of this 
     * string buffer. 
     *
     * @param      offset   the offset.
     * @param      l        a <code>long</code>.
     * @return     a reference to this <code>StringBufferExt</code> object.
     * @exception  StringIndexOutOfBoundsException  if the offset is invalid.
     * @see        java.lang.String#valueOf(long)
     * @see        com.amx.duet.tools.text.StringBufferExt#insert(int, java.lang.String)
     * @see        com.amx.duet.tools.text.StringBufferExt#length()
     */
    public StringBufferExt insert(int offset, long l)
    {
    	buffer.insert(offset, l);
    	return this;
    }

    /**
     * Inserts the string representation of the <code>float</code> 
     * argument into this string buffer. 
     * <p>
     * The second argument is converted to a string as if by the method 
     * <code>String.valueOf</code>, and the characters of that 
     * string are then inserted into this string buffer at the indicated 
     * offset. 
     * <p>
     * The offset argument must be greater than or equal to 
     * <code>0</code>, and less than or equal to the length of this 
     * string buffer. 
     *
     * @param      offset   the offset.
     * @param      f        a <code>float</code>.
     * @return     a reference to this <code>StringBufferExt</code> object.
     * @exception  StringIndexOutOfBoundsException  if the offset is invalid.
     * @see        java.lang.String#valueOf(float)
     * @see        com.amx.duet.tools.text.StringBufferExt#insert(int, java.lang.String)
     * @see        com.amx.duet.tools.text.StringBufferExt#length()
     */
    public StringBufferExt insert(int offset, float f)
    {
    	buffer.insert(offset, f);
    	return this;
    }

    /**
     * Inserts the string representation of the <code>double</code> 
     * argument into this string buffer. 
     * <p>
     * The second argument is converted to a string as if by the method 
     * <code>String.valueOf</code>, and the characters of that 
     * string are then inserted into this string buffer at the indicated 
     * offset. 
     * <p>
     * The offset argument must be greater than or equal to 
     * <code>0</code>, and less than or equal to the length of this 
     * string buffer. 
     *
     * @param      offset   the offset.
     * @param      d        a <code>double</code>.
     * @return     a reference to this <code>StringBufferExt</code> object.
     * @exception  StringIndexOutOfBoundsException  if the offset is invalid.
     * @see        java.lang.String#valueOf(double)
     * @see        com.amx.duet.tools.text.StringBufferExt#insert(int, java.lang.String)
     * @see        com.amx.duet.tools.text.StringBufferExt#length()
     */
    public StringBufferExt insert(int offset, double d)
    {
    	buffer.insert(offset, d);
    	return this;
    }

    /**
     * Returns the index within this string of the first occurrence of the
     * specified character. If a character with value <code>ch</code> occurs 
     * in the character sequence represented by this <code>String</code> 
     * object, then the index of the first such occurrence is returned -- 
     * that is, the smallest value <i>k</i> such that: 
     * <blockquote><pre>
     * this.charAt(<i>k</i>) == ch
     * </pre></blockquote>
     * is <code>true</code>. If no such character occurs in this string, 
     * then <code>-1</code> is returned.
     *
     * @param   ch   a character.
     * @return  the index of the first occurrence of the character in the
     *          character sequence represented by this object, or
     *          <code>-1</code> if the character does not occur.
     */
    public int indexOf(int ch)
    {
    	return indexOf(ch, 0);
    }
    
    /**
     * Returns the index within this string of the first occurrence of the
     * specified character, starting the search at the specified index.
     * <p>
     * If a character with value <code>ch</code> occurs in the character 
     * sequence represented by this <code>String</code> object at an index 
     * no smaller than <code>fromIndex</code>, then the index of the first
     * such occurrence is returned--that is, the smallest value <i>k</i> 
     * such that: 
     * <blockquote><pre>
     * (this.charAt(<i>k</i>) == ch) && (<i>k</i> >= fromIndex)
     * </pre></blockquote>
     * is true. If no such character occurs in this string at or after 
     * position <code>fromIndex</code>, then <code>-1</code> is returned.
     * <p>
     * There is no restriction on the value of <code>fromIndex</code>. If it 
     * is negative, it has the same effect as if it were zero: this entire 
     * string may be searched. If it is greater than the length of this 
     * string, it has the same effect as if it were equal to the length of 
     * this string: <code>-1</code> is returned.
     *
     * @param   ch          a character.
     * @param   fromIndex   the index to start the search from.
     * @return  the index of the first occurrence of the character in the
     *          character sequence represented by this object that is greater
     *          than or equal to <code>fromIndex</code>, or <code>-1</code>
     *          if the character does not occur.
     */
    public int indexOf(int ch, int fromIndex)
    {
    	if (fromIndex > buffer.length())
    		return -1;
        return buffer.toString().indexOf(ch, fromIndex);
    }
    
    /**
     * Returns the index within this string of the last occurrence of the
     * specified character. That is, the index returned is the largest 
     * value <i>k</i> such that:
     * <blockquote><pre>
     * this.charAt(<i>k</i>) == ch
     * </pre></blockquote>
     * is true. 
     * The String is searched backwards starting at the last character. 
     *
     * @param   ch   a character.
     * @return  the index of the last occurrence of the character in the
     *          character sequence represented by this object, or
     *          <code>-1</code> if the character does not occur.
     */
    public int lastIndexOf(int ch)
    {
        return lastIndexOf(ch, buffer.length());
    }
    
    /**
     * Returns the index within this string of the last occurrence of the
     * specified character, searching backward starting at the specified 
     * index. That is, the index returned is the largest value <i>k</i> 
     * such that:
     * <blockquote><pre>
     * this.charAt(k) == ch) && (k <= fromIndex)
     * </pre></blockquote>
     * is true.
     *
     * @param   ch          a character.
     * @param   fromIndex   the index to start the search from. There is no 
     *          restriction on the value of <code>fromIndex</code>. If it is 
     *          greater than or equal to the length of this string, it has 
     *          the same effect as if it were equal to one less than the 
     *          length of this string: this entire string may be searched. 
     *          If it is negative, it has the same effect as if it were -1: 
     *          -1 is returned.
     * @return  the index of the last occurrence of the character in the
     *          character sequence represented by this object that is less
     *          than or equal to <code>fromIndex</code>, or <code>-1</code>
     *          if the character does not occur before that point.
     */
    public int lastIndexOf(int ch, int fromIndex)
    {
    	if (fromIndex > buffer.length())
    		return -1;
        return buffer.toString().lastIndexOf(ch, fromIndex);
    }

    /**
     * Returns the index within this string buffer of the first occurrence of the
     * specified substring. The integer returned is the smallest value 
     * <i>k</i> such that:
     * <blockquote><pre>
     * this.startsWith(str, <i>k</i>)
     * </pre></blockquote>
     * is <code>true</code>.
     *
     * @param   str   any string.
     * @return  if the string argument occurs as a substring within this
     *          object, then the index of the first character of the first
     *          such substring is returned; if it does not occur as a
     *          substring, <code>-1</code> is returned.
     * @exception java.lang.NullPointerException if <code>str</code> is 
     *          <code>null</code>.
     * @see        java.lang.String#indexOf(java.lang.String)
     */
    public int indexOf(String str)
    {
    	return indexOf(str, 0);
    }

    /**
     * Returns the index within this string buffer of the first occurrence 
     * of the specified substring, starting at the specified index. The  
     * integer returned is the smallest value <i>k</i> such that:
     * <blockquote><pre>
     * this.startsWith(str, <i>k</i>) && (<i>k</i> >= fromIndex)
     * </pre></blockquote>
     * is <code>true</code>.
     * <p>
     * There is no restriction on the value of <code>fromIndex</code>. If 
     * it is negative, it has the same effect as if it were zero: this entire 
     * string buffer may be searched. If it is greater than the length of this 
     * string buffer, it has the same effect as if it were equal to the length of 
     * this string buffer: <code>-1</code> is returned.
     *
     * @param   str         the substring to search for.
     * @param   fromIndex   the index to start the search from.
     * @return  If the string argument occurs as a substring within this
     *          object at a starting index no smaller than
     *          <code>fromIndex</code>, then the index of the first character
     *          of the first such substring is returned. If it does not occur
     *          as a substring starting at <code>fromIndex</code> or beyond,
     *          <code>-1</code> is returned.
     * @exception java.lang.NullPointerException if <code>str</code> is 
     *          <code>null</code>
     * @see        java.lang.String#indexOf(java.lang.String, int)
     */
    public synchronized int indexOf(String str, int fromIndex)
    {
    	if (fromIndex > buffer.length())
    		return -1;
        return buffer.toString().indexOf(str, fromIndex);
    }

    /**
     * Returns the index within this string buffer of the rightmost occurrence
     * of the specified substring.  The rightmost empty string "" is
     * considered to occur at the index value <code>this.length()</code>. 
     * The returned index is the largest value <i>k</i> such that 
     * <blockquote><pre>
     * this.startsWith(str, k)
     * </pre></blockquote>
     * is true.
     *
     * @param   str   the substring to search for.
     * @return  if the string argument occurs one or more times as a substring
     *          within this object, then the index of the first character of
     *          the last such substring is returned. If it does not occur as
     *          a substring, <code>-1</code> is returned.
     * @exception java.lang.NullPointerException  if <code>str</code> is 
     *          <code>null</code>.
     */
    public int lastIndexOf(String str)
    {
        // Note, synchronization achieved via other invocations
        return lastIndexOf(str, buffer.length());
    }

    /**
     * Returns the index within this string buffer of the last occurrence of
     * the specified substring.
     * The returned index indicates the start of the substring, and it
     * must be equal to or less than <code>fromIndex</code>. That is, 
     * the index returned is the largest value <i>k</i> such that:
     * <blockquote><pre>
     * this.startsWith(str, k) && (k <= fromIndex)
     * </pre></blockquote>
     * 
     * @param   str         the substring to search for.
     * @param   fromIndex   the index to start the search from. There is no 
     *          restriction on the value of fromIndex. If it is greater than 
     *          the length of this string buffer, it has the same effect as if it 
     *          were equal to the length of this string buffer: this entire string 
     *          buffer may be searched. If it is negative, it has the same effect 
     *          as if it were -1: -1 is returned.
     * @return  If the string argument occurs one or more times as a substring
     *          within this object at a starting index no greater than
     *          <code>fromIndex</code>, then the index of the first character of
     *          the last such substring is returned. If it does not occur as a
     *          substring starting at <code>fromIndex</code> or earlier,
     *          <code>-1</code> is returned.
     * @exception java.lang.NullPointerException if <code>str</code> is 
     *          <code>null</code>.
     */
    public synchronized int lastIndexOf(String str, int fromIndex)
    {
    	if (fromIndex > buffer.length())
    		return -1;
        return buffer.toString().lastIndexOf(str, fromIndex);
    }
    
    /**
     * Tests if this string buffer starts with the specified prefix.
     * 
     * @param str the prefix
     * @return true if the character sequence represented by the argument is a prefix of the character sequence 
     * represented by this string buffer; false otherwise. Note also that true will be returned if the argument 
     * is an empty string or is equal to this StringBufferExt object as determined by the equals(Object) method.
     */
    public synchronized boolean startsWith(String str)
    {
    	return buffer.toString().startsWith(str);
    }

    /**
     * Tests if this string ends with the specified suffix.
     * 
     * @param str the suffix.
     * @return true if the character sequence represented by the argument is a suffix of the character sequence 
     * represented by this object; false otherwise. Note that the result will be true if the argument is the 
     * empty string or is equal to this StringBufferExt object as determined by the equals(Object) method.
     */
	public synchronized boolean endsWith(String str)
    {
    	return buffer.toString().endsWith(str);
    }

    /**
     * The character sequence contained in this string buffer is 
     * replaced by the reverse of the sequence. 
     * <p>
     * Let <i>n</i> be the length of the old character sequence, the one 
     * contained in the string buffer just prior to execution of the 
     * <code>reverse</code> method. Then the character at index <i>k</i> in 
     * the new character sequence is equal to the character at index 
     * <i>n-k-1</i> in the old character sequence.
     *
     * @return  a reference to this <codeStringBuffer</code> object..
     */
    public synchronized StringBufferExt reverse()
    {
    	buffer.reverse();
    	return this;
    }

    /**
     * Converts to a <code>StringBuffer</code> representing the data in this 
     * string buffer object. The <code>StringBuffer</code> returned has already been  
     * allocated and initialized to contain the character sequence currently 
     * represented by this object. Subsequent changes to this string buffer object 
     * do not affect the contents of the returned <code>StringBuffer</code>. 
     * 
     * @return the StringBuffer representation of this object.
     */
    public synchronized StringBuffer toStringBuffer()
    {
    	return buffer;
    }
    
    /**
     * Converts to a string representing the data in this string buffer.
     * A new <code>String</code> object is allocated and initialized to 
     * contain the character sequence currently represented by this 
     * string buffer. This <code>String</code> is then returned. Subsequent 
     * changes to the string buffer do not affect the contents of the 
     * <code>String</code>. 
     * <p>
     * Implementation advice: This method can be coded so as to create a new
     * <code>String</code> object without allocating new memory to hold a 
     * copy of the character sequence. Instead, the string can share the 
     * memory used by the string buffer. Any subsequent operation that alters 
     * the content or capacity of the string buffer must then make a copy of 
     * the internal buffer at that time. This strategy is effective for 
     * reducing the amount of memory allocated by a string concatenation 
     * operation when it is implemented using a string buffer.
     *
     * @return  a string representation of the string buffer.
     */
    public synchronized String toString()
    {
    	return buffer.toString();
    }

}
