/*
 * StringUtil.java
 * 
 * Created on Jul 7, 2006
 * Created by VGonzales
 *
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.text;

import java.util.StringTokenizer;

import com.amx.duet.tools.lang.ObjectRef;

/**
 * A utility class, which contains a variety of String manipulation routines.
 * 
 * @version 1.0.0
 */
public class StringUtil
{
	private final static String digits = "0123456789ABCDEF";

	/**
	 * Returns an array of strings containing each parsed value
	 * 
	 * @param data data string to be parsed
	 * @param delim string delimiter to parse the data by
	 * 
	 * @return an array of strings
	 */
	public final static String[] parse(String data, String delim)
	{
		StringTokenizer tokenizer = new StringTokenizer(data, delim);
		String value[] = new String[tokenizer.countTokens()];
		int count = 0;
		
		while (tokenizer.hasMoreTokens()) 
		{
			value[count++] = tokenizer.nextToken();
			//diag("value[" + (count-1) + "] = " + value[count-1]);
		}
		return value;
	}

	/**
	 * Removes every instance of the passed in pattern from the string and replaces them with the 
	 * replace pattern.
	 * 
	 * @param str String to be manipulated
	 * @param pattern Pattern to remove from source string
	 * @param replace Pattern to replace removed pattern with
	 * @return the newly replaced string
	 */
    public final static String ReplaceString(String str, String pattern, String replace) 
    {
        int s = 0;
        int e = 0;
        StringBuffer result = new StringBuffer();
    
        while ((e = str.indexOf(pattern, s)) >= 0) 
        {
            result.append(str.substring(s, e));
            result.append(replace);
            s = e+pattern.length();
        }
        
        result.append(str.substring(s));
        return result.toString();
    }

	/**
	 * Removes every instance of the passed in pattern from the string and returns the new string
	 * without any of the pattern data.
	 * 
	 * @param str String to be manipulated
	 * @param pattern Pattern to remove from source string
	 * @return String that no longer contains pattern 
	 */
    public final static String RemoveStringAll(String str, String pattern)
    {
    	return ReplaceString(str, pattern, "");
    } 

	/**
	 * Removes the first instance of the passed in pattern from the source string but does not change 
	 * the source string in anyway.
	 * 
	 * @param str Source string to be manipulated
	 * @param pattern Pattern to remove from source string
	 * @return String that no longer contains pattern 
	 */
    public final static String RemoveString(String str, String pattern) 
    {
        return RemoveString(str, pattern, 0);
    }
    
    /**
	 * Removes the first instance of the passed in pattern from the source string and manipulates 
	 * the source string parameter to reflect the changes.
     * 
     * @param rStr Must be of type <STRONG>String</STRONG>. ObjectRef containing the source string data 
     * to be manipulated (value is passed by reference).<P>
     * 
	 * Create rStr with a <STRONG>valueOf(String)</STRONG> Example: <br>
	 * <code>ObjectRef rStr = new ObjectRef("") or rStr.valueOf("")</code>
	 * @param pattern Pattern to remove from source string
	 * @return String that no longer contains pattern 
     */
    public final static String RemoveString(ObjectRef rStr, String pattern) 
    {
        return RemoveString(rStr, pattern, 0);
    }
    
    /**
	 * Removes the first instance of the passed in pattern from the source string starting from the 
	 * offset value but does not change the source string in anyway.
     * 
	 * @param str Source string to be manipulated
	 * @param pattern Pattern to remove from source string
     * @param start offset value
	 * @return String that no longer contains pattern 
     */
    public final static String RemoveString(String str, String pattern, int start) 
    {
        return RemoveString(new ObjectRef(str), pattern, start);
    }
    
    /**
	 * Removes the first instance of the passed in pattern from the source string starting from the 
	 * offset value and manipulates the source string parameter to reflect the changes.
     * 
     * @param rStr Must be of type <STRONG>String</STRONG>. ObjectRef containing the source string data 
     * to be manipulated (value is passed by reference).<P>
     * 
	 * Create rStr with a <STRONG>valueOf(String)</STRONG> Example: <br>
	 * <code>ObjectRef rStr = new ObjectRef("") or rStr.valueOf("")</code>
	 * @param pattern Pattern to remove from source string
     * @param start offset value
	 * @return String that no longer contains pattern 
     */
    public final static String RemoveString(ObjectRef rStr, String pattern, int start)
    { 
        int e = 0;
        StringBuffer result = new StringBuffer();
        
    	if ((e = rStr.toString().indexOf(pattern, start)) >= 0)
    	{
	        result.append(rStr.toString().substring(start, e));
    		e += pattern.length();
	        rStr.valueOf(rStr.toString().substring(e));
    	}
        
        return result.toString();
    }
    
	/**
	 * This function is used to remove a number of characters from a string
	 * 
     * @param rStr Must be of type <STRONG>String</STRONG>. ObjectRef containing the source string data 
     * to be manipulated (value is passed by reference).<P>
     * 
	 * Create rStr with a <STRONG>valueOf(String)</STRONG> Example: <br>
	 * <code>ObjectRef rStr = new ObjectRef("") or rStr.valueOf("")</code>
	 * @param count the number of characters to remove
	 * @return String that contains all of the characters that were removed from the string
	 */
    public final static String RemoveString(ObjectRef rStr, int count)
    {
    	String result = "";
    	
    	result = rStr.toString().substring(0, count);
    	rStr.valueOf(rStr.toString().substring(count));
    	
        return result;
    }
    
    /**
	 * This function is used to remove a number of characters from a string
     * 
	 * @param str Source string to be manipulated
	 * @param count the number of characters to remove
     * @return Only the character that were not removed from the string
     */
    public final static String RemoveString(String str, int count) 
    {
    	ObjectRef ref = new ObjectRef(str);
    	RemoveString(ref, count);
        return ref.toString();
    }
    
	/**
	 * Writes len bytes from the specified byte array starting at offset off to this byte array
	 * (Code that I just didn't want to throw away, but is already being done in other places. 
	 * Acts like a mid string or sub string method.)
	 * 
	 * @param b source byte array containing all of the data
	 * @param off offset value to start copying source array from 
	 * @param len number of bytes to copy after offset
	 * @return new byte array containg the copied bytes
	 */
	public final static byte[] writeBytes(byte[] b, int off, int len)
	{
		byte[] bArray = null;
        int offlen = off + len;
		if (len > 0 && offlen < b.length)
		{
			bArray = new byte[len];
			for (int i = off; i < offlen; i++)
			{
				bArray[i-off] = b[i];
			}
		}
		
		return bArray;
	}
	
	/**
	 * Generates an ASCII hex string from the byte array passed in.
	 * ex. {0x1a, 0x1b, 0x1c} returns "1A1B1C"
	 * 
	 * @return ASCII hex representation of a byte array
	 */
	public final static String toAsciiHexString(byte[] bArr)
	{
		int dig1;
		int dig2;
		
		byte[] tempArr = new byte[bArr.length*2];
		
		for (int i=0; i<bArr.length; i++)
		{
			dig1 = (bArr[i] >> 4) & 0x0F;
			dig2 = bArr[i] & 0x0F;
			
			tempArr[i*2] = (byte)digits.charAt(dig1);
			tempArr[(i*2)+1] = (byte)digits.charAt(dig2);
		}
		
		return new String(tempArr);
	}

    /**
     * This function is used to remove characters from a buffer. GetBufferString() has a two-part operation:<P>
     * 
     * 1. Retrieve the characters from the buffer up to the specified count.<br>
     * 2. Remove the retrieved characters from the buffer and shift the remaining characters up to fill the gap.
     * 
     * @param obj Must be of type <STRONG>String</STRONG>. ObjectRef containing the source string data to be 
     * manipulated (value is passed by reference).<P>
     * 
	 * Create rbuffer with a <STRONG>valueOf(String)</STRONG> Example: <br>
	 * <code>ObjectRef rBuffer = new ObjectRef("") or rBuffer.valueOf("")</code>
	 * @param count the number of characters to be retrieved and then shifted
	 * @return String that contains all of the characters that were removed from the string
     */
    public final static String GetBufferString(ObjectRef rbuffer, int count)
    {
    	return StringUtil.RemoveString(rbuffer, count);
    }
    
    /**
     * This function is used to remove characters from a buffer. GetBufferChar() has a two-part operation:<P>
     * 
     * 1. Retrieve the first character in the buffer.<br>
     * 2. Remove the retrieved character from the buffer and shift the remaining characters by one to fill the gap.
     * 
     * @param obj Must be of type <STRONG>String</STRONG>. ObjectRef containing the source string data to be 
     * manipulated (value is passed by reference).<P>
     * 
	 * Create rbuffer with a <STRONG>valueOf(String)</STRONG> Example: <br>
	 * <code>ObjectRef rBuffer = new ObjectRef("") or rBuffer.valueOf("")</code>
     * @return The character that was removed from the string.
     */
    public final static char GetBufferChar(ObjectRef rbuffer)
    {
        char c = 0;
        String str = rbuffer.toString();
        if (str.length() > 0)
        {
	        c = str.charAt(0);
	        if (str.length() > 1)
	        	rbuffer.valueOf(str.substring(1, str.length()));
	        else
	        	rbuffer.valueOf("");
        }
        return c;
    }

    /**
     * Retrieves a String containing the specified number of characters from the beginning of a string.
     * 
     * @param str the string from which to extract the characters.
     * @param count the number of character to copy from the beginning of the string.
     * @return A String containing a copy of the first count characters from the string.
     */
    public final static String LeftString(String str, int count)
    {
        return str.substring(0, count);
    }

	/**
     * Retrieves a String containing the specified number of characters from the end of a string.
     * 
     * @param str the string from which to extract the characters.
	 * @param count the number of character to copy from the end of the string.
	 * @return A String containing a copy of the last count characters from the string.
	 */
    public final static String RightString(String str, int count)
    {
    	int len = str.length() - count;
    	if (len < 0)
    		len = 0;
        return str.substring(len);
    }
    
    /**
     * Retrieves a String containing the specified number of characters starting at the specified 
     * location in the source string.
     * 
     * @param str the string from which to extract the characters.
     * @param start the starting location in the string.
     * @param count the number of characters to extract.
	 * @return A String containing a copy of the specified characters from the string.
     */
    public final static String MidString(String str, int start, int count)
    {
        return str.substring(start, count);
    }
    
    /**
     * Searches through a string for a specified sequence of characters.
     * 
     * @param str the string of characters to search.
     * @param pattern the sequence of characters to search for
     * @param start the starting character position for the search
     * @return An integer representing the character location of pattern in the string. If the character 
     * sequence is found at the beginning of the string, this method returns 0. Any error condition 
     * returns -1.
     */
    public final static int FindString(String str, String pattern, int start)
    {
    	return str.indexOf(pattern, start);
    }

}
