/*
 * @author VGonzales
 * 
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.util;

import com.amx.duet.core.master.netlinx.Custom;
import com.amx.duet.core.master.netlinx.DPS;
import com.amx.duet.core.master.netlinx.Event;
import com.amx.duet.core.master.netlinx.Level;

/**
 * The <code>EventPacker</code> class was created to package up NetLinx event data into a single 
 * <code>Event</code> object.  The class provides a list of methods that can be used to create 
 * <code>Event</code> objects for routines such as processing advanced events or listening to 
 * loopback data coming from a derived <code>NetLinxDevice</code>.
 *  
 * @version 1.0.0
 * @since AMXTools 1.0.1
 *
 */
public class EventPacker
{
	/**
	 * Create an ERROR <code>Event</code> for the passed in DPS.
	 * 
	 * @param dps a DPS object representing the D:P:S of an IcspNetLinxDevice
	 * @param error Error value to be reported
	 * @return an Event object containing all necessary event information.
	 */
	public static final Event packError(DPS dps, int error)
	{
		Event e = new Event(dps);
		e.dps = dps;
		e.type = Event.E_ERROR;
		e.dataNumber = error;
		return e;
	}
	
	/**
	 * Create an ONLINE <code>Event</code> for the passed in DPS.
	 * 
	 * @param dps a DPS object representing the D:P:S of an IcspNetLinxDevice
	 * @return an Event object containing all necessary event information.
	 */
	public static final Event packOnline(DPS dps)
	{
		//diag("packOnline()");
		Event e = new Event(dps);
		e.dps = dps;
		e.type = Event.E_ONLINE;
		return e;
	}

	/**
	 * Create an OFFLINE <code>Event</code> for the passed in DPS.
	 * 
	 * @param dps a DPS object representing the D:P:S of an IcspNetLinxDevice
	 * @return an Event object containing all necessary event information.
	 */
	public static final Event packOffline(DPS dps)
	{
		//diag("packOffline()");
		Event e = new Event(dps);
		e.dps = dps;
		e.type = Event.E_OFFLINE;
		return e;
	}

	/**
	 * Create an OFF <code>Event</code> for the passed in DPS.
	 * 
	 * @param dps a DPS object representing the D:P:S of an IcspNetLinxDevice
	  * @param channelId is the index of the channel (1 based)
	 * @return an Event object containing all necessary event information.
	 */
	public static final Event packOff(DPS dps, int channelId)
	{
		//diag("packOff()");
		Event e = new Event(dps);
		e.dps = dps;
		e.type = Event.E_OFF;
		e.idx = channelId;
		return e;
	}

	/**
	 * Create a RELEASE <code>Event</code> for the passed in DPS.
	 * 
	 * @param dps a DPS object representing the D:P:S of an IcspNetLinxDevice
	  * @param channelId is the index of the channel (1 based)
	 * @return an Event object containing all necessary event information.
	 */
	public static final Event packOffInputChannel(DPS dps, int channelId)
	{
		//diag("packOffInputChannel()");
		Event e = new Event(dps);
		e.dps = dps;
		e.type = Event.E_RELEASE;
		e.idx = channelId;
		return e;
	}

	/**
	 * Create an ON <code>Event</code> for the passed in DPS.
	 * 
	 * @param dps a DPS object representing the D:P:S of an IcspNetLinxDevice
	  * @param channelId is the index of the channel (1 based)
	 * @return an Event object containing all necessary event information.
	 */
	public static final Event packOn(DPS dps, int channelId)
	{
		//diag("packOn()");
		Event e = new Event(dps);
		e.dps = dps;
		e.type = Event.E_ON;
		e.idx = channelId;
		return e;
	}

	/**
	 * Create a PUSH <code>Event</code> for the passed in DPS.
	 * 
	 * @param dps a DPS object representing the D:P:S of an IcspNetLinxDevice
	  * @param channelId is the index of the channel (1 based)
	 * @return an Event object containing all necessary event information.
	 */
	public static final Event packOnInputChannel(DPS dps, int channelId)
	{
		//diag("packOnInputChannel()");
		Event e = new Event(dps);
		e.dps = dps;
		e.type = Event.E_PUSH;
		e.idx = channelId;
		return e;
	}

	/**
	 * Create a SEND_COMMAND <code>Event</code> for the passed in DPS.
	 * 
	 * @param dps a DPS object representing the D:P:S of an IcspNetLinxDevice
	 * @param str is a string containing the command data
	 * @return an Event object containing all necessary event information.
	 */
	public static final Event packSendCommand(DPS dps, byte[] str)
	{
		//diag("packSendCommand(byte[])");
		Event e = new Event(dps);
		e.dps = dps;
		e.type = Event.E_COMMAND;
		e.dataType = Event.D_STRING8;
		e.dataValue = str;
		return e;
	}

	/**
	 * Create a SEND_COMMAND <code>Event</code> for the passed in DPS.
	 * 
	 * @param dps a DPS object representing the D:P:S of an IcspNetLinxDevice
	 * @param str is a string containing the command data
	 * @return an Event object containing all necessary event information.
	 */
	public static final Event packSendCommand(DPS dps, char[] str)
	{
		//diag("packSendCommand(char[])");
		return packSendCommand(dps, new String(str));
	}

	/**
	 * Create a SEND_COMMAND <code>Event</code> for the passed in DPS.
	 * 
	 * @param dps a DPS object representing the D:P:S of an IcspNetLinxDevice
	 * @param str is a string containing the command data
	 * @return an Event object containing all necessary event information.
	 */
	public static final Event packSendCommand(DPS dps, String str)
	{
		//diag("packSendCommand(String)");
		return packSendCommand(dps, str.getBytes());
	}

	/**
	 * Create a wide string SEND_COMMAND <code>Event</code> for the passed in DPS.
	 * 
	 * @param dps a DPS object representing the D:P:S of an IcspNetLinxDevice
	 * @param str is a string containing the command data
	 * @return an Event object containing all necessary event information.
	 */
	public static final Event packSendCommandWide(DPS dps, String str)
	{
		//diag("packSendCommandWide(String)");
		Event e = new Event(dps);
		e.dps = dps;
		e.type = Event.E_COMMAND;
		e.dataType = Event.D_STRING16;
		e.dataValue = str.toCharArray();
		return e;
	}

	/**
	 * Create a CUSTOM <code>Event</code> for the passed in DPS.
	 * 
	 * @param dps a DPS object representing the D:P:S of an IcspNetLinxDevice
	 * @param id
	 * @param eventType
	 * @param flag
	 * @param value1
	 * @param value2
	 * @param value3
	 * @param eventData a string containing the custom event data
	 * @param encode
	 * @return an Event object containing all necessary event information.
	 */
	public static final Event packSendCustom(DPS dps, int id, int eventType, int flag, int value1, int value2, int value3, byte[] eventData, byte[] encode)
	{
		//diag("packSendCustom(byte[])");
		Custom cEvt = new Custom();
		cEvt.id = id;
		cEvt.eventType = eventType;
		cEvt.flag = flag;
		cEvt.value1 = value1;
		cEvt.value2 = value2;
		cEvt.value3 = value3;
		cEvt.dataType = Event.D_STRING8;
		cEvt.dataValue = eventData;
		cEvt.encode = encode;

		Event e = new Event(dps);
		e.dps = dps;
		e.dataType = Event.D_CUSTOM;
		e.type = Event.E_CUSTOM;
		e.dataValue = cEvt;
		return e;
	}

	/**
	 * Create a CUSTOM <code>Event</code> for the passed in DPS.
	 * 
	 * @param dps a DPS object representing the D:P:S of an IcspNetLinxDevice
	 * @param id
	 * @param eventType
	 * @param flag
	 * @param value1
	 * @param value2
	 * @param value3
	 * @param eventData a string containing the custom event data
	 * @param encode
	 * @return an Event object containing all necessary event information.
	 */
	public static final Event packSendCustom(DPS dps, int id, int eventType, int flag, int value1, int value2, int value3, char[] eventData, byte[] encode)
	{
		//diag("packSendCustom(char[])");
		return packSendCustom(dps, id, eventType, flag, value1, value2, value3, new String(eventData), encode);
	}

	/**
	 * Create a CUSTOM <code>Event</code> for the passed in DPS.
	 * 
	 * @param dps a DPS object representing the D:P:S of an IcspNetLinxDevice
	 * @param id
	 * @param eventType
	 * @param flag
	 * @param value1
	 * @param value2
	 * @param value3
	 * @param eventData a string containing the custom event data
	 * @param encode
	 * @return an Event object containing all necessary event information.
	 */
	public static final Event packSendCustom(DPS dps, int id, int eventType, int flag, int value1, int value2, int value3, String eventData, byte[] encode)
	{
		//diag("packSendCustom(String)");
		return packSendCustom(dps, id, eventType, flag, value1, value2, value3, eventData.getBytes(), encode);
	}

	/**
	 * Create a wide string CUSTOM <code>Event</code> for the passed in DPS.
	 * 
	 * @param dps a DPS object representing the D:P:S of an IcspNetLinxDevice
	 * @param id
	 * @param eventType
	 * @param flag
	 * @param value1
	 * @param value2
	 * @param value3
	 * @param eventData a string containing the custom event data
	 * @param encode
	 * @return an Event object containing all necessary event information.
	 */
	public static final Event packSendCustomWide(DPS dps, int id, int eventType, int flag, int value1, int value2, int value3, String eventData, byte[] encode)
	{
		//diag("packSendCustomWide(String)");
		Custom cEvt = new Custom();
		cEvt.id = id;
		cEvt.eventType = eventType;
		cEvt.flag = flag;
		cEvt.value1 = value1;
		cEvt.value2 = value2;
		cEvt.value3 = value3;
		cEvt.dataType = Event.D_STRING16;
		cEvt.dataValue = eventData.toCharArray();
		cEvt.encode = encode;

		Event e = new Event(dps);
		e.dps = dps;
		e.dataType = Event.D_CUSTOM;
		e.type = Event.E_CUSTOM;
		e.dataValue = cEvt;
		return e;
	}

	/**
	 * Create a LEVEL <code>Event</code> for the passed in DPS.
	 * 
	 * @param dps a DPS object representing the D:P:S of an IcspNetLinxDevice
	 * @param levelId a one based integer representing the level to send
	 * @param value a Level object containing the level value to send
	 * @return an Event object containing all necessary event information.
	 */
	public static final Event packSendLevel(DPS dps, int levelId, Level value)
	{
		//diag("packSendLevel()");
		Event e = new Event(dps);
		e.dps = dps;
		e.type = Event.E_LEVEL;
		e.idx = levelId;
		e.dataType = Event.D_LEVEL;
		e.dataValue = value;
		return e;
	}

	/**
	 * Create a LEVEL <code>Event</code> for the passed in DPS.
	 * 
	 * @param dps a DPS object representing the D:P:S of an IcspNetLinxDevice
	 * @param levelId a one based integer representing the level to send
	 * @param value int level value to send
	 * @return an Event object containing all necessary event information.
	 */
	public static final Event packSendLevel(DPS dps, int levelId, int value)
	{
		//diag("packSendLevel()");
		return packSendLevel(dps, levelId, new Level(value));
	}

	/**
	 * Create a LEVEL <code>Event</code> for the passed in DPS.
	 * 
	 * @param dps a DPS object representing the D:P:S of an IcspNetLinxDevice
	 * @param levelId a one based integer representing the level to send
	 * @param value int level value to send
	 * @return an Event object containing all necessary event information.
	 */
	public static final Event packSendLevel(DPS dps, int levelId, float value)
	{
		//diag("packSendLevel()");
		return packSendLevel(dps, levelId, new Level(value));
	}
	
	/**
	 * Create a SEND_STRING <code>Event</code> for the passed in DPS.
	 * 
	 * @param dps a DPS object representing the D:P:S of an IcspNetLinxDevice
	 * @param str is a string buffer containing the string data
	 * @return an Event object containing all necessary event information.
	 */
	public static final Event packSendString(DPS dps, byte[] str)
	{
		//diag("packSendString(byte[])");
		Event e = new Event(dps);
		e.dps = dps;
		e.type = Event.E_STRING;
		e.dataType = Event.D_STRING8;
		e.dataValue = str;
		return e;
	}

	/**
	 * Create a SEND_STRING <code>Event</code> for the passed in DPS.
	 * 
	 * @param dps a DPS object representing the D:P:S of an IcspNetLinxDevice
	 * @param str is a string buffer containing the string data
	 * @return an Event object containing all necessary event information.
	 */
	public static final Event packSendString(DPS dps, char[] str)
	{
		//diag("packSendString(char[])");
		return packSendString(dps, new String(str));
	}

	/**
	 * Create a SEND_STRING <code>Event</code> for the passed in DPS.
	 * 
	 * @param dps a DPS object representing the D:P:S of an IcspNetLinxDevice
	 * @param str is a string buffer containing the string data
	 * @return an Event object containing all necessary event information.
	 */
	public static final Event packSendString(DPS dps, String str)
	{
		//diag("packSendString(String)");
		return packSendString(dps, str.getBytes());
	}

	/**
	 * Create a wide string SEND_STRING <code>Event</code> for the passed in DPS.
	 * 
	 * @param dps a DPS object representing the D:P:S of an IcspNetLinxDevice
	 * @param str is a string buffer containing the string data
	 * @return an Event object containing all necessary event information.
	 */
	public static final Event packSendStringWide(DPS dps, String str)
	{
		//diag("packSendStringWide(String)");
		Event e = new Event(dps);
		e.dps = dps;
		e.type = Event.E_STRING;
		e.dataType = Event.D_STRING16;
		e.dataValue = str.toCharArray();
		return e;
	}

}
