/*
 * @author VGonzales
 * Created on May 3, 2005
 * 
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.util;

import java.util.Random;

import com.amx.duet.core.master.netlinx.Level;
import com.amx.duet.da.NetLinxDevice;

/**
 * Class containing NetLinx API java equivalent methods. If you are not extending
 * this class, then calls should be used like so: 
 * <br><code>NetLinx.SendLevel(nlDev, nLevel, nValue)</code>
 * 
 * <P> <STRONG>Note: </STRONG>Most methods are using camel case as apposed to the 
 * mixed case or upper case method used by NetLinx. Also, the underscore has been
 * removed.
 *  
 * @version 1.0.3
 * @since AMXTools 1.0.0
 *
 */
public class NetLinxUtil
{
	public static final int FIRST_VIRTUAL_DEVICE   = 32768;
	public static final int FIRST_LOCAL_PORT       = 2;

	public static final int TIMELINE_ONCE          = 0;
	public static final int TIMELINE_REPEAT        = 1;
	
//	private static Calendar calendar = Calendar.getInstance();
	
	private static Pulser pulser = new Pulser(5);
	
	/**
	 * Send the level value to the device.
	 * 
	 * @param nlDev
	 * @param nLevel
	 * @param nValue
	 * @return
	 */
	public final static int SendLevel(NetLinxDevice nlDev, int nLevel, int nValue)
	{
		if (nlDev == null)
			return (0);
			
		diag("SEND_LEVEL " + nlDev.getDPS().toString() + " level = " + nLevel + " value = " + nValue);
		Level val = null;
		// This works for all level channel feedback including
		// brightness, color, contrast, sharpness, tint
		try
		{
			val = new Level(nValue);
		}
		catch (Exception e)
		{
			error("Exception thrown! " + e.getMessage());
		}
		nlDev.sendLevel(nLevel, val);
		
		return (1);
	}

	/**
	 * Send the level value to the device.
	 * 
	 * @param nlDev
	 * @param nLevel
	 * @param fValue
	 * @return
	 */
	public final static int SendLevel(NetLinxDevice nlDev, int nLevel, float fValue)
	{
		if (nlDev == null)
			return (0);
			
		diag("SEND_LEVEL " + nlDev.getDPS().toString() + " level = " + nLevel + " value = " + fValue);
		Level val = null;
		// This works for all level channel feedback including
		// brightness, color, contrast, sharpness, tint
		try
		{
			val = new Level(fValue);
		}
		catch (Exception e)
		{
			error("Exception thrown! " + e.getMessage());
		}
		nlDev.sendLevel(nLevel, val);
		
		return (1);
	}
	
	/**
	 * Send the string to the device.
	 * 
	 * @param nlDev
	 * @param str
	 * @return
	 */
	public final static int SendString(NetLinxDevice nlDev, String str)
	{
		if (nlDev == null)
			return (0);
			
		diag("SEND_STRING " + nlDev.getDPS().toString() + " string = " + str);
		nlDev.sendString(str);

		return (1);
	}

	/**
	 * Send the command to the device.
	 * 
	 * @param nlDev
	 * @param str
	 * @return
	 */
	public final static int SendCommand(NetLinxDevice nlDev, String str)
	{
		if (nlDev == null)
			return (0);
			
		diag("SEND_COMMAND " + nlDev.getDPS().toString() + " command = " + str);
		nlDev.sendCommand(str);

		return (1);
	}

	/**
	 * Send a push command to the device.
	 * 
	 * @param nlDev
	 * @param nChan
	 * @return
	 */
	public final static int Push(NetLinxDevice nlDev, int nChan)
	{
		if (nlDev == null)
			return (0);
			
		diag("PUSH " + nlDev.getDPS().toString() + " channel = " + nChan);
		// We should only be sending feedback to our device
		nlDev.onInputChannel(nChan);

		return (1);
	}

	/**
	 * Send a release command to the device.
	 * 
	 * @param nlDev
	 * @param nChan
	 * @return
	 */ 	
	public final static int Release(NetLinxDevice nlDev, int nChan)
	{
		if (nlDev == null)
			return (0);
			
		diag("RELEASE " + nlDev.getDPS().toString() + " channel = " + nChan);
		// We should only be sending feedback to our device
		nlDev.offInputChannel(nChan);

		return (1);
	}

	/**
	 * Send the feedback on state to the device.
	 * 
	 * @param nlDev
	 * @param nChan
	 * @return
	 */ 	
	public final static int On(NetLinxDevice nlDev, int nChan)
	{
		if (nlDev == null)
			return (0);
			
		diag("CHANNEL_ON " + nlDev.getDPS().toString() + " channel = " + nChan);
		// We should only be sending feedback to our device
		nlDev.on(nChan);

		return (1);
	}

	/**
	 * Send the feedback off state to the device.
	 * 
	 * @param nlDev
	 * @param nChan
	 * @return
	 */ 	
	public final static int Off(NetLinxDevice nlDev, int nChan)
	{
		if (nlDev == null)
			return (0);
			
		diag("CHANNEL_OFF " + nlDev.getDPS().toString() + " channel = " + nChan);
		// We should only be sending feedback to our device
		nlDev.off(nChan);

		return (1);
	}
	
	/**
	 * Send a pulse command to the device.
	 * 
	 * @param nlDev
	 * @param nChan
	 * @return
	 */
	public final static int Pulse(NetLinxDevice nlDev, int nChan)
	{
		if (nlDev == null)
			return (0);
			
		diag("PULSE " + nlDev.getDPS().toString() + " channel = " + nChan);

		pulser.pulse(nlDev, nChan);
		return (1);
	}
	
	/**
	 * This function sets the PULSE time in .10 second units. The default PULSE 
	 * time is 5 (0.5 seconds).
	 *  
	 * @param nTime
	 */
	public final static void setPulseTime(int nTime)
	{
		pulser.setPulseTime(nTime);
	}
	
	/**
	 * The function returns the current duration of PULSE as set by <code>setPulseTime()</code>. 
	 * Time is measured in tenths of a second; the default is 5 (0.5 seconds).
	 *  
	 * @return
	 */
	public final static int getPulseTime()
	{
		return pulser.getPulseTime();
	}
	
	/**
	 * This method cancels the specified wait timer.
	 * 
	 * @param timer WaitTimer to be cancelled.
	 */
	public final static void CancelWait(WaitTimer timer)
	{
		if (timer != null)
		{
			if (timer.isStarted())
				timer.cancel();
		}
	}

	/**
	 * This method is used to create and process an event of one or more statements 
	 * at a specified period of time.
	 * 
	 * @param timer WaitTimer object
	 * @param nTimeOut period of time to reset the timer to
	 * @return true if the timer was able to start
	 */
	public final static boolean Wait(WaitTimer timer, int nTimeOut)
	{
		boolean bStarted = false;
		
		if (timer != null)
		{
			timer.setTimeout(nTimeOut);
			bStarted = timer.start();
		}
		
		return bStarted;
	}
	
	/**
	 * This method is used to create and process an event of one or more statements 
	 * at a later time.
	 * 
	 * @param timer WaitTimer object
	 * @return true if the timer was able to start
	 */
	public final static boolean Wait(WaitTimer timer)
	{
		boolean bStarted = false;
		
		if (timer != null)
			bStarted = timer.start();
		
		return bStarted;
	}
	
	/**
	 * This function returns a random number X in the range 0 &lt= X &lt number.
	 * 
	 * @param number integer (must be greater than zero) that will serve as 
	 * the upper limit for the random number generator.
	 * @return An integer &gt= 0 and &lt number
	 */
	public final static int RandomNumber(int number)
	{
        Random rand = new Random();
        int n = rand.nextInt(number);
		return n;
	}

	/**
	 * This routine takes two variables and compares for the lowest value.
	 * 
	 * @param lvar1 variable that must be a long or casted to a long
	 * @param lvar2 variable that must be a long or casted to a long
	 * @return the lower value
	 */
	public final static long MinValue(long lvar1, long lvar2)
	{
		return Math.min(lvar1, lvar2);
	}

	/**
	 * This routine takes two variables and compares for the highest value.
	 * 
	 * @param lvar1 variable that must be a long or casted to a long
	 * @param lvar2 variable that must be a long or casted to a long
	 * @return the higher variable.
	 */
	public final static long MaxValue(long lvar1, long lvar2)
	{
		return Math.max(lvar1, lvar2);
	}
	
	
	/*
	 * wraps up the diag method so that we don't have to pass in an instance everytime
	 * 
	 * @param str
	 */
	private static void diag(String str)
	{
		System.out.println("NetLinxUtil: " + str);
	}

	/*
	 * 
	 * @param str
	 */
	private static void error(String str)
	{
		diag("ERROR - " + str);
	}
}

