/*
 * Created on Jul 15, 2005
 * @author VGonzales
 * 
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.util;

import com.amx.duet.core.registry.Registry;
import com.amx.duet.tools.lang.Logger;
/**
 * This class is a wrapper class to simplify Registry editting.
 * 
 * @version 1.0.0
 * @since AMXTools 1.0.0
 * 
 */
public class Reg //extends NetLinx
{
	private Registry registry = new Registry();
	
	private String sAppKey = "";
	private String sCurrentKey = "";
	
	private int nDebug = Logger.LOG_ERROR;
	
	private Object sync = new Object();
	
	/**
	 * Default constructor.
	 *
	 */
	public Reg()
	{
	}
	
	/**
	 * 
	 * @param appkey
	 */
	public Reg(String appkey)
	{
		setAppKey(appkey);
	}
	
	/**
	 * Sets the registry application root key. This is the key that all your other keys will fall under.
	 *   
	 * If the key has not been created, this method will create it for you.
	 * 
	 * @param key String specifying the key path
	 */
	public void setAppKey(String key)
	{
		try
		{
			key = key.replace('\\', '/');
			
			// Get rid of the last "/"
			if (key.endsWith("/"))
				key = key.substring(0, key.length() - 1);
			
			// Create the application registry key or set it as the current key
			// if it is already created.
			registry.createKey(key, false);
			sAppKey = key;
			sCurrentKey = key;
		}
		catch (Exception e)
		{
			if (getDebugState() == Logger.LOG_DEBUG)
				debug("Exception in setAppKey() " + e.getMessage());
			if (getDebugState() == Logger.LOG_DEBUG)
				e.printStackTrace();
		}
	}
	
	/**
	 * Returns the application registry root key, which was created through <code>setAppKey()</code>.
	 * 
	 * @param key String containing the application root key path
	 */
	public String getAppKey()
	{
		return sAppKey;
	}
	
	/**
	 * Sets the <i>currentKey</i> setting for the Registry object to the specified <i>key</i>.
	 * 
	 * The access control values are validated 
	 * using the Registry objects classname and properties supplied at construction.  
	 * 
	 * If the key has not been created, this method will create it for you.
	 *  
	 * @param key a String object containing the desired key path
	 * @return SUCCESS, NO_CURRENT_KEY, INVALID_KEY, FAILURE
	 */
	public int setCurrentKey(String key)
	{
		try
		{
			key = key.replace('\\', '/');
			
			// Get rid of the last "/"
			if (key.endsWith("/"))
				key = key.substring(0, key.length() - 1);
	
			if (key.startsWith(sCurrentKey))
			{
				sCurrentKey = key;
			}
			// Check to see if we're going backwards
			else if (sCurrentKey.startsWith(key))
			{
				if (key.startsWith(sAppKey))
				{
					sCurrentKey = key;
				}
				
				// This should never happen
				else
				{
					return Registry.INVALID_KEY;
				}
				
			}
			else
			{
				if (key.startsWith(sAppKey))
				{
					sCurrentKey = key;
				}
				else
				{
					// Prepend a "/"
					if (!key.startsWith("/"))
						key = "/" + key;
	
					sCurrentKey = sAppKey + key;
				}
			}
			
			return registry.createKey(sCurrentKey, false);
		}
		catch (Exception e)
		{
			if (getDebugState() == Logger.LOG_DEBUG)
				debug("Exception in setCurrentKey() " + e.getMessage());
			if (getDebugState() == Logger.LOG_DEBUG)
				e.printStackTrace();
		}
		
		return Registry.INVALID_KEY;
	}
	
	/**
	 * Returns the <i>currentKey</i> set by calls to <code>setCurrentKey()</code> or <code>setAppKey</code>.
	 * 
	 * @return key a String object containing the current key path
	 */
	public String getCurrentKey()
	{
		return sCurrentKey;
	}
	
	/**
	 * Writes the entry specified by <i>name</i> to <i>value</i>.  If the entry does not exist, 
	 * it is created.
	 * 
	 * @param regKey a String object containing the name to set
	 * @param value a String object containing the value to set
	 * @return true if the key was successfully writen to; false otherwise
	 */
	public boolean writeKey(String regKey, String value)
	{
		debug("writeRegKey(" + regKey + "," + value + ")");
		
		boolean bSuccess = false;
		
		try
		{
			if (registry != null)
			{
				if (registry.setCurrentKey(sCurrentKey, false) == Registry.SUCCESS)
				{
					synchronized (sync)
					{
						if (registry.setValue(regKey, value) == Registry.SUCCESS)
						{
							bSuccess = true;
						}
					}
				}
			}
		}
		catch (Exception e)
		{
			if (getDebugState() == Logger.LOG_DEBUG)
				debug("Exception in writeRegKey() " + e.getMessage());
		}
		
		return bSuccess;
	}
	
	/**
	 * Retrieves an entrys (specified by <i>name</i>) value from the <i>currentKey</i>.
	 * 
	 * @param regKey a String containing the name of the value to retrieve
	 * @return a String object containing the requested value.
	 */
	public String readKey(String regKey)
	{
		debug("readRegKey(" + regKey + ")");
		
		String value = "";
		try
		{
			if (registry != null)
			{
				if (registry.setCurrentKey(sCurrentKey, false) == Registry.SUCCESS)
				{
					value = new String(registry.getValue(regKey));
				}
			}
		}
		catch (Exception e)
		{
			if (getDebugState() == Logger.LOG_DEBUG)
				debug("Exception in readRegKey() " + e.getMessage());
		}
		
		return value;
	}
	
	/**
	 * Removes a <i>key</i> from the registry including all of the keys values and any sub-keys 
	 * and their values.
	 * 
	 * @param regKey a String object containing the key path
	 * @return true if the key was successfully deleted; false otherwise
	 */
	public boolean removeKey(String regKey)
	{
		debug("removeRegKey(" + regKey + ")");
		
		boolean bSuccess = false;
		try
		{
			if (registry != null)
			{
				if (registry.setCurrentKey(sCurrentKey, false) == Registry.SUCCESS)
				{
					synchronized (sync)
					{
						if (registry.deleteKey(regKey, true) == Registry.SUCCESS)
						{
							bSuccess = true;
						}
					}
				}
			}
		}
		catch (Exception e)
		{
			if (getDebugState() == Logger.LOG_DEBUG)
				debug("Exception in removeRegKey() " + e.getMessage());
		}

		return bSuccess;
	}
	
	/**
	 * @deprecated Do not use this method; use setDebugState instead
	 * @param bState
	 */
	public void setDebug(boolean bState)
	{
		if (bState)
			nDebug = Logger.LOG_DEBUG;
		else
			nDebug = Logger.LOG_ERROR;
	}

	/**
	 * Puts the class into a debug diagnostic state.
	 * 
	 * @param nState
	 */
	public void setDebugState(int nState)
	{
		nDebug = nState;
	}
	
	/**
	 * Gets the current diagnostic state.
	 * 
	 * @return
	 */
	public int getDebugState()
	{
		return nDebug;
	}
	
	private void debug(String str)
	{
		if (nDebug > Logger.LOG_WARNING)
			System.out.println("Reg: " + str);
	}
}
