/*
 * Created on May 4, 2005
 * 
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.util;

import com.amx.duet.util.Timer;
import com.amx.duet.util.TimerListener;

/**
 * The WaitTimer class extends the Timer class, which provides API's for creating single value timeouts.  
 * The <code>WaitTimer</code> uses a standard listener model interface, <code>TimerListener</code>.
 * <p>
 * A WaitTimer is created by supplying a <code>TimerListener</code> object, a <i>timeout</i> value in milliseconds and
 * optional <i>repeat</i> value and <i>argument</i> object.  Once a WaitTimer is created an ID is automatically assigned 
 * to it. WaitTimer are <b>NOT</b> started at construction.  The timer commences upon the call to <i>start()</i>.  
 * <p>
 * Upon timeout, the <code>TimerListener</code>'s <i>handleTimerEvent()</i> is called with the <b>Timer</b> 
 * object associated with the timeout passed as a parameter.
 * <p>
 * WaitTimer are cancelled by calling the <i>cancel()</i> method.
 * <p>
 * The main difference between the Timer and the WaitTimer classes is that the WaitTimer keeps track of its started
 * state.  A call to <code>isStarted()</code> will verify if the timer has been started or not.
 * <p>
 * Internally, timers are implemented using a single thread.  Because of this, processing within the
 * <code>TimerListener</code> <i>handleTimerEvent()</i> should be kept to a minimum as this can affect the 
 * accuracy of other timer's in the system.
 * 
 * @version 1.0.0
 * @since AMXTools 1.0.0
 */
public class WaitTimer extends Timer
{
	private static int id = 0;
	private int nTimerID = 0;
	private boolean bStarted = false;
	
	/**
	 * Constructs a one-time, non-repeating timer.
	 * 
	 * @param listener a <i>TimerListener</i> object that will handle timeout events
	 * @param timeout length of timeout in milliseconds
	 */
	public WaitTimer(TimerListener listener, long timeout)
	{
		super(listener, timeout);
		nTimerID = id++;
	}
	
	/**
	 * Constructs a repeatable timer with an optional Object argument.  The argument is not used
	 * for any Timer processing, but is retrievable from the Timer instance through the 
	 * <i>getArgument()</i> API.
	 * 
	 * @param listener a <i>TimerListener</i> object that will handle timer events
	 * @param timeout length of timeout in milliseconds
	 * @param repeat number of times the timer should repeat.  0 indicates no repeat, <i>Timer.INFINITE</i>
	 * indicates the Timer should repeat until cancelled. For example, a repeat value of 1 will cause the 
	 * timer to trigger twice.
	 * @param argument an Object instance that is retained by the Timer object.
	 */
	public WaitTimer(TimerListener listener, long timeout, int repeat, Object argument)
	{
		super(listener, timeout, repeat, argument);
		nTimerID = id++;
	}
	
	/**
	 * Retrieve's the internal ID value assigned to this timer.
	 * 
	 * @return int value containing the timer ID.
	 */
	public int getTimerID()
	{
		return nTimerID;
	}
	
	/**
	 * Cancels a timer.
	 *
	 */
	public void cancel()
	{
		super.cancel();
		if (bStarted)
			bStarted = false;
	}
	
	/**
	 * Starts a timer.  If the timer's timeout value or listener is invalid, false is returned.
	 * 
	 * @return status indicating if the timer was started
	 */
	public boolean start()
	{
		bStarted = super.start();
		return bStarted;
	}
	
	/**
	 * Retrieve's the timer's internal started state.
	 * 
	 * @return status indicating if the timer was started
	 */
	public boolean isStarted()
	{
		return bStarted;
	}
}
