/*
 * @author VGonzales
 * Created on May 23, 2005
 * @since 1.0.0 
 * @version 1.0.0, 05/23/05
 *  
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.xml;

import java.util.Vector;

/**
 * The <code>Tag</code> class was created to hold the properties and attributes of an XML 
 * tag object, but it could also be used to hold tag information for HTML tags and other 
 * user defined tags.
 * 
 * <P>Attributes are stored in a list of {@link Attribute} objects that will store  
 * information for each of the individual attributes.
 * 
 * @version 1.0.2
 * @since AMXTools 1.0.0
 * 
 */
public class Tag
{
	private Vector attributes = new Vector();

	private int columnnumber = 0;
	private int depth = 0;
	private int linenumber = 0;
	private String name = "";
	private String namespace = "";
	private String description = "";
	private String prefix = "";
	private String text = "";
	
	private Object property = null; // place holder for map object
	
	/**
	 * Ensure that we will always have a tag name
	 * @param sName <TT>String</TT>
	 */
	public Tag(String sName)
	{
		setName(sName);
	}
	
	//***************************************************************************
	// Attribute Methods
	//***************************************************************************
	
	/**
	 * Retrieve the number of attributes for this tag
	 * @return <TT>int</TT> containing the number of attributes for this tag
	 */
	public int getAttributeCount() { return attributes.size(); }
	/**
	 * Retrieve the entire list of attributes for this tag
	 * @return <TT>Vector</TT> containing a list of all the attributes for this tag
	 */
	public Vector getAttributes() { return attributes; }
	/**
	 * Retrieve a single attribute at the asking index position for this tag
	 * @param index
	 * @return <code>Attribute</code> object that contains all of the information that 
	 * makes up a single attribute
	 */
	public Attribute getAttribute(int index)
	{
		Attribute attr = null;
		if (attributes.size() > 0)
			attr = (Attribute)attributes.get(index);
		return attr;
	}
	/**
	 * Retrieve a single attribute matching the name for this attribute
	 * @param name Name of the attribute we are searching for 
	 * @return <code>Attribute</code> object that contains all of the information that 
	 * makes up a single attribute
	 */
	public Attribute getAttribute(String attrName)
	{
		for (int i = 0; i < attributes.size(); i++)
		{
			Attribute attr = (Attribute)attributes.get(i);
			if (attr != null)
			{
				if (attrName.compareTo(attr.getName()) == 0)
					return attr;
			}
		}
		return null;
	}
	
	/**
	 * Add an <code>Attribute</code> object to this tag
	 * @param attr <code>Attribute</code>
	 */
	public void addAttribute(Attribute attr) { attributes.add(attr); }
	/**
	 * Clear all of the attributes listed for this tag
	 */
	public void clearAttributes() { attributes.clear(); }
	
	//***************************************************************************
	// General Tag Methods
	//***************************************************************************
	/**
	 * Retrieve the file column number of this tag
	 * @return <TT>int</TT> value of the current column
	 */
	public int getColumnNumber() { return columnnumber; }
	/**
	 * Retrieve the depth count for this tag
	 * @return <TT>int</TT> value of the current depth
	 */
	public int getDepth() { return depth; }
	/**
	 * Retrieve the file line number of this tag
	 * @return <TT>int</TT> value of the current line number
	 */
	public int getLineNumber() { return linenumber; }
	/**
	 * Retrieve the name of this tag
	 * @return <TT>String</TT> value of the tag name
	 */
	public String getName() { return name; }
	/**
	 * Retrieve the name space for this tag
	 * @return <TT>String</TT> value of the tag name space
	 */
	public String getNameSpace() { return namespace; }
	/**
	 * Retrieve a description of the position for this tag
	 * @return <TT>String</TT> containing the description of this position
	 */
	public String getPositionDescription() { return description; }
	/**
	 * Retrieve the prefix value of this tag
	 * @return <TT>String</TT> containing the prefix for this tag
	 */
	public String getPrefix() { return prefix; }
	/**
	 * Retrieve the text contained within start and end of the tag set that this 
	 * <code>Tag</code> object was created from.
	 * @return <TT>String</TT> value of the text within a complete tag set
	 */
	public String getText() { return text; }

	/**
	 * Set the file column number of this tag. This should only be called by the parser.
	 * @param nColumn <TT>int</TT>
	 */
	public void setColumnNumber(int nColumn) { columnnumber = nColumn; }
	/**
	 * Set the XML tag depth count of this tag. This should only be called by the parser.
	 * @param nDepth <TT>int</TT>
	 */
	public void setDepth(int nDepth) { depth = nDepth; }
	/**
	 * Set the file line number of this tag. This should only be called by the parser.
	 * @param nLine <TT>int</TT>
	 */
	public void setLineNumber(int nLine) { linenumber = nLine; }
	/**
	 * Set the name of this tag.
	 * @param sName <TT>String</TT>
	 */
	public void setName(String sName) { name = sName; }
	/**
	 * Set the name space for this tag.
	 * @param sNameSpace <TT>String</TT>
	 */
	public void setNameSpace(String sNameSpace) { namespace = sNameSpace; }
	/**
	 * Set the position description for this tag.
	 * @param sDescription <TT>String</TT>
	 */
	public void setPositionDescription(String sDescription) { description = sDescription; }
	/**
	 * Set the prefix value for this tag. This should only be called by the parser.
	 * @param sPrefix <TT>String</TT>
	 */
	public void setPrefix(String sPrefix) { prefix = sPrefix; }
	/**
	 * Set the tag set text of this tag.
	 * @param sText <TT>String</TT>
	 */
	public void setText(String sText) { text = sText; }

//	/*
//	 * Not implemented yet: will need a map to hold properties - I think
//	 * @param sProperty <TT>String</TT>
//	 * @return <TT>Object</TT>
//	 */
//	private Object getProperty(String sProperty) { return property; }
//	/*
//	 * Not implemented yet: will need a map to hold properties - I think
//	 * @param sProperty <TT>String</TT>
//	 * @param oProperty <TT>Object</TT>
//	 */
//	private void setProperty(String sProperty, Object oProperty) { property = oProperty; }
    /**
	 * Outputs all of the tag data to the print stream
     */
    public void dump()
    {
    	System.out.println("Tag: ============= Tag ==============");
    	System.out.println("Tag: Name:          " + name);
    	System.out.println("Tag: Text:          " + text);
    	System.out.println("Tag: Name space:    " + namespace);
    	System.out.println("Tag: Description:   " + description);
    	System.out.println("Tag: Prefix:        " + prefix);
    	System.out.println("Tag: Column Number: " + columnnumber);
    	System.out.println("Tag: Line Number:   " + linenumber);
    	System.out.println("Tag: Depth:         " + depth);
		for (int i = 0; i < attributes.size(); i++)
		{
			Attribute attr = (Attribute)attributes.get(i);
			if (attr != null)
				attr.dump();
		}
		System.out.println("Tag: ================================");
    }

	public String toString()
	{
		String tag = "-- TAG --" + "\n" +
			"Name:          " + name + "\n" +
			"Text:          " + text + "\n" +
			"Name space:    " + namespace + "\n" +
			"Description:   " + description + "\n" +
			"Prefix:        " + prefix + "\n" +
			"Column Number: " + columnnumber + "\n" +
			"Line Number:   " + linenumber + "\n" +
			"Depth:         " + depth;
		return tag;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj)
	{
		if (!(obj instanceof Tag))
			return false;
		
		Tag tag = (Tag)obj;
		return (this.linenumber == tag.linenumber &&
				this.columnnumber == tag.columnnumber &&
				this.depth == tag.depth &&
				this.name == tag.name &&
				this.text == tag.text &&
				this.namespace == tag.namespace &&
				this.description == tag.description &&
				this.prefix == tag.prefix &&
				this.property == tag.property);
	}
	
}
