/*
 * @author VGonzales
 * 
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.xml;

import java.util.Vector;

import com.amx.duet.tools.lang.Logger;

/**
 * The <code>XMLNode</code> class was created to store the contents of XML files into  
 * <code>Tag</code> and <code>Attribute</code> objects, but it could also be used to 
 * store tag information for HTML tags and other user defined tags.
 * 
 * <P>The <TT>XMLNode</TT> can be traversed foward by looping through the list of nodes 
 * stored at the root, then looping through the individual nodes until <TT>null</TT> is 
 * reached.  An <TT>XMLNode</TT> can also be traversed top-down using the {@link #getParent()}
 * method until <TT>null</TT> is reached, which signifies the "root" level.
 * 
 * <P>A {@link Tag} is stored at the base level of each <TT>XMLNode</TT> in the list
 * 
 * @version 1.0.2
 * @since AMXTools 1.0.0
 * 
 */
public class XMLNode //extends Diag
{
	private Vector nodes = new Vector();
	private Tag data = null;
	private XMLNode parent = null;
	
	private int nDebug = Logger.LOG_ERROR;
	
	/**
	 * 
	 *
	 */
	public XMLNode()
	{
		this(null, null);
	}
	
	/**
	 * Creates an XML node object with the passed in <TT>Tag</TT> as its data
	 * 
	 * @param data the first object in the node
	 */
	public XMLNode(XMLNode nodeParent, Tag data)
	{
		this.parent = nodeParent;
		this.data = data;
	}
	
	/**
	 * Adds and entire sub tree as a leaf to the current node
	 * 
	 * @param node an <TT>XMLNode</TT> object containing its own tags and nodes
	 */
	public void addNode(XMLNode node)
	{
		nodes.add(node);
	}
	
	/**
	 * Takes the passed in <TT>Tag</TT> and adds it adds a leaf to the current node. It then 
	 * returns the <TT>XMLNode</TT> object created from this tag so that user can add and  
	 * retrieve nodes to and from this new leaf.
	 * 
	 * @param tag an XML <TT>Tag</TT> object
	 * @return an <TT>XMLNode</TT> object 
	 */
	public synchronized XMLNode addTag(XMLNode nodeParent, Tag tag)
	{
		XMLNode node = new XMLNode(nodeParent, tag);
		nodes.add(node);
		return node;
	}
	
	/**
	 * Search through nodes on the same level as this leaf at this index
	 * 
	 * @param index position of the leaf at this level
	 * @return an <TT>XMLNode</TT> object 
	 */
	public synchronized XMLNode getNode(int index)
	{
		XMLNode node = null;
		if (nodes.size() > 0)
			node = (XMLNode)nodes.get(index);
		return node;
	}

	/**
	 * Search through nodes on the current level for a matching tag name.
	 * 
	 * @param name
	 * @return an <TT>XMLNode</TT> object 
	 */
	public synchronized XMLNode getNode(String name)
	{
		debug("getNode(String " + name + ")");
		debug("nodes.size() = " + nodes.size());
		XMLNode node = null;
		for (int i = 0; i < nodes.size(); i++)
		{
			node = (XMLNode)nodes.get(i);
			if (node != null)
			{
				Tag tag = node.data;
				if (tag != null && name.equals(tag.getName()))
				{
					debug("tag = " + tag);
					return node;
				}
			}
		}

		return null;
	}
	
	/**
	 * Search through nodes on the current level for a matching tag.
	 * 
	 * @param tag
	 * @return an <TT>XMLNode</TT> object 
	 */
	public synchronized XMLNode getNode(Tag tag)
	{
		if (tag != null)
			debug("getNode(Tag " + tag.getName() + ")");
		
		debug("nodes.size() = " + nodes.size());
		XMLNode node = null;
		for (int i = 0; i < nodes.size(); i++)
		{
			node = (XMLNode)nodes.get(i);
			if (node != null)
			{
				if (tag.equals(node.data))
				{
					debug("tag = " + tag);
					return node;
				}
			}
		}

		return null;
	}
	
	/**
	 * Retrieves the root node of the entire node tree no matter where current node's 
	 * position is.
	 * <p>
	 * <strong>Note:</strong> This method does not retrieve the actual root node, instead it 
	 * retrieves the node stored in the root node, which was the first node that was added 
	 * to the tree.  If for any reason you require the actual root node, you can call 
	 * {@link getParent()} or {@link getAbsoluteRoot()}.
	 * 
	 * @return an <TT>XMLNode</TT> object 
	 */
	public synchronized XMLNode getRoot()
	{
		XMLNode node = null;
		
		if (getData() != null)
		{
			XMLNode root = getParent();
			if (root != null)
				node = root.getRoot();
			else
				node = this;
		}
		else if (nodes.size() == 1)
			node = (XMLNode)nodes.get(0);
				
		return node;
	}

	/**
	 * Retrieves the absolute root node of the entire node tree no matter where current 
	 * node's position is. This is the node where the data and parent are both set to null.
	 * <P>
	 * <strong>Note:</strong> This method is rarely used. Use {@link getRoot()} instead.
	 * 
	 * @return an <TT>XMLNode</TT> object 
	 */
	public synchronized XMLNode getAbsoluteRoot()
	{
		XMLNode node = null;
		
		XMLNode root = getParent();
		if (root != null)
			node = root.getAbsoluteRoot();
		else
			node = this;
				
		return node;
	}
	
	/**
	 * Search through nodes on the same level as this tag at this index
	 * 
	 * @param index position of the tag at this level
	 * @return an XML <TT>Tag</TT> object 
	 */
	public synchronized Tag getTag(int index)
	{
		Tag tag = null;
		XMLNode node = getNode(index);
		if (node != null)
			tag = node.data;
		return tag;
	}
	
	/**
	 * Search through nodes on the same level as the matching named tag
	 * 
	 * @param name of tag to search for
	 * @return an XML <TT>Tag</TT> object 
	 */
	public synchronized Tag getTag(String name)
	{
		debug("getTag(String " + name + ")");
		debug("nodes.size() = " + nodes.size());
		XMLNode node = null;
		for (int i = 0; i < nodes.size(); i++)
		{
			node = (XMLNode)nodes.get(i);
			if (node != null)
			{
				Tag tag = node.data;
				if (tag != null && name.equals(tag.getName()))
				{
					debug("tag = " + tag);
					return tag;
				}
			}
		}
		
		return null;
	}

	/**
	 * Recursive search through the node leaves for the matching named tag
	 * 
	 * @param name of tag to search for
	 * @return an XML <TT>Tag</TT> object 
	 */
	public Tag getNextTag(String name)
	{
		Tag tag = null;
		
		// Loop through the current level first
		tag = getTag(name);
		if (tag == null)
		{
			synchronized (XMLNode.this)
			{
				// If the tag was not found in the current level
				// then check the next level and so on...
				for (int i = 0; i < nodes.size(); i++)
				{
					XMLNode node = (XMLNode)nodes.get(i);
					if (node != null)
					{
						tag = node.getNextTag(name);
						if (tag != null)
							return tag;
					}
				}
				
				if (tag == null)
				{
					Tag temp = getData();
					if (temp != null && name.equals(temp.getName()))
					{
						tag = getData();
						debug("tag = " + tag);
					}
				}
			}
		}

		return tag;
	}
	
	/**
	 * Recursive search through the node leaves for the matching named tag
	 * 
	 * @param name of tag to search for
	 * @return an <TT>XMLNode</TT> object 
	 */
	public XMLNode getNextNode(String name)
	{
		XMLNode node = null;
		
		// Loop through the current level first
		node = getNode(name);
		if (node == null)
		{
			synchronized (XMLNode.this)
			{
				XMLNode listnode = null;
				// If the node was not found in the current level
				// then check the next level and so on...
				for (int i = 0; i < nodes.size(); i++)
				{
					listnode = (XMLNode)nodes.get(i);
					if (listnode != null)
					{
						XMLNode temp = listnode.getNextNode(name);
						if (temp != null)
							return temp;
					}
				}
			}
		}
		
		return node;
	}
	
	/**
	 * Retrieve a list of nodes at this level of the tree that match the node name
	 *  
	 * @param name of tag to search for
	 * @return <TT>Vector</TT> containing a list of all the nodes for this tag
	 */
	public Vector getNodeList(String name)
	{
		Vector list = new Vector();

		synchronized (XMLNode.this)
		{
			for (int i = 0; i < nodes.size(); i++)
			{
				XMLNode node = (XMLNode)nodes.get(i);
				if (node != null)
				{
					Tag tag = node.data;
					if (tag != null)
					{
						if (name.compareTo(tag.getName()) == 0)
							list.add(node);
						
					} // if (tag != null)
					
				} // if (enum != null)
				
			} // for (int j = 0; j < params.size(); j++)
		}
		
		return list;
	}
	
	/**
	 * Recursive search through the node leaves for the matching tag
	 * 
	 * @param tag the Tag to search for
	 * @return an <TT>XMLNode</TT> object 
	 */
	public XMLNode getNextNode(Tag tag)
	{
		XMLNode node = null;
		
		// Loop through the current level first
		node = getNode(tag);
		if (node == null)
		{
			synchronized (XMLNode.this)
			{
				XMLNode listnode = null;
				// If the node was not found in the current level
				// then check the next level and so on...
				for (int i = 0; i < nodes.size(); i++)
				{
					listnode = (XMLNode)nodes.get(i);
					if (listnode != null)
					{
						XMLNode temp = listnode.getNextNode(tag);
						if (temp != null)
							return temp;
					}
				}
			}
		}

		return node;
	}

	/**
	 * Retrieve the parent node for this node. The node will be <TT>null</TT> if this is the 
	 * root node.
	 * 
	 * @return an <TT>XMLNode</TT> object 
	 */
	public synchronized XMLNode getParent()
	{
		return parent;
	}
	
	/**
	 * Retrieves the first <TT>Tag</TT> object from this node
	 *  
	 * @return an XML <TT>Tag</TT> object 
	 */
	public synchronized Tag getData() { return data; }
	
	/**
	 * Retrieve the entire list of nodes for this tag
	 * 
	 * @return <TT>Vector</TT> containing a list of all the nodes for this tag
	 */
	public synchronized Vector getNodes() { return nodes; }
	
	/**
	 * Retrieve the number of nodes for this tag
	 * 
	 * @return <TT>int</TT> containing the number of nodes for this tag
	 */
	public int size() { return nodes.size(); }
	
	/**
	 * Clear all of the nodes listed for this node
	 */
	public void clear() { nodes.clear(); }
	
	/**
	 * Outputs all of the node data to the print stream
	 */
	public void dump()
	{
		for (int i = 0; i < size(); i++)
		{
			XMLNode node = (XMLNode)nodes.get(i);
			if (node != null)
				node.dump();
		}
		
		if (data != null)
		{
			if (parent == null)
				diag("root");
			else if (parent.getData() != null)
				diag("parent = " + parent.getData().getName());
			else
				diag("parent = " + parent);
			data.dump();
		}
	}
	
	/**
	 * @deprecated Do not use this method; use setDebugState instead
	 * @param bState
	 */
	public void setDebug(boolean bState)
	{
		if (bState)
			nDebug = Logger.LOG_DEBUG;
		else
			nDebug = Logger.LOG_ERROR;
	}

	/**
	 * Puts the class into a debug diagnostic state.
	 * 
	 * @param nState
	 */
	public void setDebugState(int nState)
	{
		nDebug = nState;
	}
	
	/**
	 * Gets the current diagnostic state.
	 * 
	 * @return
	 */
	public int getDebugState()
	{
		return nDebug;
	}
	
	private void diag(String str)
	{
		System.out.println("XMLNode: " + str);
	}
	
	private void debug(String str)
	{
		if (nDebug > Logger.LOG_WARNING)
			diag(str);
	}
	
}
