/*
 * @author VGonzales
 * Created on May 23, 2005
 * @since 1.0.0 
 * @version 1.0.0, 05/23/05
 *  
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.xml;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.StringTokenizer;

import org.kxml2.io.KXmlParser;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

import com.amx.duet.tools.lang.Logger;

/**
 * This <code>XMLParser</code> was created as a wrapper class to simplify the parsing of XML files. 
 * The user of this class simply needs to create an <code>XMLParser</code> object (hopefully they
 * passed in some sort of file or io stream processing object), and then just call the {@link #getNextTag()},  
 * {@link #getNextTag(String)}, or {@link #getNextNode(String)} method to get the tag information that 
 * they are searching for.
 * <P>
 * Here are a few examples on how to create an XMLParser object:<br>
 * <pre>
 * XMLParser parser = null;
 * 
 * // You can create the parser object using an existing file
 * parser = new XMLParser("C:\\DEV\\XMLFile.xml");
 *
 * // Or you can create the parser object using an InputStream object
 * ByteArrayInputStream stream = new ByteArrayInputStream(xmlBuffer.toString().getBytes());
 * parser = new XMLParser(stream);
 * 
 * // Or you could create the parser object with the default constructor and start the parser at a later time
 * InputStream stream = new ByteArrayInputStream(xmlBuffer.toString().getBytes());
 * parser = new XMLParser();
 * if (parser != null)
 * 	parser.start(stream);
 * </pre>
 * <P>
 * These are some examples showing how to use the XMLParser object once it has been created:<br>
 * <pre>
 * // This will get the {@link Tag} values matching the "tag_name"
 * if (parser != null)
 * {
 * 	Tag tag = parser.getNextTag("tag_name");
 * 	String text = "";
 * 	if (tag != null)
 * 		text = tag.getText();
 * }
 * 
 * // This will give you the next {@link Tag} in the XML stream from the stream's current position
 * if (parser != null)
 * {
 * 	Tag tag = parser.getNextTag();
 * 	String text = "";
 * 	if (tag != null)
 * 		text = tag.getText();
 * }
 *
 * // This will give you an {@link XMLNode} or a branch of XML tags starting with the one matching your request
 * if (parser != null)
 * {
 * 	XMLNode parserNode = parser.getNextNode("tag_name");
 * 	// At this point you can start traversing the tree starting at the current tag
 * 	if (parserNode != null)
 * 		XMLNode root = parserNode.getRoot();
 * }
 * </pre>
 * <P>
 * <strong>Note:</strong> If you do not want to change the XML tag stream position, you could create the parser object
 * and then get the root tag like so:<br>
 * <pre>
 * // This will allow you to walk the XML tree in both forward and reverse order.
 * 
 * // parser creation routine goes here...
 * 
 * if (parser != null)
 * {
 * 	XMLNode parserNode = parser.getNextNode("first_tag"); // whatever your first tag is
 * 	// At this point you can start traversing the tree starting at the current tag
 * 	if (parserNode != null)
 * 		XMLNode root = parserNode.getRoot();
 * }
 * </pre> 
 * <P>
 * If you want to traverse a tag list straight from the XML stream you could do the following: 
 * <pre>
 * XMLNode nextNode = null;
 * if (parser != null)
 * {
 * 	while ((nextNode = parser.getNextNode("tags")) != null)
 * 	{
 * 		XMLNode tagNode = nextNode.getRoot();
 * 	}
 * }
 * // You cannot go back up the XML stream once you make any XMLParse call.
 * </pre>
 * Otherwise, see {@link XMLNode} for examples on how to traverse the XML node tree.
 * 
 * @version 1.0.2
 * @since AMXTools 1.0.0
 * @see XMLNode
 * @see Tag
 * @see Attribute
 */
public class XMLParser //extends Diag
{
	private XmlPullParser xpp = new KXmlParser();
//	private boolean bStarted = false;
	private Tag currentTag = null;
	private XMLNode xmlTree = new XMLNode();
	private XMLNode subTree = null;
	private String strTreeDepth = "";
	private int currentDepth = 0;
	private int currentIndex = 0;

	private int nDebug = Logger.LOG_ERROR;
	
	/**
	 * Default constructor
	 *
	 */
	public XMLParser()
	{
	}
	
	/**
	 * Initialize the parser to an <TT>InputStream</TT> object
	 * 
	 * @param is <TT>InputStream</TT>
	 */
	public XMLParser(InputStream is) throws XmlPullParserException, IOException 
	{
		start(is);
	}
	
	/**
	 * Creates an XML parser with the <TT>InputStream</TT> taken from <TT>File</TT> 
	 * object that was passed in
	 * 
	 * @param file <TT>File</TT> object pointing to the file to read from
	 * @throws FileNotFoundException
	 */
	public XMLParser(File file) throws FileNotFoundException, XmlPullParserException, IOException
	{
		this(new FileInputStream(file));
	}
	
	/**
	 * Creates an XML parser with the <TT>InputStream</TT> taken from <TT>File</TT> 
	 * object that was opened from the file path passed in.
	 * 
	 * @param sFilePath <TT>String</TT> of the file path to read a file from
	 * @throws FileNotFoundException
	 */
	public XMLParser(String sFilePath) throws FileNotFoundException, XmlPullParserException, IOException
	{
		this(new File(sFilePath));
	}
	
	/**
	 * Initialize the parser to an <TT>InputStream</TT> object
	 * 
	 * @param is <TT>InputStream</TT>
	 * @return <TT>int</TT>
	 */
	public int start(InputStream is) throws XmlPullParserException, IOException
	{
//		this.enableDebug(true);
		xpp.setFeature(XmlPullParser.FEATURE_PROCESS_NAMESPACES, true);
		xpp.setInput(is, null);
//		bStarted = true;

		parse();
		
		return 0;
	}

	/*
	 * start by opening the document and skipping all of the header info.
	 * loop through all of the tags checking for the start and end of tag.
	 * get the attribute values and text value.
	 * Fill an XML object with the attributes and text.
	 * 
	 * @throws XmlPullParserException
	 * @throws IOException
	 */
	private void parse() throws XmlPullParserException, IOException
	{
		if (xpp == null)
			return;
		
		int eventType = xpp.getEventType();
		
		if (eventType == XmlPullParser.END_DOCUMENT)
			return;

		synchronized (XMLParser.this)
		{
			do // while (!found && eventType != XmlPullParser.END_DOCUMENT);
			{ 
				switch (eventType)
				{
					case XmlPullParser.START_DOCUMENT:
					{
//						debug("START_DOCUMENT");
						break;
					}
					case XmlPullParser.END_DOCUMENT:
					{
//						debug("END_DOCUMENT");
						break;
					}
					case XmlPullParser.START_TAG:
					{
//						debug("START_TAG = " + xpp.getName());
						String tagName = xpp.getName();
						currentTag = new Tag(tagName);
						
						currentTag.setColumnNumber(xpp.getColumnNumber());
						currentTag.setDepth(xpp.getDepth());
						currentTag.setLineNumber(xpp.getLineNumber());
						currentTag.setNameSpace(xpp.getNamespace());
						currentTag.setPositionDescription(xpp.getPositionDescription());
						currentTag.setPrefix(xpp.getPrefix());

						int count = xpp.getAttributeCount();
						for (int i = 0; i < count; i++)
						{
							Attribute attr = new Attribute(xpp.getAttributeName(i));
							attr.setNameSpace(xpp.getAttributeNamespace(i));
							attr.setPrefix(xpp.getAttributePrefix(i));
							attr.setType(xpp.getAttributeType(i));
							attr.setValue(xpp.getAttributeValue(i));
							currentTag.addAttribute(attr);
						}

						// Set the previous node as this node's parent, then
						// add the tag to the node, and then add it as a node
						// to the root node
						xmlTree = xmlTree.addTag(xmlTree, currentTag);
						break;
					}
					case XmlPullParser.END_TAG:
					{
//						debug("END_TAG");
						// If the proper end tag was found then come back down the tree
						if (xmlTree != null)
							xmlTree = xmlTree.getParent();
						break;
					}
					case XmlPullParser.TEXT:
					{
						String text = xpp.getText();
//						debug("TEXT = " + text);
						
						if (currentTag != null)
						{
							// Make sure we're not putting "null" or 
							// empty whitespace into the tag text.
//							text = xpp.getText();
							if (text != null)
								text = text.trim();
							else
								text = "";
							
							if (text != null && text.length() > 0)
								currentTag.setText(text);
							
//							debug("currentTag = " + currentTag.getName());
						}
						
						break;
					}
					default:
					{
//						debug("EVENT_TYPE = " + eventType);
						break;
					}
				}
				
				eventType = xpp.next();
				
			}
			while (eventType != XmlPullParser.END_DOCUMENT);
			
			// Set the walk tree to the current parse tree and keep the original
			subTree = xmlTree;
			
		} // synchronized (sync)
	}

	/**
	 * 
	 * @return
	 */
	public XMLNode getRootNode()
	{
		return subTree = xmlTree;
	}
	
	/**
	 * Get the next set of tags in the XML file.
	 * <P>
	 * <strong>Note:</strong> This method updates the xml stream position.
	 * 
	 * @return <code>XMLNode</code> containg all of the information between the start and end node.
	 */
	public XMLNode getNextNode()
	{
//		debug("getNextNode()");
		if (subTree != null)
		{
			if (currentIndex < subTree.size())
			{
				XMLNode node = getListNode(currentIndex);
				if (node != null)
					return node;
				return getNextListNode();
			}

			// Step back until we find a new node
			subTree = getNextListNode();
			if (subTree != null)
			{
				currentIndex = 0;
				return subTree;
			}
			debug("End of file");
		}
		
		return null;
	}

	/**
	 * Get the next set of tags in the XML file that match the tag name passed in.
	 * <P>
	 * <strong>Note:</strong> This method updates the xml stream position.
	 * 
	 * @param sTagName <TT>String</TT> name to search for.
	 * @return <code>XMLNode</code> containg all of the information between the start and end node.
	 */
	public XMLNode getNextNode(String sTagName)
	{
		Tag tag = null;
		XMLNode node = null;
		while ((node = getNextNode()) != null)
		{
			tag = node.getData();
			if (tag != null)
			{
				if (sTagName.compareTo(tag.getName()) == 0)
					return node;
			}
		}
		
		return null;
	}

	/**
	 * This method traverses the XML tree from top to bottom until it finds the 
	 * matching tag name. 
	 * <P>
	 * <B>NOTE: </B>This is a slow method due to the process always starting 
	 * from the ROOT node of the XML tree and working its way down through all 
	 * of its leafs until it finds the matching text.
	 * <P>
	 * If it does not find the matching tag the XML tree pointer will be set
	 * to null. In this case, simply call {@link #getRootNode()} to restart the 
	 * tree traversal.
	 * 
	 * @param sTagName <TT>String</TT> name to search for.
	 * @return <code>XMLNode</code> containg all of the information between the start and end node.
	 */
	public XMLNode getNode(String sTagName)
	{
		Tag tag = null;
		XMLNode node = getRootNode();
		while ((node = getNextNode()) != null)
		{
			tag = node.getData();
			if (tag != null)
			{
				if (sTagName.compareTo(tag.getName()) == 0)
					return node;
			}
		}
		
		return null;
	}
	
	/**
	 * Get the next tag in the XML file.
	 * <P>
	 * <strong>Note:</strong> This method updates the xml stream position.
	 * 
	 * @return <code>Tag</code> containg all of the information between the start and end tags.
	 */
	public Tag getNextTag()
	{
		Tag tag = null;
		XMLNode node = null;
		while ((node = getNextNode()) != null)
		{
			tag = node.getData();
			if (tag != null)
				return tag;
		}
		return null;
	}

	/**
	 * Get the next tag in the XML file that matches the tag name passed in.
	 * <P>
	 * <strong>Note:</strong> This method updates the xml stream position.
	 * 
	 * @param sTagName <TT>String</TT> name to search for.
	 * @return <code>Tag</code> containg all of the information between the start and end tags.
	 */
	public Tag getNextTag(String sTagName)
	{
		Tag tag = null;
		XMLNode node = null;
		while ((node = getNextNode()) != null)
		{
			tag = node.getData();
			if (tag != null)
			{
				if (sTagName.compareTo(tag.getName()) == 0)
					return tag;
			}
		}
		return null;
	}

	/**
	 * This method traverses the XML tree from top to bottom until it finds the 
	 * matching tag name. 
	 * <P>
	 * <B>NOTE: </B>This is a slow method due to the process always starting 
	 * from the ROOT node of the XML tree and working its way down through all 
	 * of its leafs until it finds the matching text.
	 * <P>
	 * If it does not find the matching tag the XML tree pointer will be set
	 * to null. In this case, simply call {@link #getRootNode()} to restart the 
	 * tree traversal.
	 * 
	 * @param sTagName <TT>String</TT> name to search for.
	 * @return <code>Tag</code> containg all of the information between the start and end tags.
	 */
	public Tag getTag(String sTagName)
	{
		Tag tag = null;
		XMLNode node = getRootNode();
		while ((node = getNextNode()) != null)
		{
			tag = node.getData();
			if (tag != null)
			{
				if (sTagName.compareTo(tag.getName()) == 0)
					return tag;
			}
		}
		return null;
	}
	
	/*
	 * This method recursively searches for the next node in the tree. It works 
	 * its way down starting from the first node, to the first node's first 
	 * child, to the child's first child and so on, returning a node for 
	 * each call made until it reaches a leaf. Once it reaches a leaf it will 
	 * walk the tree backwards once and check to see if there is another node 
	 * within the previous node list. It will basically walk back and forth 
	 * for each call until it has returned all of the leaves for this node 
	 * (array walking). After it has reached all of this node's leaves it will 
	 * step back once again and see if there is another set of nodes to traverse 
	 * at this position.
	 * 
	 * @return
	 */
	private XMLNode getNextListNode()
	{
//		debug("getNextListNode()");
		if (subTree != null)
		{
			decrementDepth();
			currentIndex = getDepthIndex();
			subTree = subTree.getParent();
			if (subTree != null && (subTree.size()-1) > currentIndex)
			{
				currentIndex = getDepthIndex();
				subTree = getListNode(++currentIndex);
				if (subTree != null)
				{
					resetDepthIndex(currentIndex);
					return subTree;
				}
				return getNextListNode();
			}
			return getNextListNode();
		}
		
		return null;
	}
	
	/*
	 * See if we can find and return a node in the current index of this sub 
	 * tree. The tree is an n-ary tree which means there is an array of unknown 
	 * size of nodes for each and every node. This method tries to access an 
	 * index of the array for the current sub tree.
	 * 
	 * @param index
	 * @return
	 */
	private XMLNode getListNode(int index)
	{
//		debug("getListNode(" + index + ")");
		XMLNode node = null;
		if (subTree != null)
		{
			if (index < subTree.size())
			{
				node = subTree.getNode(index);
				if (node != null)
				{
					subTree = node;
//					if (bIncrement)
						incrementDepth(index);
//					else
//						setDepthIndex(index);
				}
			}
		}
		return node;
	}
	
	/*
	 * 
	 * @param index
	 */
	private void incrementDepth(int index)
	{
		currentDepth++;
		String[] depths = parse(strTreeDepth, "|");
		if (depths != null && depths.length > 0 && currentDepth < depths.length)
			setDepthIndex(index);
		else
		{
			if (strTreeDepth.length() == 0)
			{
				try
				{
					strTreeDepth = Integer.toString(index);
				}
				catch (Exception e) {}
			}
			else
				strTreeDepth += "|" + index;
		}
//		debug("incrementDepth(" + index + "): " + strTreeDepth);
	}
	
	/*
	 * 
	 *
	 */
	private void decrementDepth()
	{
		if (currentDepth > 0)
			currentDepth--;
//		debug("decrementDepth(): " + currentDepth);
	}
	
	/*
	 * 
	 * @return
	 */
	private int getDepthIndex()
	{
		int index = Integer.MIN_VALUE;
		String[] depths = parse(strTreeDepth, "|");
		if (depths != null && depths.length > 0 && currentDepth < depths.length)
		{
			try
			{
				index = Integer.parseInt(depths[currentDepth]);
			}
			catch (Exception e) {}
		}
//		debug("getDepthIndex(): " + index);
		return index;
	}
	
	/*
	 * 
	 * @param index
	 */
	private void setDepthIndex(int index)
	{
		String[] depths = parse(strTreeDepth, "|");
		if (depths != null && depths.length > 0 && currentDepth < depths.length)
		{
			try
			{
				depths[currentDepth] = Integer.toString(index);
			}
			catch (Exception e) {}
			strTreeDepth = "";
			for (int i = 0; i < depths.length; i++)
			{
				if (i == 0)
					strTreeDepth = depths[i];
				else
					strTreeDepth += "|" + depths[i];
			}
		}
//		debug("setDepthIndex(" + index + "): " + strTreeDepth);
	}

	/*
	 * 
	 * @param index
	 */
	private void resetDepthIndex(int index)
	{
		String[] depths = parse(strTreeDepth, "|");
		if (depths != null && depths.length > 0 && currentDepth < depths.length && currentDepth > 0)
		{
			try
			{
				depths[currentDepth-1] = Integer.toString(index);
			}
			catch (Exception e) {}
			for (int i = (currentDepth); i < depths.length; i++)
				depths[i] = "0";
			
			strTreeDepth = "";
			for (int i = 0; i < depths.length; i++)
			{
				if (i == 0)
					strTreeDepth = depths[i];
				else
					strTreeDepth += "|" + depths[i];
			}
		}
//		debug("resetDepthIndex(" + index + "): " + strTreeDepth);
	}
	
//	/*
//	 * 
//	 * @return
//	 */
//	private XMLNode getParentNode()
//	{
//		debug("getParentNode()");
//		XMLNode node = null;
//		if (subTree != null)
//		{
//			decrementDepth();
//			return subTree.getParent();
//		}
//		
//		return null;
//	}
	
//	/*
//	 * 
//	 * @return
//	 */
//	private XMLNode getNextListNode()
//	{
//		debug("getNextListNode()");
//		XMLNode node = null;
//		
//		subTree = getParentNode();
//		if (subTree != null)
//		{
//			currentIndex = getDepthIndex();
//			node = getListNode(++currentIndex, true);
//		}
//		else
//		{
//			debug("getNextListNode(): subTree is NULL!!!!!");
//		}
//		
//		return node;
//	}
	
	/**
	 * @deprecated Do not use this method; use setDebugState instead
	 * @param bState
	 */
	public void setDebug(boolean bState)
	{
		if (bState)
			nDebug = Logger.LOG_DEBUG;
		else
			nDebug = Logger.LOG_ERROR;
	}

	/**
	 * Puts the class into a debug diagnostic state.
	 * 
	 * @param nState
	 */
	public void setDebugState(int nState)
	{
		nDebug = nState;
	}
	
	/**
	 * Gets the current diagnostic state.
	 * 
	 * @return
	 */
	public int getDebugState()
	{
		return nDebug;
	}
	
	private void diag(String str)
	{
		System.out.println("XMLParer: " + str);
	}
	
	private void debug(String str)
	{
		if (nDebug > Logger.LOG_WARNING)
			diag(str);
	}
	
	private final static String[] parse(String data, String delim)
	{
		StringTokenizer tokenizer = new StringTokenizer(data, delim);
		String value[] = new String[tokenizer.countTokens()];
		int count = 0;
		
		while (tokenizer.hasMoreTokens()) 
		{
			value[count++] = tokenizer.nextToken();
		}
		return value;
	}
}
